/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.rational.connector.hudson.ui;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import com.ibm.rational.connector.hudson.internal.client.IHudsonServiceClient;
import com.ibm.rational.connector.hudson.internal.common.HudsonConfigurationData;
import com.ibm.rational.connector.hudson.internal.common.HudsonConfigurationElement;
import com.ibm.rational.connector.hudson.internal.common.IHudsonConstants;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.builddefinition.BuildDefinitionEditor;
import com.ibm.team.build.ui.editors.builddefinition.AbstractConfigurationElementEditor;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Creates the UI contributor that implements the Hudson Build Definition.
 * 
 * @author Spencer Murata
 */
@SuppressWarnings("restriction")
public class HudsonConfigurationEditor extends
		AbstractConfigurationElementEditor {

	private static final String JOB = "job"; //$NON-NLS-1$
	private static final String SLASH = "/"; //$NON-NLS-1$
	private static final String JOB_NAME_FORMAT = "{0} (job/{1})"; //$NON-NLS-1$
	private FormToolkit fToolkit;
	private BuildDefinitionEditor fEditor;
	private IBuildDefinition fBuildDefinitionWorkingCopy;
	private HudsonConfigurationData fConfigurationData;
	private Table fHudsonJobTable;
	private Button fHudsonGetJobs;
	private Status fSaveStatus;
	private Text fHudsonJobFilter;
	private IBuildDefinition[] currentJobList;
	private boolean fIgnoreFilter = true;

	/**
	 * Creates the editor object.
	 * 
	 * @param elementId
	 * @param title
	 */
	public HudsonConfigurationEditor(String elementId, String title) {
		super(elementId, title);
	}

	@Override
	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	public void setWorkingCopy(IBuildDefinition definition) {
		fConfigurationData = new HudsonConfigurationData(definition);
		fBuildDefinitionWorkingCopy = definition;
	}

	@Override
	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	public void createContent(Composite parent, FormToolkit toolkit) {
		fToolkit = toolkit;
		parent.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL_GRAB));
		TableWrapLayout twl = new TableWrapLayout();
		twl.numColumns = 2;
		twl.makeColumnsEqualWidth = true;
		twl.horizontalSpacing = 10;
		parent.setLayout(twl);

		Section hudsonJobSection = fToolkit.createSection(parent, Section.DESCRIPTION
				| ExpandableComposite.TITLE_BAR);
		hudsonJobSection.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB));
		hudsonJobSection.setLayout(new TableWrapLayout());
		hudsonJobSection
				.setText(Messages.HudsonConfigurationEditor_LABEL_JOB_SECTION);
		hudsonJobSection
				.setDescription(NLS.bind(Messages.HudsonConfigurationEditor_DESC_JOB_SECTION, IHudsonConstants.PROPERTY_BUILD_RESULT_UUID));

		Composite jobComposite = fToolkit.createComposite(hudsonJobSection);
		jobComposite.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL_GRAB));
		TableWrapLayout projLayout = new TableWrapLayout();
		projLayout.numColumns = 1;
		projLayout.horizontalSpacing = 10;
		jobComposite.setLayout(projLayout);

		createHudsonJobWidgets(jobComposite);
		createHudsonGetJobWidgets(jobComposite);

		hudsonJobSection.setClient(jobComposite);

	}

	/**
	 * Creates the Job widgets in the given composite
	 * 
	 * @param jobComposite
	 */
	private void createHudsonJobWidgets(Composite jobComposite) {
		String job = getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_JOB, ""); //$NON-NLS-1$
		String displayName = simplifyJobUrl(job); //$NON-NLS-1$

		createSpacer(jobComposite, 5, 1);
		fHudsonJobFilter = fToolkit.createText(jobComposite, Messages.HudsonConfigurationEditor_FILTER_TEXT);
		fHudsonJobFilter.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		fHudsonJobFilter.setEnabled(false);
		fHudsonJobFilter.addKeyListener(getHudsonFilterKeyListener());
		fHudsonJobFilter.addFocusListener(getHudsonFilterFocusListener());		
		fHudsonJobTable = fToolkit.createTable(jobComposite, SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		TableWrapData layoutData = new TableWrapData(
				TableWrapData.FILL_GRAB);
		layoutData.heightHint = fHudsonJobTable.getItemHeight() * 10;
		fHudsonJobTable.setLayoutData(layoutData);

		if (!job.trim().equalsIgnoreCase("")) { //$NON-NLS-1$
			TableItem tableItem = new TableItem(fHudsonJobTable, 0);
			tableItem.setData(HudsonConfigurationElement.PROPERTY_HUDSON_JOB, job);
			if (displayName.equals(job)) {
				tableItem.setText(displayName);
			} else {
				tableItem.setText(NLS.bind(JOB_NAME_FORMAT, displayName, job));
			}
			fHudsonJobTable.setSelection(0);
		}

		fHudsonJobTable.addSelectionListener(getHudsonJobSelectionListener());
		Label descriptionLabel = fToolkit
				.createLabel(jobComposite,
						Messages.HudsonConfigurationEditor_LABEL_DESCRIPTION,
						SWT.WRAP);
		descriptionLabel.setLayoutData(new TableWrapData(TableWrapData.FILL));
		descriptionLabel.setForeground(Display.getDefault().getSystemColor(
				SWT.COLOR_DARK_GRAY));

	}

	/**
	 * Remove some segments from the url to simplify what the user sees.
	 * @param job The url of the job
	 * @return a display version of the url
	 */
	private String simplifyJobUrl(String job) {
		String[] segments = job.split(SLASH);
		if (segments.length > 1) {
			String displayName = ""; //$NON-NLS-1$
			for (int i=0; i<segments.length; i++) {
				// if i is odd and the segment is "job" strip it out to beautify Cloud Bees folder urls
				if (i/2*2 == i) {
					// i is even so include the segment automatically
					displayName += segments[i] + SLASH;
				} else if (!segments[i].equals(JOB)) {
					// surprising, so include it in the name
					displayName += segments[i] + SLASH;
				}
			}
			displayName = displayName.substring(0, displayName.length()-1);
			return displayName;
		} else {
			return job;
		}
	}

	/**
	 * Creates and returns the listener for the Job List box.
	 * 
	 * @return SelectionListener
	 */
	private SelectionListener getHudsonJobSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean clearMessages = true;
				String currentJob = getConfigProperty(
						HudsonConfigurationElement.PROPERTY_HUDSON_JOB, null);
				if (validate()) {
					TableItem[] tableItem = fHudsonJobTable.getSelection();
					if (tableItem.length != 1) {
						addErrorMessage(
								fHudsonGetJobs,
								Messages.HudsonConfigurationEditor_ERROR_SELECT_ONLY_ONE_JOB,
								fHudsonGetJobs);
						clearMessages = false;
					} else {
						currentJob = (String) tableItem[0].getData(HudsonConfigurationElement.PROPERTY_HUDSON_JOB);
					}
					// if the properties are not there don't add them with a value of null.
					if (currentJob != null) {
    					setConfigProperty(
    							HudsonConfigurationElement.PROPERTY_HUDSON_JOB,
    							currentJob);
    					setDirty(true);

    					// If its not our plugin on the other end it could be due to an older version of the plugin; user is still setting
    					// things up or they are not using tight integration. So no warning.
    					// If the pluginBuildDefinition is not set it could be due to an older version of the plugin (it is not exposed
    					// in 1.1.8 and prior); user is still setting things up or its pointing to a workspace. We will warn. I am not so worried
    					// about an earlier plugin version since the plugin changes will be out 4 months before the client changes.
    					// Also its just a warning - doesn't prevent a save. Its supposed to help the user figure out their possible mistakes.
    					Boolean pluginConfigured = Boolean.valueOf((String) tableItem[0].getData(IHudsonConstants.PLUGIN_CONFIGURED));
    					String pluginBuildDefinition = (String) tableItem[0].getData(IHudsonConstants.PLUGIN_BUILD_DEFINITION);
    					if (pluginConfigured && (pluginBuildDefinition == null || !pluginBuildDefinition.equals(fBuildDefinitionWorkingCopy.getId()))) {
    						addWarningMessage(fHudsonGetJobs, Messages.HudsonConfigurationEditor_DIFFERENT_JOB_CONFIG, fHudsonGetJobs);
    						clearMessages = false;
    					}
					}
					if (clearMessages) {
						removeMessage(fHudsonGetJobs, fHudsonGetJobs);
					}
				}
			}
		};
	}
	
    @Override
	public void performExtraSave(IProgressMonitor monitor) {
        // When the build definition is saved, add the buildResultUUID parameter to the Hudson/Jenkins job
		// no status unless a save of the H/J job was done
    	fSaveStatus = null;
    	try {
    		String jobName = getConfigProperty(HudsonConfigurationElement.PROPERTY_HUDSON_JOB, null);
    		if (jobName != null) {
    			boolean jobChanged = parameterizeHudsonJob(getFirstHudsonEngine(), jobName, monitor);
    			if (jobChanged) {
    			    fSaveStatus = new Status(IStatus.WARNING, BuildUIPlugin.getUniqueIdentifier(), 
    			            NLS.bind(Messages.HudsonConfigurationEditor_PARAMETER_ADDED,
    			                    jobName, IHudsonConstants.PROPERTY_BUILD_RESULT_UUID));
    			}
    		}
        } catch (TeamRepositoryException tre) {
            String message;
            if (tre.getCause() != null && tre.getCause().getMessage() != null) {
                message = NLS.bind(Messages.HudsonConfigurationEditor_ERROR_ADDING_PARAMETER, tre.getCause().getMessage());
            } else {
            	message = NLS.bind(Messages.HudsonConfigurationEditor_ERROR_ADDING_PARAMETER, tre.getMessage());
            }
            fSaveStatus = new Status(Status.ERROR, BuildUIPlugin.getUniqueIdentifier(), message, tre);
            BuildUIPlugin.log(fSaveStatus);
        }
    }

	protected boolean parameterizeHudsonJob(IBuildEngine engine, String jobName, IProgressMonitor monitor) throws TeamRepositoryException {

		IHudsonServiceClient hudsonClient = (IHudsonServiceClient) getTeamRepository()
				.getClientLibrary(IHudsonServiceClient.class);
		return hudsonClient.parameterizeHudsonJob(engine, jobName, monitor);
	}

	
	@Override
	public void postSave(boolean saveSucceeded) {
		if (fSaveStatus != null) {
    		// Tell the user if we changed the H/J job or if there were problems
            final IWorkbenchPartSite site = getSite();
            Display.getDefault().asyncExec(new Runnable() {
                public void run() {
                	if (!site.getShell().isDisposed()) {
						if (fSaveStatus.getSeverity() == IStatus.ERROR) {
							MessageDialog.openError(
									site.getShell(),
									Messages.HudsonConfigurationEditor_JOB_CHANGE_ERROR_TITLE,
									fSaveStatus.getMessage());
						} else {
							MessageDialog.openInformation(site.getShell(),
									Messages.HudsonConfigurationEditor_JOB_CHANGE_TITLE, fSaveStatus.getMessage());
						}
                	}
            		fSaveStatus = null;
                }
            });
		}
	}

	/**
	 * Creates the Get Jobs button in the given composite.
	 * 
	 * @param jobComposite
	 */
	private void createHudsonGetJobWidgets(Composite jobComposite) {
		createSpacer(jobComposite, 5, 1);

		fHudsonGetJobs = fToolkit.createButton(jobComposite,
				Messages.HudsonConfigurationEditor_LABEL_GET_JOBS, SWT.PUSH);
		fHudsonGetJobs
				.setToolTipText(Messages.HudsonConfigurationEditor_DESC_GET_JOBS);
		fHudsonGetJobs
				.setText(Messages.HudsonConfigurationEditor_LABEL_GET_JOBS);
		fHudsonGetJobs
				.addSelectionListener(getHudsonGetJobsSelectionListener());

		Label getProjectsLabel = fToolkit.createLabel(jobComposite,
				Messages.HudsonConfigurationEditor_DESC_GET_JOBS, SWT.WRAP);
		getProjectsLabel.setLayoutData(new TableWrapData(TableWrapData.FILL));
		getProjectsLabel.setForeground(Display.getDefault().getSystemColor(
				SWT.COLOR_DARK_GRAY));

	}

	/**
	 * Creates and returns the listener for the Get Jobs button. It will check
	 * that a Hudson Engine is associated to the Build Definition, clear the
	 * existing job list, and call the getJobs() function.
	 * 
	 * @return SelectionListener
	 */
	private SelectionListener getHudsonGetJobsSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (validateHudsonEngineExists()) {
					if (fHudsonJobTable.getItemCount() > 0) {
						fHudsonJobTable.removeAll();
					}
					if(!fHudsonJobFilter.getEnabled()) {
						fHudsonJobFilter.setEnabled(true);
					}
					getJobs();
				}
			}
		};
	}
	
	/**
	 * Creates and returns a KeyListener for the filter text box.
	 * When a key is released, the table is populated with jobs that
	 * match the filter text.
	 * 
	 * @return KeyListener
	 */
	
	private KeyListener getHudsonFilterKeyListener() {
		return new KeyListener() {
			@Override
			public void keyPressed(KeyEvent e) {
				// Does nothing
				
			}
			@Override
			public void keyReleased(KeyEvent e) {
				fHudsonJobTable.removeAll();
				buildJobTable();			
			}
		};
	}
	
	/**
	 * Creates and returns a FocusListener for the filter text box.
	 * When focus is gained, an ignoreFilter flag is set to false and the 
	 * contents of the filter is set no an empty string. When focus is lost,
	 * if the contents is empty, the text is set to default filter text and an 
	 * ignoreFilter flag is set to true.
	 * 
	 * @return FocusListener
	 */
	
	private FocusListener getHudsonFilterFocusListener() {
		return new FocusListener() {

			@Override
			public void focusGained(FocusEvent e) {
				if (fIgnoreFilter) {
					fIgnoreFilter = false;
					fHudsonJobFilter.setText("");
				}
			}

			@Override
			public void focusLost(FocusEvent e) {
				if (fHudsonJobFilter.getText() == null || fHudsonJobFilter.getText().isEmpty()) {
					fIgnoreFilter = true;
					fHudsonJobFilter.setText(Messages.HudsonConfigurationEditor_FILTER_TEXT);
				}			
			}
		};
	}

	/**
	 * Gets the Jobs on the Hudson server and puts them in the Hudson Job box
	 * on the UI. It will also highlight the currently selected Job.
	 */
	protected void getJobs() {

		IHudsonServiceClient hudsonClient = (IHudsonServiceClient) getTeamRepository()
				.getClientLibrary(IHudsonServiceClient.class);
		currentJobList = null;
		
		try {
			currentJobList = hudsonClient.getJobs(
					(IBuildEngine[]) fEditor.getSupportingEngines().toArray(
							new IBuildEngine[fEditor.getSupportingEngines()
									.size()]), null);
		} catch (TeamRepositoryException e) {
			String message;
			if (e.getCause() != null && e.getCause().getMessage() != null) {
				message = NLS.bind(Messages.HudsonConfigurationEditor_ERROR_RETRIEVING_JOBS, e.getCause().getMessage());
			} else {
				message = NLS.bind(Messages.HudsonConfigurationEditor_ERROR_RETRIEVING_JOBS, e.getMessage());
			}
			addErrorMessage(fHudsonGetJobs, message, fHudsonGetJobs);
			BuildUIPlugin.log(new Status(Status.ERROR, BuildUIPlugin.getUniqueIdentifier(), message, e));
		}
		buildJobTable();
		
	}
	
	/**
	 * Generates the Job Table.  This will populate the table with the names of the Hudson/Jenkins jobs and 
	 * filter the list if there are contents in the filter text field.
	 */

	protected void buildJobTable() {
		if (currentJobList == null) {
			return;
		}

		for (int i = 0; i < currentJobList.length; i++) {
			IBuildDefinition currBuildDef = currentJobList[i];
			String jobName = currBuildDef.getId();
			String displayName = currBuildDef.getDescription();
			String buildDefinition = currBuildDef.getPropertyValue(IHudsonConstants.PLUGIN_BUILD_DEFINITION, null);
			String pluginConfigured = currBuildDef.getPropertyValue(IHudsonConstants.PLUGIN_CONFIGURED, "false"); //$NON-NLS-1$
			
			if (fIgnoreFilter || displayName.toLowerCase().contains(fHudsonJobFilter.getText().toLowerCase())) {
				TableItem ti = new TableItem(fHudsonJobTable, SWT.NULL);
				if (displayName.equals(jobName)) {
					ti.setText(displayName);
				} else {
					ti.setText(NLS.bind(JOB_NAME_FORMAT, displayName, jobName));
				}
				ti.setData(HudsonConfigurationElement.PROPERTY_HUDSON_JOB, jobName);
				ti.setData(IHudsonConstants.PLUGIN_BUILD_DEFINITION, buildDefinition);
				ti.setData(IHudsonConstants.PLUGIN_CONFIGURED, pluginConfigured);
				String configurationProjectName = fConfigurationData.getJobName();
				if (configurationProjectName != null) {
					if (configurationProjectName.equals(jobName)) {
						fHudsonJobTable.setSelection(i);
					}
				}
			}
		}
		removeMessage(fHudsonGetJobs, fHudsonGetJobs);
	}

	/**
	 * Helper method that will verify that a Hudson engine is associated to the
	 * Build Definition.
	 * 
	 * @return boolean
	 */
	protected boolean validateHudsonEngineExists() {
		IBuildEngine buildEngine = getFirstHudsonEngine();
		if (buildEngine == null) {
			MessageDialog
					.openError(
							getSite().getShell(),
							Messages.HudsonConfigurationEditor_ERROR_DIALOG_NO_ENGINE_FOUND_TITLE,
							Messages.HudsonConfigurationEditor_ERROR_DIALOG_NO_ENGINE_FOUND_DESC);
			return false;
		}
		return true;
	}

	/**
	 * Helper method that will verify that a Hudson engine is associated to the
	 * Build Definition.
	 * 
	 * @return boolean
	 */
	protected boolean validateHudsonProjectSelected() {
		
		if (fHudsonJobTable.getSelectionIndex() < 0) {
			MessageDialog
					.openError(
							getSite().getShell(),
							Messages.HudsonConfigurationEditor_ERROR_DIALOG_NO_PROJECT_SELECTED_TITLE,
							Messages.HudsonConfigurationEditor_ERROR_DIALOG_NO_PROJECT_SELECTED_DESC);
			return false;
		}
		return true;
	}
	
	/**
	 * Helper method that iterates through the engines associated to the Build
	 * Definition and returns the first Hudson one it finds.
	 * 
	 * @return IBuildEngine
	 */
	private IBuildEngine getFirstHudsonEngine() {
		List<IBuildEngine> engines = fEditor.getSupportingEngines();
		if (!engines.isEmpty()) {
			for (IBuildEngine engine : engines) {
				if (engine
						.getConfigurationElement(HudsonConfigurationElement.ENGINE_ELEMENT_ID) != null) {
					return engine;
				}
			}
		}
		return null;
	}

	@Override
	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	public boolean validate() {
		return validateHudsonEngineExists() && validateHudsonProjectSelected();
	}

	@Override
	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	public void initialize(BuildDefinitionEditor editor) {
		super.initialize(editor);
		fEditor = editor;
	}

	/**
	 * Returns a string configuration property if set, or the default value
	 * otherwise.
	 * 
	 * @return String
	 */
	private String getConfigProperty(String propertyName, String defaultValue) {
		return fBuildDefinitionWorkingCopy.getConfigurationPropertyValue(
				HudsonConfigurationElement.ELEMENT_ID, propertyName,
				defaultValue);
	}

	/**
	 * Sets a string-valued configuration property.
	 */
	private void setConfigProperty(String propertyName, String value) {
		fBuildDefinitionWorkingCopy.setConfigurationProperty(
				HudsonConfigurationElement.ELEMENT_ID, propertyName, value);
	}

	/**
	 * Creates a spacer for the UI dialog within the Composite.
	 */
	private void createSpacer(Composite parent, int height, int horizontalSpan) {
		Label label = fToolkit.createLabel(parent, ""); //$NON-NLS-1$
		TableWrapData tableWrapData = new TableWrapData(TableWrapData.FILL,
				TableWrapData.MIDDLE, 1, horizontalSpan);
		tableWrapData.heightHint = height;
		label.setLayoutData(tableWrapData);
	}

}
