/*******************************************************************************
 * Licensed Materials - Property of IBM
 *  Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.rational.connector.hudson.ui;

import java.io.UnsupportedEncodingException;
import java.security.GeneralSecurityException;
import java.util.Locale;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import com.ibm.rational.connector.hudson.internal.client.IHudsonServiceClient;
import com.ibm.rational.connector.hudson.internal.common.HudsonConfigurationElement;
import com.ibm.rational.connector.hudson.internal.common.HudsonEngineConfigurationData;
import com.ibm.rational.connector.hudson.internal.common.IHudsonConstants;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.internal.ui.editors.buildengine.BasicEngineConfigurationElementEditor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.common.util.ObfuscationHelper;

/**
 * Creates a UI contributor that implements the Hudson engine editor.
 * 
 * @since 0.9 (RTC 4.0)
 */
public class HudsonEngineConfigurationEditor extends
		BasicEngineConfigurationElementEditor {

    private static final int SPACER_HEIGHT = 10;

	private FormToolkit fToolkit;
	private Text fHudsonUserId;
	private Text fHudsonURL;
	private Text fHudsonConnectionText;
	private Button fHudsonTestConnection;
	private Text fHudsonPassword;
	private Button fHudsonAuthRequired;
	private Button fHudsonHostnameValidate;

	/**
	 * Create the editor.
	 */
	public HudsonEngineConfigurationEditor(String elementId, String title) {
		super(elementId, title);
	}

	@Override
	public void createContent(Composite parent, FormToolkit toolkit) {
		// Generate the base composite part for the Engine UI.
		fToolkit = toolkit;
		parent.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL_GRAB));
		TableWrapLayout twl = new TableWrapLayout();
		twl.numColumns = 2;
		twl.makeColumnsEqualWidth = true;
		twl.horizontalSpacing = 10;
		parent.setLayout(twl);
        
        // server section
        Section serverSection = createSection(parent,
        		Messages.HudsonEditor_LABEL_SERVER_INFO,
        		Messages.HudsonEditor_DESC_SERVER_INFO, false);
        Composite serverComposite = (Composite) serverSection.getClient();
        serverSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB, 1, 1));

		createHudsonURLWidgets(serverComposite);
		createSpacer(serverComposite, SPACER_HEIGHT, 2);
		createHudsonHostnameValidateWidget(serverComposite);
		createSpacer(serverComposite, SPACER_HEIGHT, 2);
		createHudsonAuthRequiredWidgets(serverComposite);
		createSpacer(serverComposite, SPACER_HEIGHT, 2);
		createHudsonUserIdWidgets(serverComposite);
		createHudsonPasswordWidget(serverComposite);
        
        // test connection section
        Section testConnectionSection = createSection(parent,
        		Messages.HudsonEditor_LABEL_TEST_CONNECTION_SECTION_TITLE,
        		Messages.HudsonEditor_DESC_TEST_CONNECTION, false, 1);
        Composite testConnectionComposite = (Composite) testConnectionSection.getClient();
        testConnectionSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB, 1, 1));

		createHudsonTestResultsWidgets(testConnectionComposite);
		createHudsonTestConnectionWidgets(testConnectionComposite);

		testConnectionSection.setClient(testConnectionComposite);
	}

	/**
	 * Creates the URL widgets in the given composite element.
	 */
	private void createHudsonURLWidgets(Composite composite) {
		String propValue = getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_URL, ""); //$NON-NLS-1$

		fHudsonURL = createLabeledText(composite,
				Messages.HudsonEditor_LABEL_URL,
				Messages.HudsonEditor_DESC_URL, propValue);

		fHudsonURL.addModifyListener(getHudsonURLModifiedListener());
	}

	/**
	 * Creates a listener that will dirty the editor and set the config element
	 * to the URL when the text is updated.
	 */
	private ModifyListener getHudsonURLModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setConfigProperty(
						HudsonConfigurationElement.PROPERTY_HUDSON_URL,
						fHudsonURL.getText().trim());
				
				// force user to re-enter password when URL changes
				if (fHudsonPassword.getText().length() > 0) {
					fHudsonPassword.setText(""); //$NON-NLS-1$
				}
				// the property is already cleared by the password field's modify text listener, but paranoia is fun
				setConfigProperty(HudsonConfigurationElement.PROPERTY_HUDSON_PASSWORD, ""); //$NON-NLS-1$
				
				setDirty(true);
				validate();
			}
		};
	}

	/**
	 * Creates the Hostname Validate button on the UI.
	 * 
	 * @param composite
	 */
	private void createHudsonHostnameValidateWidget(Composite composite) {
		String propValue = getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_HOSTNAME_VALIDATE,
				Boolean.TRUE.toString());

		fHudsonHostnameValidate = createLabeledCheckbox(composite, Messages.HudsonEditor_LABEL_VALIDATE_HOSTNAME,
				Messages.HudsonEditor_DESC_HOSTNAME_VALIDATE, Boolean.parseBoolean(propValue));
		fHudsonHostnameValidate.addSelectionListener(getHudsonValidateHostnameSelectionListener());
	}

	/**
	 * Creates and returns the listener that will check whether to validate the
	 * certificate hostname with the URL hostname. Dirty the editor.
	 * 
	 * @return
	 */
	private SelectionListener getHudsonValidateHostnameSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean validateHostname = fHudsonHostnameValidate.getSelection();
				setConfigProperty(
						HudsonConfigurationElement.PROPERTY_HUDSON_HOSTNAME_VALIDATE,
						Boolean.toString(validateHostname));
				setDirty(true);
				validate();
			}
		};
	}

	/**
	 * Creates the Auth Required button on the UI.
	 */
	private void createHudsonAuthRequiredWidgets(Composite composite) {
		String propValue = getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_AUTH_REQUIRED,
				Boolean.FALSE.toString());

		fHudsonAuthRequired = createLabeledCheckbox(composite, Messages.HudsonEditor_LABEL_AUTH_REQUIRED,
				Messages.HudsonEditor_DESC_AUTH_REQUIRED, Boolean.parseBoolean(propValue));
		fHudsonAuthRequired
				.addSelectionListener(getHudsonAuthRequiredSelectionListener());
	}

	/**
	 * Creates and returns the listener that will check if authorization is
	 * required. Depending on the result the user information text boxes may be
	 * disabled. The editor will also be marked as dirty.
	 */
	private SelectionListener getHudsonAuthRequiredSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean isAuthRequired = fHudsonAuthRequired.getSelection();
				setConfigProperty(
						HudsonConfigurationElement.PROPERTY_HUDSON_AUTH_REQUIRED,
						Boolean.toString(isAuthRequired));
				if (isAuthRequired) {
					fHudsonUserId.setEnabled(true);
					fHudsonPassword.setEnabled(true);
				} else {
					fHudsonUserId.setEnabled(false);
					fHudsonPassword.setEnabled(false);
				}
				setDirty(true);
				validate();
			}
		};
	}

	/**
	 * Creates the User ID UI widgets.
	 */
	private void createHudsonUserIdWidgets(Composite composite) {
		String propValue = getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_USERID, ""); //$NON-NLS-1$

		fHudsonUserId = createLabeledText(composite,
				Messages.HudsonEditor_LABEL_USER_ID,
				Messages.HudsonEditor_DESC_USER_ID, propValue);
		fHudsonUserId.setEnabled(Boolean.parseBoolean(getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_AUTH_REQUIRED,
				Boolean.FALSE.toString())));

		createSpacer(composite, SPACER_HEIGHT, 2);


		fHudsonUserId.addModifyListener(getHudsonUserIdModifiedListener());
	}

	/**
	 * Creates and returns a listener for the User Id text box. When detected it
	 * will save the config element and mark the editor as dirty.
	 */
	private ModifyListener getHudsonUserIdModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setConfigProperty(
						HudsonConfigurationElement.PROPERTY_HUDSON_USERID,
						fHudsonUserId.getText().trim());
				setDirty(true);
				validate();
			}
		};
	}

	/**
	 * Creates the password UI widget in the given composite.
	 */
	private void createHudsonPasswordWidget(Composite composite) {
		String propValue = getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_PASSWORD, ""); //$NON-NLS-1$
		boolean isEnabled = Boolean.parseBoolean(getConfigProperty(
				HudsonConfigurationElement.PROPERTY_HUDSON_AUTH_REQUIRED,
				Boolean.FALSE.toString()));


		// Password field
		fHudsonPassword = createLabeledText(composite,
				Messages.HudsonEditor_LABEL_PASSWORD,
				Messages.HudsonEditor_DESC_PASSWORD,
				"", SWT.DEFAULT, SWT.PASSWORD); //$NON-NLS-1$

		String passwordString = propValue;
		try {
			passwordString = new String(
					ObfuscationHelper.decryptString(propValue));
			fHudsonPassword.setText(passwordString);
		} catch (GeneralSecurityException gse) {
			addErrorMessage(fHudsonPassword,
					Messages.HudsonEditor_ERROR_PASSWORD, fHudsonPassword);
		} catch (UnsupportedEncodingException e) {
			addErrorMessage(fHudsonPassword,
					Messages.HudsonEditor_ERROR_ENCODING_PASSWORD,
					fHudsonPassword);
		}
		fHudsonPassword.addModifyListener(getHudsonPasswordModifiedListener());
		fHudsonPassword.setEnabled(isEnabled);
	}

	/**
	 * Creates and returns the listener for the password UI element. When
	 * detected it will obfuscate the password and set in the configuration
	 * element. The editor will then be marked dirty.
	 */
	private ModifyListener getHudsonPasswordModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				String passwordString = fHudsonPassword.getText().trim();
				if (passwordString.length() > 0) {
					try {
						passwordString = new String(
								ObfuscationHelper.encryptString(passwordString));
					} catch (GeneralSecurityException gse) {
						addErrorMessage(fHudsonPassword,
								Messages.HudsonEditor_ERROR_RETRIEVING_PASSWORD,
								fHudsonPassword);
					} catch (UnsupportedEncodingException uee) {
						addErrorMessage(fHudsonPassword,
								Messages.HudsonEditor_ERROR_ENCODING_PASSWORD,
								fHudsonPassword);
					}
				}
				setConfigProperty(
						HudsonConfigurationElement.PROPERTY_HUDSON_PASSWORD,
						passwordString);
				setDirty(true);
				validate();
			}
		};
	}

	/**
	 * Creates the text box area for the testConnection results.
	 */
	private void createHudsonTestResultsWidgets(Composite composite) {
		fHudsonConnectionText = fToolkit.createText(composite, null,
				SWT.READ_ONLY | SWT.NO_FOCUS | SWT.MULTI | SWT.V_SCROLL | SWT.WRAP);
		fHudsonConnectionText.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB, 40, 1));
	}

	/**
	 * Creates the UI test connection widgets in the given composite object.
	 */
	private void createHudsonTestConnectionWidgets(Composite composite) {
		fHudsonTestConnection = fToolkit.createButton(composite,
				Messages.HudsonEditor_LABEL_TEST_CONNECTION_BUTTON, SWT.PUSH);
		fHudsonTestConnection
				.addSelectionListener(getBuildForgeTestConnectionSelectionListener());
		Label descriptionLabel = fToolkit.createLabel(composite,
				Messages.HudsonEngineConfigurationEditor_0, SWT.WRAP);
		TableWrapData layoutData = new TableWrapData(TableWrapData.FILL, TableWrapData.FILL);
        layoutData.maxWidth = 250; // magic!
        descriptionLabel.setLayoutData(layoutData);
        descriptionLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_GRAY));
	}

	/**
	 * Creates and returns a new listener that will detect when the Text
	 * Connection button is pressed and will then call the testConnection()
	 * method.
	 */
	private SelectionListener getBuildForgeTestConnectionSelectionListener() {
		return new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				removeMessage(fHudsonTestConnection, fHudsonTestConnection);
				testConnection();
			}
		};
	}

	/**
	 * Tests the connection to the Hudson server using the current
	 * fHudsonEngineWorkingCopy. The results from the test connection attempt
	 * are then parsed and added to the Test Connection text box.
	 */
	protected void testConnection() {
		
		fHudsonTestConnection.setEnabled(false);
		
		removeMessage(fHudsonTestConnection, fHudsonTestConnection);
		fHudsonConnectionText
				.setText(Messages.HudsonEditor_MESSAGE_TEST_REQUESTED + "\n"); //$NON-NLS-1$
		
		Job job = new Job(Messages.HudsonEditor_LABEL_TEST_CONNECTION_BUTTON) {
			protected IStatus run(IProgressMonitor monitor) {
				try {
					IHudsonServiceClient servicesLayerClient = (IHudsonServiceClient) getTeamRepository()
							.getClientLibrary(IHudsonServiceClient.class);
					HudsonEngineConfigurationData configurationData = new HudsonEngineConfigurationData(fWorkingCopy);
					final StringBuilder message = new StringBuilder("\t" //$NON-NLS-1$
							+ Messages.HudsonEditor_MESSAGE_START_CONNECTION
							+ " " + configurationData.getURL() + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
					IBuildProperty[] connResults = null;
					try {
						connResults = servicesLayerClient.requestConnectionTest(fWorkingCopy, monitor);
					} catch (TeamRepositoryException e) {
						// BuildForgePlugin.log(_class, e);
						message.append(e.getClass().getName() + " - " //$NON-NLS-1$
								+ e.getMessage());
					}
					Map<String, String> resultMap = HudsonEngineConfigurationData
							.convertResultPropertiesToMap(connResults);
					if (resultMap != null && !resultMap.isEmpty()) {
						if (Boolean.parseBoolean(resultMap
								.get(IHudsonConstants.HUDSON_TEST_RESULT))) {
							if (resultMap
									.containsKey(IHudsonConstants.JENKINS_HEADER)) {
								message.append("\t" + resultMap //$NON-NLS-1$
												.get(IHudsonConstants.JENKINS_HEADER)
										+ "\n"); //$NON-NLS-1$
							}
							if (resultMap
									.containsKey(IHudsonConstants.HUDSON_HEADER)) {
								message.append("\t" + resultMap //$NON-NLS-1$
										.get(IHudsonConstants.HUDSON_HEADER) + "\n"); //$NON-NLS-1$
							}
							message.append("\t" //$NON-NLS-1$
									+ Messages.HudsonEditor_MESSAGE_CONNECTION_SUCCESS);
						} else {
							message.append("\t" + resultMap //$NON-NLS-1$
									.get(IHudsonConstants.HUDSON_EXCEPTION) + "\n"); //$NON-NLS-1$
		                    Display.getDefault().syncExec(new Runnable() {
		                        public void run() {
		                            if (!fHudsonTestConnection.isDisposed()) {
		    							addWarningMessage(
		    									fHudsonTestConnection,
		    									Messages.HudsonEditor_MESSSAGE_SERVER_NOT_FOUND,
		    									fHudsonTestConnection);
		                            }
		                        }
		                    });
							message.append("\t" + Messages.HudsonEditor_MESSAGE_CONNECTION_FAILED); //$NON-NLS-1$
						}
					}
					appendTestConnectionText(message.toString());
                } finally {

                    Display.getDefault().syncExec(new Runnable() {
                        public void run() {
                            if (!fHudsonTestConnection.isDisposed()) {
                            	fHudsonTestConnection.setEnabled(true);
                            }
                        }
                    });
                }

				return Status.OK_STATUS;
			}

            private void appendTestConnectionText(final String text) {
                if (text != null) {
                    Display.getDefault().syncExec(new Runnable() {
                        public void run() {
                            if (!fHudsonConnectionText.isDisposed()) {
                            	fHudsonConnectionText.append(text + "\n"); //$NON-NLS-1$
                            }
                        }
                    });
                }
            }
		};
		job.schedule();
	}

	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	@Override
	public boolean validate() {
        boolean isValid = true;

        if (fHudsonURL.getText().trim().isEmpty()) {
			addErrorMessageForRequiredField(fHudsonURL,
					Messages.HudsonEditor_ERROR_URL_REQUIRED, fHudsonURL);
			isValid = false;
        } else if((fHudsonURL.getText().toLowerCase(Locale.ROOT).startsWith("http://localhost:"))){
        	addWarningMessage(fHudsonURL, NLS.bind(
        			Messages.HudsonEngineConfigurationEditor_2, "http", "localhost"), fHudsonURL);
        } else if (fHudsonURL.getText().toLowerCase(Locale.ROOT).contains("://localhost:")) { //$NON-NLS-1$
			addWarningMessage(fHudsonURL, NLS.bind(
					Messages.HudsonEngineConfigurationEditor_3, "localhost"), //$NON-NLS-1$
					fHudsonURL);
        } else if (fHudsonURL.getText().toLowerCase(Locale.ROOT).startsWith("http:")){
        	addWarningMessage(fHudsonURL, NLS.bind(
        			Messages.HudsonEngineConfigurationEditor_1, "http"), fHudsonURL);
        } else {
			removeMessage(fHudsonURL, fHudsonURL);
		}

        if (fHudsonAuthRequired.getSelection()) {

    		if (fHudsonUserId.getText().trim().isEmpty()) {
    			addErrorMessageForRequiredField(fHudsonUserId,
    					Messages.HudsonEditor_ERROR_USERID_REQUIRED, fHudsonUserId);
    			isValid = false;
    		} else {
    			removeMessage(fHudsonUserId, fHudsonUserId);
    		}

    		if (fHudsonPassword.getText().trim().isEmpty()) {
    			addErrorMessageForRequiredField(fHudsonPassword,
    					Messages.HudsonEditor_ERROR_PASSWORD_REQUIRED,
    					fHudsonPassword);
    			isValid = false;
    		} else {
    			removeMessage(fHudsonPassword, fHudsonPassword);
    		}
        } else {
			removeMessage(fHudsonUserId, fHudsonUserId);
			removeMessage(fHudsonPassword, fHudsonPassword);
        }

		setPageStatusIndicator(!isValid, false);
		
		return isValid;
	}
	
	/**
	 * Creates a labeled checkbox spanning two columns.
	 */
	private Button createLabeledCheckbox(Composite parent, String label, String description, boolean selection) {
		Button button = fToolkit.createButton(parent, label, SWT.CHECK);
		button.setLayoutData(new TableWrapData(TableWrapData.FILL, TableWrapData.MIDDLE, 1, 2));
		button.setSelection(selection);
		createDescriptionLabel(parent, description, 2);
		return button;
	}

	/**
	 * Creates a description label widget (gray text, wrapped, small max width, grabs excess horizontal space).
	 */
	private void createDescriptionLabel(Composite parent, String description, int colSpan) {
		Label descriptionLabel = fToolkit.createLabel(parent, description, SWT.WRAP);
		TableWrapData layoutData = new TableWrapData(TableWrapData.FILL_GRAB);
		layoutData.colspan = colSpan;
		layoutData.maxWidth = 100;		
		descriptionLabel.setLayoutData(layoutData);
		descriptionLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_GRAY));
	}

}
