/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui;

import java.io.File;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.core.search.TypeNameMatch;
import org.eclipse.jdt.core.search.TypeNameMatchRequestor;
import org.eclipse.jdt.core.search.TypeNameRequestor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.internal.ide.dialogs.OpenResourceDialog;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;

import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.common.model.ICompileSource;
import com.ibm.team.build.internal.common.helper.FileHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;
import com.ibm.team.build.internal.ui.helper.BuildUIHelperMessages;
import com.ibm.team.build.internal.ui.helper.CompileSourceHelper;
import com.ibm.team.build.internal.ui.helper.WorkspaceHelper;

/**
 * Finds a file in the work space for a Java class
 */
@SuppressWarnings("restriction")
public class JdtWorkspaceHelper extends WorkspaceHelper {

    private static JdtWorkspaceHelper SINGLETON_WORKSPACE_HELPER = new JdtWorkspaceHelper();

    /**
     * Open the specified java workspace file in the specified editor and
     * navigate to the specified line number.
     * <p>
     * Must be called from a UI thread.
     * 
     * @param fullyQualifiedTypeName
     *            The fully qualified class name of the java file to open.
     * @param lineNumber
     *            The line number to open to.
     * @param editorSite
     *            The editor site to open the file in.
     * @return Returns true if the hyperlink opened successfully, false if the
     *         file could not be found.
     */
    public static boolean openJavaSourceFile(String fullyQualifiedTypeName, int lineNumber, IEditorSite editorSite) {
        ValidationHelper.validateNotEmpty("fullyQualifiedTypeName", fullyQualifiedTypeName); //$NON-NLS-1$
        ValidationHelper.validateNotNull("editorSite", editorSite); //$NON-NLS-1$
        return SINGLETON_WORKSPACE_HELPER.doOpenJavaSourceFile(fullyQualifiedTypeName, lineNumber, editorSite);
    }

    /**
     * Open the specified JUnit workspace file in the specified editor and
     * navigate to the specified JUnit test case.
     * <p>
     * Must be called from a UI thread.
     * 
     * @param fullyQualifiedTypeName
     *            The fully qualified class name of the java file to open.
     * @param testCaseName
     *            The name of the JUnit test case to navigate to.
     * @param editorSite
     *            The editor site to open the file in.
     * @return Returns true if the hyperlink opened successfully, false if the
     *         file could not be found.
     */
    public static boolean openJUnitSourceFile(String fullyQualifiedTypeName, String testCaseName, IEditorSite editorSite) {
        ValidationHelper.validateNotEmpty("fullyQualifiedTypeName", fullyQualifiedTypeName); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("testCaseName", testCaseName); //$NON-NLS-1$
        ValidationHelper.validateNotNull("editorSite", editorSite); //$NON-NLS-1$
        return SINGLETON_WORKSPACE_HELPER.doOpenJUnitSourceFile(fullyQualifiedTypeName, testCaseName, editorSite);
    }

    protected boolean doOpenJavaSourceFile(String className, int lineNumber, IEditorSite editorSite) {
        IFile javaFile = findJavaSourceFile(className);
        if (javaFile == null) {
            return false;
        }
        return openWorkspaceFileToLocation(javaFile, lineNumber, editorSite);
    }

    protected boolean doOpenJUnitSourceFile(String className, String testCaseName, IEditorSite editorSite) {
        IFile javaFile = findJavaSourceFile(className);
        if (javaFile == null) {
            return false;
        }

        return openWorkspaceFileToLocation(javaFile, testCaseName, editorSite);
    }
    
    protected boolean doOpenJavaSourceFile(String filePath, String className, int lineNumber,
			IEditorSite editorSite) {
		IFile javaFile = findJavaSourceFile(className, false);
		if (javaFile != null) {
			return openWorkspaceFileToLocation(javaFile, lineNumber, editorSite);
		}

		// open the open resource dialog to try to get to the file
		return openSourceFileToLocation(filePath, className, lineNumber, editorSite);
	}
    
    protected boolean doOpenJUnitSourceFile(String filePath, int lineNum,
			String className, String testCaseName, IEditorSite editorSite) {
		IFile javaFile = findJavaSourceFile(className, false);
		if (javaFile != null) {
			return openWorkspaceFileToLocation(javaFile, testCaseName,
					editorSite);
		}

		// open the open resource dialog to try to get to the file
		// (doing java syntax pattern matching)
		return openSourceFileToLocation(filePath, className,
				(lineNum > 0 ? lineNum : 1), editorSite);
	}
    
    /**
	 * Open the specified java workspace file in the specified editor and
	 * navigate to the specified line number.
	 * <p>
	 * Must be called from a UI thread.
	 * 
	 * @param filePath
	 *        The path to the file to open
	 * @param fullyQualifiedTypeName
	 *            The fully qualified class name of the java file to open.
	 * @param lineNumber
	 *            The line number to open to.
	 * @param editorSite
	 *            The editor site to open the file in.
	 * @return Returns true if the hyperlink opened successfully, false if the
	 *         file could not be found.
	 */
	public static boolean openJavaSourceFile(String filePath, String fullyQualifiedTypeName,
			int lineNumber, IEditorSite editorSite) {
		ValidationHelper.validateNotEmpty(
				"fullyQualifiedTypeName", fullyQualifiedTypeName); //$NON-NLS-1$
		ValidationHelper.validateNotNull("editorSite", editorSite); //$NON-NLS-1$
		return SINGLETON_WORKSPACE_HELPER.doOpenJavaSourceFile(filePath,
				fullyQualifiedTypeName, lineNumber, editorSite);
	}
    
    /**
	 * Open the specified JUnit workspace file in the specified editor and
	 * navigate to the specified JUnit test case.
	 * <p>
	 * Must be called from a UI thread.
	 *
	 * @param filePath 
	 *            The path to the file that needs to be opened.
	 * @param fullyQualifiedTypeName
	 *            The fully qualified class name of the java file to open.
	 * @param testCaseName
	 *            The name of the JUnit test case to navigate to.
	 * @param editorSite
	 *            The editor site to open the file in.
	 * @return Returns true if the hyperlink opened successfully, false if the
	 *         file could not be found.
	 */
	public static boolean openJUnitSourceFile(String filePath, int lineNumber, String fullyQualifiedTypeName, String testCaseName,
			IEditorSite editorSite) {
		ValidationHelper.validateNotEmpty(
				"fullyQualifiedTypeName", fullyQualifiedTypeName); //$NON-NLS-1$
		ValidationHelper.validateNotEmpty("testCaseName", testCaseName); //$NON-NLS-1$
		ValidationHelper.validateNotNull("editorSite", editorSite); //$NON-NLS-1$
		return SINGLETON_WORKSPACE_HELPER.doOpenJUnitSourceFile(filePath, lineNumber,
				fullyQualifiedTypeName, testCaseName, editorSite);
	}
	
	protected IFile findJavaSourceFile(String className) {
		return findJavaSourceFile(className, true);
	}
	
    protected IFile findJavaSourceFile(String className, boolean showErrorDlg) {
        IFile javaFile = null;

        try {
            javaFile = doFindJavaFile(className);
        } catch (JavaModelException exception) {
            showErrorDialog(NLS.bind(BuildUIHelperMessages.WorkspaceHelper_ERROR_OPENING_FILE_MESSAGE, className),
                    null, exception);
            return null;
        }

        if (javaFile == null && showErrorDlg == true) {
            showErrorDialog(NLS.bind(BuildUIHelperMessages.WorkspaceHelper_JAVA_SOURCE_FILE_NOT_FOUND_ERROR_MESSAGE,
                    className), BuildUIHelperMessages.WorkspaceHelper_FILE_NOT_FOUND_DIALOG_TITLE, null);
            return null;
        }

        return javaFile;
    }

    /**
     * Open the specified workspace file in the specified editor and navigate to
     * the specified line number.
     * <p>
     * Must be called from a UI thread.
     * 
     * @param file
     *            The file to open.
     * @param lineNumber
     *            The line number to open to.
     * @param editorSite
     *            The editor site to open the file in.
     * @return Returns true if the hyperlink opened successfully, false if the
     *         file could not be found.
     */
    public static boolean openWorkspaceFile(IFile file, int lineNumber, IEditorSite editorSite) {
        ValidationHelper.validateNotNull("file", file); //$NON-NLS-1$
        ValidationHelper.validateNotNull("editorSite", editorSite); //$NON-NLS-1$
        return SINGLETON_WORKSPACE_HELPER.openWorkspaceFileToLocation(file, lineNumber, editorSite);
    }
    
	private boolean openSourceFileToLocation(String filePath, String fileName,
			int lineNumber, IEditorSite editorSite) {
		IFile iFile = findFileInWorkspace(filePath);
		if (iFile != null) {
			return openWorkspaceFile(iFile, lineNumber, editorSite);
		} else {
			OpenResourceDialog dialog = new OpenResourceDialog(
					editorSite.getShell(), ResourcesPlugin.getWorkspace()
							.getRoot(),
					org.eclipse.core.resources.IResource.FILE);
			String fileNameFromFilePath = getFileName(filePath);
			fileName = fileNameFromFilePath != null ? fileNameFromFilePath : fileName;
			dialog.setInitialPattern(fileName);
			if (dialog.open() == Window.OK) {
				if (dialog.getResult().length > 0) {
					if (dialog.getResult()[0] instanceof IFile) {
						IFile file = (IFile) dialog.getResult()[0];
						return openWorkspaceFile(file, lineNumber, editorSite);
					}
				} else {
					// see if we can find the editor part just opened and set
					// the line number if its a form of text editor
					IEditorPart activeEditor = editorSite.getPage()
							.getActiveEditor();
					SINGLETON_WORKSPACE_HELPER.positionToLocationInEditor(
							activeEditor, lineNumber, fileName);
					return true;
				}
			}
		}
		return false;
	}
	
	private String getFileName(String filePath) {
		try {
			if (filePath != null) {
				filePath = filePath.replace("\\", "/"); //$NON-NLS-1$ //$NON-NLS-2$
				File f = new File(filePath);
				return f.getName();
			}
		} catch (Exception e) {
			// do nothing
		}
		return null;
	}
	
	private IFile findFileInWorkspace(String filePath) {
		if (filePath != null) {
			return findFileInWorkspace(ResourcesPlugin.getWorkspace().getRoot(),filePath,getFileName(filePath));
		}
		return null;
	}

	private IFile findFileInWorkspace(IContainer container, String filePath, String fileName) {
		org.eclipse.core.resources.IResource[] members;
		try {
			members = container.members();
			for (org.eclipse.core.resources.IResource member : members) {
				if (member instanceof IContainer)
					return findFileInWorkspace((IContainer) member, filePath, fileName);
				else if (member instanceof IFile
						&& ((IFile) member).getName().equals(fileName)) {
					return ((IFile) member);
				}
			}
			return null;
		} catch (CoreException e) {
			return null;
		}
	}
    

    protected boolean openWorkspaceFileToLocation(IFile file, Object location, IEditorSite editorSite) {
        if (!file.exists()) {
            String errorMessage = NLS.bind(BuildUIHelperMessages.WorkspaceHelper_FILE_NOT_FOUND_ERROR_MESSAGE,
                    file.getName());
            showErrorDialog(errorMessage, BuildUIHelperMessages.WorkspaceHelper_FILE_NOT_FOUND_DIALOG_TITLE, null);
            return false;
        }

        try {
            IEditorPart editorPart = openEditor(editorSite.getPage(), file);
            String fileName = file.getName();

            positionToLocationInEditor(editorPart, location, fileName);

            return true;
        } catch (CoreException exception) {
            BuildUIPlugin.log(exception.getStatus());
            showErrorDialog(BuildUIHelperMessages.WorkspaceHelper_ERROR_OPENING_FILE_MESSAGE, null, exception);
        }

        return false;
    }

    protected void navigateToLineNumber(ITextEditor textEditor, IDocument document, int lineNumber, String fileName) {
        try {
            // Line numbers are 1-based, but the editor API is
            // 0-based.
            int editorLineNumber = lineNumber;
            if (editorLineNumber != 0) {
                editorLineNumber--;
            }

            IRegion line = document.getLineInformation(editorLineNumber);
            textEditor.selectAndReveal(line.getOffset(), line.getLength());
        } catch (BadLocationException e) {
            openWarningDialog(NLS.bind(BuildUIHelperMessages.WorkspaceHelper_LOCATION_NOT_FOUND_ERROR_MESSAGE,
                    new String[] { Integer.toString(lineNumber), fileName }),
                    BuildUIHelperMessages.WorkspaceHelper_LOCATION_NOT_FOUND_DIALOG_TITLE);
        }
    }

    /**
     * We're searching for "public void testFooBar(" - we use the regular
     * expression character class for whitespace (\s) so that we don't have to
     * worry about number of spaces, tabs, newlines etc. between the elements of
     * the method signature.
     */
    protected void navigateToTestMethod(ITextEditor textEditor, IDocument document, String testCaseName, String fileName) {
        Pattern testCasePattern = Pattern.compile("public[\\s]*void[\\s]*" + testCaseName + "[\\s]*\\("); //$NON-NLS-1$ //$NON-NLS-2$
        Matcher testCaseMatcher = testCasePattern.matcher(document.get());
        if (testCaseMatcher.find()) {
            int start = testCaseMatcher.start() + testCaseMatcher.group().indexOf(testCaseName);
            textEditor.selectAndReveal(start, testCaseName.length());
        } else {
            openWarningDialog(NLS.bind(BuildUIHelperMessages.WorkspaceHelper_JUNIT_TESTCASE_NOT_FOUND_ERROR_MESSAGE,
                    new String[] { testCaseName, fileName }),
                    BuildUIHelperMessages.WorkspaceHelper_LOCATION_NOT_FOUND_DIALOG_TITLE);
        }
    }

    /**
     * Get the java <code>IFile</code> in the work space for the qualified
     * type name.
     * <p>
     * TODO: bug 18779 This works as a proof of concept, but it doesn't handle
     * inner types.
     * 
     * @param qualifiedTypeName
     *            the type name to search for in the workspace.
     * @return the file, or <code>null</code> if the type name is
     *         <code>null</code> or the file cannot be found
     * @throws JavaModelException
     */
    public static IFile findJavaFile(String qualifiedTypeName) throws JavaModelException {
        ValidationHelper.validateNotEmpty("qualifiedTypeName", qualifiedTypeName); //$NON-NLS-1$
        return SINGLETON_WORKSPACE_HELPER.doFindJavaFile(qualifiedTypeName);
    }

    protected IFile doFindJavaFile(String typeName) throws JavaModelException {
        SearchEngine engine = new SearchEngine();
        IJavaSearchScope scope = SearchEngine.createWorkspaceScope();

        final IFile[] file = new IFile[1];

        TypeNameRequestor requestor = new TypeNameRequestor() {
            public void acceptType(int modifiers, char[] packageNameChars, char[] simpleTypeName,
                    char[][] enclosingTypeNames, String path) {
                file[0] = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));
            }
        };

        engine.searchAllTypeNames(getPackageName(typeName).toCharArray(), SearchPattern.R_EXACT_MATCH, getClassName(
                typeName).toCharArray(), SearchPattern.R_EXACT_MATCH, IJavaSearchConstants.TYPE, scope, requestor,
                IJavaSearchConstants.WAIT_UNTIL_READY_TO_SEARCH, null);

        return file[0];
    }

    /**
     * Get the java <code>IType</code> in the work space for the qualified
     * type name.
     * <p>
     * 
     * @param qualifiedTypeName
     *            the type name to search for in the workspace.
     * @return the type object, or <code>null</code> if the type name is
     *         <code>null</code> or the type cannot be found
     * @throws JavaModelException
     */
    public static IType findJavaType(String qualifiedTypeName) throws JavaModelException {
        ValidationHelper.validateNotEmpty("qualifiedTypeName", qualifiedTypeName); //$NON-NLS-1$
        return SINGLETON_WORKSPACE_HELPER.doFindJavaType(qualifiedTypeName);
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected IType doFindJavaType(String typeName) throws JavaModelException {
        final ArrayList<TypeNameMatch> typeNameMatches = new ArrayList<TypeNameMatch>();
        IJavaSearchScope scope = SearchEngine.createWorkspaceScope();

        TypeNameMatchRequestor requestor = new TypeNameMatchRequestor() {
            public void acceptTypeNameMatch(TypeNameMatch match) {
                typeNameMatches.add(match);
            }
        };

        new SearchEngine().searchAllTypeNames(getPackageName(typeName).toCharArray(), SearchPattern.R_EXACT_MATCH
                | SearchPattern.R_CASE_SENSITIVE, getClassName(typeName).toCharArray(), SearchPattern.R_EXACT_MATCH
                | SearchPattern.R_CASE_SENSITIVE, IJavaSearchConstants.CLASS, scope, requestor,
                IJavaSearchConstants.WAIT_UNTIL_READY_TO_SEARCH, null);

        if (!typeNameMatches.isEmpty()) {
            return typeNameMatches.get(0).getType();
        }

        return null;
    }

    protected IEditorPart openEditor(IWorkbenchPage page, IFile file) throws PartInitException {
        return IDE.openEditor(page, file);
    }

    protected void connectProvider(IDocumentProvider provider, IEditorInput editorInput) throws CoreException {
        provider.connect(editorInput);
    }

    protected void showErrorDialog(String message, String title, Exception exception) {
        BuildUIHelper.showErrorDialog(message, title, exception);
    }

    protected MessageDialog createWarningDialog(String message, String title) {
        return new MessageDialog(null, title, null, message, MessageDialog.WARNING,
                new String[] { IDialogConstants.OK_LABEL }, 0);
    }

    protected void openWarningDialog(String message, String title) {
        createWarningDialog(message, title).open();
    }

    private String getPackageName(String typeName) {
        int index = typeName.lastIndexOf("."); //$NON-NLS-1$
        if (index == -1) {
            // Assume default package.
            return ""; //$NON-NLS-1$
        } else {
            String packageName = typeName.substring(0, index);
            return packageName;
        }
    }

    private String getClassName(String typeName) {
        int index = typeName.lastIndexOf("."); //$NON-NLS-1$
        if (index == -1) {
            // Assume default package.
            return typeName;
        } else {
            String className = typeName.substring(index + 1);
            return className;
        }
    }
    /**
     * This implementation should be closer to the real thing, but it doesn't
     * work yet.
     */
    // protected IFile getFilePattern(String line) {
    // if (line == null || "".equals(line)) { //$NON-NLS-1$
    // return null;
    // }
    // final String pattern = line.replace('$', '.');
    // final IProgressMonitor monitor = new NullProgressMonitor();
    // final IFile[] file = new IFile[1];
    // final SearchRequestor collector = new SearchRequestor() {
    // public void endReporting() {
    // // Intentionally Empty
    // }
    //
    // public void acceptSearchMatch(SearchMatch match) throws CoreException {
    // IResource resource = match.getResource();
    // if (resource.getType() == IResource.FILE) {
    // file[0] = (IFile) resource;
    // }
    // // cancel once we have one match
    // monitor.setCanceled(true);
    // }
    // };
    //
    // SearchEngine engine = new SearchEngine(JavaCore.getWorkingCopies(null));
    // SearchPattern searchPattern = SearchPattern.createPattern(pattern,
    // IJavaSearchConstants.TYPE,
    // IJavaSearchConstants.DECLARATIONS, SearchPattern.R_EXACT_MATCH |
    // SearchPattern.R_CASE_SENSITIVE);
    // IJavaSearchScope scope = SearchEngine.createWorkspaceScope();
    // SearchParticipant[] participants = new SearchParticipant[] {
    // SearchEngine.getDefaultSearchParticipant() };
    // try {
    // engine.search(searchPattern, participants, scope, collector, monitor);
    // } catch (CoreException e) {
    // BuildUIPlugin.log(e.getStatus());
    // } catch (OperationCanceledException e) {
    // // Intentionally Empty
    // }
    // return file[0];
    // }

	public static boolean openSourceFile(ICompileSource compileSource,
			int lineNumber, IEditorSite editorSite) {
		if (isJavaSource(compileSource)) {
	        String fullTypeName = CompileSourceHelper.getFullyQualifiedTypeName(compileSource);
	        return JdtWorkspaceHelper.openJavaSourceFile(fullTypeName, lineNumber, editorSite);
		} else {
			String initialPattern = compileSource.getSourceName();
//			String packageName = compileSource.getCompilePackage().getPackageName();
//			if (packageName != null && packageName.length() > 0) {
//				if (ICompilePackage.PACKAGE_TYPE_JAVA_PACKAGE.equals(compileSource.getCompilePackage().getPackageType())) {
//					packageName = packageName.replace('.', '/');
//				}
//				initialPattern = "*/" + packageName + "/" + initialPattern; //$NON-NLS-1$ //$NON-NLS-2$
//			}
			OpenResourceDialog dialog = new OpenResourceDialog(editorSite.getShell(), ResourcesPlugin.getWorkspace().getRoot(), org.eclipse.core.resources.IResource.FILE);
			dialog.setInitialPattern(initialPattern);
			if (dialog.open() == Window.OK) {
				if (dialog.getResult().length > 0) {
					if (dialog.getResult()[0] instanceof IFile) {
						IFile file = (IFile) dialog.getResult()[0];
						return openWorkspaceFile(file, lineNumber, editorSite);
					}
				} else {
					// see if we can find the editor part just opened and set the line number if its a form of text editor
					IEditorPart activeEditor = editorSite.getPage().getActiveEditor();
					SINGLETON_WORKSPACE_HELPER.positionToLocationInEditor(activeEditor, lineNumber, initialPattern);
				}
			}
			return false;
		}
	}

	/**
	 * Returns whether the given compile source represents a Java source file.
	 */
	private static boolean isJavaSource(ICompileSource compileSource) {
		return ICompilePackage.PACKAGE_TYPE_JAVA_PACKAGE.equals(compileSource
				.getCompilePackage().getPackageType())
				&& FileHelper.hasFileSuffix(compileSource.getSourceName(), ".java"); //$NON-NLS-1$
	}
}
