/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.compile;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompileContributionHandle;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * The root node of the Compile results tree. Represents an IBuildResult.
 */
public class CompileBuildResultNode extends AbstractCompileTreeNode {

    private static final List PARTIAL_COMPILE_CONTRIBUTION_PROPERTIES = new LinkedList();

    static {
        PARTIAL_COMPILE_CONTRIBUTION_PROPERTIES.add(ICompileContribution.PROPERTY_ERRORS);
        PARTIAL_COMPILE_CONTRIBUTION_PROPERTIES.add(ICompileContribution.PROPERTY_WARNINGS);
        PARTIAL_COMPILE_CONTRIBUTION_PROPERTIES.add(ICompileContribution.PROPERTY_ERROR_COUNT);
        PARTIAL_COMPILE_CONTRIBUTION_PROPERTIES.add(ICompileContribution.PROPERTY_WARNING_COUNT);
        PARTIAL_COMPILE_CONTRIBUTION_PROPERTIES.add(ICompileContribution.PROPERTY_CLASS_COUNT);
    }

    private IBuildResultContribution[] fBuildResultContributions;
    private Hashtable<String, CompileComponentNode> fComponentNodeMap = new Hashtable<String, CompileComponentNode>();

    /**
     * Create a BuildResultNode representing the given buildResult.
     * 
     * @param buildResultContributions
     *            The {@link IBuildResultContribution}s related to the build
     *            result.
     */
    public CompileBuildResultNode(IBuildResultContribution[] buildResultContributions) {
        super(null);
        ValidationHelper.validateNotNull("buildResultContributions", buildResultContributions); //$NON-NLS-1$
        fBuildResultContributions = buildResultContributions;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean hasChildren() {
        return true;
    }

    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        ICompileTreeNode[] children = null;

        if (fBuildResultContributions.length > 0) {
            List<ICompileContribution> compileContributions = fetchCompileContributions(fBuildResultContributions,
                    monitor);

            createComponentNodeChildren(fBuildResultContributions, compileContributions);

            /*
             * If there is only 1 component and it has no name, don't show a
             * single [no component] node. Instead, put that component's
             * children at the top level.
             */
            if (fComponentNodeMap.size() == 1) {
                Enumeration componentNames = fComponentNodeMap.keys();
                String name = (String) componentNames.nextElement();
                if (name.length() == 0) {
                    children = (ICompileTreeNode[]) fComponentNodeMap.get(name).fetchChildren(monitor);
                }
            }

            if (children == null) {
                children = fComponentNodeMap.values().toArray(new ICompileTreeNode[fComponentNodeMap.size()]);
            }
        }

        return children;
    }

    protected void logException(TeamRepositoryException exception) {
        BuildUIPlugin.log(exception);
    }

    /**
     * @param contributions
     * @param compileContributions
     */
    private void createComponentNodeChildren(IBuildResultContribution[] contributions,
            List<ICompileContribution> compileContributions) {
        for (int i = 0; i < contributions.length; i++) {
            String componentName = contributions[i].getComponentName();
            if (componentName == null) {
                componentName = ""; //$NON-NLS-1$
            }

            CompileComponentNode node = fComponentNodeMap.get(componentName);
            if (node == null) {
                node = new CompileComponentNode(this, componentName);
                fComponentNodeMap.put(componentName, node);
            }
            node.addCompileContribution(contributions[i], compileContributions.get(i));
        }
    }

    private List<ICompileContribution> fetchCompileContributions(IBuildResultContribution[] contributions,
            IProgressMonitor monitor) throws TeamRepositoryException {

        List<ICompileContribution> compileContributions = new ArrayList<ICompileContribution>();
        List<ICompileContributionHandle> compileContributionHandles = new ArrayList<ICompileContributionHandle>();

        // Collect handles for all the compile contributions
        for (IBuildResultContribution contribution : contributions) {
            compileContributionHandles.add((ICompileContributionHandle) contribution.getExtendedContribution());
        }

        ITeamRepository teamRepository = (ITeamRepository) contributions[0].getExtendedContribution().getOrigin();
        compileContributions.addAll(teamRepository.itemManager().fetchPartialItems(compileContributionHandles,
                IItemManager.DEFAULT, PARTIAL_COMPILE_CONTRIBUTION_PROPERTIES, monitor));
        return compileContributions;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        return null;
    }
    
    @Override
    public boolean hasProblems() {
        return false;
    }

    @Override
    public String getNodeName() {
        return null;
    }

    @Override
    public boolean hasWarnings() {
        return false;
    }

    @Override
    public boolean hasErrors() {
        return false;
    }
}
