/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.compile;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorMessages;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * A node representing a compile component. The node contains both the
 * <code>IBuildResultContributions</code> and the
 * <code>ICompileContributions</code>.
 */
public class CompileComponentNode extends AbstractCompileTreeNode {

    static final List PARTIAL_COMPILE_PACKAGE_PROPERTIES = new LinkedList();

    static {
        PARTIAL_COMPILE_PACKAGE_PROPERTIES.add(ICompilePackage.PROPERTY_ERROR_COUNT);
        PARTIAL_COMPILE_PACKAGE_PROPERTIES.add(ICompilePackage.PROPERTY_WARNING_COUNT);
        PARTIAL_COMPILE_PACKAGE_PROPERTIES.add(ICompilePackage.PROPERTY_CLASS_COUNT);
        PARTIAL_COMPILE_PACKAGE_PROPERTIES.add(ICompilePackage.PROPERTY_PACKAGE_NAME);
        PARTIAL_COMPILE_PACKAGE_PROPERTIES.add(ICompilePackage.PROPERTY_PACKAGE_TYPE);
    }

    private List<ICompileContribution> fCompileContributions = new ArrayList<ICompileContribution>();
    private List<IBuildResultContribution> fBuildResultContributions = new ArrayList<IBuildResultContribution>();

    private Hashtable<String, CompilePackageNode> fPackageNodeMap = new Hashtable<String, CompilePackageNode>();
    private List<ICompileTreeNode> fChildNodeList = new ArrayList<ICompileTreeNode>();

    private String fComponentName = null;
    private int fWarningCount = 0;
    private int fErrorCount = 0;

    /**
     * Create a compile component node.
     * 
     * @param parentNode
     *            the parent node of this node
     * @param componentName
     *            the name of this component
     */
    public CompileComponentNode(CompileBuildResultNode parentNode, String componentName) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$
        ValidationHelper.validateNotNull("componentName", componentName); //$NON-NLS-1$
        fComponentName = componentName.length() == 0 ? BuildResultEditorMessages.NO_COMPONENT_LABEL : componentName;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean hasChildren() {
        return true;
    }

    /**
     * Return the number of contributions under this node.
     * 
     * @return The number of contributions represented by this node.
     */
    public int getContributionCount() {
        return fBuildResultContributions.size();
    }

    /**
     * Add the specified contribution to this node.
     * 
     * @param buildResultContribution
     * @param compileContribution
     */
    public void addCompileContribution(IBuildResultContribution buildResultContribution,
            ICompileContribution compileContribution) {
        fBuildResultContributions.add(buildResultContribution);
        fCompileContributions.add(compileContribution);
        fWarningCount += compileContribution.getWarningCount();
        fErrorCount += compileContribution.getErrorCount();
    }

    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        // Fetch the complete compile contribution, then fetch the compile
        // packages.
        fChildNodeList.clear();
        fPackageNodeMap.clear();
        ICompileTreeNode[] children = null;

        for (int i = 0; i < fBuildResultContributions.size(); i++) {
            IBuildResultContribution buildResultContribution = fBuildResultContributions.get(i);
            ICompileContribution compileContribution = fCompileContributions.get(i);

            compileContribution = fetchCompileContribution(compileContribution, monitor);

            if (isSimpleContribution(compileContribution)) {
                fChildNodeList.add(new SimpleCompileContributionNode(this, buildResultContribution, compileContribution));
            } else {
                ITeamRepository teamRepository = (ITeamRepository) compileContribution.getOrigin();
                List compilePackages = compileContribution.getCompilePackages();
                List fetchedCompilePackages = teamRepository.itemManager().fetchPartialItems(compilePackages,
                        IItemManager.DEFAULT, PARTIAL_COMPILE_PACKAGE_PROPERTIES, monitor);

                for (Object object : fetchedCompilePackages) {
                    addCompilePackageNode((ICompilePackage) object);
                }
            }
        }

        fChildNodeList.addAll(fPackageNodeMap.values());
        children = fChildNodeList.toArray(new ICompileTreeNode[fChildNodeList.size()]);
        return children;
    }

    protected ICompileContribution fetchCompileContribution(ICompileContribution compileContribution,
            IProgressMonitor monitor) throws TeamRepositoryException {
        ITeamRepository teamRepository = (ITeamRepository) compileContribution.getOrigin();
        return (ICompileContribution) teamRepository.itemManager().fetchCompleteItem(compileContribution,
                IItemManager.DEFAULT, monitor);

    }

    protected void logException(TeamRepositoryException exception) {
        BuildUIPlugin.log(exception);
    }

    /**
     * @param compileContribution
     * @return
     */
    private boolean isSimpleContribution(ICompileContribution compileContribution) {
        return compileContribution.getClassCount() == 0;
    }

    /**
     * @param compilePackage
     */
    private void addCompilePackageNode(ICompilePackage compilePackage) {
        String packageName = compilePackage.getPackageName();

        CompilePackageNode node = fPackageNodeMap.get(packageName);
        if (node == null) {
            node = new CompilePackageNode(this, packageName, compilePackage.getPackageType());
            fPackageNodeMap.put(packageName, node);
        }

        node.addCompilePackage(compilePackage);
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == CompileResultsPage.NAME_COLUMN_INDEX) {

            Image image = BuildUIPlugin.getImage("icons/obj16/component_obj.gif"); //$NON-NLS-1$

            BuildStatus status = BuildStatus.OK;
            if (fErrorCount > 0) {
                status = BuildStatus.ERROR;
            } else if (fWarningCount > 0) {
                status = BuildStatus.WARNING;
            }

            return BuildCompositeImageRegistry.getImage(image, status, false);
        }

        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.NAME_COLUMN_INDEX:
                return fComponentName;
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return Integer.toString(fWarningCount);
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return Integer.toString(fErrorCount);
        }
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return (long) fWarningCount;
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return (long) fErrorCount;
        }
        return null;
    }
    
    @Override
    public boolean hasProblems() {
        return fErrorCount > 0 || fWarningCount > 0;
    }

    @Override
    public String getNodeName() {
        return fComponentName;
    }

    @Override
    public boolean hasWarnings() {
        return fWarningCount > 0;
    }

    @Override
    public boolean hasErrors() {
        return fErrorCount > 0;
    }

}
