/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.compile;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.forms.editor.FormEditor;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompileContributionHandle;
import com.ibm.team.build.ui.editors.result.AbstractBuildResultContributionProvider;
import com.ibm.team.build.ui.editors.result.AbstractBuildResultPage;
import com.ibm.team.build.ui.editors.result.IBuildResultContext;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Summary provider for compile information.
 */
public class CompileContributionProvider extends AbstractBuildResultContributionProvider {

    private int fErrorCount;
    private int fWarningCount;
    private IBuildResultContribution[] fContributions;

    /**
     * Create a CompileContributionProvider object.
     * 
     * @param buildEditor
     *            The build result form editor
     * @param id
     *            The unique id of the compile contribution extension.
     * @param context
     *            The context object giving access to build result data.
     */
    public CompileContributionProvider(FormEditor buildEditor, String id, IBuildResultContext context) {
        super(buildEditor, id, context);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void initializeContributionProvider(IProgressMonitor monitor) throws Exception {
        fErrorCount = 0;
        fWarningCount = 0;
        fContributions = new IBuildResultContribution[0];

        ITeamRepository teamRepository = getBuildResultContext().getTeamRepository();
        List<ICompileContributionHandle> compileHandles = new ArrayList<ICompileContributionHandle>();

        fContributions = getContributions();
        for (IBuildResultContribution contribution : fContributions) {
            compileHandles.add((ICompileContributionHandle) contribution.getExtendedContribution());
        }

        if (fContributions.length > 0) {
            List properties = Arrays.asList(new String[] { ICompileContribution.PROPERTY_ERROR_COUNT,
                    ICompileContribution.PROPERTY_WARNING_COUNT });

            List compiles = teamRepository.itemManager().fetchPartialItems(compileHandles, IItemManager.DEFAULT,
                    properties, monitor);
            for (Object object : compiles) {
                if (object != null) {
                    ICompileContribution compile = (ICompileContribution) object;
                    fErrorCount += compile.getErrorCount();
                    fWarningCount += compile.getWarningCount();
                }
            }
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public BuildStatus getStatus() {
        if (fErrorCount > 0) {
            return BuildStatus.ERROR;
        }

        if (fWarningCount > 0) {
            return BuildStatus.WARNING;
        }

        return BuildStatus.OK;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public String getSummaryText() {
        if (fContributions.length > 0) {
            return MessageFormat.format(Messages.CompileContributionProvider_SUMMARY_TEXT, String.valueOf(fErrorCount),
                    getErrorString(), String.valueOf(fWarningCount), getWarningString());
        }
        return Messages.CompileContributionProvider_NO_COMPILES;
    }

    private String getWarningString() {
        if (fWarningCount == 1) {
            return Messages.CompileContributionProvider_ONE_WARNING;
        }
        return Messages.CompileContributionProvider_MULTIPLE_WARNINGS;
    }

    private String getErrorString() {
        if (fErrorCount == 1) {
            return Messages.CompileContributionProvider_ONE_ERROR;
        }
        return Messages.CompileContributionProvider_MULTIPLE_ERRORS;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public String[] getExtendedContributionIds() {
        return new String[] { ICompileContribution.EXTENDED_CONTRIBUTION_TYPE_ID };
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public boolean isSummaryLink() {
        if (fContributions.length > 0) {
            return true;
        }
        return false;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void summaryLinkActivated() {
        getBuildResultEditor().setActivePage(getExtensionId());
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getSummaryTitle() {
        return Messages.CompileContributionProvider_SUMMARY_TITLE;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public AbstractBuildResultPage getBuildResultPage() {
        return new CompileResultsPage(getBuildResultEditor(), getExtensionId(),
                Messages.CompileContributionProvider_PAGE_TITLE, getBuildResultContext());
    }

}
