/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.compile;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.RetargetAction;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.part.Page;
import org.eclipse.ui.part.PageBook;

import com.ibm.team.build.common.model.ICompileProblem;
import com.ibm.team.build.internal.java.ui.JdtWorkspaceHelper;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.jface.tooltip.TooltipSupport;

/**
 * Displays details of a compile source in the BuildResultEditor.
 */
public class CompileDetailBlock {

    private static final int NUMBER_OF_COLUMNS = 2;

    private ICompileDetailProvider fProvider;
    private boolean fShowErrors;

    protected FormEditor fEditor;
    protected PageBook fPageBook;
    protected Page fSimplePage;
    protected Page fComplexPage;

    protected Control fSimpleDetailsControl;
    protected Control fComplexDetailsControl;

    protected Tree fTree;
    protected TreeViewer fTreeViewer;

    protected Text fText;

    /**
     * Creates the contents of the details block.
     * 
     * @param parent
     *            the parent of the details block; never <code>null</code>.
     * @param toolkit
     *            the form toolkit; never <code>null</code>.
     * @param editor
     *            the form editor; never <code>null</code>.
     * @return a new <code>Composite</code> for compile details.
     */
    public Composite createContents(Composite parent, final FormToolkit toolkit, FormEditor editor) {

        fPageBook = new PageBook(parent, SWT.NONE);
        fEditor = editor;

        GridDataFactory.fillDefaults().grab(true, true).applyTo(fPageBook);

        fSimplePage = new Page() {
            public void createControl(Composite innerParent) {
                createSimpleDetailsControl(innerParent, toolkit);
            }

            public Control getControl() {
                return fSimpleDetailsControl;
            }

            public void setFocus() {
                // nothing to do
            }
        };

        fSimplePage.createControl(fPageBook);

        fComplexPage = new Page() {
            public void createControl(Composite innerParent) {
                createComplexDetailsControl(innerParent, toolkit);
            }

            public Control getControl() {
                return fComplexDetailsControl;
            }

            public void setFocus() {
                // nothing to do
            }
        };

        fComplexPage.createControl(fPageBook);

        return fPageBook;
    }

    protected void createSimpleDetailsControl(Composite parent, FormToolkit toolkit) {

        Composite composite = toolkit.createComposite(parent, SWT.NONE);
        GridLayout gridLayout = new GridLayout(1, false);
        gridLayout.marginHeight = 0;
        gridLayout.marginWidth = 0;
        composite.setLayout(gridLayout);
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        fText = toolkit.createText(composite, "", SWT.READ_ONLY | SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL); //$NON-NLS-1$
        GridData gridData = new GridData(GridData.FILL_BOTH);
        fText.setLayoutData(gridData);
        fText.setFont(JFaceResources.getTextFont());

        fSimpleDetailsControl = composite;
    }

    protected void createComplexDetailsControl(final Composite parent, FormToolkit toolkit) {
        final Composite composite = toolkit.createComposite(parent, SWT.NONE);
        GridLayout gridLayout = new GridLayout(1, false);
        gridLayout.marginHeight = 0;
        gridLayout.marginWidth = 0;
        composite.setLayout(gridLayout);
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        fTree = toolkit.createTree(composite, SWT.MULTI | SWT.FULL_SELECTION);
        fTree.setLayoutData(new GridData(GridData.FILL_BOTH));

        for (int i = 0; i < NUMBER_OF_COLUMNS; i++) {
            TreeColumn column = new TreeColumn(fTree, SWT.NONE, i);
            column.setResizable(true);
        }

        fTreeViewer = new TreeViewer(fTree);
        fTreeViewer.addOpenListener(getOpenListener());

        createContextMenu();
        addTooltipSupport();

        fComplexDetailsControl = composite;
    }

    protected TooltipSupport addTooltipSupport() {
        return new TooltipSupport(fTree, true, false) {
            @Override
            protected String getMarkup(Object element, boolean isFocus) {
                String markup = super.getMarkup(element, isFocus);

                if (element instanceof ICompileProblem) {
                    if (isFocus) {
                        setPreferredFocusSize(new Point(500, 40));
                    }
                }
                return markup;
            }
        };
    }

    protected IOpenListener getOpenListener() {
        return new IOpenListener() {
            public void open(final OpenEvent event) {
                IStructuredSelection selection = (IStructuredSelection) event.getSelection();
                openCompileProblem((ICompileProblem) selection.getFirstElement());
            }
        };
    }

    protected void createContextMenu() {
        MenuManager mgr = new MenuManager();
        mgr.setRemoveAllWhenShown(true);
        mgr.addMenuListener(getMenuListener());
        Menu menu = mgr.createContextMenu(fTreeViewer.getControl());
        fTreeViewer.getControl().setMenu(menu);

        fEditor.getEditorSite().registerContextMenu(mgr, fTreeViewer);
    }

    protected IMenuListener getMenuListener() {
        return new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                ActionFactory.IWorkbenchAction copyAction = ActionFactory.COPY.create(fEditor.getEditorSite().getWorkbenchWindow());
                copyAction.setEnabled(true);
                // TODO: 50525 - Remove this when we switch to Eclipse 3.4
                if (copyAction instanceof RetargetAction) {
                    ((RetargetAction) copyAction).partActivated(fEditor);
                }
                manager.add(copyAction);
                manager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
            }
        };
    }

    protected boolean openCompileProblem(ICompileProblem compileProblem) {
        return JdtWorkspaceHelper.openSourceFile(compileProblem.getCompileSource(), compileProblem.getLineNumber(), fEditor.getEditorSite());
    }

    /**
     * Set the compile detail provider and update the controls.
     * 
     * @param provider
     *            <code>ICompileDetailProvider</code> to use to fill controls.
     * @param showErrors
     *            if true, show error details, else show warning details
     */
    public void setDetailProvider(ICompileDetailProvider provider, boolean showErrors) {
        if (provider == fProvider && fShowErrors == showErrors) {
            return;
        }

        fProvider = provider;
        fShowErrors = showErrors;

        if (provider == null) {
            setVisible(false);
            return;
        }

        updateDetailsAsync();
    }

    protected void updateDetailsAsync() {
        TeamBuildJob job = new TeamBuildJob(Messages.CompileDetailBlock_RETRIEVING_PROBLEM_DETAILS, true) {
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                final boolean complexDetails = fProvider instanceof IComplexCompileDetailProvider;

                final ICompileProblem[][] problems = new ICompileProblem[1][];
                final String details[] = new String[] { "" }; //$NON-NLS-1$

                if (complexDetails) {
                    IComplexCompileDetailProvider complexProvider = (IComplexCompileDetailProvider) fProvider;
                    problems[0] = fShowErrors ? complexProvider.getErrors(monitor)
                            : complexProvider.getWarnings(monitor);
                } else {
                    details[0] = fShowErrors ? fProvider.getErrorDetails(monitor)
                            : fProvider.getWarningDetails(monitor);
                }

                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        handleShowDetailsAsync(complexDetails, problems[0], details[0]);
                    }
                });

                return Status.OK_STATUS;
            }

            protected void jobFinished(IStatus status) {
                if (status.getSeverity() == IStatus.ERROR) {
                    Display.getDefault().asyncExec(new Runnable() {
                        public void run() {
                            setVisible(false);
                        }
                    });
                }
            }
        };

        scheduleJob(job);
    }

    protected void scheduleJob(TeamBuildJob job) {
        job.schedule();
    }

    protected void handleShowDetailsAsync(boolean complex, ICompileProblem[] problems, String details) {
        if (!fPageBook.isDisposed()) {
            if (complex) {
                showComplexDetails(problems);
            } else {
                showSimpleDetails(details);
            }
        }
    }

    protected void showSimpleDetails(String problemString) {
        if (problemString != null && problemString.length() > 0) {
            fText.setText(problemString);
            showSimplePage();
            setVisible(true);
        } else {
            setVisible(false);
        }
    }

    protected void showComplexDetails(ICompileProblem[] problems) {
        cleanupTree();

        ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
        Image image = fShowErrors ? sharedImages.getImage(ISharedImages.IMG_OBJS_ERROR_TSK)
                : sharedImages.getImage(ISharedImages.IMG_OBJS_WARN_TSK);

        if (problems == null || problems.length == 0) {
            return;
        }

        for (int i = 0; i < problems.length; i++) {
            final ICompileProblem problem = problems[i];
            final TreeItem TreeItem = new TreeItem(fTree, SWT.NONE);
            TreeItem.setImage(image);
            String lineNumber = NLS.bind(Messages.CompileDetailBlock_LINE_NUMBER, problem.getLineNumber());
            TreeItem.setText(0, lineNumber);
            TreeItem.setText(1, problem.getMessageText());
            TreeItem.setData(problem);
        }

        for (int i = 0; i < NUMBER_OF_COLUMNS; i++) {
            fTree.getColumn(i).pack();
        }

        showComplexPage();
        setVisible(true);
    }

    protected void showSimplePage() {
        fPageBook.showPage(fSimpleDetailsControl);
    }

    protected void showComplexPage() {
        fPageBook.showPage(fComplexDetailsControl);
    }

    protected void setVisible(boolean visible) {
        fPageBook.setVisible(visible);
    }

    private void cleanupTree() {
        for (TreeItem item : fTree.getItems()) {
            item.dispose();
        }
    }
}