/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.compile;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.common.model.ICompileSource;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Represents a compile package in the compile results tree.
 */
public class CompilePackageNode extends AbstractCompileTreeNode {

    private String fPackageName = null;
    private String fPackageType = null;
    private int fWarningCount = 0;
    private int fErrorCount = 0;

    private List<ICompilePackage> fCompilePackages = new ArrayList<ICompilePackage>();

    private List<CompileSourceNode> fSourceNodes = new ArrayList<CompileSourceNode>();
    private Hashtable<String, CompilePackageNode> fSubPackageNodeMap = new Hashtable<String, CompilePackageNode>();

    /**
     * Create a CompilePackageNode representing the specified
     * {@link ICompilePackage} and using the specified {@link ICompileTreeNode}
     * as the parent.
     * 
     * @param parentNode
     *            The {@link ICompileTreeNode} that is the parent of this node.
     * @param packageName
     *            The name of this compile package.
     * @param packageType
     *            The type of this compile package.
     */
    public CompilePackageNode(ICompileTreeNode parentNode, String packageName, String packageType) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$
        ValidationHelper.validateNotNull("packageName", packageName); //$NON-NLS-1$
        ValidationHelper.validateNotNull("packageType", packageType); //$NON-NLS-1$
        fPackageName = packageName;
        fPackageType = packageType;
    }

    /**
     * Add the specified package to this node.
     * 
     * @param compilePackage
     */
    public void addCompilePackage(ICompilePackage compilePackage) {
        fCompilePackages.add(compilePackage);
        fWarningCount += compilePackage.getWarningCount();
        fErrorCount += compilePackage.getErrorCount();
    }

    /**
     * Return the number of compile packages under this node.
     * 
     * @return The number of compile packages represented by this node.
     */
    public int getPackageCount() {
        return fCompilePackages.size();
    }

    @Override
    public boolean hasChildren() {
        return true;
    }

    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        // Fetch the complete compile package, then add the test cases.
        for (ICompilePackage compilePackage : fCompilePackages) {
            compilePackage = fetchCompilePackage(compilePackage, monitor);

            ITeamRepository teamRepository = (ITeamRepository) compilePackage.getOrigin();
            List subCompilePackages = compilePackage.getSubCompilePackages();
            List fetchedCompilePackages = teamRepository.itemManager().fetchPartialItems(subCompilePackages,
                    IItemManager.DEFAULT, CompileComponentNode.PARTIAL_COMPILE_PACKAGE_PROPERTIES, monitor);
            for (Object object : fetchedCompilePackages) {
                addSubCompilePackageNode((ICompilePackage) object);
            }

            List compileSources = compilePackage.getCompileSources();
            for (Object object : compileSources) {
                fSourceNodes.add(new CompileSourceNode(this, (ICompileSource) object));
            }
        }

        List<ICompileTreeNode> childNodes = new ArrayList<ICompileTreeNode>(fSourceNodes.size() + fSubPackageNodeMap.size());
        childNodes.addAll(fSubPackageNodeMap.values());
        childNodes.addAll(fSourceNodes);
        return childNodes.toArray();
    }

    protected ICompilePackage fetchCompilePackage(ICompilePackage compilePackage, IProgressMonitor monitor)
            throws TeamRepositoryException {
        ITeamRepository teamRepository = (ITeamRepository) compilePackage.getOrigin();
        return (ICompilePackage) teamRepository.itemManager().fetchCompleteItem(compilePackage, IItemManager.DEFAULT,
                monitor);

    }

    private void addSubCompilePackageNode(ICompilePackage compilePackage) {
        String packageName = compilePackage.getPackageName();

        CompilePackageNode node = fSubPackageNodeMap.get(packageName);
        if (node == null) {
            node = new CompilePackageNode(this, packageName, compilePackage.getPackageType());
            fSubPackageNodeMap.put(packageName, node);
        }

        node.addCompilePackage(compilePackage);
    }

    /**
     * Return the package type of this node.
     * 
     * @return the package type.
     */
    public String getPackageType() {
        return fPackageType;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == CompileResultsPage.NAME_COLUMN_INDEX) {

            Image newImage = null;
            if (fPackageType.equals(ICompilePackage.PACKAGE_TYPE_PLUGIN)) {
                newImage = BuildUIPlugin.getImage("icons/obj16/plugin_obj.gif"); //$NON-NLS-1$
            } else if (fPackageType.equals(ICompilePackage.PACKAGE_TYPE_PROJECT)) {
                newImage = PlatformUI.getWorkbench().getSharedImages().getImage(IDE.SharedImages.IMG_OBJ_PROJECT);
            } else if (fPackageType.equals(ICompilePackage.PACKAGE_TYPE_FOLDER)) {
                newImage = PlatformUI.getWorkbench().getSharedImages().getImage(org.eclipse.ui.ISharedImages.IMG_OBJ_FOLDER);
            } else {
                newImage = JavaUI.getSharedImages().getImage(ISharedImages.IMG_OBJS_PACKAGE);
            }

            BuildStatus status = BuildStatus.OK;
            if (hasWarnings()) {
                status = BuildStatus.WARNING;
            }
            if (hasErrors()) {
                status = BuildStatus.ERROR;
            }

            return BuildCompositeImageRegistry.getImage(newImage, status, false);
        }
        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.NAME_COLUMN_INDEX:
                String packageName = fPackageName;
                if (fPackageType.equals(ICompilePackage.PACKAGE_TYPE_PLUGIN)) {
                    if (packageName.indexOf('_') != -1) {
                        packageName = packageName.substring(0, packageName.indexOf('_'));
                    }
                } else if (fPackageType.equals(ICompilePackage.PACKAGE_TYPE_JAVA_PACKAGE)) {
                    if (packageName.length() == 0) {
                        packageName = Messages.CompilePackageNode_DEFAULT_PACKAGE_LABEL;
                    }
                }
                return packageName;
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return Integer.toString(fWarningCount);
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return Integer.toString(fErrorCount);
        }
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return (long) fWarningCount;
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return (long) fErrorCount;
        }
        return null;
    }
    
    @Override
    public boolean hasProblems() {
        return (fErrorCount > 0 || fWarningCount > 0);
    }

    @Override
    public String getNodeName() {
        return fPackageName;
    }

    @Override
    public boolean hasWarnings() {
        return fWarningCount > 0;
    }

    @Override
    public boolean hasErrors() {
        return fErrorCount > 0;
    }
}
