/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.compile;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TreeColumnLayout;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.common.model.ICompileSource;
import com.ibm.team.build.internal.java.ui.JdtWorkspaceHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.IHelpContextIds;
import com.ibm.team.build.internal.ui.editors.result.BuildDeferredContentProvider;
import com.ibm.team.build.internal.ui.editors.result.BuildLabelProvider;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorMessages;
import com.ibm.team.build.internal.ui.editors.result.BuildResultPage;
import com.ibm.team.build.internal.ui.editors.result.BuildResultsTreeViewerComparator;
import com.ibm.team.build.internal.ui.editors.result.IDeferredBuildTreeNode;
import com.ibm.team.build.ui.editors.result.IBuildResultContext;

/**
 * A build result tab for compilation results.
 */
public class CompileResultsPage extends BuildResultPage {
    /**
     * Context menu ID for the compile results page.
     */
    public final static String CONTEXT_MENU_ID = "com.ibm.team.build.ui.compilePageContextMenu"; //$NON-NLS-1$

    /**
     * The index of the error count column.
     */
    public final static int ERROR_COUNT_COLUMN_INDEX = 1;

    /**
     * The index of the warning count column.
     */
    public final static int WARNING_COUNT_COLUMN_INDEX = 2;

    /**
     * The index of the time taken column.
     */
    public final static int TIME_TAKEN_COLUMN_INDEX = 3;

    protected TreeViewer fViewer;
    protected Tree fCompileResultsTree;
    private CompileDetailBlock fDetailBlock = new CompileDetailBlock();
    protected Button fShowErrorsButton;
    protected Button fShowWarningsButton;

    protected boolean fShowSuccessNodes;
    protected boolean fShowWarningNodes;

    /**
     * Create the compile results tab.
     * 
     * @param editor
     *            the form editor
     * @param id
     *            the page id
     * @param title
     *            the page title
     * @param buildResultContext
     *            the context that provides access to data related to the build
     *            result
     */
    public CompileResultsPage(FormEditor editor, String id, String title, IBuildResultContext buildResultContext) {
        super(editor, id, title, new String[] { ICompileContribution.EXTENDED_CONTRIBUTION_TYPE_ID },
                buildResultContext);
        fShowSuccessNodes = fShowWarningNodes = !containsFailures(getContributions());
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public PageGroup getPageGroup() {
        return PageGroup.COMPILATION;
    }

    protected void doCreatePageContent(Composite parent) {
        FormToolkit toolkit = getManagedForm().getToolkit();
        toolkit.setBorderStyle(SWT.BORDER);

        FormLayout formLayout = new FormLayout();
        parent.setLayout(formLayout);

        addResultTree(parent, toolkit);
        addDetails(parent, toolkit);
        createContextMenu(CONTEXT_MENU_ID, getSite());

        PlatformUI.getWorkbench().getHelpSystem().setHelp(parent,
                IHelpContextIds.HELP_CONTEXT_BUILD_RESULT_EDITOR_COMPILE_PAGE);
    }

    private void addResultTree(final Composite body, FormToolkit toolkit) {
        Section section = toolkit.createSection(body, ExpandableComposite.TITLE_BAR);
        section.setText(Messages.CompileResultsPage_COMPILE_SECTION_TITLE);
        GridLayoutFactory.fillDefaults().applyTo(section);

        ToolBar toolbar = createToolbar(toolkit, section);
        section.setTextClient(toolbar);

        Composite treeComposite = toolkit.createComposite(body);

        FormData formData = new FormData();
        formData.left = new FormAttachment(0, 0);
        formData.top = new FormAttachment(0, 0);
        formData.right = new FormAttachment(100, 0);
        formData.bottom = new FormAttachment(treeComposite);
        section.setLayoutData(formData);

        formData = new FormData();
        formData.left = new FormAttachment(0, 0);
        formData.top = new FormAttachment(section, 7);
        formData.right = new FormAttachment(100, 0);
        formData.bottom = new FormAttachment(50, 0);
        treeComposite.setLayoutData(formData);

        fCompileResultsTree = toolkit.createTree(treeComposite, SWT.BORDER | SWT.VIRTUAL | SWT.FULL_SELECTION
                | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
        fCompileResultsTree.setLinesVisible(true);
        fCompileResultsTree.setHeaderVisible(true);
        fCompileResultsTree.addListener(SWT.SetData, new Listener() {
            public void handleEvent(Event event) {
                fCompileResultsTree.removeListener(SWT.SetData, this);
                body.layout(true, true);
            }
        });

        TreeColumnLayout treeColumnLayout = new TreeColumnLayout();

        createTreeColumn(fCompileResultsTree, treeColumnLayout, BuildResultEditorMessages.NAME_COLUMN_TITLE, 60,
                SWT.LEFT, NAME_COLUMN_INDEX);
        createTreeColumn(fCompileResultsTree, treeColumnLayout, BuildResultEditorMessages.ERROR_COUNT_COLUMN_TITLE, 20,
                SWT.RIGHT, ERROR_COUNT_COLUMN_INDEX);
        createTreeColumn(fCompileResultsTree, treeColumnLayout, BuildResultEditorMessages.WARNING_COUNT_COLUMN_TITLE,
                20, SWT.RIGHT, WARNING_COUNT_COLUMN_INDEX);

        treeComposite.setLayout(treeColumnLayout);

        fViewer = new TreeViewer(fCompileResultsTree);
        fViewer.setLabelProvider(new BuildLabelProvider());
        fViewer.setContentProvider(new BuildDeferredContentProvider(fViewer));
        fViewer.setComparator(new BuildResultsTreeViewerComparator());

        TreeColumn nameColumn = fViewer.getTree().getColumn(NAME_COLUMN_INDEX);
        fViewer.getTree().setSortColumn(nameColumn);
        fViewer.getTree().setSortDirection(SWT.UP);

        fViewer.addOpenListener(getOpenListener());

        Image showSuccessImage = BuildCompositeImageRegistry.getImage(
                BuildUIPlugin.getImage("icons/view16/compile.gif"), //$NON-NLS-1$
                BuildStatus.OK, true);

        Image showWarningImage = BuildCompositeImageRegistry.getImage(
                BuildUIPlugin.getImage("icons/view16/compile.gif"), //$NON-NLS-1$
                BuildStatus.WARNING, false);

        createFilterButton(showSuccessImage, Messages.CompileResultsPage_SUCCESSFUL_PROJECTS_FILTER_TOOLTIP, toolbar,
                fSuccessNodeFilter, fViewer, fShowSuccessNodes);

        createFilterButton(showWarningImage, Messages.CompileResultsPage_PROJECT_WITH_WARNINGS_FILTER_TOOLTIP, toolbar,
                fWarningNodeFilter, fViewer, fShowWarningNodes);

        // Set the input last to avoid refresh conflicts with the previous call
        // to createFilterButton.
        fViewer.setInput(new CompileBuildResultNode(getContributions()));
    }

    @Override
    protected void toggleFilter(boolean buttonSelected, StructuredViewer viewer, ViewerFilter filter) {
        if (filter == fSuccessNodeFilter) {
            fShowSuccessNodes = buttonSelected;
        } else {
            fShowWarningNodes = buttonSelected;
        }
        super.toggleFilter(buttonSelected, viewer, filter);
    }

    protected IOpenListener getOpenListener() {
        return new IOpenListener() {
            public void open(OpenEvent event) {
                IStructuredSelection selection = (IStructuredSelection) event.getSelection();
                if (selection.size() == 1) {
                    Object element = selection.getFirstElement();
                    if (element instanceof CompileSourceNode) {
                        CompileSourceNode sourceNode = (CompileSourceNode) element;
                        openSourceFile(sourceNode.getCompileSource());
                    }
                }
            }
        };
    }

    protected boolean openSourceFile(ICompileSource compileSource) {
        return JdtWorkspaceHelper.openSourceFile(compileSource, 1, getEditorSite());
    }

    protected SelectionAdapter fRadioListener = new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
            IStructuredSelection selection = (IStructuredSelection) getViewer().getSelection();
            if (selection.size() == 1) {
                if (selection.getFirstElement() instanceof ICompileDetailProvider) {
                    setDetailProvider((ICompileDetailProvider) selection.getFirstElement(),
                            fShowErrorsButton.getSelection());
                }
            }
        }
    };

    protected void setDetailProvider(ICompileDetailProvider provider, boolean showErrors) {
        fDetailBlock.setDetailProvider(provider, showErrors);
    }

    private void addDetails(Composite body, FormToolkit toolkit) {
        Section outputSection = toolkit.createSection(body, ExpandableComposite.TITLE_BAR | Section.DESCRIPTION);
        outputSection.setDescription(Messages.CompileResultsPage_OUTPUT_SECTION_DESCRIPTION);
        outputSection.setText(Messages.CompileResultsPage_OUTPUT_SECTION_TITLE);
        GridLayoutFactory.fillDefaults().applyTo(outputSection);

        FormData formData = new FormData();
        formData.left = new FormAttachment(0, 0);
        formData.top = new FormAttachment(50, 7);
        formData.right = new FormAttachment(100, 0);
        formData.bottom = new FormAttachment(100, 0);
        outputSection.setLayoutData(formData);

        Composite composite = toolkit.createComposite(outputSection);
        GridLayoutFactory.fillDefaults().applyTo(composite);
        GridDataFactory.fillDefaults().grab(true, true).minSize(300, 150).applyTo(composite);
        outputSection.setClient(composite);

        Composite radioComposite = toolkit.createComposite(composite);
        GridLayoutFactory.fillDefaults().numColumns(2).applyTo(radioComposite);
        GridDataFactory.fillDefaults().grab(true, false).applyTo(radioComposite);

        fShowErrorsButton = toolkit.createButton(radioComposite, Messages.CompileResultsPage_ERRORS_BUTTON_TITLE,
                SWT.RADIO);
        fShowErrorsButton.setSelection(true);
        fShowErrorsButton.addSelectionListener(fRadioListener);
        fShowWarningsButton = toolkit.createButton(radioComposite, Messages.CompileResultsPage_WARNINGS_BUTTON_TITLE,
                SWT.RADIO);

        fDetailBlock.createContents(composite, toolkit, getEditor());
        fViewer.addSelectionChangedListener(getSelectionChangedListener());
    }

    protected ISelectionChangedListener getSelectionChangedListener() {
        return new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                IStructuredSelection selection = (IStructuredSelection) event.getSelection();
                if (selection.size() == 1) {

                    /*
                     * The refresh that occurs at the end of a removeFilter call
                     * generates a bogus SelectionEvent. TODO: remove after
                     * Eclipse Bug 176106 is fixed. Update - bug is fixed in 3.4
                     * M1, so we can remove this when we move to 3.4.
                     */
                    if (inRemoveFilterCall()) {
                        return;
                    }

                    Object element = selection.getFirstElement();
                    if (element instanceof ICompileDetailProvider) {
                        ICompileDetailProvider provider = (ICompileDetailProvider) element;
                        if (provider.hasErrors()) {
                            fShowWarningsButton.setSelection(false);
                            fShowErrorsButton.setSelection(true);
                        } else if (provider.hasWarnings()) {
                            fShowWarningsButton.setSelection(true);
                            fShowErrorsButton.setSelection(false);
                        }
                        setDetailProvider((ICompileDetailProvider) element, fShowErrorsButton.getSelection());
                        return;
                    }
                }

                setDetailProvider(null, false);
            }
        };
    }

    protected Object getSelectedItem() {
        IStructuredSelection selection = (IStructuredSelection) fViewer.getSelection();
        return selection.getFirstElement();
    }

    protected Tree getCompileResultsTree() {
        return fCompileResultsTree;
    }

    @Override
    protected CopyStrategy getCopyStrategy(Control control) {
        return control == getCompileResultsTree() ? CopyStrategy.FIRST : CopyStrategy.DEFAULT;
    }

    @Override
    protected String getCopyActionItemNameForSelectedItem(Control control) {
        Object selectedItem = getSelectedItem();

        if (selectedItem instanceof CompileComponentNode) {
            return Messages.CompileResultsPage_COMPONENT_NAME;
        } else if (selectedItem instanceof CompilePackageNode) {
            if (((CompilePackageNode) selectedItem).getPackageType().equals(ICompilePackage.PACKAGE_TYPE_PLUGIN)) {
                return Messages.CompileResultsPage_PLUGIN_NAME;
            }
            return Messages.CompileResultsPage_PACKAGE_NAME;
        } else { // if (selectedItem instanceof CompileSourceNode)
            return Messages.CompileResultsPage_FILENAME;
        }
    }

    /*
     * (non-Javadoc)
     */
    protected Viewer getViewer() {
        return fViewer;
    }

    protected void createTreeColumn(Tree tree, TreeColumnLayout layout, String columnName, int weight, int style,
            final int index) {
        TreeColumn treeColumn = new TreeColumn(tree, style, index);
        layout.setColumnData(treeColumn, new ColumnWeightData(weight));
        treeColumn.setText(columnName);
        treeColumn.addSelectionListener(getColumnSelectionListener(tree, treeColumn));
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getContextHelpId() {
        return IHelpContextIds.HELP_CONTEXT_BUILD_RESULT_EDITOR_COMPILE_PAGE;
    }

    protected ViewerFilter fSuccessNodeFilter = new ViewerFilter() {
        public boolean select(Viewer viewer, Object parentElement, Object element) {
            if (element instanceof ICompileTreeNode) {
                // Filter out success elements by only allowing elements which
                // have errors or warnings.
                IDeferredBuildTreeNode node = (IDeferredBuildTreeNode) element;
                return node.hasProblems();
            }
            return true;
        }
    };

    protected ViewerFilter fWarningNodeFilter = new ViewerFilter() {
        public boolean select(Viewer viewer, Object parentElement, Object element) {
            if (element instanceof ICompileTreeNode) {
                // Filter out warning elements by allowing elements which have
                // no warnings.
                // Also allow elements which have errors (even if they have
                // warnings).
                ICompileTreeNode node = (ICompileTreeNode) element;
                return (!node.hasWarnings()) || node.hasErrors();
            }
            return true;
        }
    };
}