/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.compile;

import java.util.Collection;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.ICompileProblem;
import com.ibm.team.build.common.model.ICompileSource;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Represents a compile source file in the compile results tree.
 */
public class CompileSourceNode extends AbstractCompileTreeNode implements IComplexCompileDetailProvider {

    private ICompileSource fCompileSource;

    /**
     * Create a JUnitTestCaseNode representing the given test case with the
     * given test class node as parent.
     * 
     * @param parentNode
     *            The {@link CompilePackageNode} that is the parent of this
     *            node.
     * @param compileSource
     *            The {@link ICompileSource} object that this node represents.
     */
    public CompileSourceNode(CompilePackageNode parentNode, ICompileSource compileSource) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$
        ValidationHelper.validateNotNull("compileSource", compileSource); //$NON-NLS-1$
        fCompileSource = compileSource;
    }

    /**
     * Return the {@link ICompileSource} object that this node represents.
     * 
     * @return The {@link ICompileSource} object this node represents.
     */
    public ICompileSource getCompileSource() {
        return fCompileSource;
    }

    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        return null;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == CompileResultsPage.NAME_COLUMN_INDEX) {
            ImageDescriptor imageDesc = PlatformUI.getWorkbench().getEditorRegistry().getImageDescriptor(getCompileSource().getSourceName());
            Image newImage = BuildUIPlugin.getImage(imageDesc);

            BuildStatus status = BuildStatus.OK;
            if (fCompileSource.getWarningCount() > 0) {
                status = BuildStatus.WARNING;
            }
            if (fCompileSource.getErrorCount() > 0) {
                status = BuildStatus.ERROR;
            }

            return BuildCompositeImageRegistry.getImage(newImage, status, false);
        }
        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.NAME_COLUMN_INDEX:
                return fCompileSource.getSourceName();
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return Integer.toString(fCompileSource.getWarningCount());
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return Integer.toString(fCompileSource.getErrorCount());
        }
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return (long) fCompileSource.getWarningCount();
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return (long) fCompileSource.getErrorCount();
        }
        return null;
    }
    
    @Override
    public boolean hasChildren() {
        return false;
    }

    @Override
    public String getNodeName() {
        return fCompileSource.getSourceName();
    }

    @Override
    public boolean hasProblems() {
        return fCompileSource.getErrorCount() > 0 || fCompileSource.getWarningCount() > 0;
    }

    @Override
    public boolean hasWarnings() {
        return fCompileSource.getWarningCount() > 0;
    }

    @Override
    public boolean hasErrors() {
        return fCompileSource.getErrorCount() > 0;
    }

    @Override
    public ICompileProblem[] getErrors(IProgressMonitor monitor) throws TeamRepositoryException {
        List<ICompileProblem> errors = fCompileSource.getErrors();
        return errors.toArray(new ICompileProblem[errors.size()]);
    }

    @Override
    public ICompileProblem[] getWarnings(IProgressMonitor monitor) throws TeamRepositoryException {
        List<ICompileProblem> warnings = fCompileSource.getWarnings();
        return warnings.toArray(new ICompileProblem[warnings.size()]);
    }

    @Override
    public String getErrorDetails(IProgressMonitor monitor) throws TeamRepositoryException {
        return getDetails("ERROR", fCompileSource.getErrors()); //$NON-NLS-1$
    }

    @Override
    public String getWarningDetails(IProgressMonitor monitor) throws TeamRepositoryException {
        return getDetails("WARNING", fCompileSource.getWarnings()); //$NON-NLS-1$
    }

    private String getDetails(String type, Collection problems) {
        StringBuffer details = new StringBuffer();
        for (Object object : problems) {
            ICompileProblem problem = (ICompileProblem) object;
            details.append(NLS.bind(Messages.CompileSourceNode_PROBLEM_STRING, new String[] {
                    String.valueOf(problem.getLineNumber()), type, problem.getMessageText(), problem.getSourceText() }));
            int spaces = problem.getSourceStart() + 4;
            while (spaces-- > 0) {
                details.append(' ');
            }
            int carets = problem.getSourceEnd() - problem.getSourceStart();
            while (carets-- >= 0) {
                details.append('^');
            }
            details.append('\n');
            details.append('\n');
        }
        return new String(details);
    }
}
