/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.junit;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.common.model.IJUnitTestSuiteContributionHandle;
import com.ibm.team.build.internal.ui.editors.result.AbstractBuildTreeNode;
import com.ibm.team.build.internal.ui.editors.result.IDeferredBuildTreeNode;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * The root node of the JUnit results tree. Represents an IBuildResult.
 */
public class JUnitBuildResultNode extends AbstractBuildTreeNode {

    private static final List PARTIAL_TEST_SUITE_PROPERTIES = new LinkedList();

    static {
        PARTIAL_TEST_SUITE_PROPERTIES.add(IJUnitTestSuiteContribution.PROPERTY_ERROR_DATA);
        PARTIAL_TEST_SUITE_PROPERTIES.add(IJUnitTestSuiteContribution.PROPERTY_ERROR_COUNT);
        PARTIAL_TEST_SUITE_PROPERTIES.add(IJUnitTestSuiteContribution.PROPERTY_FAILURE_COUNT);
        PARTIAL_TEST_SUITE_PROPERTIES.add(IJUnitTestSuiteContribution.PROPERTY_TEST_COUNT);
        PARTIAL_TEST_SUITE_PROPERTIES.add(IJUnitTestSuiteContribution.PROPERTY_TEST_SUITE_CLASS_NAME);
    }

    private IBuildResultContribution[] fBuildResultContributions;
    private Map<String, JUnitComponentNode> fComponentNodeMap = new HashMap<String, JUnitComponentNode>();
    private JUnitComponentNode fUnnamedComponentNode;

    /**
     * Create a BuildResultNode representing the given buildResult.
     * 
     * @param buildResultContributions
     *            The {@link IBuildResultContribution}s related to the build
     *            result.
     */
    public JUnitBuildResultNode(IBuildResultContribution[] buildResultContributions) {
        super(null);
        fBuildResultContributions = buildResultContributions;
    }

    @Override
    public boolean hasChildren() {
        return true;
    }

    @Override
    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        IDeferredBuildTreeNode[] children = null;

        if (fBuildResultContributions.length > 0) {
            List<IJUnitTestSuiteContribution> testSuites = fetchTestSuites(fBuildResultContributions, monitor);

            if (hasComponents(fBuildResultContributions)) {
                children = getComponentNodes(fBuildResultContributions, testSuites);
            } else {
                children = getTestSuiteNodes(fBuildResultContributions, testSuites, monitor);
            }
        }

        return children;
    }

    protected List<IJUnitTestSuiteContribution> fetchTestSuites(IBuildResultContribution[] contributions,
            IProgressMonitor monitor) throws TeamRepositoryException {

        List<IJUnitTestSuiteContribution> testSuites = new ArrayList<IJUnitTestSuiteContribution>();

        List<IJUnitTestSuiteContributionHandle> testSuiteHandles = new ArrayList<IJUnitTestSuiteContributionHandle>();

        ITeamRepository teamRepository = (ITeamRepository) contributions[0].getExtendedContribution().getOrigin();

        // Collect handles for all the test suites.
        for (IBuildResultContribution contribution : contributions) {
            testSuiteHandles.add((IJUnitTestSuiteContributionHandle) contribution.getExtendedContribution());
        }

        // Fetch the test suites and create nodes for them.
        testSuites.addAll(teamRepository.itemManager().fetchPartialItems(testSuiteHandles, IItemManager.DEFAULT,
                PARTIAL_TEST_SUITE_PROPERTIES, monitor));

        return testSuites;
    }

    private IDeferredBuildTreeNode[] getTestSuiteNodes(IBuildResultContribution[] contributions,
            List<IJUnitTestSuiteContribution> testSuites, IProgressMonitor monitor) {
        List<JUnitTestSuiteNode> testSuiteNodes = new ArrayList<JUnitTestSuiteNode>(testSuites.size());

        for (int i = 0; i < testSuites.size(); i++) {
            testSuiteNodes.add(new JUnitTestSuiteNode(this, contributions[i], testSuites.get(i)));
        }
        return (IDeferredBuildTreeNode[]) testSuiteNodes.toArray(new IDeferredBuildTreeNode[testSuiteNodes.size()]);
    }

    private IDeferredBuildTreeNode[] getComponentNodes(IBuildResultContribution[] contributions,
            List<IJUnitTestSuiteContribution> testSuites) {
        fComponentNodeMap.clear();

        for (int i = 0; i < contributions.length; i++) {
            JUnitComponentNode node = getComponentNode(contributions[i]);
            node.addContribution(contributions[i], testSuites.get(i));
        }

        return (IDeferredBuildTreeNode[]) fComponentNodeMap.values().toArray(
                new IDeferredBuildTreeNode[fComponentNodeMap.size()]);
    }

    private JUnitComponentNode getComponentNode(IBuildResultContribution contribution) {
        JUnitComponentNode node = null;
        if (contribution.getComponentName() != null) {
            node = fComponentNodeMap.get(contribution.getComponentName());
            if (node == null) {
                node = new JUnitComponentNode(this, contribution.getComponentName());
                fComponentNodeMap.put(contribution.getComponentName(), node);
            }
        } else {
            node = getUnnamedComponentNode();
        }

        return node;
    }

    private JUnitComponentNode getUnnamedComponentNode() {
        if (fUnnamedComponentNode == null) {
            fUnnamedComponentNode = new JUnitComponentNode(this, null);
        }
        return fUnnamedComponentNode;
    }

    private boolean hasComponents(IBuildResultContribution[] contributions) {
        for (IBuildResultContribution contribution : contributions) {
            if (contribution.getComponentName() != null && contribution.getComponentName().length() > 0) {
                return true;
            }
        }
        return false;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        return null;
    }
    
    @Override
    public boolean hasProblems() {
        return false;
    }

    @Override
    public String getNodeName() {
        return null;
    }

}
