/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.junit;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.AbstractBuildTreeNode;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorMessages;
import com.ibm.team.build.internal.common.helper.TimeFormatHelper;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * A node representing a component grouping of JUnit test suite contributions.
 * The children of the node are JUnit test suites.
 */
public class JUnitComponentNode extends AbstractBuildTreeNode {
    private List<IBuildResultContribution> fBuildResultContributions = new ArrayList<IBuildResultContribution>();
    private List<IJUnitTestSuiteContribution> fTestSuites = new ArrayList<IJUnitTestSuiteContribution>();

    private String fComponentName;

    /**
     * Create a test suite node.
     * 
     * @param parentNode
     *            the parent node of this node
     * @param componentName
     *            the component name; may be null or empty to indicate the
     *            component name is not set
     */
    public JUnitComponentNode(JUnitBuildResultNode parentNode, String componentName) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$

        fComponentName = componentName;
    }

    /**
     * Add a contribution as a child of this component node.
     * 
     * @param contribution
     *            The <code>IBuildResultContribution</code>.
     * @param testSuite
     *            The corresponding <code>IJUnitTestSuiteContribution</code>.
     */
    public void addContribution(IBuildResultContribution contribution, IJUnitTestSuiteContribution testSuite) {
        fBuildResultContributions.add(contribution);
        fTestSuites.add(testSuite);
    }

    @Override
    public boolean hasChildren() {
        return true;
    }

    @Override
    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        JUnitTestSuiteNode[] testSuiteNodes = new JUnitTestSuiteNode[fTestSuites.size()];
        for (int i = 0; i < fTestSuites.size(); i++) {
            IJUnitTestSuiteContribution testSuite = (IJUnitTestSuiteContribution) fTestSuites.get(i);
            testSuiteNodes[i] = new JUnitTestSuiteNode(this, fBuildResultContributions.get(i), testSuite);
        }

        return testSuiteNodes;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == JUnitResultsPage.NAME_COLUMN_INDEX) {
            Image image = BuildUIPlugin.getImage("icons/obj16/component_obj.gif"); //$NON-NLS-1$

            BuildStatus status = BuildStatus.OK;
            if (componentContainsErrors()) {
                status = BuildStatus.ERROR;
            }

            return BuildCompositeImageRegistry.getImage(image, status);
        }

        return null;
    }

    private boolean componentContainsErrors() {
        for (IBuildResultContribution contribution : fBuildResultContributions) {
            if (contribution.getStatus() == BuildStatus.ERROR) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.NAME_COLUMN_INDEX:
                return getNodeName();
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return Integer.toString(getTestCount());
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return Integer.toString(getFailureCount());
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return Integer.toString(getErrorCount());
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return TimeFormatHelper.formatAbbreviatedTime(getTimeTaken());
        }
        return null;
    }

    private int getTestCount() {
        int testCount = 0;
        for (IJUnitTestSuiteContribution testSuite : fTestSuites) {
            testCount += testSuite.getTestCount();
        }
        return testCount;
    }

    private int getFailureCount() {
        int failureCount = 0;
        for (IJUnitTestSuiteContribution testSuite : fTestSuites) {
            failureCount += testSuite.getFailureCount();
        }
        return failureCount;
    }

    private int getErrorCount() {
        int errorCount = 0;
        for (IJUnitTestSuiteContribution testSuite : fTestSuites) {
            errorCount += testSuite.getErrorCount();
        }
        return errorCount;
    }

    private long getTimeTaken() {
        long timeTaken = 0;
        for (IBuildResultContribution contribution : fBuildResultContributions) {
            timeTaken += contribution.getTimeTaken();
        }
        return timeTaken;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return (long) getTestCount();
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return (long) getFailureCount();
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return (long) getErrorCount();
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return getTimeTaken();
        }
        return null;
    }
    
    @Override
    public boolean hasProblems() {
        return getErrorCount() > 0 || getFailureCount() > 0;
    }

    @Override
    public String getNodeName() {
        if (fComponentName == null || fComponentName.equals("")) { //$NON-NLS-1$
            return BuildResultEditorMessages.NO_COMPONENT_LABEL;
        }
        return fComponentName;
    }
}
