/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.junit;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.forms.editor.FormEditor;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.common.model.IJUnitTestSuiteContributionHandle;
import com.ibm.team.build.ui.editors.result.AbstractBuildResultContributionProvider;
import com.ibm.team.build.ui.editors.result.AbstractBuildResultPage;
import com.ibm.team.build.ui.editors.result.IBuildResultContext;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Summary provider for JUnit information.
 */
public class JUnitContributionProvider extends AbstractBuildResultContributionProvider {
    private int fTestCount;
    private int fErrorCount;
    private int fFailureCount;

    /**
     * Create a JUnitContributionProvider object.
     * 
     * @param buildEditor
     *            The build result form editor
     * @param id
     *            The unique id of the JUnit contribution extension.
     * @param context
     *            The context object giving access to build result data.
     */
    public JUnitContributionProvider(FormEditor buildEditor, String id, IBuildResultContext context) {
        super(buildEditor, id, context);
    }

    public void initializeContributionProvider(IProgressMonitor monitor) throws Exception {
        fTestCount = 0;
        fErrorCount = 0;
        fFailureCount = 0;

        ITeamRepository teamRepository = getBuildResultContext().getTeamRepository();
        List<IJUnitTestSuiteContributionHandle> testSuiteHandles = new ArrayList<IJUnitTestSuiteContributionHandle>();

        IBuildResultContribution[] contributions = getContributions();

        for (int i = 0; i < contributions.length; i++) {
            testSuiteHandles.add((IJUnitTestSuiteContributionHandle) contributions[i].getExtendedContribution());
        }

        if (contributions.length > 0) {
            List properties = Arrays.asList(new String[] { IJUnitTestSuiteContribution.PROPERTY_TEST_COUNT,
                    IJUnitTestSuiteContribution.PROPERTY_ERROR_COUNT,
                    IJUnitTestSuiteContribution.PROPERTY_FAILURE_COUNT });

            List testSuites = teamRepository.itemManager().fetchPartialItems(testSuiteHandles, IItemManager.DEFAULT,
                    properties, monitor);
            for (Object object : testSuites) {
                if (object != null) {
                    IJUnitTestSuiteContribution testSuite = (IJUnitTestSuiteContribution) object;
                    fTestCount += testSuite.getTestCount();
                    fErrorCount += testSuite.getErrorCount();
                    fFailureCount += testSuite.getFailureCount();
                }
            }
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public BuildStatus getStatus() {
        if (fErrorCount > 0 || fFailureCount > 0) {
            return BuildStatus.ERROR;
        }

        return BuildStatus.OK;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public String getSummaryText() {
        if (fTestCount > 0) {
            return MessageFormat.format(Messages.JUnitContributionProvider_SUMMARY_TEXT, String.valueOf(fTestCount),
                    getTestString(), String.valueOf(fFailureCount), getFailureString(), String.valueOf(fErrorCount),
                    getErrorString());
        }
        return Messages.JUnitContributionProvider_NO_TESTS;
    }

    private String getTestString() {
        if (fTestCount == 1) {
            return Messages.JUnitContributionProvider_ONE_TEST;
        }
        return Messages.JUnitContributionProvider_MULTIPLE_TESTS;
    }

    private String getFailureString() {
        if (fFailureCount == 1) {
            return Messages.JUnitContributionProvider_ONE_FAILURE;
        }
        return Messages.JUnitContributionProvider_MULTIPLE_FAILURES;
    }

    private String getErrorString() {
        if (fErrorCount == 1) {
            return Messages.JUnitContributionProvider_ONE_ERROR;
        }
        return Messages.JUnitContributionProvider_MULTIPLE_ERRORS;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getSummaryTitle() {
        return Messages.JUnitContributionProvider_SUMMARY_TITLE;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public boolean isSummaryLink() {
        if (fTestCount > 0) {
            return true;
        }
        return false;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void summaryLinkActivated() {
        getBuildResultEditor().setActivePage(getExtensionId());
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public AbstractBuildResultPage getBuildResultPage() {
        return new JUnitResultsPage(getBuildResultEditor(), getExtensionId(),
                Messages.JUnitContributionProvider_PAGE_TITLE, getBuildResultContext());
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public String[] getExtendedContributionIds() {
        return new String[] { IJUnitTestSuiteContribution.EXTENDED_CONTRIBUTION_TYPE_ID };
    }

}
