/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.junit;

import java.util.ListResourceBundle;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.commands.ActionHandler;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.ITextOperationTarget;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.RetargetAction;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.events.IHyperlinkListener;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.handlers.IHandlerActivation;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.texteditor.FindReplaceAction;
import org.eclipse.ui.texteditor.IWorkbenchActionDefinitionIds;

import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.java.ui.JdtWorkspaceHelper;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditor;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorMessages;
import com.ibm.team.build.internal.common.helper.TimeFormatHelper;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Displays details of a JUnitTest in the BuildResultEditor.
 */
public class JUnitDetailBlock {
	
	/**
     * {@link IMenuListener} that fills the context menu.
     */
    private final class MenuListener implements IMenuListener {
        public void menuAboutToShow(IMenuManager mgr) {
            fillContextMenu(mgr);
        }
    }

    /**
     * FocusListner that sets find and replace target when focus is gained or
     * lost.
     */
    private final class DetailTextFocusListener implements FocusListener {
        public void focusLost(FocusEvent e) {
            ((BuildResultEditor) fEditor).setFindReplaceTarget(null);
            fFind.update();
        }

        public void focusGained(FocusEvent e) {
            ((BuildResultEditor) fEditor).setFindReplaceTarget(fDetailTextViewer.getFindReplaceTarget());
            fFind.update();
        }
    }

    private Hyperlink fTestHyperlink;
    private Hyperlink fClassNameHyperlink;
    private Label fTimeTakenLabel;
    private StyledText fDetailText;
    private SourceViewer fDetailTextViewer;
    private Label fDetailLabel;
    private IWorkbenchAction fCopyAction;
    private IWorkbenchAction fSelectAllAction;
    private FindReplaceAction fFind;
    private FormEditor fEditor;
    private CopyToClipboardAction fCopyTrace;
    private DetailTextFocusListener fFocusListener;
    private IMenuListener fMenuListener;
    private Composite detailComposite;

    /**
     * Creates the contents of the details block.
     * 
     * @param parent
     *            the parent of the details block; never <code>null</code>.
     * @param toolkit
     *            the form toolkit; never <code>null</code>.
     * @param editor
     *            the form editor; never <code>null</code>.
     * @return a new <code>Composite</code> for JUnit test case details.
     */
    public Composite createContents(Composite parent, FormToolkit toolkit, FormEditor editor) {
        fEditor = editor;
        Composite composite = toolkit.createComposite(parent, SWT.NONE);
        GridLayoutFactory.fillDefaults().applyTo(composite);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(composite);

        detailComposite = toolkit.createComposite(composite, SWT.NONE);
        GridLayoutFactory.fillDefaults().numColumns(2).applyTo(detailComposite);
        GridDataFactory.fillDefaults().grab(true, false).applyTo(detailComposite);

        createLabel(detailComposite, toolkit, Messages.JUnitDetailBlock_CLASS_NAME_LABEL);

        fClassNameHyperlink = toolkit.createHyperlink(detailComposite, "", SWT.NONE); //$NON-NLS-1$
        GridDataFactory.swtDefaults().applyTo(fClassNameHyperlink);
        fClassNameHyperlink.addHyperlinkListener(getHyperlinkListener(false));
        fClassNameHyperlink.setVisible(false);

        createLabel(detailComposite, toolkit, Messages.JUnitDetailBlock_METHOD_LABEL);

        fTestHyperlink = toolkit.createHyperlink(detailComposite, "", SWT.NONE); //$NON-NLS-1$
        GridDataFactory.swtDefaults().applyTo(fTestHyperlink);
        fTestHyperlink.addHyperlinkListener(getHyperlinkListener(true));
        fTestHyperlink.setVisible(false);

        createLabel(detailComposite, toolkit, Messages.JUnitDetailBlock_TIME_TAKEN_LABEL);
        fTimeTakenLabel = createLabel(detailComposite, toolkit, ""); //$NON-NLS-1$

        fDetailLabel = createLabel(detailComposite, toolkit, ""); //$NON-NLS-1$
        GridDataFactory.fillDefaults().span(2, 1).applyTo(fDetailLabel);
        fDetailLabel.setVisible(false);

        fDetailTextViewer = new SourceViewer(composite, null, SWT.MULTI | SWT.READ_ONLY | SWT.BORDER | SWT.H_SCROLL
                | SWT.V_SCROLL);
        fDetailTextViewer.configure(new JavaStackTraceConfiguration(editor.getEditorSite()));
        fDetailTextViewer.setDocument(new Document());
        fDetailText = fDetailTextViewer.getTextWidget();
        GridDataFactory.fillDefaults().grab(true, true).span(2, 1).applyTo(fDetailText);
        fDetailText.setVisible(false);
        fDetailText.addFocusListener(getDetailTextFocusListener());
        createActions(fEditor.getEditorSite(), parent);
        createContextMenu(fEditor.getEditorSite());

        return composite;
    }

    /**
     * Create and return the hyperlink listener for opening the java source
     * file.
     * 
     * @param testcase
     *            If true, this is a JUnit testcase, so attempt to open the file
     *            to the testcase method. Otherwise just open the file at the
     *            beginning.
     * @return the hyperlink listener for opening the java source file.
     */
    protected IHyperlinkListener getHyperlinkListener(final boolean testcase) {
        return new HyperlinkAdapter() {
            public void linkActivated(HyperlinkEvent e) {
            	String locInfo = fDetailText !=null ? 
            			UntiTestLocationUtils.getLocationInfo(fDetailText.getText()) : null;
                if (testcase) {
                	if(locInfo != null) {
                		openJUnitSourceFile(locInfo, fClassNameHyperlink.getText(), 
                        		fTestHyperlink.getText(), fEditor.getEditorSite());
                	}else {
                		openJUnitSourceFile(fClassNameHyperlink.getText(), 
                        		fTestHyperlink.getText(), fEditor.getEditorSite());
                	}
                } else {
					if (locInfo != null) {
						openJavaSourceFile(locInfo, fClassNameHyperlink.getText(),
								fEditor.getEditorSite());
					} else {
						openJavaSourceFile(fClassNameHyperlink.getText(),
								fEditor.getEditorSite());
					}
                }
            }
        };
    }
   
    protected boolean openJUnitSourceFile(String className, String junitTestName, IEditorSite editorSite) {
        return JdtWorkspaceHelper.openJUnitSourceFile(className, junitTestName, editorSite);
    }

    protected boolean openJavaSourceFile(String className, IEditorSite editorSite) {
        return JdtWorkspaceHelper.openJavaSourceFile(className, 1, editorSite);
    }
    
    protected boolean openJUnitSourceFile(String locationInfo, String className,
			String junitTestName, IEditorSite editorSite) {
		return JdtWorkspaceHelper.openJUnitSourceFile(UntiTestLocationUtils.getFilePath(locationInfo), 
				UntiTestLocationUtils.getLineLocation(locationInfo), className, junitTestName,
				editorSite);
	}

	protected boolean openJavaSourceFile(String locationInfo,
			String className, IEditorSite editorSite) {
		return JdtWorkspaceHelper.openJavaSourceFile(
				UntiTestLocationUtils.getFilePath(locationInfo), className, 1, editorSite);
	}

    /**
     * Allows access to the focus listener that is used in the detail area.
     * 
     * @return {@link DetailTextFocusListener}, never null
     */
    protected FocusListener getDetailTextFocusListener() {
        if (fFocusListener == null) {
            fFocusListener = new DetailTextFocusListener();
        }
        return fFocusListener;
    }

    private void createContextMenu(IEditorSite site) {
        MenuManager menuMgr = new MenuManager("#PopUp"); //$NON-NLS-1$
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(getMenuListener());

        fCopyAction = ActionFactory.COPY.create(site.getWorkbenchWindow());
        setActionPart(fCopyAction);
        fSelectAllAction = ActionFactory.SELECT_ALL.create(site.getWorkbenchWindow());
        setActionPart(fSelectAllAction);
        fSelectAllAction.setEnabled(true);

        Menu menu = menuMgr.createContextMenu(fDetailText);
        fDetailText.setMenu(menu);
        site.registerContextMenu(menuMgr, fDetailTextViewer.getSelectionProvider());
    }

    // TODO: 50525 - Remove this when we switch to Eclipse 3.4
    protected void setActionPart(IWorkbenchAction action) {
        if (action instanceof RetargetAction) {
            ((RetargetAction) action).partActivated(getEditor());
        }
    }

    /**
     * Allows access to the menu listener used by this class.
     * <p>
     * This method is protected for testing. Subclasses should not override this
     * method.
     * 
     * @return {@link IMenuListener}, never null.
     */
    protected IMenuListener getMenuListener() {
        if (fMenuListener == null) {
            fMenuListener = new MenuListener();
        }
        return fMenuListener;
    }

    /**
     * Adds the context menu actions for the details view.
     */
    protected void fillContextMenu(IMenuManager menu) {
        if (fDetailTextViewer.getDocument() == null) {
            return;
        }

        fCopyAction.setEnabled(fDetailText.getSelectionText().length() > 0);

        menu.add(new Separator());
        menu.add(fCopyAction);
        menu.add(fCopyTrace);
        menu.add(fSelectAllAction);
        menu.add(new Separator());
        menu.add(fFind);
        menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
    }

    protected void createActions(final IEditorSite site, Composite parent) {
        IHandlerService handlerService = getHandlerService(site);

        ResourceBundle findReplaceBundle = new ListResourceBundle() {
            private final Object[][] CONTENTS = new Object[][] {
                    { "JUnit.Find.label", Messages.JUnitDetailBlock_FIND_ACTION_LABEL }, //$NON-NLS-1$
                    { "JUnit.Find.tooltip", Messages.JUnitDetailBlock_FIND_ACTION_TOOLTIP }, //$NON-NLS-1$
                    { "JUnit.Find.image", "" }, //$NON-NLS-1$ //$NON-NLS-2$
                    { "JUnit.Find.description", Messages.JUnitDetailBlock_FIND_ACTION_DESCRIPTION } //$NON-NLS-1$
            };

            protected Object[][] getContents() {
                return CONTENTS;
            }
        };
        fFind = new FindReplaceAction(findReplaceBundle, "JUnit.Find.", site.getPart()); //$NON-NLS-1$
        fFind.setActionDefinitionId(IWorkbenchActionDefinitionIds.FIND_REPLACE);

        ActionHandler findHandler = new ActionHandler(fFind);
        final IHandlerActivation findActivation = handlerService.activateHandler(fFind.getActionDefinitionId(),
                findHandler);

        fCopyTrace = new CopyToClipboardAction(fDetailTextViewer);
        fCopyTrace.setText(Messages.JUnitDetailBlock_COPY_STACK_TRACE_ACTION_LABEL);

        parent.addDisposeListener(new DisposeListener() {
            public void widgetDisposed(DisposeEvent e) {
                IHandlerService service = getHandlerService(site);
                if (service != null) {
                    service.deactivateHandler(findActivation);
                }
            }
        });
    }

    /**
     * Create a new {@link ViewAction} with the given ITextOperationTarget and
     * operationCode. This method exists so that tests can override it.
     * 
     * @param target
     *            the {@link ITextOperationTarget} for the ViewAction
     * @param operationCode
     *            the code for the ViewAction
     * @return a new {@link ViewAction}
     */
    protected ViewAction createViewAction(ITextOperationTarget target, int operationCode) {
        return new ViewAction(target, operationCode);
    }

    /**
     * Allows access to the handler service so that items in the context menu of
     * the detail section can hook into global actions. This method allows test
     * classes to provide a mock handler for testing.
     * 
     * @param site
     *            IEditorSite to use to get the {@link IHandlerService}
     * @return {@link IHandlerService} or null if the service could not be
     *         found.
     */
    protected IHandlerService getHandlerService(final IEditorSite site) {
        return (IHandlerService) site.getService(IHandlerService.class);
    }

    /**
     * Set the test case and update the controls.
     * 
     * @param provider
     *            <code>IJUnitTestDetailProvider</code> to use to fill
     *            controls.
     * @param teamRepository
     *            <code>ITeamRepository</code> where the test case resides.
     */
    protected void setDetailProvider(IJUnitDetailProvider provider, ITeamRepository teamRepository) {
        if (provider == null) {
            fClassNameHyperlink.setText(""); //$NON-NLS-1$
            fClassNameHyperlink.setVisible(false);
            fTestHyperlink.setText(""); //$NON-NLS-1$
            fTestHyperlink.setVisible(false);
            fTimeTakenLabel.setText(""); //$NON-NLS-1$
            fDetailLabel.setText(""); //$NON-NLS-1$
            fDetailText.setVisible(false);
        } else {
            fClassNameHyperlink.setText(provider.getJUnitClassName());
            ((GridData) fClassNameHyperlink.getLayoutData()).widthHint = fClassNameHyperlink.computeSize(SWT.DEFAULT,
                    SWT.DEFAULT, true).x;
            fClassNameHyperlink.setVisible(fClassNameHyperlink.getText().length() > 0);
            fClassNameHyperlink.pack(true);
            fTestHyperlink.setText(provider.getJUnitName());
            ((GridData) fTestHyperlink.getLayoutData()).widthHint = fTestHyperlink.computeSize(SWT.DEFAULT,
                    SWT.DEFAULT, true).x;
            fTestHyperlink.pack(true);
            fTestHyperlink.setVisible(fTestHyperlink.getText().length() > 0);
            fTimeTakenLabel.setText(TimeFormatHelper.formatTime(provider.getTimeTaken()));
            detailComposite.pack(true);

            if (provider instanceof IJUnitRunErrorDetailProvider) {
                updateDetailsAsynch((IJUnitRunErrorDetailProvider) provider, teamRepository);
            } else {
                fDetailText.setVisible(false);
                fDetailLabel.setText(""); //$NON-NLS-1$
            }
        }
    }

    /**
     * Update the details section with content from the teamRepository.
     */
    private void updateDetailsAsynch(final IJUnitRunErrorDetailProvider provider, final ITeamRepository teamRepository) {
        Job job = getUpdateDetailsJob(provider, teamRepository);
        job.schedule();
    }

    protected Job getUpdateDetailsJob(final IJUnitRunErrorDetailProvider provider, final ITeamRepository teamRepository) {
        return new Job(BuildResultEditorMessages.JUnitDetailBlock_FETCHING_JUNIT_DETAILS) {
            protected IStatus run(IProgressMonitor monitor) {
                String problemString = provider.getDetailString();
                String detailLabel = ""; //$NON-NLS-1$

                try {
                    if (provider.isError()) {
                        detailLabel = Messages.JUnitDetailBlock_ERROR_DETAIL_LABEL;
                        if (problemString == null) {
                            problemString = getProblemContentAsString(provider.getErrorData(), teamRepository, monitor);
                            provider.setDetailString(problemString);
                        }
                    } else if (provider.isFailure()) {
                        detailLabel = Messages.JUnitDetailBlock_FAILURE_DETAIL_LABEL;
                        if (problemString == null) {
                            problemString = getProblemContentAsString(provider.getFailureData(), teamRepository,
                                    monitor);
                            provider.setDetailString(problemString);
                        }
                    }
                } catch (TeamRepositoryException e) {
                    BuildUIPlugin.log(e);
                    problemString = NLS.bind(Messages.JUnitDetailBlock_FAILURE_TO_RETRIEVE_PROBLEM_DATA, e.toString());
                }

                final String finalProblemString = problemString == null ? "" : problemString; //$NON-NLS-1$
                final String finalDetailLabel = detailLabel;
                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        updateDetailArea(finalProblemString, finalDetailLabel);
                    }
                });
                return Status.OK_STATUS;
            }
        };
    }

    private Label createLabel(Composite parent, FormToolkit toolkit, String text) {
        Label label = toolkit.createLabel(parent, text);
        label.setLayoutData(new GridData(text.length() == 0 ? GridData.FILL_HORIZONTAL : SWT.NONE));
        return label;
    }

    /**
     * Allows access to the text viewer in the details section.
     * 
     * @return
     */
    protected SourceViewer getDetailTextViewer() {
        return fDetailTextViewer;
    }

    /**
     * @return the testHyperlink
     */
    protected Hyperlink getTestHyperlink() {
        return fTestHyperlink;
    }

    /**
     * @return the classNameLabel
     */
    protected Hyperlink getClassNameHyperlink() {
        return fClassNameHyperlink;
    }

    /**
     * @return the timeTakenLabel
     */
    protected Label getTimeTakenLabel() {
        return fTimeTakenLabel;
    }

    /**
     * @return the detailLabel
     */
    protected Label getDetailLabel() {
        return fDetailLabel;
    }

    /**
     * @return the detailText
     */
    protected StyledText getDetailText() {
        return fDetailText;
    }

    /**
     * Allows access to the editor passed to createContents.
     * 
     * @return the {@link FormEditor}, which is a {@link BuildResultEditor}.
     *         May be null if createContents.
     */
    protected FormEditor getEditor() {
        return fEditor;
    }

    /**
     * Retrieve the contentToRetrieve from teamRepository. This method is
     * protected so that test methods can override it.
     * 
     * @param contentToRetrieve
     *            {@link IContent} to be retrieved from the repository
     * @param teamRepository
     *            {@link ITeamRepository} in which contentToRetrieve resides
     * @param monitor
     *            {@link IProgressMonitor} used to communicate progress and
     *            check for cancellation
     * @return {@link String} that is text of contentToRetrieve
     * @throws TeamRepositoryException
     *             if there is a problem communicating with the teamRepository.
     */
    protected String getProblemContentAsString(IContent contentToRetrieve, ITeamRepository teamRepository,
            IProgressMonitor monitor) throws TeamRepositoryException {
        return ContentUtil.contentToString(teamRepository, contentToRetrieve, monitor);
    }

    /**
     * Sets the information in the detail area with the problem string and
     * label. This method must be called from a UI thread.
     * <p>
     * This method is protected for testing. Sub-classes should not override
     * this behavior
     * </p>
     * 
     * @param finalProblemString
     *            String to display as the detail text.
     * @param finalDetailLabel
     *            String to display as the detail label.
     */
    protected void updateDetailArea(String problemString, String detailLabel) {
        if (!fDetailLabel.isDisposed()) {
            fDetailLabel.setText(detailLabel);
            fDetailLabel.setVisible(true);
            fDetailTextViewer.getDocument().set(problemString);
            fDetailText.setVisible(problemString.length() != 0);
        }
    }
}
