/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2006, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.junit;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IJUnitTestCase;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.AbstractBuildTreeNode;
import com.ibm.team.build.internal.common.helper.TimeFormatHelper;
import com.ibm.team.repository.common.IContent;

/**
 * Represents a JUnit test class in the JUnits results tree.
 */
public class JUnitTestCaseNode extends AbstractBuildTreeNode implements IJUnitRunErrorDetailProvider {

    private IJUnitTestCase fTestCase = null;
    private String fDetailString = null;

    /**
     * Create a JUnitTestCaseNode representing the given test case with the
     * given test class node as parent.
     * 
     * @param parentNode
     *            The {@link JUnitTestClassNode} that is the parent of this
     *            node.
     * @param testCase
     *            The {@link IJUnitTestCase} object that this node represents.
     */
    public JUnitTestCaseNode(JUnitTestClassNode parentNode, IJUnitTestCase testCase) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$
        ValidationHelper.validateNotNull("testCase", testCase); //$NON-NLS-1$
        fTestCase = testCase;
    }

    /**
     * Return the JUnit test case this node represents.
     * 
     * @return {@link IJUnitTestCase}
     */
    public IJUnitTestCase getTestCase() {
        return fTestCase;
    }

    @Override
    public Object[] doFetchChildren(IProgressMonitor monitor) {
        return null;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == JUnitResultsPage.NAME_COLUMN_INDEX) {
            Image newImage = BuildUIPlugin.getImage("icons/obj16/test_obj.gif"); //$NON-NLS-1$
            BuildStatus status = BuildStatus.OK;
            if (fTestCase.isError() || fTestCase.isFailure()) {
                status = BuildStatus.ERROR;
            }

            return BuildCompositeImageRegistry.getImage(newImage, status);
        }
        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.NAME_COLUMN_INDEX:
                return getJUnitName();
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return "1"; //$NON-NLS-1$
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return isFailure() ? "1" : "0"; //$NON-NLS-1$//$NON-NLS-2$
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return isError() ? "1" : "0"; //$NON-NLS-1$//$NON-NLS-2$
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return TimeFormatHelper.formatAbbreviatedTime((getTimeTaken()));
            case JUnitResultsPage.RUN_ORDER_COLUMN_INDEX:
                return Integer.toString(fTestCase.getOrdinal()) + "    "; //$NON-NLS-1$
        }
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return 1L;
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return isFailure() ? 1L : 0L;
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return isError() ? 1L : 0L;
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return getTimeTaken();
            case JUnitResultsPage.RUN_ORDER_COLUMN_INDEX:
                return (long) fTestCase.getOrdinal();
        }
        return null;
    }
    
    @Override
    public boolean hasChildren() {
        return false;
    }

    @Override
    public boolean hasProblems() {
        return (fTestCase.isError() || fTestCase.isFailure());
    }

    @Override
    public String getNodeName() {
        return getJUnitClassName();
    }

    @Override
    public String getJUnitClassName() {
        return fTestCase.getClassName();
    }

    @Override
    public IContent getErrorData() {
        return fTestCase.getErrorData();
    }

    @Override
    public IContent getFailureData() {
        return fTestCase.getFailureData();
    }

    @Override
    public String getJUnitName() {
        return fTestCase.getName();
    }

    @Override
    public long getTimeTaken() {
        return fTestCase.getTimeTaken();
    }

    @Override
    public boolean isError() {
        return fTestCase.isError();
    }

    @Override
    public boolean isFailure() {
        return fTestCase.isFailure();
    }

    @Override
    public String getDetailString() {
        return fDetailString;
    }

    @Override
    public void setDetailString(String detailString) {
        fDetailString = detailString;
    }
}
