/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.junit;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IJUnitTestCase;
import com.ibm.team.build.common.model.IJUnitTestClass;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.AbstractBuildTreeNode;
import com.ibm.team.build.internal.common.helper.TimeFormatHelper;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Represents a JUnit test class in the JUnits results tree.
 */
public class JUnitTestClassNode extends AbstractBuildTreeNode implements IJUnitDetailProvider {

    private IJUnitTestClass fTestClass = null;

    /**
     * Create a JUnitTestClassNode representing the given test class with the
     * given test suite contribution node as parent.
     * 
     * @param parentNode
     *            The {@link JUnitTestSuiteNode} that is the parent of this
     *            node.
     * @param testClass
     *            The {@link IJUnitTestClass} object that this node represents.
     */
    public JUnitTestClassNode(JUnitTestSuiteNode parentNode, IJUnitTestClass testClass) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$
        ValidationHelper.validateNotNull("testClass", testClass); //$NON-NLS-1$
        fTestClass = testClass;
    }

    @Override
    public boolean hasChildren() {
        return true;
    }

    @Override
    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        // Fetch the complete test class, then add the test cases.
        ITeamRepository teamRepository = (ITeamRepository) fTestClass.getOrigin();
        fTestClass = (IJUnitTestClass) teamRepository.itemManager().fetchCompleteItem(fTestClass, IItemManager.DEFAULT,
                monitor);

        List testCases = fTestClass.getTestCases();
        JUnitTestCaseNode[] testCaseNodes = new JUnitTestCaseNode[testCases.size()];
        int i = 0;
        for (Object testCase : testCases) {
            testCaseNodes[i++] = new JUnitTestCaseNode(this, (IJUnitTestCase) testCase);
        }

        return testCaseNodes;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == JUnitResultsPage.NAME_COLUMN_INDEX) {
            Image newImage = BuildUIPlugin.getImage("icons/obj16/test_obj.gif"); //$NON-NLS-1$

            BuildStatus status = BuildStatus.OK;
            if (fTestClass.getErrorCount() > 0 || fTestClass.getFailureCount() > 0) {
                status = BuildStatus.ERROR;
            }

            return BuildCompositeImageRegistry.getImage(newImage, status);
        }
        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.NAME_COLUMN_INDEX:
                return fTestClass.getClassName();
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return Integer.toString(fTestClass.getTestCount());
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return Integer.toString(fTestClass.getFailureCount());
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return Integer.toString(fTestClass.getErrorCount());
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return TimeFormatHelper.formatAbbreviatedTime((fTestClass.getTimeTaken()));
            case JUnitResultsPage.RUN_ORDER_COLUMN_INDEX:
                return Integer.toString(fTestClass.getOrdinal());
        }
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return (long) fTestClass.getTestCount();
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return (long) fTestClass.getFailureCount();
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return (long) fTestClass.getErrorCount();
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return fTestClass.getTimeTaken();
            case JUnitResultsPage.RUN_ORDER_COLUMN_INDEX:
                return (long) fTestClass.getOrdinal();
        }
        return null;
    }
    
    @Override
    public boolean hasProblems() {
        return (fTestClass.getErrorCount() > 0 || fTestClass.getFailureCount() > 0);
    }

    @Override
    public String getNodeName() {
        return getJUnitClassName();
    }

    @Override
    public String getJUnitClassName() {
        return fTestClass.getClassName();
    }

    @Override
    public String getJUnitName() {
        return ""; //$NON-NLS-1$
    }

    @Override
    public long getTimeTaken() {
        return fTestClass.getTimeTaken();
    }
}
