/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.junit;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IJUnitTestClass;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.AbstractBuildTreeNode;
import com.ibm.team.build.internal.ui.editors.result.IDeferredBuildTreeNode;
import com.ibm.team.build.internal.common.helper.TimeFormatHelper;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * A node representing a JUnit test suite contribution. The node contains both
 * the <code>IBuildResultContribution</code> and the
 * <code>IJUnitTestSuiteContribution</code>.
 */
public class JUnitTestSuiteNode extends AbstractBuildTreeNode implements IJUnitRunErrorDetailProvider {

    private static final List PARTIAL_TEST_CLASS_PROPERTIES = new LinkedList();
    private String fProblemString;

    static {
        PARTIAL_TEST_CLASS_PROPERTIES.add(IJUnitTestClass.PROPERTY_CLASS_NAME);
        PARTIAL_TEST_CLASS_PROPERTIES.add(IJUnitTestClass.PROPERTY_ERROR_COUNT);
        PARTIAL_TEST_CLASS_PROPERTIES.add(IJUnitTestClass.PROPERTY_FAILURE_COUNT);
        PARTIAL_TEST_CLASS_PROPERTIES.add(IJUnitTestClass.PROPERTY_TEST_COUNT);
        PARTIAL_TEST_CLASS_PROPERTIES.add(IJUnitTestClass.PROPERTY_TIME_TAKEN);
        PARTIAL_TEST_CLASS_PROPERTIES.add(IJUnitTestClass.PROPERTY_ORDINAL);
    }

    private IJUnitTestSuiteContribution fTestSuiteContribution;
    private IBuildResultContribution fBuildResultContribution;

    /**
     * Create a test suite node.
     * 
     * @param parentNode
     *            the parent node of this node
     * @param buildResultContribution
     *            the build result contribution
     * @param testSuite
     *            the test suite
     */
    public JUnitTestSuiteNode(IDeferredBuildTreeNode parentNode, IBuildResultContribution buildResultContribution,
            IJUnitTestSuiteContribution testSuite) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildResultContribution", buildResultContribution); //$NON-NLS-1$
        ValidationHelper.validateNotNull("testSuite", testSuite); //$NON-NLS-1$
        fBuildResultContribution = buildResultContribution;
        fTestSuiteContribution = testSuite;
    }

    /**
     * Get the test suite represented by this node.
     * 
     * @return the test suite represented by this node
     */
    public IJUnitTestSuiteContribution getTestSuiteContribution() {
        return fTestSuiteContribution;
    }

    /**
     * Get the build result contribution represented by this node.
     * 
     * @return the build result contribution represented by this node
     */
    public IBuildResultContribution getBuildResultContribution() {
        return fBuildResultContribution;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean hasChildren() {
        if (getChildren() != null) {
            return getChildren().length > 0;
        } else {
            // If we haven't fetched the children yet, check the error data. If
            // there was an error at the test suite level, there should be no
            // children. Otherwise we just assume there are children until we
            // fetch and know for sure.
            return fTestSuiteContribution.getErrorData() == null;
        }
    }

    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        // Fetch the complete test suite, then fetch the test classes.
        ITeamRepository teamRepository = (ITeamRepository) fTestSuiteContribution.getOrigin();
        fTestSuiteContribution = (IJUnitTestSuiteContribution) teamRepository.itemManager().fetchCompleteItem(
                fTestSuiteContribution, IItemManager.DEFAULT, monitor);

        List testClassHandles = fTestSuiteContribution.getTestClasses();
        List fetchedTestClasses = teamRepository.itemManager().fetchPartialItems(testClassHandles,
                IItemManager.DEFAULT, PARTIAL_TEST_CLASS_PROPERTIES, monitor);

        JUnitTestClassNode[] testClasses = new JUnitTestClassNode[fetchedTestClasses.size()];
        int i = 0;
        for (Object testClass : fetchedTestClasses) {
            testClasses[i++] = new JUnitTestClassNode(this, (IJUnitTestClass) testClass);
        }

        return testClasses;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == JUnitResultsPage.NAME_COLUMN_INDEX) {
            Image newImage = BuildUIPlugin.getImage("icons/obj16/test_obj.gif"); //$NON-NLS-1$

            BuildStatus status = BuildStatus.OK;

            if (fTestSuiteContribution.getErrorCount() > 0 || fTestSuiteContribution.getFailureCount() > 0) {
                status = BuildStatus.ERROR;
            }

            return BuildCompositeImageRegistry.getImage(newImage, status);
        }

        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.NAME_COLUMN_INDEX:
                return fBuildResultContribution.getLabel();
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return Integer.toString(fTestSuiteContribution.getTestCount());
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return Integer.toString(fTestSuiteContribution.getFailureCount());
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return Integer.toString(fTestSuiteContribution.getErrorCount());
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return TimeFormatHelper.formatAbbreviatedTime((fBuildResultContribution.getTimeTaken()));
        }
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case JUnitResultsPage.TEST_COUNT_COLUMN_INDEX:
                return (long) fTestSuiteContribution.getTestCount();
            case JUnitResultsPage.FAILURE_COUNT_COLUMN_INDEX:
                return (long) fTestSuiteContribution.getFailureCount();
            case JUnitResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return (long) fTestSuiteContribution.getErrorCount();
            case JUnitResultsPage.TIME_TAKEN_COLUMN_INDEX:
                return fBuildResultContribution.getTimeTaken();
        }
        return null;
    }
    
    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean hasProblems() {
        return fTestSuiteContribution.getErrorCount() > 0 || fTestSuiteContribution.getFailureCount() > 0;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getJUnitName() {
        return ""; //$NON-NLS-1$
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getNodeName() {
        return getJUnitClassName();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getJUnitClassName() {
        return fTestSuiteContribution.getTestSuiteClassName();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean isError() {
        return fTestSuiteContribution.getErrorData() != null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean isFailure() {
        return false;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IContent getErrorData() {
        return fTestSuiteContribution.getErrorData();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IContent getFailureData() {
        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public long getTimeTaken() {
        return fBuildResultContribution.getTimeTaken();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getDetailString() {
        return fProblemString;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setDetailString(String detailString) {
        fProblemString = detailString;
    }
}
