/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.junit;

import org.eclipse.jface.text.IRegion;
import org.eclipse.ui.IEditorSite;

import com.ibm.team.build.internal.java.ui.AbstractJavaSourceHyperlink;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorMessages;

/**
 * Hyperlink for a line in a java stack traces.
 */
public class JavaStackTraceHyperlinkNoConsole extends AbstractJavaSourceHyperlink {

    private final String fLine;
    private final IRegion fRegion;
    private String fFileName;

    /**
     * Create a new {@link JavaStackTraceHyperlinkNoConsole}.
     * 
     * @param line
     *            line of stack trace, minus the beginning "at"
     * @param region
     *            {@link IRegion} region in of line.
     * @param editorSite
     *            the {@link IEditorSite} in which the hyperlink exists.
     */
    public JavaStackTraceHyperlinkNoConsole(String line, IRegion region, IEditorSite editorSite) {
        super(editorSite);
        fLine = line;
        fRegion = region;
    }

    /**
     * Returns this link's text
     */
    protected String getLinkText() {
        int linkMiddle = fLine.indexOf(".java:"); //$NON-NLS-1$
        int linkStart = fLine.lastIndexOf(' ', linkMiddle);
        int linkEnd = fLine.indexOf(')', linkMiddle);
        return fLine.substring(linkStart == -1 ? 0 : linkStart + 1, linkEnd + 1);
    }

    public String getHyperlinkText() {
        return fLine;
    }

    public String getTypeLabel() {
        return BuildResultEditorMessages.JavaStackTraceHyperlinkNoConsole_JAVA_STACK_TRACE_LABEL;
    }

    public IRegion getHyperlinkRegion() {
        return fRegion;
    }

    protected String getFileName() {
        return fFileName;
    }
    
    /**
     * Returns the fully qualified name of the type to open
     * 
     * @return fully qualified type name
     */
    public String getTypeName() {
        String linkText = getLinkText();
        int start = linkText.indexOf('(');
        int end = linkText.indexOf(')');
        if (start >= 0 && end > start) {
            // linkText could be something like packageA.TypeB(TypeA.java:45)
            // need to look in packageA.TypeA for line 45 since TypeB is defined
            // in TypeA.java
            // Inner classes can be ignored because we're using file and line
            // number

            // get File name
            String typeName = linkText.substring(start + 1, end);
            
            // remove the :<line number>
            int index = typeName.indexOf(":"); //$NON-NLS-1$
            if (index >= 0) {
                typeName = typeName.substring(0, index);
            }
            
            fFileName = typeName;
            
            String qualifier = linkText.substring(0, start);
            // remove the method name
            start = qualifier.lastIndexOf('.');

            if (start >= 0) {
                // remove the class name
                start = qualifier.substring(0, start).lastIndexOf('.');
                if (start == -1) {
                    start = 0; // default package
                }
            }

            if (start >= 0) {
                qualifier = qualifier.substring(0, start);
            }

            // remove the .java
            index = typeName.indexOf(".java"); //$NON-NLS-1$
            if (index >= 0) {
                typeName = typeName.substring(0, index);
            }

            if (qualifier.length() > 0) {
                typeName = qualifier + "." + typeName; //$NON-NLS-1$
            }
            return typeName;
        }
        return null;
    }

    /**
     * Returns the line number associated with the stack trace or -1 if none.
     */
    public int getLineNumber() {
        String linkText = getLinkText();
        int index = linkText.lastIndexOf(':');
        if (index >= 0) {
            String numText = linkText.substring(index + 1);
            index = numText.indexOf(')');
            if (index >= 0) {
                numText = numText.substring(0, index);
            }
            try {
                return Integer.parseInt(numText);
            } catch (NumberFormatException e) {
                BuildUIPlugin.log(e);
            }
        }
        return -1;
    }

}
