/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.junit;

import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.junit.launcher.JUnitLaunchShortcut;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.internal.java.ui.JdtWorkspaceHelper;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.IDeferredBuildTreeNode;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;

/**
 * Action delegate for launching a JUnit test from the JUnit tab of the build
 * result editor.
 */
public abstract class AbstractJUnitLaunchActionDelegate extends ActionDelegate implements IEditorActionDelegate {
    private String fTestClassName;

    /**
     * Run launch mode.
     */
    public static final String LAUNCH_MODE_RUN = "run"; //$NON-NLS-1$

    /**
     * Debug launch mode.
     */
    public static final String LAUNCH_MODE_DEBUG = "debug"; //$NON-NLS-1$

    /**
     * Return the launch mode.
     * 
     * @return Either <code>LAUNCH_MODE_RUN</code> or
     *         <code>LAUNCH_MODE_DEBUG</code>.
     */
    public abstract String getLaunchMode();

    /**
     * Return the <code>JUnitLaunchShortcut</code> to use. Typically, this is
     * <code>JUnitLaunchShortcut</code> for running a test as a regular JUnit
     * test. Or, <code>JUnitWorkbenchShortcut</code> for running a test as a
     * JUnit Plugin test.
     * 
     * @return A <code>JUnitLaunchShortcut</code> to use for launching the
     *         test.
     */
    public abstract JUnitLaunchShortcut getLaunchShortcut();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
        try {
            IType type = findJavaType(fTestClassName);
            if (type != null) {
                JUnitLaunchShortcut shortcut = getLaunchShortcut();
                shortcut.launch(new StructuredSelection(type), getLaunchMode());
            } else {
                showErrorDialog(Messages.AbstractJUnitLaunchActionDelegate_ERROR_TITLE, NLS.bind(
                        Messages.AbstractJUnitLaunchActionDelegate_TYPE_NOT_FOUND, fTestClassName), null);
            }
        } catch (JavaModelException exception) {
            BuildUIPlugin.log(exception);
            showErrorDialog(Messages.AbstractJUnitLaunchActionDelegate_ERROR_TITLE, NLS.bind(
                    Messages.AbstractJUnitLaunchActionDelegate_ERROR_OPENING_TYPE, fTestClassName), exception);
        }
    }

    protected IType findJavaType(String name) throws JavaModelException {
        return JdtWorkspaceHelper.findJavaType(name);
    }

    protected void showErrorDialog(final String title, final String message, final Exception exception) {
        BuildUIHelper.showErrorDialog(message, title, exception);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        fTestClassName = getTestClassNameFromSelection(selection);
        if (action != null) {
            if (fTestClassName != null && fTestClassName.length() > 0) {
                action.setEnabled(true);
            } else {
                action.setEnabled(false);
            }
        }
    }

	private String getTestClassNameFromSelection(ISelection selection) {
		String testClassName = null;

		if (selection instanceof IStructuredSelection) {
			IStructuredSelection ss = (IStructuredSelection) selection;

			if (ss.size() == 1) { // We only allow single selection
				Object element = ss.getFirstElement();

				if (element instanceof IDeferredBuildTreeNode) {
					IDeferredBuildTreeNode node = (IDeferredBuildTreeNode) element;
					testClassName = node.getNodeName();
				}
			}
		}

		return testClassName;
	}

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IEditorActionDelegate#setActiveEditor(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IEditorPart)
     */
    public void setActiveEditor(IAction action, IEditorPart targetEditor) {
        // Nothing to do.
    }

}
