/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.workitems;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.java.ui.editors.result.junit.IJUnitRunErrorDetailProvider;
import com.ibm.team.build.internal.ui.actions.BuildResultEditorActionDelegate;
import com.ibm.team.build.internal.ui.editors.result.workitem.WorkItemHelper;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Action that creates a work item from a failing JUnit in the build result
 * editor.
 */
public class CreateWorkItemFromJUnitTestAction extends BuildResultEditorActionDelegate {

    protected ISelection fSelection;

    public void run(IAction action) {
        final Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
        ITeamRepository teamRepository = (ITeamRepository) fBuildResult.getOrigin();

        final Job job = new TeamBuildJob(Messages.CreateWorkItemFromJUnitTestAction_CREATING_WORK_ITEM, true,
                teamRepository) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                SubMonitor subMonitor = SubMonitor.convert(monitor, 2);
                IJUnitRunErrorDetailProvider detailProvider = getJUnitDetailProvider(fSelection);
                final IProjectAreaHandle projectAreaHandle = getProjectAreaHandle(subMonitor.newChild(1));
                final String summary = getSummary(detailProvider);
                final String description = getDescription(detailProvider, subMonitor.newChild(1));

                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        handleCreateWorkItemAsync(shell, summary, description, projectAreaHandle);
                    }

                });
                return Status.OK_STATUS;
            }

            @Override
            protected void jobFinished(IStatus status) {
                handleJobDone(status);
            }
        };
        job.schedule();
    }

    protected void handleJobDone(IStatus status) {
        /*
         * Default implementation does nothing. Tests should override to wait on
         * job completion.
         */
    }

    protected IProjectAreaHandle getProjectAreaHandle(IProgressMonitor monitor) throws TeamRepositoryException {
        IBuildDefinitionHandle buildDefinitionHandle = fBuildResult.getBuildDefinition();
        ITeamRepository teamRepository = (ITeamRepository) buildDefinitionHandle.getOrigin();

        IBuildDefinition buildDefinition = (IBuildDefinition) teamRepository.itemManager().fetchCompleteItem(
                buildDefinitionHandle, IItemManager.DEFAULT, monitor);

        IProcessAreaHandle processAreaHandle = buildDefinition.getProcessArea();

        IProcessArea processArea = (IProcessArea) teamRepository.itemManager().fetchCompleteItem(processAreaHandle,
                IItemManager.DEFAULT, monitor);

        return processArea.getProjectArea();
    }

    protected void handleCreateWorkItemAsync(Shell shell, String summary, String description,
            IProjectAreaHandle projectAreaHandle) {

        if (shell != null && !shell.isDisposed()) {
            WorkItemHelper.getInstance().createAndLinkWorkItem(shell, summary, description, fBuildResult,
                    projectAreaHandle);
        }
    }

    public void selectionChanged(IAction action, ISelection selection) {
        fSelection = selection;
        updateEnablement(action, fSelection);
    }

    protected String getSummary(IJUnitRunErrorDetailProvider detailProvider) {
        if (detailProvider.getJUnitName().length() == 0) {
            return NLS.bind(Messages.CreateWorkItemFromJUnitTestAction_TESTSUITE_FAILURE, fBuildResult.getLabel(),
                    detailProvider.getJUnitClassName());
        } else {
            return NLS.bind(Messages.CreateWorkItemFromJUnitTestAction_TESTCASE_FAILURE, new String[] {
                    fBuildResult.getLabel(), detailProvider.getJUnitClassName(), detailProvider.getJUnitName() });
        }
    }

    protected String getDescription(IJUnitRunErrorDetailProvider detailProvider, IProgressMonitor monitor)
            throws TeamRepositoryException {
        String description = detailProvider.getDetailString();
        if (description == null) {
            ITeamRepository teamRepository = (ITeamRepository) fBuildResult.getOrigin();

            if (detailProvider.isError()) {
                description = ContentUtil.contentToString(teamRepository, detailProvider.getErrorData(), monitor);
            } else {
                description = ContentUtil.contentToString(teamRepository, detailProvider.getFailureData(), monitor);
            }
        }

        return description;
    }

    protected void updateEnablement(IAction action, ISelection selection) {
        IJUnitRunErrorDetailProvider detailProvider = getJUnitDetailProvider(selection);
        action.setEnabled(detailProvider != null && (detailProvider.isError() || detailProvider.isFailure()));
    }

	protected IJUnitRunErrorDetailProvider getJUnitDetailProvider(ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			IStructuredSelection ss = (IStructuredSelection) selection;
			if (ss.size() == 1) { // We only allow single selection.
				Object element = ((IStructuredSelection) selection).getFirstElement();
				if (element instanceof IJUnitRunErrorDetailProvider) {
					return (IJUnitRunErrorDetailProvider) element;
				}
			}
		}
		return null;
	}

}
