/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import org.eclipse.jface.resource.CompositeImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;

/**
 * An AstractBuildCompositeImageDescription consists of a main icon and several
 * adornments. The adornments are computed according to type and state of the
 * adornment object.
 */
public abstract class AbstractBuildCompositeImageDescriptor extends CompositeImageDescriptor {

    private Image fBaseImage;
    private Object fAdornmentObject;
    private Point fSize;

    /**
     * Create a new BuildCompositeImageDescription.
     * 
     * @param baseImage
     *            an image descriptor used as the base image
     * @param adornmentObject
     *            the object used to determine the image's adornment
     */
    public AbstractBuildCompositeImageDescriptor(Image baseImage, Object adornmentObject) {
        fBaseImage = baseImage;
        fAdornmentObject = adornmentObject;
    }

    /*
     * Allow access to the adornment object to child classes.
     */
    protected Object getAdornmentObject() {
        return fAdornmentObject;
    }

    /**
     * Get the location, relative to the base image, for the adornment object.
     * Default is left-bottom.
     * 
     * @return the location for the adornment object
     */
    protected Point getOverlayLocation(ImageData overlay) {
        return new Point(0, getSize().y - overlay.height);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.resource.CompositeImageDescriptor#getSize()
     */
    protected Point getSize() {
        if (fSize == null) {
            ImageData data = fBaseImage.getImageData();
            fSize = new Point(data.width, data.height);
        }
        return fSize;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object object) {
        if (!(object instanceof AbstractBuildCompositeImageDescriptor)) {
            return false;
        }

        AbstractBuildCompositeImageDescriptor other = (AbstractBuildCompositeImageDescriptor) object;
        return (fBaseImage.equals(other.fBaseImage) && fAdornmentObject.equals(other.fAdornmentObject));
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return fBaseImage.hashCode() | fAdornmentObject.hashCode();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.resource.CompositeImageDescriptor#drawCompositeImage(int,
     *      int)
     */
    protected void drawCompositeImage(int width, int height) {
        ImageData bg = fBaseImage.getImageData();
        if (bg == null) {
            bg = DEFAULT_IMAGE_DATA;
        }
        drawImage(bg, 0, 0);

        ImageData data = getOverlay();
        if (data != null) {
            Point location = getOverlayLocation(data);
            drawImage(data, location.x, location.y);
        }
    }

    /**
     * Get the overlay to be drawn on top of the base image.
     * 
     * @return the overlay, or <code>null</code> to indicate no overlay
     */
    protected abstract ImageData getOverlay();

    protected void setSize(Point size) {
        fSize = size;
    }
}