/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import java.util.Hashtable;

import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.internal.common.helper.ValidationHelper;

/**
 * Registry for storing decorated images dynamically created using
 * {@link AbstractBuildCompositeImageDescriptor}.
 */
public class BuildCompositeImageRegistry {

    private static Hashtable<Image, Image[]> fBuildStatusImageMap = new Hashtable<Image, Image[]>();
    private static Hashtable<Image, Image[]> fBuildActivityStateImageMap = new Hashtable<Image, Image[]>();
    private static Hashtable<Image, Image> fPersonalBuildImageMap = new Hashtable<Image, Image>();

    /**
     * Takes an image and a BuildStatus and returns the decorated images.
     * 
     * @param baseImage
     *            The base image to be decorated.
     * @param status
     *            The BuildStatus that determines the decoration to be added.
     * @return The decorated image.
     */
    public static Image getImage(Image baseImage, BuildStatus status) {
        return getImage(baseImage, status, true);
    }

    /**
     * Takes an image and a BuildStatus and returns the decorated images.
     * 
     * @param baseImage
     *            The base image to be decorated.
     * @param status
     *            The BuildStatus that determines the decoration to be added.
     * @param decorateSuccessfulImage
     *            Whether or not to decorate an image if status == OK
     * @return The decorated image.
     */
    public static Image getImage(Image baseImage, BuildStatus status, boolean decorateSuccessfulImage) {
        if (status == BuildStatus.OK && !decorateSuccessfulImage) {
            return baseImage;
        }

        Image decoratedImage = null;

        synchronized (fBuildStatusImageMap) {
            Image[] decoratedImages = fBuildStatusImageMap.get(baseImage);
            if (decoratedImages == null) {
                decoratedImages = new Image[BuildStatus.values().length];
                fBuildStatusImageMap.put(baseImage, decoratedImages);
            }
            if (decoratedImages[status.ordinal()] == null) {
                BuildStatusCompositeImageDescriptor descriptor = new BuildStatusCompositeImageDescriptor(baseImage,
                        status);
                decoratedImage = descriptor.createImage();
                decoratedImages[status.ordinal()] = decoratedImage;
            }

            decoratedImage = decoratedImages[status.ordinal()];
        }

        return decoratedImage;
    }

    /**
     * Takes a BuildState object and return a build activity image decorated
     * according to the state.
     * <p>
     * Valid values for <code>state</code> are:
     * 
     * <pre>
     *     IN_PROGRESS
     *     INCOMPLETE
     *     COMPLETED
     * </pre>
     * 
     * @param state
     *            The BuildState that determines the decoration to be added.
     * @return The decorated image.
     */
    public static Image getBuildActivityImage(BuildState state) {
        ValidationHelper.validateNotNull("state", state); //$NON-NLS-1$
        if (state != BuildState.IN_PROGRESS && state != BuildState.INCOMPLETE && state != BuildState.COMPLETED) {
            throw new IllegalArgumentException(NLS.bind(
                    BuildUIMessages.BuildCompositeImageRegistry_INVALID_STATE_FOR_BUILD_ACTIVITY, state.toString()));
        }

        Image image = BuildUIPlugin.getImage("icons/obj16/bld_act_obj.gif"); //$NON-NLS-1$

        synchronized (fBuildActivityStateImageMap) {
            Image[] decoratedImages = fBuildActivityStateImageMap.get(image);
            if (decoratedImages == null) {
                decoratedImages = new Image[BuildState.values().length];
                fBuildActivityStateImageMap.put(image, decoratedImages);
            }

            if (decoratedImages[state.ordinal()] == null) {
                BuildActivityCompositeImageDescriptor descriptor = new BuildActivityCompositeImageDescriptor(image,
                        state);
                image = descriptor.createImage();
                decoratedImages[state.ordinal()] = image;
            }

            image = decoratedImages[state.ordinal()];
        }

        return image;
    }

    /**
     * Takes a base image decorates it with the personal build overlay in the
     * upper left corner.
     * 
     * @param baseImage
     *            The base image.
     * @return The decorated image.
     */
    public static Image getPersonalBuildImage(Image baseImage) {
        Image decoratedImage = null;

        synchronized (fPersonalBuildImageMap) {
            decoratedImage = fPersonalBuildImageMap.get(baseImage);
            if (decoratedImage == null) {
                PersonalBuildCompositeImageDescriptor descriptor = new PersonalBuildCompositeImageDescriptor(baseImage);
                decoratedImage = descriptor.createImage();
                fPersonalBuildImageMap.put(baseImage, decoratedImage);
            }
        }

        return decoratedImage;
    }
}
