/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.build.internal.ui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.jface.viewers.ViewerLabel;
import org.eclipse.osgi.util.NLS;
import org.eclipse.osgi.util.TextProcessor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.internal.common.model.dto.IBuildDefinitionStatusRecord;
import com.ibm.team.build.internal.ui.helper.BuildDefinitionHandleHelper;
import com.ibm.team.jface.labelProviders.TypeLabelProvider;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;

/**
 * {@link ILabelProvider} for an {@link IBuildDefinition} or
 * {@link IBuildDefinitionStatusRecord}.
 */
public class BuildDefinitionLabelProvider extends TypeLabelProvider {

    private static final Object FETCH_LOCK = new Object();
    private static BuildJob fFetchJob;
    private static Map<UUID, IBuildDefinitionHandle> fCurrentlyFetching = new Hashtable<UUID, IBuildDefinitionHandle>();

    private Set<IBuildDefinition> fCachedBuildDefinitions = new HashSet<IBuildDefinition>();
    private Set<UUID> fDeletedBuildDefinitions = new HashSet<UUID>();

    protected enum FetchJobStatus {
        /**
         * Signifies that the job has been scheduled, but is not yet running
         */
        SCHEDULED,

        /**
         * Signifies the job is currently running
         */
        RUNNING,

        /**
         * The job is not running and is not scheduled to run
         */
        NOT_RUNNING
    }

    private static FetchJobStatus fFetchJobStatus = FetchJobStatus.NOT_RUNNING;

    private boolean fShowRepository;

    /**
     * Create a new BuildDefinitionLabelProvider.
     */
    public BuildDefinitionLabelProvider() {
        fShowRepository = false;
    }

    /**
     * Create a new BuildDefinitionLabelProvider.
     * 
     * @param showRepository
     *            whether or not to show the repository name in the label.
     */
    public BuildDefinitionLabelProvider(boolean showRepository) {
        fShowRepository = showRepository;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void updateLabel(ViewerLabel label, Object element) {
        Object[] labelObjects = getBuildDefinitionLabelObjects(element);
        String definitionId = (String) labelObjects[0];
        IBuildDefinitionHandle handle = (IBuildDefinitionHandle) labelObjects[1];

        if (handle != null) {
            label.setImage(getDefinitionImage());

            if (definitionId != null) {
                label.setText(getLabel(handle, definitionId));
            } else {
                if (fDeletedBuildDefinitions.contains(handle.getItemId())) {
                    label.setText(BuildUIMessages.BUILD_DEFINITION_LABEL_DELETED);
                } else {
                    fetchDefinition(handle, element);
                }
            }
        }
    }

    protected Object[] getBuildDefinitionLabelObjects(Object element) {
        String definitionId = null;
        IBuildDefinitionHandle handle = null;

        if (element instanceof IBuildDefinitionStatusRecord) {
            handle = ((IBuildDefinitionStatusRecord) element).getBuildDefinition();
            definitionId = ((IBuildDefinitionStatusRecord) element).getBuildDefinition().getId();
        } else if (element instanceof IBuildDefinition) {
            handle = (IBuildDefinition) element;
            definitionId = ((IBuildDefinition) element).getId();
        } else if (element instanceof BuildDefinitionHandleHelper) {
            BuildDefinitionHandleHelper helper = (BuildDefinitionHandleHelper) element;
            handle = helper.getBuildDefinitionHandle();
            definitionId = helper.getBuildDefinitionId();
        } else if (element instanceof IBuildDefinitionHandle) {
            handle = (IBuildDefinitionHandle) element;
            ITeamRepository repository = (ITeamRepository) getOrigin(handle);
            IBuildDefinition buildDefinition = (IBuildDefinition) repository.itemManager().getSharedItemIfKnown(handle);
            if (buildDefinition != null && buildDefinition.isPropertySet(IBuildDefinition.PROPERTY_ID)) {
                definitionId = buildDefinition.getId();
            }
        }

        return new Object[] { definitionId, handle };
    }

    protected Image getDefinitionImage() {
        return BuildUIPlugin.getImage("icons/obj16/build_type_obj.gif"); //$NON-NLS-1$
    }

    protected IBuildDefinition getBuildDefinition(IBuildDefinitionHandle handle) {
        ITeamRepository repository = getOrigin(handle);
        IBuildDefinition buildDefinition = (IBuildDefinition) repository.itemManager().getSharedItemIfKnown(handle);
        if (buildDefinition == null || !buildDefinition.isPropertySet(IBuildDefinition.PROPERTY_ID)) {
            return null;
        }

        fCachedBuildDefinitions.add(buildDefinition);
        return buildDefinition;
    }

    /**
     * Returns the label for the definition.
     */
    protected String getLabel(IBuildDefinitionHandle definitionHandle, String id) {
       return TextProcessor.process(getUnprocessedLabel(definitionHandle, id),"()"); //$NON-NLS-1$
    }
    
    /**
     * Returns the label for the definition  without the use of {@link TextProcessor}.
     */
    private String getUnprocessedLabel(IBuildDefinitionHandle definitionHandle, String id) {
        if (fShowRepository && getOrigin(definitionHandle) != null) {
            ITeamRepository repo = (ITeamRepository) getOrigin(definitionHandle);
            String repoId = repo.getName();
            if (repoId == null || repoId.length() == 0) {
                repoId = repo.getRepositoryURI();
            }
            return NLS.bind(BuildUIMessages.BUILD_DEFINITION_LABEL_WITH_REPOSITORY, id, repoId);
        } else {
            return id;
        }
    }

    protected void fetchDefinition(IBuildDefinitionHandle handle, Object element) {
        synchronized (FETCH_LOCK) {
            if (fCurrentlyFetching.containsKey(handle.getItemId())) {
                return;
            }

            fCurrentlyFetching.put(handle.getItemId(), handle);

            if (fFetchJob == null) {
                fFetchJob = createFetchJob();
            }
            if (fFetchJobStatus == FetchJobStatus.NOT_RUNNING) {
                scheduleFetchJob(fFetchJob);
                fFetchJobStatus = FetchJobStatus.SCHEDULED;
            }
        }
    }

    protected void scheduleFetchJob(BuildJob fetchJob) {
        fetchJob.scheduleJob();
    }

    protected BuildJob createFetchJob() {
        return new BuildJob(BuildUIMessages.BuildDefinitionLabelProvider_FETCH_BUILD_DEFINITION_JOB_TITLE) {
            @Override
            public IStatus run(IProgressMonitor monitor) {
                try {
                    fetchBuildItems(monitor);
                } finally {
                    fFetchJobStatus = FetchJobStatus.NOT_RUNNING;
                }
                return Status.OK_STATUS;
            }
        };
    }

    protected void fetchBuildItems(IProgressMonitor monitor) {
        final List<IBuildDefinitionHandle> definitionsToResolve = new ArrayList<IBuildDefinitionHandle>();
        List<IBuildDefinition> resolvedDefinitions = new ArrayList<IBuildDefinition>();

        ITeamRepository teamRepository = null;

        while (true) {
            /*
             * Get the lock so that no other threads can add new build
             * definitions or decide whether or not to reschedule the fetch job
             * while we are determining whether or not we are done for now.
             */
            synchronized (FETCH_LOCK) {
                if (fCurrentlyFetching.size() > 0) {
                    fFetchJobStatus = FetchJobStatus.RUNNING;
                } else {
                    fFetchJobStatus = FetchJobStatus.NOT_RUNNING;
                    break;
                }

                /*
                 * The build definitions may come from multiple repositories. We
                 * will fetch from the repository of the first build definition
                 * we see. Definitions from other repositories will be fetched
                 * during a subsequent loop-through.
                 */
                definitionsToResolve.clear();

                for (IBuildDefinitionHandle handle : fCurrentlyFetching.values()) {
                    if (teamRepository == null) {
                        definitionsToResolve.add(handle);
                        teamRepository = (ITeamRepository) getOrigin(handle);
                    } else {
                        if (teamRepository.getId().equals(((ITeamRepository) getOrigin(handle)).getId())) {
                            definitionsToResolve.add(handle);
                        }
                    }
                }
            }

            try {
                List<IBuildDefinitionHandle> definitionsToFetch = new ArrayList<IBuildDefinitionHandle>();

                for (IBuildDefinitionHandle handle : definitionsToResolve) {
                    IBuildDefinition definition = getBuildDefinition(handle);
                    if (definition == null) {
                        definitionsToFetch.add(handle);
                    } else {
                        resolvedDefinitions.add(definition);
                    }
                }

                if (definitionsToFetch.size() > 0) {
                    List<IBuildDefinition> fetchedDefinitions = teamRepository.itemManager().fetchPartialItems(
                            definitionsToFetch, IItemManager.DEFAULT,
                            Collections.singleton(IBuildDefinition.PROPERTY_ID), monitor);

                    for (int i = 0; i < fetchedDefinitions.size(); i++) {
                        IBuildDefinition fetchedDefinition = fetchedDefinitions.get(i);
                        if (fetchedDefinition != null) {
                            resolvedDefinitions.add(fetchedDefinition);
                        } else {
                            fDeletedBuildDefinitions.add(definitionsToFetch.get(i).getItemId());
                        }
                    }
                }

                fCachedBuildDefinitions.addAll(resolvedDefinitions);

            } catch (TeamRepositoryException exception) {
                BuildUIPlugin.log(new Status(IStatus.ERROR, BuildUIPlugin.getUniqueIdentifier(),
                        BuildUIMessages.BuildResultLabelProvider_FETCH_ERROR, exception));
            } finally {
                synchronized (FETCH_LOCK) {
                    for (IBuildDefinitionHandle handle : definitionsToResolve) {
                        fCurrentlyFetching.remove(handle.getItemId());
                    }
                }

                fireLabelProviderChangeEvent(definitionsToResolve);
            }
        }
    }

    protected void fireLabelProviderChangeEvent(Collection<IBuildDefinitionHandle> buildDefinitionHandles) {
        final List<IBuildDefinitionHandle> buildDefinitions = new ArrayList<IBuildDefinitionHandle>();
        buildDefinitions.addAll(buildDefinitionHandles);

        asyncExec(new Runnable() {
            public void run() {
                fireLabelProviderChanged(new LabelProviderChangedEvent(BuildDefinitionLabelProvider.this,
                        buildDefinitions.toArray(new IBuildDefinitionHandle[buildDefinitions.size()])));
            }
        });
    }

    protected void asyncExec(Runnable runnable) {
        Display.getDefault().asyncExec(runnable);
    }

    protected ITeamRepository getOrigin(IBuildDefinitionHandle buildItemHandle) {
        return (ITeamRepository) buildItemHandle.getOrigin();
    }
}
