/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import com.ibm.team.build.internal.ui.jobs.TeamBuildJobFamily;

/**
 * Utility job class used in the UI code to facilitate testing.
 * <p>
 * By making the run method public, we can call it directly in our test code
 * instead of relying on <code>job.schedule()</code>.
 */
public abstract class BuildJob extends Job {

    /**
     * Set to true before running tests. Causes the scheduleJob method to run
     * the job synchronously instead of adding it to the scheduled queue.
     */
    public static boolean RUN_JOBS_SYNCHRONOUSLY_FOR_TESTING = false;

    private IProgressMonitor fProgressMonitor = null;

    /**
     * Create a new job.
     * 
     * @param name
     *            The job name.
     */
    public BuildJob(String name) {
        super(name);
    }

    /**
     * Schedule this BuildJob to be run. If the TESTING flag is set to true, the
     * job will be run synchronously instead of being scheduled.
     * 
     * @return The status returned from the run method if {@link #RUN_JOBS_SYNCHRONOUSLY_FOR_TESTING} is
     *         true, OK status otherwise
     */
    public IStatus scheduleJob() {
        if (RUN_JOBS_SYNCHRONOUSLY_FOR_TESTING) {
            IProgressMonitor monitor = fProgressMonitor == null ? new NullProgressMonitor() : fProgressMonitor;
            return run(monitor);
        } else {
            schedule();
        }

        return Status.OK_STATUS;
    }

    /**
     * Testing method. Allows tester to set progress monitor to be used if the
     * job is run synchronously in the {@link #scheduleJob} method.
     * 
     * @param monitor
     *            The progress monitor to use when running the job synchronously
     *            for testing.
     */
    public void setTestProgressMonitor(IProgressMonitor monitor) {
        fProgressMonitor = monitor;
    }
    
    public boolean belongsTo(Object family) {
        return (family instanceof TeamBuildJobFamily);
    }
}
