/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchCommandConstants;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.internal.common.helper.BuildEngineHelper;
import com.ibm.team.build.internal.ui.dialogs.BuildPropertyDialog;
import com.ibm.team.build.internal.ui.dialogs.BuildPropertyKindSelectionDialog;
import com.ibm.team.build.internal.ui.editors.builddefinition.PropertyTableComparator;
import com.ibm.team.build.internal.ui.editors.builddefinition.PropertyTableContentProvider;
import com.ibm.team.build.internal.ui.editors.builddefinition.PropertyTableLabelProvider;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;
import com.ibm.team.build.internal.ui.helper.ClipboardHelper;
import com.ibm.team.build.internal.ui.properties.BuildPropertyEditorExtension;
import com.ibm.team.build.internal.ui.properties.BuildPropertyEditorExtensionManager;
import com.ibm.team.build.internal.ui.properties.PulldownPropertyEditor;
import com.ibm.team.build.ui.properties.AbstractBuildPropertyEditor;
import com.ibm.team.build.ui.properties.IBuildPropertyEditorContext;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * A UI control for displaying and, optionally, adding, editing, and removing
 * <code>BuildProperty</code> objects from a supplied
 * <code>java.util.List</code>.
 */
public class BuildPropertyControl {

    protected ITeamRepository fTeamRepository;
    protected Shell fParentShell;
    protected List fBuildProperties;
    protected Button fAddPropertyButton;
    protected Button fEditPropertyButton;
    protected Button fRemovePropertyButton;
    protected TableViewer fTableViewer;
    protected Label fDescriptionLabel;
    private PropertyTableComparator fPropertyTableSorter;
    private ListenerList fListenerList = new ListenerList();
    private Composite fDescriptionComposite;

    private static final String COLUMN_NAME = "name"; //$NON-NLS-1$
    private static final String COLUMN_VALUE = "value"; //$NON-NLS-1$
    private static final String COLUMN_DESCRIPTION = "description"; //$NON-NLS-1$

    private static final String[] FULL_LAYOUT_COLUMN_PROPERTIES = new String[] { COLUMN_NAME, COLUMN_VALUE,
            COLUMN_DESCRIPTION };
    private static final String[] COMPACT_LAYOUT_COLUMN_PROPERTIES = new String[] { COLUMN_NAME, COLUMN_VALUE };

    private static final String[] FULL_LAYOUT_COLUMN_HEADERS = new String[] {
            BuildUIMessages.BuildPropertyControl_NAME, BuildUIMessages.BuildPropertyControl_VALUE,
            BuildUIMessages.BuildPropertyControl_DESCRIPTION };
    private static final String[] COMPACT_LAYOUT_COLUMN_HEADERS = new String[] {
            BuildUIMessages.BuildPropertyControl_NAME, BuildUIMessages.BuildPropertyControl_VALUE };

    private static final int[] COMPACT_LAYOUT_COLUMN_WEIGHTS = new int[] { 50, 50 };
    private static final int[] FULL_LAYOUT_COLUMN_WEIGHTS = new int[] { 30, 40, 30 };

    private FormToolkit fToolkit;
    private Composite fComposite;
    
    private BuildPropertyCopyAction fCopyAction;
    private BuildPropertyPasteAction fPasteAction;
    
    private boolean fEditable;

    /**
     * Enum used to indicate whether the control should minimize its layout.
     */
    public enum LayoutSize {
        /**
         * Use the compact version of the control.
         */
        COMPACT,

        /**
         * Use the full-sized version of the control.
         */
        FULL
    }

    /**
     * Create a build property control.
     * 
     * @param parent
     *            The parent composite.
     * @param toolkit
     *            May be null. If not null, this toolkit will be used for
     *            creating widgets in the build property control. If null, the
     *            standard SWT constructors will be used.
     * @param buildProperties
     *            The list of properties
     * @param teamRepository
     *            The repository context in which to edit the properties.
     * @param isEditable
     *            Flag that determines whether or not this control lets the user
     *            change the properties.
     * @param layoutSize
     *            Whether to use the compact or full version of the form.
     */
    public BuildPropertyControl(Composite parent, FormToolkit toolkit, List buildProperties,
            ITeamRepository teamRepository, boolean isEditable, LayoutSize layoutSize) {
        fBuildProperties = buildProperties;
        fParentShell = parent.getShell();
        fToolkit = toolkit;
        fTeamRepository = teamRepository;
        fEditable = isEditable;

        fComposite = createComposite(parent);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(fComposite);
        GridLayoutFactory.fillDefaults().numColumns(2).applyTo(fComposite);

        boolean isCompact = layoutSize == LayoutSize.COMPACT;

        Composite tableComposite = createComposite(fComposite);
        GridDataFactory.fillDefaults().grab(true, true).span(isEditable ? 1 : 2, 1).applyTo(tableComposite);

        Table table = createTable(tableComposite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.FULL_SELECTION);
        TableColumnLayout layout = new TableColumnLayout();

        String[] columnProperties = isCompact ? COMPACT_LAYOUT_COLUMN_PROPERTIES : FULL_LAYOUT_COLUMN_PROPERTIES;
        String[] columnHeaders = isCompact ? COMPACT_LAYOUT_COLUMN_HEADERS : FULL_LAYOUT_COLUMN_HEADERS;
        int[] columnWeights = isCompact ? COMPACT_LAYOUT_COLUMN_WEIGHTS : FULL_LAYOUT_COLUMN_WEIGHTS;

        for (int i = 0; i < columnHeaders.length; i++) {
            TableColumn column = new TableColumn(table, SWT.NONE);
            column.setText(columnHeaders[i]);
            setSelectionListenerOnColumn(column, columnProperties[i]);
            layout.setColumnData(column, new ColumnWeightData(columnWeights[i]));
        }

        tableComposite.setLayout(layout);
        table.setHeaderVisible(true);
        table.setLinesVisible(true);
        table.setSortColumn(table.getColumn(0));
        table.setSortDirection(SWT.UP);
        fTableViewer = getTableViewer(table);
        fTableViewer.setColumnProperties(columnProperties);
        PropertyTableLabelProvider labelProvider = getPropertyTableLabelProvider();
        fPropertyTableSorter = new PropertyTableComparator(labelProvider);
        fTableViewer.setComparator(fPropertyTableSorter);

        fTableViewer.setContentProvider(new PropertyTableContentProvider());
        fTableViewer.setLabelProvider(labelProvider);
        fTableViewer.setInput(fBuildProperties);
        fTableViewer.setFilters(new ViewerFilter[] { getViewerFilter() });

        fTableViewer.addSelectionChangedListener(getSelectionChangedListener());

        if (isEditable) {
            fTableViewer.addOpenListener(new IOpenListener() {
                public void open(OpenEvent event) {
                    handleOpen(event);
                }
            });

            Composite buttonBar = createComposite(fComposite);
            buttonBar.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, true));
            buttonBar.setLayout(new GridLayout(1, false));

            fAddPropertyButton = createButton(buttonBar, BuildUIMessages.ADD_BUTTON_LABEL_WITH_ELLIPSIS, SWT.PUSH);
            fAddPropertyButton.addSelectionListener(getAddPropertyListener());

            fEditPropertyButton = createButton(buttonBar, BuildUIMessages.EDIT_BUTTON_LABEL_WITH_ELLIPSIS, SWT.PUSH);
            fEditPropertyButton.addSelectionListener(getEditPropertyListener());
            
            fRemovePropertyButton = createButton(buttonBar, BuildUIMessages.REMOVE_BUTTON_LABEL, SWT.PUSH);
            fRemovePropertyButton.addSelectionListener(getRemovePropertyListener());

            updatePropertyButtonEnablement();
            setButtonLayoutData();
        }

        /*
         * In compact form the description column is not displayed. Instead, we
         * show the description for the selected property below the table.
         */
        if (isCompact) {
            fDescriptionComposite = createComposite(fComposite);
            GridDataFactory.fillDefaults().grab(true, false).span(isEditable ? 2 : 1, 1).applyTo(fDescriptionComposite);
            GridLayoutFactory.fillDefaults().applyTo(fDescriptionComposite);

            Label descriptionTitle = new Label(fDescriptionComposite, SWT.NONE);
            descriptionTitle.setText(BuildUIMessages.BuildPropertyControl_DESCRIPTION_LABEL);
            descriptionTitle.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));

            fDescriptionLabel = new Label(fDescriptionComposite, SWT.WRAP);
            fDescriptionLabel.setText(BuildUIMessages.BuildPropertyControl_NO_PROPERTY_SELECTED_DESCRIPTION);
            GridDataFactory.fillDefaults().indent(10, 0).grab(true, false).applyTo(fDescriptionLabel);
        }
        
        initGlobalActions();
    }
    
    /**
     * Create and register the default context menu on the table for this
     * control. The default context menu contains the build edit actions group,
     * the global copy/paste actions, and the additions group.
     * 
     * @param partSite
     *            The workbench part site. If not <code>null</code>, then the
     *            context menu is created and registered. If <code>null</code>,
     *            the context menu is created but not registered.
     */
    public void createAndRegisterDefaultContextMenu(IWorkbenchPartSite partSite) {
        MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(getMenuListener());
        
        Menu menu = menuMgr.createContextMenu(getViewer().getControl());
        getViewer().getControl().setMenu(menu);

        if (partSite != null) {
            partSite.registerContextMenu(menuMgr, getViewer());
        }
    }
    
    private IMenuListener getMenuListener() {
        return new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                manager.add(new Separator(IBuildUIConstants.ID_BUILD_EDIT_ACTIONS_GROUP));
                //Get the registered global copy action and add it to the context menu if it exists
                if (fCopyAction != null) {
                    manager.add(fCopyAction);
                    fCopyAction.setEnabled(!fTableViewer.getSelection().isEmpty());
                }
                
                //Get the registered global paste action and add it to the context menu if it exists
                if (fPasteAction != null) {
                    manager.add(fPasteAction);
                }
                
                manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
            }
        };
    }
    
    /**
     * Replace the action bar contributors for the copy, paste and cut actions
     * on focus and revert back to the default eclipse contributors after focus
     * is lost. This is a workaround around to avoid implementing a global
     * action bar contributor for the build component editors.
     * 
     * @param editorSite
     *            The editor site that will have its global actions replaced.
     *            Cannot be <code>null</code>.
     * 
     */
    public void replaceGlobalActionsOnFocus(final IEditorSite editorSite) {
        /**
         * Workaround to avoid implementing a global action bar contributor.
         */
        fCopyAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_COPY);
        
        if (fPasteAction != null) {
            fPasteAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_PASTE);
        }
        fTableViewer.getTable().addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {
                editorSite.getActionBars().setGlobalActionHandler(ActionFactory.COPY.getId(), null);
                if (isEditable()) {
                    editorSite.getActionBars().setGlobalActionHandler(ActionFactory.PASTE.getId(), null);
                }
                editorSite.getActionBars().updateActionBars();
            }

            @Override
            public void focusGained(FocusEvent e) {
                editorSite.getActionBars().setGlobalActionHandler(ActionFactory.COPY.getId(), fCopyAction);
                if (isEditable()) {
                    editorSite.getActionBars().setGlobalActionHandler(ActionFactory.PASTE.getId(), fPasteAction);
                }
                editorSite.getActionBars().updateActionBars();
            }
        });

    }
    
    protected SelectionListener getEditPropertyListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                IBuildProperty buildProperty = getSelectedProperty();
                if (buildProperty != null) {
                    editProperty(buildProperty);
                }
            }
        };
    }
    
    protected PropertyTableLabelProvider getPropertyTableLabelProvider() {
        return new PropertyTableLabelProvider(fTableViewer, fTeamRepository);
    }

    private void initGlobalActions() {
        fCopyAction = new BuildPropertyCopyAction(fTableViewer, fTableViewer.getTable().getShell());
        
        if (isEditable()) {
        	fPasteAction = new BuildPropertyPasteAction(fTableViewer.getTable().getShell());
        }
    }
    
    
    private boolean isEditable() {
        return fEditable;
    }
    
    /**
     * Perform the copy operation on the selected build properties
     */
    public void performCopy() {
        fCopyAction.run();
    }
    
    protected Action getCopyAction() {
        return fCopyAction;
    }
    
    protected Action getPasteAction() {
        return fPasteAction;
    }
    
    private class BuildPropertyCopyAction extends Action {
        protected final Shell fShell;
        
        public BuildPropertyCopyAction(TableViewer viewer, Shell shell) {
            super(BuildUIMessages.BuildPropertyControl_COPY_ACTION_TEXT);
            
            ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
            setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
            setDisabledImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_COPY_DISABLED));
            
            fShell= shell;
        }
        
        public void run() {
            ClipboardHelper clipboardHelper = ClipboardHelper.getDefault();

            ArrayList<IBuildProperty> selectedProperties = getSelectedProperties();
            if (selectedProperties.size() > 0) {
                clipboardHelper.copyBuildPropertiesToClipboard(selectedProperties, fShell.getDisplay());
            }
        }
    }
    
    protected List<IBuildProperty> getBuildPropertiesFromClipboard(Shell shell) {
        ClipboardHelper clipboardHelper = ClipboardHelper.getDefault();
        return clipboardHelper.getBuildPropertiesFromClipboard(shell.getDisplay());
    }
    
    private class BuildPropertyPasteAction extends Action {
        private final Shell fShell;
        
        public BuildPropertyPasteAction(Shell shell) {
            super(BuildUIMessages.BuildPropertyControl_PASTE_ACTION_TEXT);
            
            ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
            setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));
            setDisabledImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE_DISABLED));
            
            fShell= shell;
        }
        
        public void run() {
            List<IBuildProperty> buildPropertiesToAdd = getBuildPropertiesFromClipboard(fShell);
            if (buildPropertiesToAdd != null) {
                final HashSet<String> ignoredBuildProperties = new HashSet<String>();
                final HashSet<String> overwrittenBuildProperties = new HashSet<String>();
                
                for (IBuildProperty propertyToAdd: buildPropertiesToAdd) {
                    AddPropertyResult result = addProperty(propertyToAdd);
                    switch (result) {
                        case OVERWRITTEN:
                            overwrittenBuildProperties.add(propertyToAdd.getName());
                            break;
                        case IGNORED:
                            ignoredBuildProperties.add(propertyToAdd.getName());
                            break;
                    }
                }
                
                final IPreferenceStore store = BuildUIPlugin.getDefault().getPreferenceStore();
                
                if (ignoredBuildProperties.size() > 0) {
                    final boolean prompt = store.getBoolean(BuildUIPlugin.PREF_SHOW_IGNORED_PROPERTY_DIALOG);
                    if (prompt) {
                        showDialogWithToggle(fParentShell.getShell(),
                                IStatus.WARNING, BuildUIMessages.BuildPropertyControl_IGNORED_PASTED_PROPERTY_TITLE,
                                BuildUIMessages.BuildPropertyControl_IGNORED_PASTED_PROPERTY_MESSAGE,
                                ignoredBuildProperties.toArray(new String[ignoredBuildProperties.size()]), BuildUIMessages.BuildPropertyControl_TOGGLE_IGNORE_DIALOG_MESSAGE,
                                !prompt, store, BuildUIPlugin.PREF_SHOW_IGNORED_PROPERTY_DIALOG);
                    }
                }
                
                if (overwrittenBuildProperties.size() > 0) {
                    final boolean prompt = store.getBoolean(BuildUIPlugin.PREF_SHOW_OVERWRITTEN_PROPERTY_DIALOG);
                    if (prompt) {
                        showDialogWithToggle(fParentShell.getShell(), IStatus.INFO,
                                BuildUIMessages.BuildPropertyControl_OVERWRITTEN_PROPERTY_TITLE, BuildUIMessages.BuildPropertyControl_OVERWRITTEN_PROPERTY_MESSAGE,
                                overwrittenBuildProperties.toArray(new String[overwrittenBuildProperties.size()]),
                                BuildUIMessages.BuildPropertyControl_TOGGLE_IGNORE_DIALOG_MESSAGE, !prompt, store, BuildUIPlugin.PREF_SHOW_OVERWRITTEN_PROPERTY_DIALOG);
                    }
                }
            }
        }
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected void showDialogWithToggle(final Shell parent, int status, final String title, final String message,
            final String[] details, final String toggleMessage, final boolean toggleState, final IPreferenceStore store, final String key) {
        if (status == IStatus.INFO) {
            boolean toggleResult = BuildUIHelper.showInfoDetailsDialogWithToggle(
                    parent,
                    title, message,
                    details,
                    toggleMessage, toggleState);
            store.setValue(key, !toggleResult);
        } else if (status == IStatus.WARNING) {
            boolean toggleResult = BuildUIHelper.showWarningDialogWithToggle(
                    parent,
                    title, message,
                    details,
                    toggleMessage, toggleState);
            store.setValue(key, !toggleResult);
        }

    }
    
    protected SelectionListener getRemovePropertyListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                removeSelectedProperties();
            }
        };
    }

    /**
     * A viewer filter that only returns elements if
     * <code>isGenericEditAllowed=true</code>.
     */
    protected ViewerFilter getViewerFilter() {
        return new ViewerFilter() {
            @Override
            public boolean select(Viewer viewer, Object parentElement, Object element) {
                IBuildProperty property = (IBuildProperty) element;
                return property.isGenericEditAllowed();
            }
        };
    }

    protected ISelectionChangedListener getSelectionChangedListener() {
        return new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                updatePropertyButtonEnablement();
                updateDescriptionLabel();
            }
        };
    }

    protected SelectionListener getAddPropertyListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                addProperty();
            }
        };
    }

    /**
     * Gets the list of properties being edited.
     * 
     * @return The list of properties being edited. Elements in the list are
     *         type <code>IBuildProperty</code>.
     */
    public List getBuildProperties() {
        return fBuildProperties;
    }

    /**
     * Set the build properties to edit in the control.
     * 
     * @param buildProperties
     *            A list of type <code>IBuildProperty</code> that will be
     *            edited in the control. The list will be modified to reflect
     *            the changes made in the control.
     */
    public void setBuildProperties(List buildProperties) {
        fBuildProperties = buildProperties;
        fTableViewer.setInput(fBuildProperties);
        updatePropertyButtonEnablement();
    }

    /**
     * Return the root composite of the property control. The client may call
     * setLayoutData() on the returned composite. This composite is a child of
     * the parent passed into the constructor.
     * 
     * @return the root composite of the property control
     */
    public Composite getComposite() {
        return fComposite;
    }

    private void addProperty() {
        String propertyKindId = getPropertyKindId(fParentShell);

        if (propertyKindId != null) {
            IBuildProperty buildProperty = BuildItemFactory.createBuildProperty();
            buildProperty.setKind(propertyKindId);

            BuildPropertyDialog dialog = getBuildPropertyDialog(buildProperty);

            if (dialog.open() == Window.OK) {
                fBuildProperties.add(buildProperty);
                fTableViewer.refresh();
                updatePropertyButtonEnablement();
                notifyPropertyAdded(buildProperty);
                StructuredSelection selection = new StructuredSelection(buildProperty);
                fTableViewer.setSelection(selection, true);
            }
        }
    }
    
    private enum AddPropertyResult {
        ADDED,
        OVERWRITTEN,
        IGNORED
    }
    
    private AddPropertyResult addProperty(final IBuildProperty buildProperty) {
        IBuildProperty existingBuildProperty = null;
        AddPropertyResult result;
        
        for (Object object : fBuildProperties) {
            IBuildProperty property = (IBuildProperty) object;
            if (property.getName().equals(buildProperty.getName())) {
                existingBuildProperty = property;
            }
        }
        
        AbstractBuildPropertyEditor buildPropertyEditor = getPropertyEditor(buildProperty);
        if (buildPropertyEditor != null && !getPropertyEditor(buildProperty).isValueSane(buildProperty.getValue())) {
            result = AddPropertyResult.IGNORED;
            return result;
        }

        if (existingBuildProperty != null) {
            if (existingBuildProperty.isGenericEditAllowed() && !existingBuildProperty.getKind().equals(IBuildProperty.PROPERTY_KIND_READONLY_STRING) && existingBuildProperty.getKind().equals(buildProperty.getKind())) {
                result = AddPropertyResult.OVERWRITTEN;
                
                existingBuildProperty.setValue(buildProperty.getValue());
                existingBuildProperty.setDescription(buildProperty.getDescription());
                
                notifyPropertyEdited(existingBuildProperty);
            } else {
                result = AddPropertyResult.IGNORED;

                return result;
            }
        } else {
            fBuildProperties.add(buildProperty);
            notifyPropertyAdded(buildProperty);
            result = AddPropertyResult.ADDED;
        }

        fTableViewer.refresh();
        updatePropertyButtonEnablement();
        
        return result;
    }
    
    private AbstractBuildPropertyEditor getPropertyEditor(final IBuildProperty buildProperty) {
        // Default to string editing.
        AbstractBuildPropertyEditor editor = null;

        String editorId = buildProperty.getKind();
        if (BuildEngineHelper.isBuildForgePulldownProperty(buildProperty)) {
            editorId = PulldownPropertyEditor.ID;
        }
        try {
            BuildPropertyEditorExtension extension = BuildPropertyEditorExtensionManager.getInstance().getBuildPropertyEditor(
                    editorId);
            if (extension != null) {
                editor = extension.createPropertyEditor();
            }
        } catch (CoreException exception) {
            BuildUIPlugin.log(exception);
        }
        return editor;
    }


    
    protected BuildPropertyDialog getBuildPropertyDialog(IBuildProperty buildProperty) {
        return new BuildPropertyDialog(fParentShell, BuildUIMessages.BuildPropertyControl_ADD_TITLE, buildProperty,
                fBuildProperties, true, fTeamRepository);
    }

    private void editProperty(final IBuildProperty buildProperty) {
        BuildPropertyDialog dialog = new BuildPropertyDialog(fParentShell,
                BuildUIMessages.BuildPropertyControl_EDIT_TITLE, buildProperty, fBuildProperties, false,
                fTeamRepository);
        
        AbstractBuildPropertyEditor propertyEditor = buildProperty.isWellKnown() ? dialog.getPropertyEditor(buildProperty) : null;
        if (propertyEditor != null) {
            // go straight through to property value editor if property is well known and it has a custom editor 
            IBuildPropertyEditorContext context = new IBuildPropertyEditorContext() {
                public IBuildProperty getProperty() {
                    return buildProperty;
                }

                public Shell getShell() {
                    return fParentShell;
                }

                public ITeamRepository getTeamRepository() {
                    return fTeamRepository;
                }
            };

            if (propertyEditor.editProperty(context)) {
                fTableViewer.refresh();
                notifyPropertyEdited(buildProperty);
            }
        } else {
            if (dialog.open() == Window.OK) {
                fTableViewer.refresh();
                notifyPropertyEdited(buildProperty);
            }
        }
    }

    /**
     * Get the property kind to use for a new property. If more than one kind is
     * available, prompt the user to choose.
     * 
     * @param shell
     *            The parent shell.
     * @return A property kind id, or <code>null</code> if the user cancels
     *         the selection.
     */
    protected String getPropertyKindId(Shell shell) {
        String id = null;

        BuildPropertyEditorExtension[] extensions = BuildPropertyEditorExtensionManager.getInstance().getBuildPropertyEditors();
        
        // remove the Pulldown extension since we don't want users to select that one when adding a new property
        for (int i = 0; i < extensions.length; i++) {
            if (PulldownPropertyEditor.ID.equals(extensions[i].getIdAttribute())) {
                BuildPropertyEditorExtension[] newExtensions = new BuildPropertyEditorExtension[extensions.length - 1];
                System.arraycopy(extensions, 0, newExtensions, 0, i);
                System.arraycopy(extensions, i + 1, newExtensions, i, extensions.length - i - 1);
                extensions = newExtensions;
                break;
            }
        }

        // remove any properties that have schedule override specified from the type menu
        Map<String, BuildPropertyEditorExtension> extensionMap = new HashMap<String, BuildPropertyEditorExtension>();
        for(BuildPropertyEditorExtension extension : extensions){
            extensionMap.put(extension.getIdAttribute(), extension);
        }
        List<IConfigurationElement> scheduleElements = calculateSchedulePropertyList();
        for(IConfigurationElement sElement : scheduleElements) {
            String targetKey = sElement.getAttribute("name");   //$NON-NLS-1$
            if(extensionMap.containsKey(targetKey))
                extensionMap.remove(targetKey);
        }
        extensions = extensionMap.values().toArray(new BuildPropertyEditorExtension[0]);

        
        // continue...
        if (extensions.length > 0) {
            String[] kinds = new String[extensions.length + 1];

            // Add String at the beginning of the list.
            kinds[0] = BuildUIMessages.BuildPropertyControl_STRING_PROPERTY_KIND_LABEL;
            for (int i = 0; i < extensions.length; i++) {
                kinds[i + 1] = extensions[i].getNameAttribute();
            }

            BuildPropertyKindSelectionDialog dialog = getPropertyKindSelectionDialog(shell, kinds);
            if (dialog.open() == Dialog.OK) {
                int selectedKind = dialog.getSelectedKind();
                if (selectedKind == 0) {
                    id = IBuildProperty.PROPERTY_KIND_STRING;
                } else if (selectedKind > 0) {
                    id = extensions[selectedKind - 1].getIdAttribute();
                }
            }
        }
        return id;
    }
    
    private List<IConfigurationElement> calculateSchedulePropertyList() {        
        ArrayList<IConfigurationElement> scheduleProperties = new ArrayList<IConfigurationElement>();
        
        IConfigurationElement[] configurationElements = Platform.getExtensionRegistry().getConfigurationElementsFor("com.ibm.team.build.common.buildConfigurationElements"); //$NON-NLS-1$
        
        for(IConfigurationElement element : configurationElements) {
            IConfigurationElement[] childElements = element.getChildren("genericProperty"); //$NON-NLS-1$
            for(IConfigurationElement childElement : childElements){
                String scheduleOverrideAttribute = childElement.getAttribute("scheduleOverride");  //$NON-NLS-1$
                if(scheduleOverrideAttribute!=null && scheduleOverrideAttribute.equalsIgnoreCase("true")) //$NON-NLS-1$
                    scheduleProperties.add(childElement);
            }
        }
        
        return scheduleProperties;
    }

    protected BuildPropertyKindSelectionDialog getPropertyKindSelectionDialog(Shell shell, String[] kinds) {
        return new BuildPropertyKindSelectionDialog(shell, BuildUIMessages.BuildPropertyControl_ADD_TITLE, kinds);
    }

    /**
     * Remove the selected build properties.
     */
    private void removeSelectedProperties() {
        ISelection selection = fTableViewer.getSelection();
        if (selection instanceof IStructuredSelection) {
            IStructuredSelection structuredSelection = (IStructuredSelection) selection;
            List propertiesRequired = new ArrayList();
            List propertiesRemoved = new ArrayList();
            List propertiesReadOnly = new ArrayList();

            for (Iterator iter = structuredSelection.iterator(); iter.hasNext();) {
                IBuildProperty buildProperty = (IBuildProperty) iter.next();
                if (buildProperty.isRequired() || buildProperty.isWellKnown()) {
                    propertiesRequired.add(buildProperty);
                } else if (buildProperty.getKind().equals(IBuildProperty.PROPERTY_KIND_READONLY_STRING)) {
                    propertiesReadOnly.add(buildProperty);
                } else {
                    fBuildProperties.remove(buildProperty);
                    propertiesRemoved.add(buildProperty);
                }
            }

            if (!propertiesRemoved.isEmpty()) {
                fTableViewer.refresh(false);
                updatePropertyButtonEnablement();
                notifyPropertiesRemoved((IBuildProperty[]) propertiesRemoved.toArray(new IBuildProperty[propertiesRemoved.size()]));
            }

            if (!propertiesRequired.isEmpty()) {
                showRequiredPropertiesError();
            }

            if (!propertiesReadOnly.isEmpty()) {
                showReadOnlyPropertiesError();
            }
        }
    }

    private ArrayList<IBuildProperty> getSelectedProperties() {
        ArrayList<IBuildProperty> buildProperties = new ArrayList<IBuildProperty>();
        ISelection selection  = fTableViewer.getSelection();
        if (selection instanceof IStructuredSelection) {
            IStructuredSelection structuredSelection = (IStructuredSelection) selection;
            for (Iterator iter = structuredSelection.iterator(); iter.hasNext();) {
                IBuildProperty buildProperty = (IBuildProperty) iter.next();
                buildProperties.add(buildProperty);
            }
        }
        return buildProperties;
   }
    
    protected void showReadOnlyPropertiesError() {
        MessageDialog.openError(fParentShell, BuildUIMessages.BuildPropertyControl_READ_ONLY_TITLE,
                BuildUIMessages.BuildPropertyControl_READ_ONLY_MESSAGE);
    }

    protected void showRequiredPropertiesError() {
        MessageDialog.openError(fParentShell, BuildUIMessages.BuildPropertyControl_REQUIRED_PROPERTIES_TITLE,
                BuildUIMessages.BuildPropertyControl_REQUIRED_PROPERTIES_MESSAGE);
    }

    protected IStructuredSelection getCurrentPropertySelection() {
        return (IStructuredSelection) fTableViewer.getSelection();
    }

    private IBuildProperty getSelectedProperty() {
        IStructuredSelection selection = getCurrentPropertySelection();
        Object element = ((IStructuredSelection) selection).getFirstElement();
        if (element instanceof IBuildProperty) {
            return (IBuildProperty) element;
        }
        return null;
    }

    private void updateDescriptionLabel() {
        if (fDescriptionLabel != null) {
            IStructuredSelection selection = getCurrentPropertySelection();
            if (selection.size() > 1) {
                fDescriptionLabel.setText(BuildUIMessages.BuildPropertyControl_MULTIPLE_PROPERTIES_SELECTED_DESCRIPTION);
            } else if (selection.size() == 1) {
                IBuildProperty buildProperty = (IBuildProperty) selection.getFirstElement();
                String description = buildProperty.getDescription();
                fDescriptionLabel.setText(description.length() == 0 ? BuildUIMessages.BuildPropertyControl_DESCRIPTION_LABEL_NO_DESCRIPTION
                        : description);
            }
            fDescriptionComposite.getParent().layout(true);
        }
    }

    private void updatePropertyButtonEnablement() {
        if (fEditPropertyButton == null) {
            return;
        }

        IStructuredSelection selection = getCurrentPropertySelection();
        fEditPropertyButton.setEnabled(isEditEnabled(selection));
        fRemovePropertyButton.setEnabled(isRemoveEnabled(selection));
    }

    /**
     * Returns whether the Edit button should be enabled, based on the given selection.
     */
    protected boolean isEditEnabled(IStructuredSelection selection) {
        boolean enableEdit = false;
        if (selection.size() == 1) {
            IBuildProperty property = (IBuildProperty) selection.getFirstElement();
            if (!property.getKind().equals(IBuildProperty.PROPERTY_KIND_READONLY_STRING)) {
                enableEdit = true;
            }
        }
        return enableEdit;
    }

    /**
     * Returns whether the Remove button should be enabled, based on the given selection.
     */
    protected boolean isRemoveEnabled(IStructuredSelection selection) {
        boolean removeEnabled = !selection.isEmpty();
        for (Object selectedObject : selection.toArray()) {
            IBuildProperty property = (IBuildProperty) selectedObject;
            if(property.isWellKnown()) {
                removeEnabled = false;
                break;
            }
        }
        return removeEnabled;
    }

    /**
     * Add a listener to the control. If the listener is already present, the
     * request is ignored.
     * 
     * @param listener
     */
    public void addListener(IBuildPropertyControlListener listener) {
        fListenerList.add(listener);
    }

    /**
     * Remove a listener from the control. If the listener isn't present, the
     * request is ignored.
     * 
     * @param listener
     */
    public void removeListener(IBuildPropertyControlListener listener) {
        fListenerList.remove(listener);
    }

    protected void notifyPropertyAdded(IBuildProperty buildProperty) {
        Object[] listeners = fListenerList.getListeners();
        for (int i = 0; i < listeners.length; i++) {
            if (listeners[i] instanceof IBuildPropertyControlListener) {
                IBuildPropertyControlListener listener = (IBuildPropertyControlListener) listeners[i];
                listener.propertyAdded(buildProperty);
            }
        }
    }

    protected void notifyPropertyEdited(IBuildProperty buildProperty) {
        Object[] listeners = fListenerList.getListeners();
        for (int i = 0; i < listeners.length; i++) {
            if (listeners[i] instanceof IBuildPropertyControlListener) {
                IBuildPropertyControlListener listener = (IBuildPropertyControlListener) listeners[i];
                listener.propertyEdited(buildProperty);
            }
        }
    }

    protected void notifyPropertiesRemoved(IBuildProperty[] buildProperties) {
        Object[] listeners = fListenerList.getListeners();
        for (int i = 0; i < listeners.length; i++) {
            if (listeners[i] instanceof IBuildPropertyControlListener) {
                IBuildPropertyControlListener listener = (IBuildPropertyControlListener) listeners[i];
                listener.propertiesRemoved(buildProperties);
            }
        }
    }

    private Composite createComposite(Composite parent) {
        if (fToolkit != null) {
            return fToolkit.createComposite(parent);
        }

        return new Composite(parent, SWT.NONE);
    }

    private Button createButton(Composite parent, String label, int style) {
        if (fToolkit != null) {
            return fToolkit.createButton(parent, label, style);
        }

        Button button = new Button(parent, style);
        button.setText(label);
        return button;
    }

    private Table createTable(Composite parent, int style) {
        if (fToolkit != null) {
            return fToolkit.createTable(parent, style);
        }
        return new Table(parent, style);
    }

    private void setSelectionListenerOnColumn(TableColumn column, final String property) {
        column.addSelectionListener(new SelectionAdapter() {
            private boolean fReverse = false;

            public void widgetSelected(SelectionEvent e) {
                int sortColumn = 0;
                if (property.equals(COLUMN_NAME)) {
                    sortColumn = 0;
                } else if (property.equals(COLUMN_VALUE)) {
                    sortColumn = 1;
                } else if (property.equals(COLUMN_DESCRIPTION)) {
                    sortColumn = 2;
                }

                fPropertyTableSorter.setSortColumn(sortColumn);
                Table table = fTableViewer.getTable();
                table.setSortColumn(table.getColumn(sortColumn));
                if (fReverse) {
                    table.setSortDirection(SWT.DOWN);
                    fPropertyTableSorter.setReverse(fReverse);
                } else {
                    table.setSortDirection(SWT.UP);
                    fPropertyTableSorter.setReverse(fReverse);
                }
                fReverse = !fReverse;
                fTableViewer.refresh();
            };
        });
    }

    protected void handleOpen(OpenEvent event) {
        ISelection selection = event.getSelection();
        if (!(selection instanceof IStructuredSelection)) {
            return;
        }
        IStructuredSelection structuredSelection = (IStructuredSelection) selection;
        if (isEditEnabled(structuredSelection)) {
            Object element = structuredSelection.getFirstElement();
            if (element instanceof IBuildProperty) {
                IBuildProperty buildProperty = (IBuildProperty) element;
                editProperty(buildProperty);
            }
        }
    };

    /**
     * Set the button layout data. Set the widths based on the longest button
     * label.
     */
    private void setButtonLayoutData() {
        GC gc = new GC(fAddPropertyButton);
        gc.setFont(fAddPropertyButton.getFont());
        FontMetrics fontMetrics = gc.getFontMetrics();
        gc.dispose();

        // Start with the default width.
        int maxWidth = Dialog.convertHorizontalDLUsToPixels(fontMetrics, IDialogConstants.BUTTON_WIDTH);

        // Determine the widest button.
        Point minSize = fAddPropertyButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
        maxWidth = Math.max(maxWidth, minSize.x);

        minSize = fEditPropertyButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
        maxWidth = Math.max(maxWidth, minSize.x);

        minSize = fRemovePropertyButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
        maxWidth = Math.max(maxWidth, minSize.x);

        // Set the width on all the buttons to match the widest.
        GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fAddPropertyButton.setLayoutData(gridData);

        gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fEditPropertyButton.setLayoutData(gridData);

        gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fRemovePropertyButton.setLayoutData(gridData);
    }

    /**
     * Globally enables or disables all the controls in the build property
     * control.
     * 
     * @param enabled
     */
    public void setEnabled(boolean enabled) {
        fAddPropertyButton.setEnabled(enabled);
        fEditPropertyButton.setEnabled(enabled);
        fRemovePropertyButton.setEnabled(enabled);
        fTableViewer.getTable().setEnabled(enabled);

        // If globally enabled, the buttons then control
        // their own enablement.
        if (enabled) {
            updatePropertyButtonEnablement();
        }
    }

    /**
     * Allows access to the {@link TableViewer} used by this control to display
     * property names, values and descriptions.
     * 
     * @return {@link Viewer}, never null.
     */
    public Viewer getViewer() {
        return fTableViewer;
    }

    /**
     * Get the repository context for the property control.
     * 
     * @return The repository, or <code>null</code> if none was set.
     */
    public ITeamRepository getTeamRepository() {
        return fTeamRepository;
    }

    protected TableViewer getTableViewer(Table table) {
        return new TableViewer(table);
    }

    /**
     * Refresh the table of properties. This can be called to update the table
     * of properties, for example, when the passed-in list of properties
     * changes.
     */
    public void refresh() {
        fTableViewer.refresh();
    }
}
