/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import java.net.URI;
import java.util.ArrayList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.helper.ItemNotFoundHelper;
import com.ibm.team.build.internal.ui.tooltips.AbstractBuildItemHyperlinkHandler;
import com.ibm.team.build.ui.BuildUI;
import com.ibm.team.foundation.common.URIReference;
import com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.Location;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Attempts to open an <code>IBuildResult</code> in its editor given either
 * the <code>itemId</code> or the <code>label</code> of the build result.
 */
public class BuildResultHyperlinkHandler extends AbstractBuildItemHyperlinkHandler {
    
    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean resolves(URI uri) {
        return handles(uri);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public Object resolve(URI uri, ContextProvider provider) throws TeamRepositoryException {
        Location location = getLocationFor(uri);
        if (location == null) {
            return null;
        }

        IBuildResultHandle buildResultHandle = getBuildResultHandle(location);
        if (buildResultHandle == null) {
            return null;
        }

        ITeamRepository teamRepository = getTeamRepository(location, null);
        if (teamRepository == null) {
            return null;
        }

        return teamRepository.itemManager().fetchCompleteItem(buildResultHandle, IItemManager.DEFAULT, null);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean links(Object object) {
        if (object instanceof IBuildResultHandle) {
            return true;
        } else {
            return super.links(object);
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public URIReference createHyperlink(Object object, IProgressMonitor monitor) {
        if (object instanceof IBuildResultHandle) {
            IBuildResultHandle buildResultHandle = (IBuildResultHandle) object;
            ITeamRepository teamRepository = (ITeamRepository) buildResultHandle.getOrigin();
            URI uri = getAbsoluteURI(buildResultHandle, teamRepository);
            if (uri != null) {
                try {
                    IBuildResult buildResult = getBuildResultForCreateHyperlink(buildResultHandle, teamRepository);
                    if (buildResult != null) {
                        String label = buildResult.getLabel();
                        if ((label == null) || (label.length() == 0)) {
                            label = BuildUIMessages.BUILD_RESULT_LABEL_UNLABELED;
                        }

                        IBuildDefinition definition = (IBuildDefinition) teamRepository.itemManager().fetchCompleteItem(
                                buildResult.getBuildDefinition(), IItemManager.DEFAULT, monitor);
                        String name = NLS.bind(BuildUIMessages.BUILD_RESULT_LABEL, definition.getId(), label);
                        String details = NLS.bind(BuildUIMessages.BuildResultHyperlinkHandler_URIReferenceDetails,
                                definition.getId(), label);
                        return new URIReference(name, details, uri);
                    }
                } catch (ItemNotFoundException exception) {
                    ItemNotFoundHelper.updateItemManager((IItemHandle) exception.getData());
                    return new URIReference(BuildUIMessages.BUILD_RESULT_LABEL_DELETED,
                            BuildUIMessages.BUILD_RESULT_DELETED_HYPERLINK_DETAIL, uri);
                } catch (TeamRepositoryException exception) {
                    BuildUIPlugin.log(exception);
                    return new URIReference(BuildUIMessages.BUILD_RESULT_HYPERLINK_ERROR,
                            BuildUIMessages.BUILD_RESULT_ERROR_HYPERLINK_DETAIL, uri);
                }
            }
        }
        return super.createHyperlink(object, monitor);
    }

    /**
     * Attempts to get an <code>IBuildResult</code> with the
     * <code>label</code> and <code>buildDefinition</code> properties for
     * use by <code>createHyperlink</code>.
     * 
     * @param buildResultHandle
     *            a <code>IBuildResultHandle</code> to get the
     *            <code>IBuildResult</code> for; never <code>null</code>.
     * @param teamRepository
     *            the <code>ITeamRepository</code> of the provided
     *            IBuildResultHandle; may be <code>null</code>.
     * @return an <code>IBuildResult</code> or <code>null</code>
     * @LongOp fetches <code>IBuildResult</code> if necessary properties are
     *         not set
     */
    protected IBuildResult getBuildResultForCreateHyperlink(IBuildResultHandle buildResultHandle,
            ITeamRepository teamRepository) throws TeamRepositoryException {
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$

        if (teamRepository == null) {
            return null;
        }

        if (buildResultHandle instanceof IBuildResult) {
            IBuildResult buildResult = (IBuildResult) buildResultHandle;
            if (hasRequiredCreateHyperlinkProperties(buildResult)) {
                return buildResult;
            }
        }

        ArrayList<String> properties = new ArrayList<String>();
        properties.add(IBuildResult.PROPERTY_LABEL);
        properties.add(IBuildResult.PROPERTY_BUILD_DEFINITION);

        return (IBuildResult) teamRepository.itemManager().fetchPartialItem(buildResultHandle, IItemManager.DEFAULT,
                properties, null);
    }

    protected boolean hasRequiredCreateHyperlinkProperties(IBuildResult buildResult) {
        return buildResult.isPropertySet(IBuildResult.PROPERTY_LABEL)
                && buildResult.isPropertySet(IBuildResult.PROPERTY_BUILD_DEFINITION);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean handles(URI uri) {
        Location location = getLocationForUriNoException(uri);
        if (location == null) {
            return false;
        }

        IBuildResultHandle buildResultHandle = getBuildResultHandle(location);
        if (buildResultHandle == null) {
            return false;
        }

        return true;
    }

    @Override
    public IStatus open2(URI uri, ContextProvider context, IProgressMonitor monitor) {
        Location location = getLocationForUriNoException(uri);
        if (location == null) {
            return new Status(Status.ERROR, BuildUIPlugin.getUniqueIdentifier(), NLS.bind(
                    BuildUIMessages.BuildResultHyperlinkHandler_BAD_URI_NO_LOCATION, uri.toString()));
        }

        IBuildResultHandle buildResultHandle = getBuildResultHandle(location);
        if (buildResultHandle == null) {
            return new Status(Status.ERROR, BuildUIPlugin.getUniqueIdentifier(), NLS.bind(
                    BuildUIMessages.BuildResultHyperlinkHandler_BAD_URI_NO_BUILD_RESULT, uri.toString()));
        }

        ITeamRepository teamRepository = null;
        try {
            teamRepository = getTeamRepository(location, monitor);
        } catch (TeamRepositoryException exception) {
            return new Status(Status.ERROR, BuildUIPlugin.getUniqueIdentifier(), NLS.bind(
                    BuildUIMessages.BuildItemHyperlinkHandler_ERROR_RESOLVING_REPOSITORY, uri.toString()), exception);
        }

        if (teamRepository == null) {
            return new Status(Status.ERROR, BuildUIPlugin.getUniqueIdentifier(), NLS.bind(
                    BuildUIMessages.BuildResultHyperlinkHandler_BAD_URI_BAD_REPOSITORY, uri.toString()));
        }

        getBuildUI().open(buildResultHandle, teamRepository, null);
        return Status.OK_STATUS;
    }

    protected BuildUI getBuildUI() {
        return BuildUI.getDefault();
    }

    protected IBuildResultHandle getBuildResultHandle(Location location) {
        ValidationHelper.validateNotNull("location", location); //$NON-NLS-1$

        if (location.getItemHandle() instanceof IBuildResultHandle) {
            return (IBuildResultHandle) location.getItemHandle();
        } else {
            return null;
        }
    }

}