/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.jface.viewers.ViewerLabel;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildItem;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.jface.labelProviders.TypeLabelProvider;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;

/**
 * Asynchronous label provider for {@link IBuildResultHandle} objects. This
 * class maintains a queue (actually a Set) of IBuildResultHandle items that
 * need to be fetched from the repository. When updateLabel(...) is called with
 * a new IBuildResultHandle object, the object is added to the collection of
 * results to be fetched. The code then creates the fetch job if it does not
 * already exist, and schedules the job if it is not already running and not
 * already scheduled. The state of the fetch job is stored internally using a
 * private enum. If the fetch job is already running, it will pick up the new
 * IBuildResult item from the collection and fetch it. Every time a new
 * IBuildResult item is fetched, a new {@link LabelProviderChangedEvent} is
 * generated to notify listeners that the associated label needs to be updated.
 */
public class BuildResultLabelProvider extends TypeLabelProvider {

    private final Object FETCH_LOCK = new Object();
    private BuildJob fFetchJob;

    private Hashtable<UUID, Image> fBuildResultImages = new Hashtable<UUID, Image>();

    private Map<UUID, IBuildResultHandle> fCurrentlyFetching = new Hashtable<UUID, IBuildResultHandle>();

    private final Collection<String> REQUIRED_BUILD_RESULT_PROPERTIES = new Vector<String>(3);
    private Set<IBuildItem> fCachedBuildItems = new HashSet<IBuildItem>();
    private Set<UUID> fDeletedBuilds = new HashSet<UUID>();

    protected enum FetchJobStatus {
        /**
         * Signifies that the job has been scheduled, but is not yet running
         */
        SCHEDULED,

        /**
         * Signifies the job is currently running
         */
        RUNNING,

        /**
         * The job is not running and is not scheduled to run
         */
        NOT_RUNNING
    }

    private FetchJobStatus fFetchJobStatus = FetchJobStatus.NOT_RUNNING;

    /**
     * Constructor
     */
    public BuildResultLabelProvider() {
        REQUIRED_BUILD_RESULT_PROPERTIES.add(IBuildResult.PROPERTY_LABEL);
        REQUIRED_BUILD_RESULT_PROPERTIES.add(IBuildResult.PROPERTY_BUILD_DEFINITION);
        REQUIRED_BUILD_RESULT_PROPERTIES.add(IBuildResult.PROPERTY_BUILD_STATUS);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void updateLabel(final ViewerLabel label, final Object element) {
        IBuildResultHandle buildResultHandle = (IBuildResultHandle) element;

        Object[] buildResultInfo = getBuildResultLabelObjects(buildResultHandle);

        synchronized (FETCH_LOCK) {
            if (buildResultInfo == null) {
                if (fDeletedBuilds.contains(buildResultHandle.getItemId())) {
                    if (label.getText() == null || label.getText().length() == 0) {
                        label.setText(BuildUIMessages.BUILD_RESULT_LABEL_DELETED);
                    }
                    label.setImage(BuildUIPlugin.getImage("icons/obj16/abanblds_obj.gif")); //$NON-NLS-1$
                } else if (!fCurrentlyFetching.containsKey(buildResultHandle.getItemId())) {
                    if (fBuildResultImages.containsKey(buildResultHandle.getItemId())) {
                        label.setImage(fBuildResultImages.get(buildResultHandle.getItemId()));
                    } else {
                        label.setImage(BuildUIPlugin.getImage("icons/obj16/bld_rslt.gif")); //$NON-NLS-1$
                    }

                    fCurrentlyFetching.put(buildResultHandle.getItemId(), buildResultHandle);
                    if (fFetchJob == null) {
                        fFetchJob = createFetchJob();
                    }
                    if (fFetchJobStatus == FetchJobStatus.NOT_RUNNING) {
                        scheduleFetchJob(fFetchJob);
                        fFetchJobStatus = FetchJobStatus.SCHEDULED;
                    }
                }
            }
        }

        if (buildResultInfo != null) {
            updateLabel(label, (IBuildResult) buildResultInfo[0], (IBuildDefinition) buildResultInfo[1]);
        }

    }

    protected void updateLabel(ViewerLabel label, IBuildResult buildResult, IBuildDefinition buildDefinition) {
        label.setText(NLS.bind(BuildUIMessages.BUILD_RESULT_LABEL, buildDefinition.getId(), buildResult.getLabel()));

        if (!fBuildResultImages.containsKey(buildResult.getItemId())) {
            fBuildResultImages.put(buildResult.getItemId(),
                    BuildStatusCompositeImageDescriptor.getBuildResultComposite(buildResult, getResources()));
        }

        label.setImage(fBuildResultImages.get(buildResult.getItemId()));
    }

    protected Object[] getBuildResultLabelObjects(IBuildResultHandle handle) {
        IBuildDefinition buildDefinition = null;
        IBuildResult buildResult = getBuildResult(handle);
        if (buildResult != null) {
            buildDefinition = getBuildDefinition(buildResult);
        }

        return buildDefinition == null ? null : new Object[] { buildResult, buildDefinition };
    }

    protected IBuildResult getBuildResult(IBuildResultHandle handle) {
        ITeamRepository repository = getOrigin(handle);
        if (repository == null) {
            return null;
        }

        IBuildResult buildResult = (handle instanceof IBuildResult) ? (IBuildResult) handle : null;
        if (buildResult == null) {
            buildResult = (IBuildResult) repository.itemManager().getSharedItemIfKnown(handle);
        }

        if (buildResult == null || !hasRequiredProperties(buildResult)) {
            return null;
        }

        fCachedBuildItems.add(buildResult);
        return buildResult;
    }

    protected IBuildDefinition getBuildDefinition(IBuildResult buildResult) {
        ITeamRepository repository = getOrigin(buildResult);
        IBuildDefinition buildDefinition = (IBuildDefinition) repository.itemManager().getSharedItemIfKnown(
                buildResult.getBuildDefinition());
        if (buildDefinition == null || !buildDefinition.isPropertySet(IBuildDefinition.PROPERTY_ID)) {
            return null;
        }

        fCachedBuildItems.add(buildDefinition);
        return buildDefinition;
    }

    private boolean hasRequiredProperties(IBuildResult buildResult) {
        for (String requiredProperty : REQUIRED_BUILD_RESULT_PROPERTIES) {
            if (!buildResult.isPropertySet(requiredProperty)) {
                return false;
            }
        }
        return true;
    }

    /**
     * @param fetchJob
     */
    protected void scheduleFetchJob(BuildJob fetchJob) {
        fetchJob.scheduleJob();
    }

    protected BuildJob createFetchJob() {
        return new BuildJob(BuildUIMessages.BuildResultLabelProvider_FETCH_BUILD_RESULT_JOB_TITLE) {
            @Override
            public IStatus run(IProgressMonitor monitor) {
                try {
                    fetchBuildResults(monitor);
                } finally {
                    fFetchJobStatus = FetchJobStatus.NOT_RUNNING;
                }
                return Status.OK_STATUS;
            }
        };
    }

    protected void fetchBuildResults(IProgressMonitor monitor) {
        Set<IBuildResultHandle> buildResultsToResolve = new HashSet<IBuildResultHandle>();
        List<IBuildResult> resolvedBuildResults = new ArrayList<IBuildResult>();

        ITeamRepository teamRepository = null;

        while (true) {
            /*
             * Get the lock so that no other threads can add new build results
             * or decide whether or not to reschedule the fetch job while we are
             * determining whether or not we are done for now.
             */
            synchronized (FETCH_LOCK) {
                if (fCurrentlyFetching.size() > 0) {
                    fFetchJobStatus = FetchJobStatus.RUNNING;
                } else {
                    fFetchJobStatus = FetchJobStatus.NOT_RUNNING;
                    break;
                }

                /*
                 * The build results may come from multiple repositories. We
                 * will fetch from the repository of the first build result we
                 * see. Results from other repositories will be fetched during a
                 * subsequent loop-through.
                 */
                buildResultsToResolve.clear();
                resolvedBuildResults.clear();
                teamRepository = null;

                for (Iterator<IBuildResultHandle> iterator = fCurrentlyFetching.values().iterator(); iterator.hasNext();) {
                    IBuildResultHandle handle = iterator.next();
                    if (getOrigin(handle) == null) {
                        iterator.remove();
                    } else if (teamRepository == null) {
                        buildResultsToResolve.add(handle);
                        teamRepository = (ITeamRepository) getOrigin(handle);
                    } else {
                        if (teamRepository.getId().equals(((ITeamRepository) getOrigin(handle)).getId())) {
                            buildResultsToResolve.add(handle);
                        }
                    }
                }
            }

            try {
                List<IBuildResultHandle> resultsToFetch = new ArrayList<IBuildResultHandle>();
                List<IBuildDefinitionHandle> definitionsToFetch = new ArrayList<IBuildDefinitionHandle>();

                for (IBuildResultHandle buildResultHandle : buildResultsToResolve) {
                    IBuildResult buildResult = getBuildResult(buildResultHandle);
                    if (buildResult == null) {
                        resultsToFetch.add(buildResultHandle);
                    } else {
                        resolvedBuildResults.add(buildResult);
                        IBuildDefinition definition = getBuildDefinition(buildResult);
                        if (definition == null) {
                            definitionsToFetch.add(buildResult.getBuildDefinition());
                        }
                    }
                }

                if (resultsToFetch.size() > 0) {
                    List<IBuildResult> fetchedBuildResults = teamRepository.itemManager().fetchPartialItems(
                            resultsToFetch, IItemManager.DEFAULT, REQUIRED_BUILD_RESULT_PROPERTIES, monitor);

                    for (int i = 0; i < fetchedBuildResults.size(); i++) {
                        IBuildResult fetchedResult = fetchedBuildResults.get(i);
                        if (fetchedResult != null) {
                            resolvedBuildResults.add(fetchedResult);
                            IBuildDefinition definition = getBuildDefinition(fetchedResult);
                            if (definition == null) {
                                definitionsToFetch.add(fetchedResult.getBuildDefinition());
                            }
                        } else {
                            fDeletedBuilds.add(resultsToFetch.get(i).getItemId());
                        }
                    }
                }

                fCachedBuildItems.addAll(resolvedBuildResults);

                if (definitionsToFetch.size() > 0) {
                    // although the process area is not needed for the label, fetch it here to avoid extra round trips elsewhere in the UI where it is needed
                    fCachedBuildItems.addAll(teamRepository.itemManager().fetchPartialItems(definitionsToFetch,
                            IItemManager.DEFAULT, Arrays.asList(IBuildDefinition.PROPERTY_ID, IBuildDefinition.PROPERTY_PROCESS_AREA), monitor));
                }
            } catch (TeamRepositoryException exception) {
                BuildUIPlugin.log(new Status(IStatus.ERROR, BuildUIPlugin.getUniqueIdentifier(),
                        BuildUIMessages.BuildResultLabelProvider_FETCH_ERROR, exception));
            } finally {
                for (IBuildResult resolvedResult : resolvedBuildResults) {
                    fBuildResultImages.put(resolvedResult.getItemId(),
                            BuildStatusCompositeImageDescriptor.getBuildResultComposite(resolvedResult, getResources()));
                }

                synchronized (FETCH_LOCK) {
                    for (IBuildResultHandle handle : buildResultsToResolve) {
                        fCurrentlyFetching.remove(handle.getItemId());
                    }
                }

                fireLabelProviderChangeEvent(buildResultsToResolve);
            }
        }
    }

    protected void fireLabelProviderChangeEvent(Collection<IBuildResultHandle> buildResultsHandles) {
        final List<IBuildResultHandle> buildResults = new ArrayList<IBuildResultHandle>();
        buildResults.addAll(buildResultsHandles);

        asyncExec(new Runnable() {
            public void run() {
                fireLabelProviderChanged(new LabelProviderChangedEvent(BuildResultLabelProvider.this,
                        buildResults.toArray(new IBuildResultHandle[buildResults.size()])));
            }
        });
    }

    protected ITeamRepository getOrigin(IBuildResultHandle buildResultHandle) {
        return (ITeamRepository) buildResultHandle.getOrigin();
    }

    public void dispose() {
        if (fFetchJob != null) {
            fFetchJob.cancel();
            fFetchJob = null;
        }
        // call super dispose to dispose of images in ResourceManager
        super.dispose();
    }

    protected void asyncExec(Runnable runnable) {
        Display.getDefault().asyncExec(runnable);
    }

}
