/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import com.ibm.team.foundation.common.util.FoundationLog;
import com.ibm.team.jface.JazzResources;

/**
 * Initializer class for the build ui plugin.
 */
public class BuildUIPlugin extends AbstractUIPlugin {

    private static BuildUIPlugin fgBuildPlugin;
    private ResourceManager fResourceManager;
    
    /**
     * The plugin id for the build ui plugin. 
     */
    public static final String PLUGIN_ID = "com.ibm.team.build.ui"; //$NON-NLS-1$
    
    /**
     * Status code indicating an unexpected internal error.
     */
    public static final int INTERNAL_ERROR = 120;
    
    /**
     * Preference for showing the ignored pasted build property dialog
     */
    public static final String PREF_SHOW_IGNORED_PROPERTY_DIALOG = "show_ignored_property_dialog"; //$NON-NLS-1$

    /**
     * Preference for showing the replaced build property dialog
     */
    public static final String PREF_SHOW_OVERWRITTEN_PROPERTY_DIALOG = "show_overwritten_property_dialog"; //$NON-NLS-1$
    
    /**
     * Preference for showing the build view folders under the build domain
     */
    public static final String PREF_SHOW_BUILD_FOLDERS = "show_build_folders"; //$NON-NLS-1$
    
    /**
     * Create the plugin.
     */
    public BuildUIPlugin() {
        super();
        fgBuildPlugin = this;
    }

    /**
     * Get the singleton plugin instance.
     * 
     * @return the singleton plugin instance
     */
    public static BuildUIPlugin getDefault() {
        return fgBuildPlugin;
    }

    /**
     * Get the plugin ID.
     * 
     * @return the plugin ID
     */
    public static String getUniqueIdentifier() {
        return PLUGIN_ID;
    }

    /**
     * Log an exception in the eclipse log. This method forwards to the log
     * method in {@link FoundationLog}.
     * 
     * @param exception
     *            the exception to log
     */
    public static void log(Throwable exception) {
        String message = exception.getMessage();
        if (message == null) {
            // The constructor for Status does not accept null for message
            message = ""; //$NON-NLS-1$
        }
        log(new Status(IStatus.ERROR, getUniqueIdentifier(), IStatus.ERROR, message, exception));
    }

    /**
     * Log an message in the eclipse log. This method forwards to the log method
     * in {@link FoundationLog}.
     * 
     * @param message
     *            the message to log
     */
    public static void log(String message) {
        if (message == null) {
            // The constructor for Status does not accept null for message
            message = ""; //$NON-NLS-1$
        }
        log(new Status(IStatus.ERROR, getUniqueIdentifier(), IStatus.ERROR, message, null));
    }

    /**
     * Log an status in the eclipse log. This method forwards to the log method
     * in {@link FoundationLog}.
     * 
     * @param status
     *            the status to log
     */
    public static void log(IStatus status) {
        FoundationLog.log(status);
    }

    /**
     * Get an image for the specified path.
     * 
     * @param path
     *            the image path
     * @return An image that is already in the image registry. The caller must
     *         not dispose the image.
     */
    public static Image getImage(String path) {
        ImageRegistry imageRegistry = getDefault().getImageRegistry();

        Image image = imageRegistry.get(path);
        if (image == null) {
            ImageDescriptor descriptor = AbstractUIPlugin.imageDescriptorFromPlugin(getUniqueIdentifier(), path);
            imageRegistry.put(path, descriptor);
            image = imageRegistry.get(path);
        }
        return image;
    }

    /**
     * Get a descriptor for an image at the specified path.
     * 
     * @param path
     *            the image path
     * @return a descriptor for an image at the specified path
     */
    public static ImageDescriptor getImageDescriptor(String path) {
        ImageRegistry imageRegistry = getDefault().getImageRegistry();

        ImageDescriptor descriptor = imageRegistry.getDescriptor(path);
        if (descriptor == null) {
            descriptor = AbstractUIPlugin.imageDescriptorFromPlugin(getUniqueIdentifier(), path);
            imageRegistry.put(path, descriptor);
        }
        return descriptor;
    }

    private ResourceManager getResourceManager() {
        if (fResourceManager == null)
            fResourceManager = new LocalResourceManager(JFaceResources.getResources());
        return fResourceManager;
    }

    /**
     * Returns an image for the given image descriptor.
     * <p>
     * WARNING: The image's lifecycle is managed by the plugin and must not be
     * disposed by the caller. It is only disposed when the application exits.
     * This method is meant to be used only for images that cannot be managed by
     * a LocalResourceManager. Most of the time you should use a
     * LocalResourceManager instead.
     * </p>
     * 
     * @param imageDesc
     *            the image descriptor
     * @return the image
     */
    public static Image getImage(ImageDescriptor imageDesc) {
        return JazzResources.getImageWithDefault(getDefault().getResourceManager(), imageDesc);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(BundleContext context) throws Exception {
        try {
            if (fResourceManager != null) {
                fResourceManager.dispose();
                fResourceManager = null;
            }

            // cancel all jobs of this family
            Job.getJobManager().cancel(fgBuildPlugin);
        } finally {
            super.stop(context);
        }
    }
    
    @Override 
    protected void initializeDefaultPreferences(IPreferenceStore store) {
        store.setDefault(PREF_SHOW_IGNORED_PROPERTY_DIALOG, true);
        store.setDefault(PREF_SHOW_OVERWRITTEN_PROPERTY_DIALOG, true);
        store.setDefault(PREF_SHOW_BUILD_FOLDERS, true);
    }
}
