/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IWorkbenchPage;

import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.client.ITeamBuildRecordClient;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorInput;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.build.internal.ui.tooltips.jobs.OpenEditorJob;
import com.ibm.team.build.ui.BuildUI;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Internal support for the API class <code>BuildUI</code>.
 */
public class BuildUISupport {

    private static final BuildUISupport fDefaultBuildUISupport = new BuildUISupport();

    /**
     * Returns the default <code>BuildUISupport</code> object; never
     * <code>null</code>.
     * 
     * @return the default <code>BuildUISupport</code> object; never
     *         <code>null</code>.
     */
    public static BuildUISupport getDefault() {
        return fDefaultBuildUISupport;
    }

    /**
     * Opens an editor for the build result indicated by the provided
     * <code>IBuildResultHandle</code>.
     * <p>
     * <b>Note</b>: <i>This method should <b>not</b> be called directly!</i>
     * It implements the contract specified in
     * {@link BuildUI#open(IBuildResultHandle, ITeamRepository, IWorkbenchPage)}.
     * All calls to this method should come through
     * <code>BuildUI.open(...)</code> which performs validation on the
     * parameters.
     * 
     * @param buildResultHandle
     *            a handle to a <code>IBuildResult</code>; never
     *            <code>null</code>.
     * @param teamRepository
     *            the <code>ITeamRepository</code> containing the build
     *            result; never <code>null</code>.
     * @param workbenchPage
     *            the <code>IWorkbenchPage</code> to open the build result
     *            editor in; may be <code>null</code>.
     */
    public void open(IBuildResultHandle buildResultHandle, ITeamRepository teamRepository, IWorkbenchPage workbenchPage) {
        /**
         * Note: parameter validation is done in
         * {@link BuildUI#open(IBuildResultHandle, ITeamRepository, IWorkbenchPage)}
         */

        FindResultJob findResultJob = createFindResultJob(buildResultHandle, teamRepository, workbenchPage);
        findResultJob.schedule();
    }

    protected FindResultJob createFindResultJob(IBuildResultHandle buildResultHandle, ITeamRepository teamRepository,
            IWorkbenchPage workbenchPage) {
        return new FindResultJob(buildResultHandle, teamRepository, workbenchPage);
    }

    protected static class FindResultJob extends TeamBuildJob {
        private final IBuildResultHandle fBuildResultHandle;
        private final ITeamRepository fTeamRepository;
        private final IWorkbenchPage fWorkbenchPage;

        protected FindResultJob(IBuildResultHandle buildResultHandle, ITeamRepository teamRepository,
                IWorkbenchPage workbenchPage) {
            super(BuildUIMessages.BuildUISupport_OPEN_BUILD_RESULT_JOB_TITLE, true, teamRepository);
            fBuildResultHandle = buildResultHandle;
            fTeamRepository = teamRepository;
            fWorkbenchPage = workbenchPage;
        }

        protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
            ITeamBuildRecordClient client = (ITeamBuildRecordClient) fTeamRepository.getClientLibrary(ITeamBuildRecordClient.class);

            IBuildResultRecord buildResultRecord = client.getBuildResultRecord(fBuildResultHandle,
                    IBuildResult.PROPERTIES_COMPLETE, monitor);

            OpenEditorJob openResultEditorJob = createOpenResultEditorJob(buildResultRecord, fTeamRepository,
                    fWorkbenchPage);
            openResultEditorJob.schedule();

            return Status.OK_STATUS;
        }

        protected OpenResultEditorJob createOpenResultEditorJob(IBuildResultRecord buildResultRecord,
                ITeamRepository teamRepository, IWorkbenchPage workbenchPage) {
            return new OpenResultEditorJob(buildResultRecord, teamRepository, workbenchPage);
        }
    }

    protected static class OpenResultEditorJob extends OpenEditorJob {
        private final IBuildResultRecord fBuildResultRecord;

        protected OpenResultEditorJob(IBuildResultRecord buildResultRecord, ITeamRepository teamRepository,
                IWorkbenchPage workbenchPage) {
            super(BuildUIMessages.BuildUISupport_OPEN_BUILD_RESULT_EDITOR_JOB_TITLE, teamRepository, workbenchPage);
            fBuildResultRecord = buildResultRecord;
        }

        /*
         * (non-Javadoc) Intentionally not documented. See parent.
         */
        @Override
        protected IEditorInput getEditorInput() {
            return new BuildResultEditorInput(fBuildResultRecord, getTeamRepository());
        }

        protected String getEditorId() {
            return IBuildUIConstants.ID_BUILD_RESULT_EDITOR;
        }

    }
}
