/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.forms.FormColors;


/**
 * A composite which displays an information, warning, or error message in
 * a Firefox style. The information bar stays invisible until a message
 * is set. <br /> 
 * 
 * Use {@link IMessageProvider} constants to set the label type.
 * @since 0.6
 */
public class InformationBar extends Composite {
    
    /**
     *  A listener for the events that occur in an information bar
     */
    public interface IInformationBarListener {
        
        /**
         * Method called when a hyperlink in the information bar's message has
         * been selected.
         * 
         * @param ref
         *            The clicked hyperlink's reference String
         */
        public void clicked(String ref);
        
        /**
         * Method called when the information bar has been closed by the user.
         */
        public void closed();
    }

    private Label fLabelImage;
    private Link fLabelMessage;
    private Composite fLabelComposite;
    private LocalResourceManager fResourceManager;
    
    private ListenerList fLinkListeners= new ListenerList(ListenerList.IDENTITY);
    private final int fStyle;
    private ToolItem fCloseButton;
    
    /**
     * Constructs a 
     * 
     * @param parent
     * @param style
     */
    public InformationBar(Composite parent, int style) {
        super(parent, SWT.NONE);
        
        fStyle= style;
        GridLayoutFactory.swtDefaults().margins(2, 0).extendedMargins(0, 0, 0, hasBit(fStyle, SWT.BORDER) ? 3 : 0).applyTo(this);
        createControl();
        setVisible(false);
        setBackground(getDisplay().getSystemColor(SWT.COLOR_INFO_BACKGROUND));
    }
    
    private static boolean hasBit(int mask, int bit) {
        return (mask & bit) != 0;
    }

    /**
     * Adds a listener to the information bar to catch the widget's events.
     * 
     * @param linkListener
     *            the information bar listener to add to the information bar.
     */
    public void addLinkListener(IInformationBarListener linkListener) {
        fLinkListeners.add(linkListener);
    }
    
    /**
     * Removes a listener from the information bar.
     * 
     * @param linkListener
     *            the information bar listener to remove from the information bar.
     */
    public void removeLinkListener(IInformationBarListener linkListener) {
        fLinkListeners.remove(linkListener);
    }

    protected void createControl() {
        
        fResourceManager= new LocalResourceManager(JFaceResources.getResources(), this);
        
        if(hasBit(fStyle, SWT.BORDER)) {
            addPaintListener(new PaintListener() {
                
                final Color fStrokeColor= fResourceManager.createColor(FormColors.blend(
                        getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND).getRGB(), 
                        getDisplay().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW).getRGB(), 
                        20));
                
                public void paintControl(PaintEvent e) {
                    Rectangle bounds= getClientArea();
                    
                    int yOffset;
                    yOffset= bounds.y + bounds.height - 3;
                    e.gc.setForeground(e.gc.getDevice().getSystemColor(SWT.COLOR_INFO_BACKGROUND));
                    e.gc.drawLine(bounds.x, yOffset, bounds.x + bounds.width, yOffset);
                    
                    yOffset+=1;
                    e.gc.setForeground(e.gc.getDevice().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
                    e.gc.drawLine(bounds.x, yOffset, bounds.x + bounds.width, yOffset);
                    
                    yOffset+=1;
                    e.gc.setForeground(fStrokeColor);
                    e.gc.drawLine(bounds.x, yOffset, bounds.x + bounds.width, yOffset);
                }
            });
        }
        
        Composite container= new Composite(this, SWT.NONE);
        container.setBackground(container.getShell().getDisplay().getSystemColor(SWT.COLOR_INFO_BACKGROUND));
        GridLayoutFactory.fillDefaults().numColumns(2).margins(0, 0).applyTo(container);
        GridDataFactory.fillDefaults().grab(true, true).span(2,1).applyTo(container);
        
        fLabelComposite= new Composite(container, SWT.NONE);
        fLabelComposite.setBackground(container.getBackground());
        GridLayoutFactory.fillDefaults().numColumns(2).equalWidth(false).applyTo(fLabelComposite);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(fLabelComposite);
        
        fLabelImage= new Label(fLabelComposite, SWT.LEFT);
        fLabelImage.setBackground(fLabelComposite.getBackground());
        GridDataFactory.swtDefaults().align(SWT.CENTER, SWT.TOP).grab(false, true).applyTo(fLabelImage);

        fLabelMessage = new Link(fLabelComposite, SWT.LEFT);
        fLabelMessage.setBackground(fLabelComposite.getBackground());
        fLabelMessage.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                Object[] listeners= fLinkListeners.getListeners();
                for (Object listener: listeners) {
                    ((IInformationBarListener)listener).clicked(e.text);
                }
            }
        });
        GridDataFactory.swtDefaults().align(SWT.FILL, SWT.CENTER).grab(true, true).applyTo(fLabelMessage);
        
        ToolBar toolBar= new ToolBar(container, SWT.FLAT | SWT.RIGHT);
        toolBar.setBackground(container.getBackground());
        GridDataFactory.swtDefaults().align(SWT.RIGHT, SWT.TOP).grab(false, true).applyTo(toolBar);

        fCloseButton= new ToolItem(toolBar, SWT.PUSH);
        fCloseButton.setImage(BuildUIPlugin.getImage("icons/elcl16/close.gif")); //$NON-NLS-1$
        fCloseButton.setHotImage(BuildUIPlugin.getImage("icons/elcl16/close_hov.gif")); //$NON-NLS-1$
        fCloseButton.addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
                widgetSelected(e);
            }
            public void widgetSelected(SelectionEvent e) {
                hide(true);
            }
        });
    }
    
    /**
     * @param message
     * @param type
     */
    public void setMessage(String message, int type) {
        boolean hasMessage = message != null && message.trim().length() > 0;
        if(!hasMessage && !isVisible())
            return;
        
        
        if(!hasMessage && isVisible()) {
            hide(false);
            
        } else if(hasMessage && (!isVisible() || !message.equals(fLabelMessage.getText()))) {
        
            fLabelMessage.setText(message);
            boolean showImage = true;
                fLabelImage.setImage(BuildUIPlugin.getImage("icons/obj16/info.gif")); //$NON-NLS-1$
            
            GridData imageGridData= ((GridData)fLabelImage.getLayoutData());
            if (showImage) {
                int closeButtonHeight= fCloseButton.getBounds().height;
                int imageHeight= fLabelImage.getImage().getImageData().height;
                int indent= Math.max(0, (closeButtonHeight - imageHeight)/2);
                imageGridData.verticalIndent= indent;
                imageGridData.horizontalIndent= indent;
            }

            fLabelImage.setVisible(showImage);
            imageGridData.exclude= !showImage;
            fLabelComposite.layout(true, true);
            
            show();
        }
    }

    /**
     * Hides the information bar in the parent composite. Can also notify
     * listeners with a closed event.
     * 
     * @param notifyListeners
     *            if true, the attached link listeners will be notified with a
     *            closed event. If false, the window is hidden without
     *            notification.
     */
    public void hide(boolean notifyListeners) {
        setVisible(false);
        getParent().layout(true, true);
        
        if (notifyListeners) {
            Object[] listeners= fLinkListeners.getListeners();
            for (Object listener: listeners) {
                ((IInformationBarListener)listener).closed();
            }
        }
    }

    /**
     * Causes the information bar to be visible in the parent composite.
     */
    public void show() {
        setVisible(true);
        getParent().layout(true, true);
    }

    @Override
    public void setLayoutData(Object data) {
        if (data != null && data instanceof GridData) {
            ((GridData) data).exclude= !isVisible();
        }
        super.setLayoutData(data);
    }

    @Override
    public void setVisible(boolean visible) {
        Object data= getLayoutData();
        if (data != null && data instanceof GridData) {
            ((GridData) data).exclude= !visible;
            super.setLayoutData(data);
        }
        super.setVisible(visible);
    }
}
