/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.build.internal.common.MicrosoftBuildItem;
import com.ibm.team.build.internal.ui.dialogs.MicrosoftBuildInputDataDialog;

/**
 * A UI control for displaying and, optionally, adding, editing, and removing
 * <code>MicrosoftSolutionBuildItem</code> objects from a supplied
 * <code>java.util.List</code>.
 */
public class MicrosoftBuildItemsTableControl {

    private Button fAddSolutionButton;
    private Button fEditSolutionButton;
    private Button fRemoveSolutionButton;
    private Button fMoveUpBuildItemButton;
    private Button fMoveDownBuildItemButton;
    
    protected Shell fParentShell;

    protected List<MicrosoftBuildItem> fBuildItemProperties;

    private TableViewer fTableViewer;

    private Composite solutionsToBuildComposite;

    private IMicrosoftBuildItemControlListener fChangeListener;

    private static final String[] COLUMN_HEADERS = { BuildUIMessages.MicrosoftBuildConfigurationEditor_SOLUTION_LABEL,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_CONFIG_LABEL,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_BUILD_TYPE_LABEL,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_BUILD_OUTPUT_LABEL,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_BUILD_ADDITIONAL_ARG_LABEL,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_WORKING_DIR_LABEL };
    private static final String[] COLUMN_HEADERS_DESC = {
            BuildUIMessages.MicrosoftBuildConfigurationEditor_SOLUTION_DESC,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_CONFIG_DESC,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_BUILD_TYPE_DESC,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_BUILD_OUTPUT_DESC,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_BUILD_ADDITIONAL_ARG_DESC,
            BuildUIMessages.MicrosoftBuildConfigurationEditor_WORKING_DIR_DESC };
    
    /**
     * Creates the table control
     * 
     * @param parent
     * @param fToolkit
     * @param buildItemProperties
     */
    public MicrosoftBuildItemsTableControl(Composite parent, FormToolkit fToolkit, List<MicrosoftBuildItem> buildItemProperties) {
        fParentShell = parent.getShell();
        
        fBuildItemProperties = buildItemProperties;
        if (fBuildItemProperties == null) {
            fBuildItemProperties = new ArrayList<MicrosoftBuildItem>(1);
        }

        Composite lableComposite = fToolkit.createComposite(parent);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(lableComposite);
        GridLayoutFactory.fillDefaults().numColumns(1).applyTo(lableComposite);
        fToolkit.createLabel(lableComposite, BuildUIMessages.MicrosoftBuildConfigurationEditor_SOLUTION_DESC);

        solutionsToBuildComposite = fToolkit.createComposite(parent);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(solutionsToBuildComposite);
        GridLayoutFactory.fillDefaults().numColumns(2).applyTo(solutionsToBuildComposite);

        Composite tableComposite = fToolkit.createComposite(solutionsToBuildComposite);
        GridDataFactory.fillDefaults().grab(true, true).span(1, 1).applyTo(tableComposite);

        Table table = fToolkit.createTable(tableComposite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.FULL_SELECTION);
        fTableViewer = new TableViewer(table);
        TableColumnLayout layout = new TableColumnLayout();

        for (int i = 0; i < COLUMN_HEADERS.length; i++) {
            TableColumn column = new TableColumn(table, SWT.NONE);
            column.setText(COLUMN_HEADERS[i]);
            column.setToolTipText(COLUMN_HEADERS_DESC[i]);
            layout.setColumnData(column, new ColumnWeightData(10));
        }

        tableComposite.setLayout(layout);
        table.setHeaderVisible(true);
        table.setLinesVisible(true);

        fTableViewer.setContentProvider(new ArrayContentProvider());
        fTableViewer.setLabelProvider(getLabelProvider());
        fTableViewer.setInput(fBuildItemProperties);
        fTableViewer.addSelectionChangedListener(getSelectionChangedListener());
        fTableViewer.addDoubleClickListener(getDoubldClickListener());

        Composite buttonBar = fToolkit.createComposite(solutionsToBuildComposite);
        buttonBar.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, true));
        buttonBar.setLayout(new GridLayout(1, false));

        fAddSolutionButton = fToolkit.createButton(buttonBar,
                BuildUIMessages.MicrosoftBuildConfigurationEditor_ADD_BUTTON_LABEL_WITH_ELLIPSIS, SWT.PUSH);
        fAddSolutionButton.addSelectionListener(getAddSolutionListener());
        fEditSolutionButton = fToolkit.createButton(buttonBar,
                BuildUIMessages.MicrosoftBuildConfigurationEditor_EDIT_BUTTON_LABEL_WITH_ELLIPSIS, SWT.PUSH);
        fEditSolutionButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                MicrosoftBuildItem buildItem = getSelectedProperty();
                if (buildItem != null) {
                    editProperty(buildItem);
                }
            }
        });
        fRemoveSolutionButton = fToolkit.createButton(buttonBar,
                BuildUIMessages.MicrosoftBuildConfigurationEditor_REMOVE_BUTTON_LABEL, SWT.PUSH);
        fRemoveSolutionButton.addSelectionListener(getRemovePropertyListener());

        fMoveUpBuildItemButton = fToolkit.createButton(buttonBar,
                BuildUIMessages.MicrosoftBuildConfigurationEditor_MOVEUP_BUTTON_LABEL, SWT.PUSH);
        fMoveUpBuildItemButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                MicrosoftBuildItem buildItem = getSelectedProperty();
                if (buildItem != null) {
                    moveProperty(true);
                }
            }
        });
        fMoveDownBuildItemButton = fToolkit.createButton(buttonBar,
                BuildUIMessages.MicrosoftBuildConfigurationEditor_MOVEDOWN_BUTTON_LABEL, SWT.PUSH);
        fMoveDownBuildItemButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                MicrosoftBuildItem buildItem = getSelectedProperty();
                if (buildItem != null) {
                    moveProperty(false);
                }
            }
        });
        
        updatePropertyButtonEnablement();
        setButtonLayoutData();
    }
   
    protected SelectionListener getRemovePropertyListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                removeSelectedProperties();
            }
        };
    }

    private void removeSelectedProperties() {
        ISelection selection = fTableViewer.getSelection();
        if (selection instanceof IStructuredSelection) {
            IStructuredSelection structuredSelection = (IStructuredSelection) selection;
            for (Iterator iter = structuredSelection.iterator(); iter.hasNext();) {
                MicrosoftBuildItem buildProperty = (MicrosoftBuildItem) iter.next();
                fTableViewer.remove(buildProperty);
                fBuildItemProperties.remove(buildProperty);
                notifyChange();
            }
        }
    }

    private void editProperty(final MicrosoftBuildItem buildItem) {
        MicrosoftBuildInputDataDialog dialog = new MicrosoftBuildInputDataDialog(fParentShell,
                BuildUIMessages.MicrosoftBuildConfigurationEditor_EDIT_BUILD_ITEM_DESC, buildItem);

        if (dialog.open() == Window.OK) {
            fTableViewer.refresh();
            notifyChange();
        }
    }

    private void moveProperty(boolean moveUp) {
        MicrosoftBuildItem buildItem = getSelectedProperty();
        int index = getElementIndex(buildItem);
        if (moveUp) {
            if (!isElementFirstItem(buildItem)) {
                Collections.swap(fBuildItemProperties, index, index - 1);
                fTableViewer.refresh();
                notifyChange();
            }
        } else {
            if (!isElementLastItem(buildItem)) {
                Collections.swap(fBuildItemProperties, index, index + 1);
                fTableViewer.refresh();
                notifyChange();
            }
        }
    }

    protected ISelectionChangedListener getSelectionChangedListener() {
        return new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                updatePropertyButtonEnablement();
            }
        };
    }
    
    private IDoubleClickListener getDoubldClickListener() {
        return new IDoubleClickListener() {
            public void doubleClick(DoubleClickEvent event) {
                MicrosoftBuildItem buildItem = getSelectedProperty();
                if (buildItem != null) {
                    editProperty(buildItem);
                }
            }
        };
    }

    private void updatePropertyButtonEnablement() {
        IStructuredSelection selection = getCurrentPropertySelection();
        fEditSolutionButton.setEnabled(false);
        fMoveUpBuildItemButton.setEnabled(false);
        fMoveDownBuildItemButton.setEnabled(false);
        fRemoveSolutionButton.setEnabled(false);
        if (selection.size() == 1) {
            fEditSolutionButton.setEnabled(true);
            fMoveUpBuildItemButton.setEnabled(!isElementFirstItem(getSelectedProperty()));
            fMoveDownBuildItemButton.setEnabled(!isElementLastItem(getSelectedProperty()));
        }
        fRemoveSolutionButton.setEnabled(!selection.isEmpty());
    }

    private ITableLabelProvider getLabelProvider() {
        return new MicrosoftBuildItemLabelProvider();
    }

    /**
     * Return the root composite of the property control. The client may call
     * setLayoutData() on the returned composite. This composite is a child of
     * the parent passed into the constructor.
     * 
     * @return the root composite of the property control
     */
    public Composite getComposite() {
        return solutionsToBuildComposite;
    }

    /**
     * Set the button layout data. Set the widths based on the longest button
     * label.
     */
    private void setButtonLayoutData() {
        GC gc = new GC(fAddSolutionButton);
        gc.setFont(fAddSolutionButton.getFont());
        FontMetrics fontMetrics = gc.getFontMetrics();
        gc.dispose();

        // Start with the default width.
        int maxWidth = Dialog.convertHorizontalDLUsToPixels(fontMetrics, IDialogConstants.BUTTON_WIDTH);

        // Determine the widest button.
        Point minSize = fAddSolutionButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
        maxWidth = Math.max(maxWidth, minSize.x);

        minSize = fEditSolutionButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
        maxWidth = Math.max(maxWidth, minSize.x);

        minSize = fRemoveSolutionButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
        maxWidth = Math.max(maxWidth, minSize.x);

        // Set the width on all the buttons to match the widest.
        GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fAddSolutionButton.setLayoutData(gridData);

        gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fEditSolutionButton.setLayoutData(gridData);

        gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fRemoveSolutionButton.setLayoutData(gridData);

        gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fMoveUpBuildItemButton.setLayoutData(gridData);

        gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
        gridData.widthHint = maxWidth;
        fMoveDownBuildItemButton.setLayoutData(gridData);
    }

    protected SelectionListener getAddSolutionListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                addSolution();
            }
        };
    }

    private void addSolution() {
        MicrosoftBuildInputDataDialog dialog = getAddSolutionDialog();
        if (dialog.open() == Dialog.OK) {
            fBuildItemProperties.add(dialog.getBuildItem());
            fTableViewer.refresh();
            updatePropertyButtonEnablement();
            notifyChange();
        }

    }

    protected MicrosoftBuildInputDataDialog getAddSolutionDialog() {
        return new MicrosoftBuildInputDataDialog(fParentShell,
                BuildUIMessages.MicrosoftBuildConfigurationEditor_ADD_BUILD_ITEM_DESC);
    }

    protected IStructuredSelection getCurrentPropertySelection() {
        return (IStructuredSelection) fTableViewer.getSelection();
    }

    private MicrosoftBuildItem getSelectedProperty() {
        IStructuredSelection selection = getCurrentPropertySelection();
        Object element = ((IStructuredSelection) selection).getFirstElement();
        if (element instanceof MicrosoftBuildItem) {
            return (MicrosoftBuildItem) element;
        }
        return null;
    }

    private int getElementIndex(MicrosoftBuildItem buildItem) {
        if (fBuildItemProperties != null) {
            for (int i = 0; i < fBuildItemProperties.size(); i++) {
                if (fBuildItemProperties.get(i) == buildItem)
                    return i;
            }
        }
        return -1;
    }

    private boolean isElementFirstItem(MicrosoftBuildItem buildItem) {
        return (getElementIndex(buildItem) == 0);
    }

    private boolean isElementLastItem(MicrosoftBuildItem buildItem) {
        return (fBuildItemProperties != null && getElementIndex(buildItem) + 1 == fBuildItemProperties.size());
    }

    protected void notifyChange() {
        if (fChangeListener != null) {
            fChangeListener.buildItemsChanged(getBuildProperties());
        }
    }

    /**
     * Gets the list of properties being edited.
     * 
     * @return The list of properties being edited. Elements in the list are
     *         type <code>MicrosoftBuildItem</code>.
     */
    public List getBuildProperties() {
        return fBuildItemProperties;
    }

    /**
     * Set the build properties to edit in the control.
     * 
     * @param buildProperties
     *            A list of type <code>MicrosoftBuildItem</code> that will be
     *            edited in the control. The list will be modified to reflect
     *            the changes made in the control.
     */
    public void setBuildProperties(List<MicrosoftBuildItem> buildProperties) {
        fBuildItemProperties = buildProperties;
        fTableViewer.setInput(fBuildItemProperties);
        updatePropertyButtonEnablement();
    }

    /**
     * @return the <code?fTableViewer</code>
     */
    public TableViewer getTableViewer() {
        return fTableViewer;
    }
   
    /**
     * @param changeListener
     *            the changeListener to set
     */
    public void setChangeListener(IMicrosoftBuildItemControlListener changeListener) {
        fChangeListener = changeListener;
    }

}

class MicrosoftBuildItemLabelProvider extends LabelProvider implements ITableLabelProvider {
    private static final int PRJ_SLN_NAME = 0;
    private static final int BUILD_CONFIG = 1;
    private static final int BUILD_TYPE = 2;
    private static final int LOG_FILE = 3;
    private static final int ADDITIONAL_ARGS = 4;
    private static final int WORKING_DIR = 5;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public Image getColumnImage(Object element, int columnIndex) {
        return null;
    }

    public String getColumnText(Object element, int columnIndex) {
        if (element instanceof MicrosoftBuildItem) {
            MicrosoftBuildItem buildItem = (MicrosoftBuildItem) element;
            switch (columnIndex) {
                case PRJ_SLN_NAME:
                    return buildItem.getBuildFileName();
                case BUILD_CONFIG:
                    return buildItem.getBuildConfig();
                case BUILD_TYPE: {
                    return buildItem.getBuildType();
                }
                case ADDITIONAL_ARGS:
                    return buildItem.getAdditionalArguments();
                case LOG_FILE:
                    return buildItem.getBuildLogFile();
                case WORKING_DIR:
                    return buildItem.getWorkingDir();
                default:
                    return null;
            }
        } else {
            return super.getText(element);
        }
    }

}
