/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildRequestClient;
import com.ibm.team.build.common.MultipleErrorTeamBuildException;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.build.internal.ui.views.query.BuildQueryView;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Allows the user to "abandon" one or more builds (ie mark result(s) as
 * abandoned). This action can work with any element that can adapt to the
 * <code>AbandonBuildAction.SelectionElement</code> and will only be enabled
 * if all build results in the selection are in the In Progress state.
 */
public class AbandonBuildActionDelegate extends AbstractRepositoryActionDelegate implements IObjectActionDelegate {

    private IStructuredSelection fSelection;
    private IWorkbenchPart fActivePart;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getConfirmationMessage() {
        if (fSelection.size() == 1) {
            return BuildUIActionMessages.AbandonBuildAction_3;
        } else {
            return NLS.bind(BuildUIActionMessages.AbandonBuildAction_2, fSelection.size());
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getJobLabel() {
        if (fSelection.size() > 1) {
            return BuildUIActionMessages.AbandonBuildAction_ABANDON_MULTIPLE_BUILDS_JOB_TITLE;
        } else {
            return BuildUIActionMessages.AbandonBuildAction_ABANDON_SINGLE_BUILD_JOB_TITLE;
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        if (fSelection.isEmpty()) {
            return;
        }

        List<TeamRepositoryException> errors = null;
        for (Object object : fSelection.toArray()) {
            IBuildResultRecord record = (IBuildResultRecord) object;
            ITeamRepository repository = (ITeamRepository) record.getBuildResult().getOrigin();
            try {
                ITeamBuildRequestClient client = ClientFactory.getTeamBuildRequestClient(repository);
                client.makeBuildIncomplete(record.getBuildResult(), IBuildResult.PROPERTIES_REQUIRED, progressMonitor);
            } catch (TeamRepositoryException exception) {
                if (errors == null) {
                    errors = new ArrayList<TeamRepositoryException>();
                }
                errors.add(exception);
            }
        }

        if (errors != null) {
            if (errors.size() == 1) {
                throw errors.get(0);
            } else {
                throw new MultipleErrorTeamBuildException(errors.toArray(new TeamRepositoryException[errors.size()]));
            }
        }
    }

    /**
     * {@inheritDoc} Update enablement of action and cached node.
     */
    public void selectionChanged(IAction action, ISelection selection) {
        if (isQueryActionInProgress()) {
            action.setEnabled(false);
        }
        fSelection = (IStructuredSelection) selection;
    }

    /**
     * Determines if the context of the action is the build query view and if
     * it's in the middle of running a build query.
     * 
     * @return <code>true</code> if the context of the action is the build
     *         query view and if it's in the middle of running a build query.
     */
    protected boolean isQueryActionInProgress() {
        if (fActivePart instanceof BuildQueryView) {
            BuildQueryView buildQueryView = (BuildQueryView) fActivePart;
            return buildQueryView.isRunBuildQueryActionInProgress();
        }
        return false;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fActivePart = targetPart;
    }

    /**
     * Retrieves a list of the currently selected elements.
     * 
     * @return <code>ArrayList</code> of <code>IBuildResultHandle</code>
     *         that are currently selected; may be empty if there is no current
     *         valid selection.
     */
    public IStructuredSelection getSelection() {
        return fSelection;
    }

}