/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.TeamBuildStateException;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJobContext;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Action delegate that performs some action in the repository. It optionally
 * supports confirming the action with the user before doing the action.
 */
public abstract class AbstractRepositoryActionDelegate extends ActionDelegate {

    private Shell fDialogParentShell = null;

    /**
     * Default constructor. Grab the current shell and store it so we can use it
     * as the parent of any dialog boxes created by the actions.
     */
    protected AbstractRepositoryActionDelegate() {
        fDialogParentShell = PlatformUI.getWorkbench().getDisplay().getActiveShell();
    }

    /**
     * Retrieves the message to be displayed to the user in a confirmation
     * dialog. The dialog will have Yes and No buttons. The message should be
     * phrased as a yes or no question.
     * 
     * @return the confirmation message, or <code>null</code> if no
     *         confirmation is desired
     */
    protected abstract String getConfirmationMessage();

    /**
     * Returns the confirmation dialog image type to use.
     *
     * The default type is {@linkMessageDialog.WARNING}.
     *
     * Actions that override this method must return one of the following:
     * <ul>
     * <li><code>MessageDialog.NONE</code> for a dialog with no image</li>
     * <li><code>MessageDialog.ERROR</code> for a dialog with an error image</li>
     * <li><code>MessageDialog.INFORMATION</code> for a dialog with an information image</li>
     * <li><code>MessageDialog.QUESTION </code> for a dialog with a question image</li>
     * <li><code>MessageDialog.WARNING</code> for a dialog with a warning image</li>
     * </ul>   
     *
     * @return the confirmation dialog image type.
     */
    protected int getConfirmationDialogImageType() {
        return MessageDialog.WARNING;
    }

    /**
     * Perform the action. This method is only called if the user has confirmed
     * the action, or if no confirmation was desired. This method is called on a
     * background thread. It is safe to perform long running operations.
     * 
     * @param progressMonitor
     *            the <code>IProgressMonitor</code> passed to the
     *            <code>run</code> method of the job created to perform this
     *            action; never <code>null</code>.
     */
    protected abstract void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * If the child action wants the job to attempt to validate a connection to
     * a repository prior to running the job, it can return the repository from
     * this method.
     * 
     * @return The repository to validate.
     */
    protected ITeamRepository getRepositoryToValidate() {
        return null;
    }

    protected void showWarningDialog(String title, String message) {
        BuildUIHelper.showWarningDialog(getShell(), title, message);
    }

    /**
     * Retrieves the label to be used for the background job in which the action
     * is performed. This label may be displayed to the user. This must not be
     * <code>null</code>.
     * 
     * @return The label to be used for the background job in which the action
     *         is performed.
     */
    protected abstract String getJobLabel();

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void run(IAction action) {
        String confirmationMessage = getConfirmationMessage();
        if (confirmationMessage != null) {
            boolean proceed = BuildUIHelper.showQueryDialog(BuildUIActionMessages.AbstractRepositoryActionDelegate_0,
                    confirmationMessage, getConfirmationDialogImageType());
            if (!proceed) {
                return;
            }
        }

        Job job = createJob();
        job.setUser(true);
        job.schedule();
    }

    /**
     * Retrieves the default shell that should be used in the action.
     * 
     * @return The default shell that should be used in the action.
     */
    protected Shell getShell() {
        return fDialogParentShell;
    }

    /**
     * Creates the background job to perform the action in.
     * 
     * @return The background job to perform the action in.
     */
    protected TeamBuildJob createJob() {
        /*
         * Create a job context that recognizes TeamBuildStateException as an
         * expected error (so we don't log it).
         */
        TeamBuildJobContext context = new TeamBuildJobContext() {
            @Override
            public boolean isExpectedStatus(Job job, IStatus status) {
                if (status.getException() instanceof TeamBuildStateException) {
                    return true;
                }
                return super.isExpectedStatus(job, status);
            }
        };
        return new TeamBuildJob(getJobLabel(), true, context) {
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                performAction(monitor);
                return Status.OK_STATUS;
            }
        };
    }
}
