/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildRequestClient;
import com.ibm.team.build.common.MultipleErrorTeamBuildException;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.internal.common.helper.PropertyHelper;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.build.internal.ui.views.query.BuildQueryView;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Action to cancel a build request; action expects selection to be a
 * <code>BuildItemNode.BuildRequest</code>.
 */
public class CancelBuildRequestActionDelegate extends AbstractRepositoryActionDelegate implements IObjectActionDelegate {

    private IStructuredSelection fSelection;
    private IWorkbenchPart fActivePart;

    /**
     * {@inheritDoc} Display message with requestor name and time request
     * created.
     */
    protected String getConfirmationMessage() {
        if (fSelection.isEmpty()) {
            return null;
        }

        if (fSelection.size() == 1) {
            return BuildUIActionMessages.CancelBuildRequestAction_1;
        } else {
            return NLS.bind(BuildUIActionMessages.CancelBuildRequestAction_0, fSelection.size());
        }
    }

    /**
     * {@inheritDoc}
     */
    protected String getJobLabel() {
        if (fSelection.size() > 1) {
            return BuildUIActionMessages.CancelBuildRequestAction_CANCEL_MULTIPLE_REQUESTS_JOB_TITLE;
        } else {
            return BuildUIActionMessages.CancelBuildRequestAction_CANCEL_SINGLE_REQUEST_JOB_TITLE;
        }
    }

    /**
     * {@inheritDoc} Cancel the requests.
     */
    protected void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        if (fSelection.isEmpty()) {
            return;
        }

        progressMonitor.beginTask(BuildUIActionMessages.CancelBuildRequestAction_CANCEL_SELECTED_BUILD_REQUESTS,
                fSelection.size() * 2);

        List<TeamRepositoryException> errors = null;
        try {
            for (Object object : fSelection.toArray()) {
                IBuildResultRecord record = (IBuildResultRecord) object;
                ITeamRepository repository = (ITeamRepository) record.getBuildResult().getOrigin();
                IBuildResult sharedResult = null;
                try {
                    ITeamBuildRequestClient client = ClientFactory.getTeamBuildRequestClient(repository);

                    String[] properties = PropertyHelper.union(IBuildRequest.PROPERTIES_REQUIRED,
                            new String[] { IBuildRequest.PROPERTY_BUILD_RESULT });
                    IBuildRequest canceledRequest = client.cancelPendingRequest(record.getBuildRequests()[0],
                            properties, new SubProgressMonitor(progressMonitor, 1));

                    // also refresh the build result associated with the request
                    // so it's state change will be reflected in UI
                    IItemManager itemManager = repository.itemManager();
                    sharedResult = (IBuildResult) itemManager.getSharedItemIfKnown(canceledRequest.getBuildResult());
                    if (sharedResult != null) {
                        itemManager.refreshSharedItems(Collections.singletonList(sharedResult), new SubProgressMonitor(
                                progressMonitor, 1));
                    }
                } catch (TeamRepositoryException exception) {
                    if (errors == null) {
                        errors = new ArrayList<TeamRepositoryException>();
                    }
                    errors.add(exception);
                }
            }
        } finally {
            progressMonitor.done();
        }

        if (errors != null) {
            if (errors.size() == 1) {
                throw errors.get(0);
            } else {
                throw new MultipleErrorTeamBuildException(errors.toArray(new TeamRepositoryException[errors.size()]));
            }
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void selectionChanged(IAction action, ISelection selection) {
        if (isQueryActionInProgress()) {
            action.setEnabled(false);
        }
        fSelection = (IStructuredSelection) selection;
    }

    /**
     * Determines if the context of the action is the build query view and if
     * it's in the middle of running a build query.
     * 
     * @return <code>true</code> if the context of the action is the build
     *         query view and if it's in the middle of running a build query.
     */
    protected boolean isQueryActionInProgress() {
        if (fActivePart instanceof BuildQueryView) {
            BuildQueryView buildQueryView = (BuildQueryView) fActivePart;
            return buildQueryView.isRunBuildQueryActionInProgress();
        }
        return false;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fActivePart = targetPart;
    }

    /**
     * Retrieves the currently selected elements.
     * 
     * @return The currently selected elements; may be <code>null</code> if
     *         there is no current valid selection.
     */
    public IStructuredSelection getSelection() {
        return fSelection;
    }
}
