/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.internal.ui.helper.ClipboardHelper;

/**
 * Action delegate that copies the value of an {@link IBuildProperty} to the
 * clipboard. If multiple {@link IBuildProperty} are selected, copies each value
 * on a separate line. This delegate expects the selected elements to be
 * {@link IBuildProperty}.
 */
public class CopyBuildPropertyActionDelegate extends ActionDelegate {

    private List fSelectedElements;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void run(IAction action) {
        copyValuesToClipboard(fSelectedElements);
    }

    /**
     * Copies the values of the {@link IBuildProperty}s in
     * <code>properties</code> to the clipboard.d
     * 
     * @param properties
     *            {@link List} of {@link IBuildProperty} elements, may be empty
     *            but not null.
     */
    public static void copyValuesToClipboard(List<IBuildProperty> properties) {
        ClipboardHelper.getDefault().copyTextToClipboard(getCopyText(properties), Display.getCurrent());
    }

    /**
     * Get the copy text for the {@link IBuildProperty}s in
     * <code>properties</code>.
     * 
     * @param properties
     *            {@link List} of {@link IBuildProperty} elements, may be empty
     *            but not null.
     * @return String containing the copy text.
     */
    public static String getCopyText(List<IBuildProperty> properties) {
        StringBuffer buffer = new StringBuffer();

        if (properties != null && !properties.isEmpty() && isValidCopy(properties)) {
            Iterator<IBuildProperty> iter = properties.iterator();
            while (iter.hasNext()) {
                Object next = iter.next();
                IBuildProperty currentProperty = (IBuildProperty) next;
                buffer.append(currentProperty.getValue());
                if (iter.hasNext()) {
                    buffer.append(System.getProperty("line.separator")); //$NON-NLS-1$
                }
            }
        }

        return new String(buffer);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void selectionChanged(IAction action, ISelection selection) {
        boolean isValueSet = false;
        if (selection instanceof IStructuredSelection) {
            fSelectedElements = ((IStructuredSelection) selection).toList();
            isValueSet = isValidCopy(fSelectedElements);
        }
        action.setEnabled(isValueSet);
    }

    /**
     * Determines whether there is a valid value to copy to the clipboard.
     * 
     * @param propertiesToCopy
     *            List of items to be copied; these should be
     *            {@link IBuildProperty} instances; if not, this method will
     *            return <code>false</code>.
     * @return true if the list contains only {@link IBuildProperty} instances
     *         and there is at least one {@link IBuildProperty} in the
     *         <code>propertiesToCopy</code> that has a non-null and non-empty
     *         value.
     */
    public static boolean isValidCopy(List propertiesToCopy) {
        boolean isValueSet = false;
        if (propertiesToCopy != null && areBuildProperties(propertiesToCopy)) {
            // verify there is at least one value to copy
            Iterator<IBuildProperty> buildProperties = propertiesToCopy.iterator();
            while (!isValueSet && buildProperties.hasNext()) {
                IBuildProperty thisProperty = buildProperties.next();
                if (!(thisProperty.getValue() == null) && !thisProperty.getValue().equals("")) { //$NON-NLS-1$
                    isValueSet = true;
                }
            }
        }
        return isValueSet;

    }

    private static boolean areBuildProperties(List propertiesToCopy) {
        Iterator iterator = propertiesToCopy.iterator();
        while (iterator.hasNext()) {
            if (!(iterator.next() instanceof IBuildProperty)) {
                return false;
            }
        }
        return true;
    }

    /**
     * @return the selectedElements
     */
    protected List<IBuildProperty> getSelectedElements() {
        return fSelectedElements;
    }

}
