/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.MultipleErrorTeamBuildException;
import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;
import com.ibm.team.process.common.advice.TeamOperationCanceledException;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;

/**
 * Action that removes one or more build folders and each of their child
 * folders. Note: Non-folder descendants are uncategorized instead of deleted.
 */
public class DeleteBuildFolderActionDelegate extends AbstractRepositoryActionDelegate implements IObjectActionDelegate {
    private List<IBuildFolderHandle> fFoldersToDelete;

    @Override
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        // Do nothing
    }

    @Override
    protected String getConfirmationMessage() {
        return BuildUIActionMessages.DeleteBuildFolderActionDelegate_CONFIRMATION_MESSAGE;
    }

    @Override
    protected void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        if (fFoldersToDelete != null && fFoldersToDelete.size() > 0) {
            Map<ITeamRepository, List<IBuildFolderHandle>> repoToFolderMap = new HashMap<ITeamRepository, List<IBuildFolderHandle>>();

            for (IBuildFolderHandle folderHandle : fFoldersToDelete) {
                ITeamRepository teamRepository = (ITeamRepository) folderHandle.getOrigin();
                if (!repoToFolderMap.containsKey(teamRepository)) {
                    List<IBuildFolderHandle> list = new ArrayList<IBuildFolderHandle>();
                    list.add(folderHandle);
                    repoToFolderMap.put(teamRepository, list);
                } else {
                    repoToFolderMap.get(teamRepository).add(folderHandle);
                }
            }

            progressMonitor.beginTask(BuildUIActionMessages.DeleteBuildFolderActionDelegate_JOB_TITLE,
                    fFoldersToDelete.size());

            try {
                List<TeamRepositoryException> exceptions = null;
                int failedDeleteCount = 0;
                for (ITeamRepository teamRepository : repoToFolderMap.keySet()) {
                    List<IBuildFolderHandle> repoFoldersToDelete = repoToFolderMap.get(teamRepository);
                    ITeamBuildClient teamBuildClient = ClientFactory.getTeamBuildClient(teamRepository);
                    try {
                        teamBuildClient.delete(
                                repoFoldersToDelete.toArray(new IBuildFolderHandle[repoFoldersToDelete.size()]),
                                new SubProgressMonitor(progressMonitor, repoFoldersToDelete.size()));
                    } catch (TeamRepositoryException e) {
                        // Ignore exceptions canceled by process, these will be
                        // visible in the Team Advisor view.
                        if (!(e instanceof TeamOperationCanceledException)) {
                            if (exceptions == null) {
                                exceptions = new ArrayList<TeamRepositoryException>();
                            }
                            exceptions.add(e);
                            failedDeleteCount += repoFoldersToDelete.size();
                        }
                    }
                }

                if (exceptions != null) {
                    if (exceptions.size() == 1) {
                        throw new TeamBuildException(NLS.bind(
                                BuildUIActionMessages.DeleteBuildFolderActionDelegate_FOLDER_DELETE_FAILED,
                                failedDeleteCount), exceptions.get(0));
                    } else {
                        throw new MultipleErrorTeamBuildException(NLS.bind(
                                BuildUIActionMessages.DeleteBuildFolderActionDelegate_FOLDER_DELETE_FAILED,
                                failedDeleteCount), exceptions.toArray(new Throwable[exceptions.size()]));
                    }
                }
            } finally {
                progressMonitor.done();
            }
        }
    }

    @Override
    protected String getJobLabel() {
        return BuildUIActionMessages.DeleteBuildFolderActionDelegate_JOB_TITLE;
    }

    @Override
    public void selectionChanged(IAction action, ISelection selection) {
        IStructuredSelection structuredSelection = (IStructuredSelection) selection;
        fFoldersToDelete = new ArrayList<IBuildFolderHandle>();

        for (Iterator i = structuredSelection.iterator(); i.hasNext();) {
            IBuildFolderHandle folderHandle = (IBuildFolderHandle) i.next();
            fFoldersToDelete.add(folderHandle);
        }
    }

    @Override
    public void run(IAction action) {
        String confirmationMessage = getConfirmationMessage();
        if (confirmationMessage != null) {
            boolean proceed = BuildUIHelper.showInfoQueryDialog(
                    BuildUIActionMessages.AbstractRepositoryActionDelegate_0, confirmationMessage);
            if (!proceed) {
                return;
            }
        }

        Job job = createJob();
        job.setUser(true);
        job.schedule();
    }

}
