/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.TeamBuildStateException;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;
import com.ibm.team.build.internal.ui.query.BuildQueryRow;
import com.ibm.team.build.internal.ui.views.query.BuildQueryView;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;

/**
 * Remove the selected build results from the repository.
 */
public class DeleteBuildResultActionDelegate extends AbstractRepositoryActionDelegate implements IViewActionDelegate {

    private IStructuredSelection fSelection;
    private BuildQueryView fQueryView;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void init(IViewPart view) {
        fQueryView = (BuildQueryView) view;
    }

    public void selectionChanged(IAction action, ISelection selection) {
        if (isQueryActionInProgress()) {
            action.setEnabled(false);
            return;
        }

        fSelection = (IStructuredSelection) selection;
        updateEnablement(action);
    }

    protected void updateEnablement(IAction action) {
        // disable the action if any of the selected rows represent
        // an in-progress or pending build.
        for (Object object : fSelection.toArray()) {
            BuildQueryRow row = (BuildQueryRow) object;
            if (row.getBuildResult().getState() == BuildState.IN_PROGRESS
                    || row.getBuildResult().getState() == BuildState.NOT_STARTED) {
                action.setEnabled(false);
                return;
            }
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getConfirmationMessage() {
        return BuildUIActionMessages.RemoveBuildResultsActionDelegate_1;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getJobLabel() {
        return BuildUIActionMessages.RemoveBuildResultsActionDelegate_0;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        final List<IBuildResult> notDeleted = new LinkedList<IBuildResult>();

        progressMonitor.beginTask(BuildUIActionMessages.RemoveBuildResultsActionDelegate_0, fSelection.size());

        try {
            for (Object object : fSelection.toArray()) {
                BuildQueryRow row = (BuildQueryRow) object;
                try {
                    if (row.getBuildResult().isDeleteAllowed()) {
                        delete(row.getTeamRepository(), row.getBuildResult(), progressMonitor);
                    } else {
                        notDeleted.add(row.getBuildResult());
                    }
                } catch (ItemNotFoundException exception) {
                    // the build result was already deleted, do nothing
                } catch (TeamBuildStateException exception) {
                    // the build result was set to not allow deletion by another
                    // client after we checked it
                    notDeleted.add(row.getBuildResult());
                }
            }

            handleDeletionNotAllowed(notDeleted);

        } finally {
            progressMonitor.done();
        }

    }

    /**
     * This method is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected void delete(ITeamRepository teamRepository, IBuildResult buildResult, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        ITeamBuildClient teamBuildClient = ClientFactory.getTeamBuildClient(teamRepository);
        teamBuildClient.delete(buildResult, new SubProgressMonitor(progressMonitor, 1));

    }

    /**
     * This method is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used or overridden by
     * subclasses.
     * 
     * @param notDeleted
     *            The list of results that were not deleted.
     */
    protected void handleDeletionNotAllowed(final List<IBuildResult> notDeleted) {
        if (!notDeleted.isEmpty()) {
            String message = NLS.bind(BuildUIActionMessages.DeleteBuildResultActionDelegate_RESULT_DELETE_NOT_ALLOWED,
                    notDeleted.size());
            BuildUIHelper.showInfoDialog(getShell(), BuildUIActionMessages.RemoveBuildResultsActionDelegate_0, message);
        }
    }

    /**
     * Determines if the context of the action is the build query view and if
     * it's in the middle of running a build query.
     * 
     * @return <code>true</code> if the context of the action is the build
     *         query view and if it's in the middle of running a build query.
     */
    protected boolean isQueryActionInProgress() {
        return fQueryView.isRunBuildQueryActionInProgress();
    }
}
