/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.IBuildUIConstants;
import com.ibm.team.build.internal.ui.editors.builddefinition.BuildDefinitionEditorInput;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Opens a build definition editor.
 */
public class EditBuildDefinitionActionDelegate extends ActionDelegate implements IObjectActionDelegate {

    private IStructuredSelection fSelection;
    private IWorkbenchPart fTargetPart;

    public void selectionChanged(IAction action, ISelection selection) {
        if (selection instanceof IStructuredSelection) {
            fSelection = (IStructuredSelection) selection;
        }
    }

    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fTargetPart = targetPart;
    }

    public void run(IAction action) {
        if (fSelection.getFirstElement() instanceof IBuildDefinition) {
            IBuildDefinition buildDefinition = (IBuildDefinition) fSelection.getFirstElement();
            run(buildDefinition, fTargetPart.getSite().getPage());
        }
    }

    /**
     * Open the build definition editor on the given build definition.
     * 
     * @param buildDefinition
     *            The build definition to edit.
     * @param workbenchPage
     *            The workbench page in which to open the editor.
     */
    public static void run(final IBuildDefinition buildDefinition, final IWorkbenchPage workbenchPage) {
        run(buildDefinition, (String) null, workbenchPage);
    }

    /**
     * Open the build definition editor for the given build definition. The
     * given initial page is activated.
     * 
     * @param buildDefinition
     *            The build definition to edit.
     * @param initialPageId
     *            The id of the initial page. If the given page id isn't found
     *            in the editor, then the editor opens with the first page
     *            showing. May be <code>null</code>.
     * @param workbenchPage
     *            The workbench page in which to open the editor.
     */
    public static void run(final IBuildDefinition buildDefinition,
            final String initialPageId, final IWorkbenchPage workbenchPage) {
        String jobName = BuildUIActionMessages.EditBuildDefinitionActionDelegate_FETCHING_TEAM_AREA;
        Job job = new TeamBuildJob(jobName, true) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();
                IProcessAreaHandle processAreaHandle = buildDefinition.getProcessArea();

                final IProcessArea processArea = (IProcessArea) teamRepository.itemManager().fetchCompleteItem(processAreaHandle,
                        IItemManager.DEFAULT, monitor);

                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        handleAsyncExec(workbenchPage, buildDefinition, initialPageId, processArea);
                    }
                });
                return Status.OK_STATUS;
            }
        };
        job.schedule();
    }

    private static void handleAsyncExec(IWorkbenchPage workbenchPage, IBuildDefinition buildDefinition,
            String initialPageId, IProcessArea processArea) {
        try {
            IWorkbenchPart activePart = workbenchPage.getActivePart();
            if (activePart != null) {
                IWorkbenchPartSite site = activePart.getSite();
                if (site != null && !site.getShell().isDisposed()) {
                    BuildDefinitionEditorInput input = new BuildDefinitionEditorInput(buildDefinition, processArea,
                            (ITeamRepository) buildDefinition.getOrigin(), false);

                    if (initialPageId != null) {
                        input.setInitialPageId(initialPageId);
                    }

                    workbenchPage.openEditor(input, IBuildUIConstants.ID_BUILD_DEFINITION_EDITOR);
                }
            }
        } catch (PartInitException exception) {
            BuildUIPlugin.log(exception);
        }
    }

}
