/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.dialogs.BuildFolderSelectionDialog;
import com.ibm.team.build.internal.ui.domain.IActionIds;
import com.ibm.team.build.internal.ui.domain.Messages;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IProjectArea;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.ITeamAreaHandle;
import com.ibm.team.process.rcp.ui.teamnavigator.ConnectedProjectAreaRegistry;
import com.ibm.team.repository.common.IItemHandle;

/**
 * Action delegate that opens a dialog to move the specified build items to the
 * specified folder.
 */
public class MoveBuildItemsAction extends Action {

    private final IPreferenceStore fPreferenceStore = BuildUIPlugin.getDefault().getPreferenceStore();
    
    private final Shell fParentShell;
    
    private final IItemHandle[] fItemHandles;
    
    private final IProcessAreaHandle fProcessAreaHandle;
    
    /**
     * Create an action for moving the specified items to the specified folder
     * using the move dialog. It is not intended to be used or overridden by
     * subclasses.
     * 
     * @param parentShell
     *            The parent shell for the dialog; may not be <code>null</code>.
     * @param itemHandles
     *            The item(s) to move.
     * @param processAreaHandle
     *            The project area to limit the scope of folder selection to
     *            within the dialog. If a <code>TeamAreaHandle</code> is
     *            provided and its parent project area is connected, then the
     *            project area will be used.
     */
    public MoveBuildItemsAction(final Shell parentShell, final IItemHandle[] itemHandles,
            final IProcessAreaHandle processAreaHandle) {
        super();
        fParentShell = parentShell;
        fItemHandles = itemHandles;
        fProcessAreaHandle = processAreaHandle;

    }
    
    @Override
    public void run() {
        if (!fParentShell.isDisposed()) {
            boolean showFolders = fPreferenceStore.getBoolean(BuildUIPlugin.PREF_SHOW_BUILD_FOLDERS);

            if (!showFolders) {
                boolean response = BuildUIHelper.showInfoQueryDialog(
                        fParentShell,
                        BuildUIActionMessages.BuildFolderActionDelegate_ENABLE_FOLDERS_TITLE,
                        BuildUIActionMessages.BuildFolderActionDelegate_ENABLE_FOLDERS_MOVE_CONFIRMATION_MESSAGE);

                if (response) {
                    fPreferenceStore.setValue(BuildUIPlugin.PREF_SHOW_BUILD_FOLDERS, true);
                } else {
                    return;
                }
            }

            List<IProcessAreaHandle> processAreas = null;
            if (fProcessAreaHandle instanceof IProjectAreaHandle) {
                processAreas = ConnectedProjectAreaRegistry.getDefault().getSelectedProcessAreas(
                        (IProjectAreaHandle) fProcessAreaHandle);
            } else if (fProcessAreaHandle instanceof ITeamAreaHandle) {
                IProjectArea projectArea = ConnectedProjectAreaRegistry.getDefault().getConnectedProjectArea(
                        (ITeamAreaHandle) fProcessAreaHandle);
                if (projectArea != null) {
                    processAreas = ConnectedProjectAreaRegistry.getDefault().getSelectedProcessAreas(
                            projectArea);
                }
            }
            if (processAreas != null) {
                BuildFolderSelectionDialog.open(fParentShell, fItemHandles,
                        processAreas.toArray(new IProcessAreaHandle[processAreas.size()]));
            }

        }
    }
    
    @Override
    public String getId() {
        return IActionIds.MOVE_BUILD_ITEM_TO_FOLDER;
    }

    @Override
    public String getText() {
        return Messages.BuildDomainActionHelper_MOVE_BUILD_ITEMS_JOB_LABEL;
    }

}
