/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.internal.ui.wizards.builddefinition.NewBuildDefinitionWizard;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Action delegate that opens the New Build Definition wizard.
 */
public class NewBuildDefinitionActionDelegate extends ActionDelegate implements IObjectActionDelegate {
    /**
     * The wizard dialog width
     */
    private static final int WIZARD_WIDTH = 500;

    /**
     * The wizard dialog height
     */
    private static final int WIZARD_HEIGHT = 500;

    /**
     * This field is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used by subclasses.
     */
    protected ITeamRepository fTeamRepository;

    /**
     * This field is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used by subclasses.
     */
    protected IWorkbenchPart fWorkbenchPart;

    private IProjectAreaHandle fProjectAreaHandle;
    
    private IBuildFolderHandle fParentFolderHandle;

    /**
     * Get an action for opening the New Team Build wizard. The build definition
     * will be uncategorized within the project area.
     * 
     * @param projectAreaHandle
     *            The project area to use as context when selecting a team area.
     *            If null, the user must choose among all project areas when
     *            selecting a team area.
     * @param parent
     *            The parent context for the dialog.
     * @return An action for opening the New Team Build wizard.
     */
    public Action getAction(final IProjectAreaHandle projectAreaHandle, final Shell parent) {
        return getNewBuildDefinitionAction(projectAreaHandle, null, parent);
    }
    
    /**
     * Get an action for opening the New Team Build wizard. The build definition
     * will be categorized under the direct ancestor as specified by the first
     * entry in <code>ancestorFolderPairs</code>.
     * 
     * @param projectAreaHandle
     *            The project area to use as context when selecting a team area.
     *            Selections will be restricted to the project area of the
     *            folder.
     * @param parentFolderHandle
     *            The folder to associate the new build definition with. If
     *            <code>null</code>, then the build definition will be
     *            uncategorized.
     * @param parent
     *            The parent context for the dialog.
     * @return An action for opening the New Team Build wizard.
     */
    public Action getAction(final IProjectAreaHandle projectAreaHandle, IBuildFolderHandle parentFolderHandle,
            final Shell parent) {
        return getNewBuildDefinitionAction(projectAreaHandle, parentFolderHandle, parent);
    }
    
    private Action getNewBuildDefinitionAction(final IProjectAreaHandle projectAreaHandle, final IBuildFolderHandle parentFolderHandle, final Shell parent) {
        return new Action() {
            @Override
            public void run() {
                ITeamRepository teamRepository = null;
                if (projectAreaHandle != null) {
                    teamRepository = (ITeamRepository) projectAreaHandle.getOrigin();
                }
                NewBuildDefinitionActionDelegate.this.run(projectAreaHandle, teamRepository, parentFolderHandle, parent);
            }

            @Override
            public String getText() {
                return BuildUIActionMessages.NewTeamBuildActionDelegate_LABEL_NEW_BUILD_DEFINITION;
            }

            @Override
            public String getId() {
                return BuildUIActionMessages.NewTeamBuildActionDelegate_ACTION_ID;
            }
        };
    }

    private void run(IProjectAreaHandle projectAreaHandle, ITeamRepository teamRepository,
            IBuildFolderHandle parentFolderHandle, Shell parent) {
    NewBuildDefinitionWizard wizard = null;
        if (projectAreaHandle != null) {
            wizard = new NewBuildDefinitionWizard(projectAreaHandle, parentFolderHandle);
        } else if (teamRepository != null) {
            wizard = new NewBuildDefinitionWizard(teamRepository);
        } else {
            wizard = new NewBuildDefinitionWizard();
        }

        WizardDialog dialog = getDialog(parent, wizard);
        dialog.create();
        dialog.getShell().pack();
        dialog.getShell().setSize(Math.max(WIZARD_WIDTH, dialog.getShell().getSize().x), WIZARD_HEIGHT);
        dialog.open();
    }

    public void run(IAction action) {
        IWorkbenchPage page = fWorkbenchPart.getSite().getPage();
        if (page != null) {
            run(fProjectAreaHandle, fTeamRepository, fParentFolderHandle, page.getWorkbenchWindow().getShell());
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fWorkbenchPart = targetPart;
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected WizardDialog getDialog(Shell parent, IWizard wizard) {
        return new WizardDialog(parent, wizard);
    }

}
