/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.dialogs.BuildFolderPropertiesDialog;
import com.ibm.team.build.internal.ui.domain.IActionIds;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Action delegate that opens a dialog to create a new build folder with either
 * a parent folder context or a default process area context.
 */
public class NewBuildFolderActionDelegate extends ActionDelegate implements IObjectActionDelegate {

    private final IPreferenceStore fPreferenceStore = BuildUIPlugin.getDefault().getPreferenceStore();
    
    /**
     * Get an action for opening the build folder properties dialog with a new
     * root-level build folder with the specified default process area.
     * 
     * @param processAreaHandle
     *            The process area to use as context when selecting a team area
     *            within the dialog; never <code>null</code>. Selections will be
     *            restricted to the project area of this process area.
     * @param parentShell
     *            The parent context for the dialog; never <code>null</code>.
     * @return An action for opening the build folder editor in the context of a
     *         new build folder.
     */
    public Action getAction(final IProcessAreaHandle processAreaHandle, final Shell parentShell) {
        ValidationHelper.validateNotNull("processAreaHandle", processAreaHandle); //$NON-NLS-1$
        return getAction(processAreaHandle, null, parentShell);
    }

    /**
     * Get an action for opening the build folder properties dialog with a new
     * build folder that has the specified parent folder.
     * 
     * @param parentFolderHandle
     *            The parent folder for the new folder; never <code>null</code>.
     * @param parentShell
     *            The parent context for the dialog; never <code>null</code>.
     * @return An action for opening the build folder editor in the context of a
     *         new build folder.
     */
    public Action getAction(final IBuildFolderHandle parentFolderHandle, final Shell parentShell) {
        ValidationHelper.validateNotNull("parentFolderHandle", parentFolderHandle); //$NON-NLS-1$
        return getAction(null, parentFolderHandle, parentShell);
    }

    private Action getAction(final IProcessAreaHandle processAreaHandle, final IBuildFolderHandle parentFolderHandle,
            final Shell parentShell) {
        return new Action() {
            @Override
            public void run() {
                ITeamRepository teamRepository = null;
                if (processAreaHandle != null) {
                    teamRepository = (ITeamRepository) processAreaHandle.getOrigin();
                } else if (parentFolderHandle != null) {
                    teamRepository = (ITeamRepository) parentFolderHandle.getOrigin();
                }
                NewBuildFolderActionDelegate.this.run(processAreaHandle, parentFolderHandle, teamRepository,
                        parentShell);
            }

            @Override
            public String getText() {
                return BuildUIActionMessages.NewBuildFolderActionDelegate_LABEL_NEW_BUILD_FOLDER;
            }

            @Override
            public String getId() {
                return IActionIds.NEW_BUILD_FOLDER;
            }
        };
    }

    /**
     * Run the action to open a build folder properties dialog with a new folder
     * context using the default process area and specified parent folder
     */
    private void run(final IProcessAreaHandle processAreaHandle, final IBuildFolderHandle parentFolderHandle,
            final ITeamRepository teamRepository, final Shell parentShell) {
        TeamBuildJob newFolderJob = new TeamBuildJob(
                BuildUIActionMessages.NewBuildFolderActionDelegate_LABEL_NEW_BUILD_FOLDER, true, teamRepository) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws TeamRepositoryException {
                try {
                    boolean showFolders = fPreferenceStore.getBoolean(BuildUIPlugin.PREF_SHOW_BUILD_FOLDERS);
                    
                    if (!showFolders) {
                        boolean response = BuildUIHelper.showInfoQueryDialogSync(parentShell,
                                BuildUIActionMessages.BuildFolderActionDelegate_ENABLE_FOLDERS_TITLE,
                                BuildUIActionMessages.BuildFolderActionDelegate_ENABLE_FOLDERS_NEW_FOLDER_CONFIRMATION_MESSAGE);
                        
                        if (response) {
                            fPreferenceStore.setValue(BuildUIPlugin.PREF_SHOW_BUILD_FOLDERS, true);
                        } else {
                            return Status.OK_STATUS;
                        }
                    }
                    
                    PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                        @Override
                        public void run() {
                            if (!parentShell.isDisposed()) {
                                if (parentFolderHandle != null) {
                                    BuildFolderPropertiesDialog.openNewFolderDialog(parentShell, parentFolderHandle);
                                } else {
                                    BuildFolderPropertiesDialog.openNewFolderDialog(parentShell, processAreaHandle);
                                }
                            }
                        }
                    });

                    return Status.OK_STATUS;
                } finally {
                    monitor.done();
                }
            }
        };
        newFolderJob.schedule();
    }

    @Override
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        // Do nothing. We don't need to keep track of the active part
    }
}
