/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.IShowInTarget;
import org.eclipse.ui.part.ShowInContext;

import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.internal.ui.feed.BuildFeedConfigurer;
import com.ibm.team.feed.core.ChannelEvent;
import com.ibm.team.feed.core.FeedManager;
import com.ibm.team.feed.core.INewsManagerListener;
import com.ibm.team.feed.core.model.Channel;
import com.ibm.team.foundation.common.util.Adapters;

/**
 * Action to subscribe to a build feed and select the feed in the Feeds domain
 * in Team Artifacts.
 */
public class SubscribeToFeedAction extends Action implements INewsManagerListener {
    /**
     * Action ID.
     */
    public static final String ACTION_ID = "build.subscribe.to.feed"; //$NON-NLS-1$

    /**
     * Team Artifacts view ID.
     */
    public static final String TEAM_ARTIFACTS_ID = "com.ibm.team.process.rcp.ui.teamArtifactsNavigator"; //$NON-NLS-1$

    private String fBuildDefinitionId;
    private IBuildDefinitionHandle fBuildDefinitionHandle;
    private Channel fChannel;

    /**
     * Create an action that subscribes to a build feed for the given
     * definition.
     * 
     * @param buildDefinitionId
     *            The build definition id.
     * @param buildDefinitionHandle
     *            The build definition handle.
     */
    public SubscribeToFeedAction(String buildDefinitionId, IBuildDefinitionHandle buildDefinitionHandle) {
        super(BuildUIActionMessages.SubscribeToFeedAction_SUBSCRIBE_TO_FEED);
        fBuildDefinitionId = buildDefinitionId;
        fBuildDefinitionHandle = buildDefinitionHandle;
    }

    @Override
    public void run() {
        FeedManager feedManager = FeedManager.getDefault();
        feedManager.addNewsManagerListener(this);

        BuildFeedConfigurer configurer = new BuildFeedConfigurer();
        fChannel = configurer.addDefinitionChannel(fBuildDefinitionId, fBuildDefinitionHandle, feedManager);

        // Explicitly call handleChannelAdded here for the case where the
        // channel already existed. In this case, our listener will not get
        // called. But, we'd still like to select the channel in Team Artifacts.
        handleChannelAdded(fChannel);
    }

    @Override
    public String getId() {
        return ACTION_ID;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void channelAdded(final ChannelEvent event) {
        if (event.getChannel().equals(fChannel)) {
            Display.getDefault().asyncExec(new Runnable() {
                public void run() {
                    handleChannelAdded(event.getChannel());
                }
            });
        }
    }

    protected void handleChannelAdded(Channel channel) {
        // Select the channel in the Feeds domain.
        try {
            IViewPart view = showTeamArtifactsView();
            if (view != null) {
                IShowInTarget showInTarget = Adapters.getAdapter(view, IShowInTarget.class);
                if (showInTarget != null) {
                    showInTarget.show(new ShowInContext(null, new TreeSelection(new TreePath(new Object[] {
                            FeedManager.getDefault(), fChannel }))));
                }
            }

        } catch (PartInitException exception) {
            Dialog dialog = getErrorDialog(BuildUIActionMessages.SubscribeToFeedAction_ERROR_ACTIVATING_VIEW,
                    BuildUIActionMessages.SubscribeToFeedAction_ERROR_ACTIVATING_VIEW_MESSAGE);
            dialog.open();
        }

        FeedManager feedManager = FeedManager.getDefault();
        feedManager.removeNewsManagerListener(this);
    }

    protected MessageDialog getErrorDialog(String title, String message) {
        return new MessageDialog(getShell(), title, null, message, MessageDialog.ERROR,
                new String[] { IDialogConstants.OK_LABEL }, 0);
    }

    private Shell getShell() {
        Shell shell = null;
        IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        if (window != null) {
            shell = window.getShell();
        }
        return shell;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void channelRemoved(ChannelEvent event) {
        // Do nothing.
    }

    protected IViewPart showTeamArtifactsView() throws PartInitException {
        IViewPart view = null;
        IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        if (window != null) {
            IWorkbenchPage page = window.getActivePage();
            page.showView(TEAM_ARTIFACTS_ID);
            view = page.findView(TEAM_ARTIFACTS_ID);
        }
        return view;
    }

}
