/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.adapters;

import org.eclipse.ui.IActionFilter;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.internal.common.model.BuildDefinition;
import com.ibm.team.build.internal.common.model.dto.IBuildDefinitionStatusRecord;
import com.ibm.team.build.internal.ui.domain.BuildDefinitionQueryNode;

/**
 * An action filter useful for controlling enablement and visibility of plugin
 * action that are part of {@link BuildDefinitionQueryNode} object contributions. 
 * The class allows filtering based on the Properties and {@link IBuildProperty} 
 * of the {@link IBuildDefinition} of this node.
 * <p>
 * Participating actions can specify <code>enablement</code> or
 * <code>visibility</code>
 * 
 * </p>
 *  
 * <p>
 * Supports attributes {@link #TEMPLATE_ID_KEY} and
 * {@link #HAS_CONFIGURATION_ELEMENT_KEY} for checking whether the definition that reflects this node in the UI
 *  was created from a given template, or has a given configuration element.
 * </p>
 * 
 * The <code>objectState</code> elements only work on objects that implement the
 * {@link IActionFilter} interface. Since {@link BuildDefinitionQueryNode} does not
 * implement {@link IActionFilter} itself, it is necessary to register an
 * adapter factory that is capable of adapting {@link BuildDefinitionQueryNode} to
 * {@link IActionFilter}. This is accomplished by registering
 * {@link BuildDefinitionQueryNodeActionFilter} in plugin.xml under
 * <code>org.eclipse.core.runtime.adapters</code>.
 */
public class BuildDefinitionQueryNodeActionFilter implements IActionFilter {

    /**
     * Attribute name for checking the definition's template id.
     */
    public final static String TEMPLATE_ID_KEY = "templateId"; //$NON-NLS-1$

    /**
     * Attribute name for checking whether the definition has a given
     * configuration element.
     */
    public final static String HAS_CONFIGURATION_ELEMENT_KEY = "hasConfigurationElement"; //$NON-NLS-1$

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean testAttribute(Object target, String name, String value) {
        IBuildDefinitionStatusRecord record = (IBuildDefinitionStatusRecord) target;
        
        if (name.equals(TEMPLATE_ID_KEY) && record.getBuildDefinition() != null) {
            String templateId = record.getBuildDefinition().getPropertyValue(BuildDefinition.INTERNAL_PROPERTY_TEMPLATE_ID, null);
            return templateId != null && templateId.equals(value);
        }
        if (name.equals(HAS_CONFIGURATION_ELEMENT_KEY) && record.getBuildDefinition() != null) {
            return record.getBuildDefinition().getConfigurationElement(value) != null;
        }

        return false;
    }

}
