/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.adapters;

import org.eclipse.ui.IActionFilter;

import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.internal.common.model.BuildDefinition;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;

/**
 * An action filter useful for controlling enablement and visibility of plugin
 * action that are part of {@link IBuildResultRecord} object contributions. The
 * class allows filtering based on the state of the build results in the
 * selected build result records.
 * <p>
 * Participating actions can specify <code>enablement</code> or
 * <code>visibility</code> elements with <code>objectState</code> children that
 * specify {@link #BUILD_STATE_ENABLEMENT_KEY} as the name of the state, and any
 * appropriate {@link BuildState} as the value. For example, the following
 * plugin.xml snippet enables the abandon build action only for IN_PROGRESS
 * builds.
 * 
 * <pre>
 *    &lt;objectContribution
 *          adaptable=&quot;true&quot;
 *          id=&quot;com.ibm.team.build.internal.common.model.dto.IBuildResultRecord&quot;
 *          objectClass=&quot;com.ibm.team.build.internal.common.model.dto.IBuildResultRecord&quot;&gt;
 *       &lt;action
 *             class=&quot;com.ibm.team.build.internal.ui.actions.AbandonBuildActionDelegate&quot;
 *             enablesFor=&quot;+&quot;
 *             id=&quot;com.ibm.team.build.ui.abandonBuildAction&quot;
 *             label=&quot;%AbandonBuildAction&quot;
 *             menubarPath=&quot;buildAdminActions&quot;&gt;
 *          &lt;enablement&gt;
 *             &lt;objectState
 *                   name=&quot;buildState&quot;
 *                   value=&quot;IN_PROGRESS&quot;&gt;
 *             &lt;/objectState&gt;
 *          &lt;/enablement&gt;
 *       &lt;/action&gt;
 *       ...
 * </pre>
 * 
 * </p>
 * 
 * <p>
 * Similarly to specifying {@link #BUILD_STATE_ENABLEMENT_KEY}, you can specify 
 * {@link #BUILD_STATUS_ENABLEMENT_KEY} as the name of the state and any appropriate 
 * {@link BuildStatus} as the value to test the build result status.
 * </p>
 * 
 * <p>
 * Also supports attributes {@link #TEMPLATE_ID_KEY} and
 * {@link #HAS_CONFIGURATION_ELEMENT_KEY} for checking whether the definition was
 * created from a given template, or has a given configuration element.
 * </p>
 * 
 * The <code>objectState</code> elements only work on objects that implement the
 * {@link IActionFilter} interface. Since {@link IBuildResultRecord} does not
 * implement {@link IActionFilter} itself, it is necessary to register an
 * adapter factory that is capable of adapting {@link IBuildResultRecord} to
 * {@link IActionFilter}. This is accomplished by registering
 * {@link BuildResultRecordAdapterFactory} in plugin.xml under
 * <code>org.eclipse.core.runtime.adapters</code>.
 */
public class BuildResultRecordActionFilter implements IActionFilter {

    /**
     * Attribute name for checking the build state.
     */
    public final static String BUILD_STATE_ENABLEMENT_KEY = "buildState"; //$NON-NLS-1$
    
    /**
     * Attribute name for checking the build status.
     */
    public final static String BUILD_STATUS_ENABLEMENT_KEY = "buildStatus"; //$NON-NLS-1$

    /**
     * Attribute name for checking the definition's template id.
     */
    public final static String TEMPLATE_ID_KEY = "templateId"; //$NON-NLS-1$

    /**
     * Attribute name for checking whether the definition has a given
     * configuration element.
     */
    public final static String HAS_CONFIGURATION_ELEMENT_KEY = "hasConfigurationElement"; //$NON-NLS-1$

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean testAttribute(Object target, String name, String value) {
        IBuildResultRecord record = (IBuildResultRecord) target;

        if (name.equals(BUILD_STATE_ENABLEMENT_KEY) && record.getBuildResult() != null) {
            return record.getBuildResult().getState().toString().equalsIgnoreCase(value);
        }
        if (name.equals(BUILD_STATUS_ENABLEMENT_KEY) && record.getBuildResult() != null) {
            return record.getBuildResult().getStatus().toString().equalsIgnoreCase(value);
        }
        if (name.equals(TEMPLATE_ID_KEY) && record.getBuildDefinition() != null) {
            String templateId = record.getBuildDefinition().getPropertyValue(BuildDefinition.INTERNAL_PROPERTY_TEMPLATE_ID, null);
            return templateId != null && templateId.equals(value);
        }
        if (name.equals(HAS_CONFIGURATION_ELEMENT_KEY) && record.getBuildDefinition() != null) {
            return record.getBuildDefinition().getConfigurationElement(value) != null;
        }

        return false;
    }

}
