/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.helper.BuildUIHelper;

/**
 * Abstract dialog for changing a collection of items.
 */
public abstract class AbstractEditItemsDialog extends Dialog {
    protected Set<String> fCheckedItems = new HashSet<String>();
    protected TableViewer fTableViewer;
    protected Button fShowAllItemsButton, fShowSubSetItemsButton, fSelectAllItemsButton, fDeselectAllItemsButton;
    protected Label fDescriptionLabel;
    private final GridData GRAB_TRUE_GRID_DATA = GridDataFactory.fillDefaults().grab(true, true).create();
    protected Text fDescriptionText;
    private boolean fSingleSelect;

    /**
     * Create a dialog for editing selection of items within a larger group. The
     * dialog consists of a description label, two radio buttons to decide what
     * to display in the checkbox table, and a checkbox table with quick select
     * push buttons. The table will display either the items stored in
     * <code>getAllItems()</code> or <code>getSubSetItems()</code> depending on
     * the currently selected radio button. The items that are initially checked
     * in the table are taken from <code>getInitialItems()</code>. This could
     * easily be augmented to be two separate groups of items instead of a full
     * list and a subset.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>.
     */
    public AbstractEditItemsDialog(Shell parentShell) {
        super(parentShell);
        ValidationHelper.validateNotNull("parentShell", parentShell); //$NON-NLS-1$
    }
    
    /**
     * Sets whether the dialog works in single select mode.
     * By default the dialog allows multiple items to be selected.
     * 
     * @param singleSelect <code>true</code> for single selection, <code>false</code> for multi selection
     */
    public void setSingleSelect(boolean singleSelect) {
        fSingleSelect = singleSelect;
    }

    @Override
    protected final Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);
        GridLayoutFactory.swtDefaults().applyTo(composite);
        GridDataFactory.createFrom(GRAB_TRUE_GRID_DATA).applyTo(composite);

        fDescriptionLabel = createLabel(composite, getDialogDescription(), (Color) null);

        // first row
        createRadioButtonBar(composite);

        // second row
        createTableRow(composite);

        // third row
        createDescriptionRow(composite);

        updateTableItems(getInitialItems());

        return composite;
    }

    /**
     * Gets the items whose status has changed since the initialization of the
     * dialog. Not intended to be overridden by subclasses, set as protected for
     * testing purposes.
     * 
     * @return the set of all items whose isChecked() status is different than
     *         after the initialization of the dialog.
     */
    protected Set<String> getChangedItems() {
        Set<String> changedItems = new HashSet<String>();
        changedItems.addAll(fCheckedItems);
        for (String item : getInitialItems()) {
            if (changedItems.contains(item)) {
                changedItems.remove(item);
            } else {
                changedItems.add(item);
            }
        }
        return changedItems;
    }

    /**
     * Sets and returns the collection of current build definition id's to be
     * displayed in the table
     */
    private final void updateTableItems(Object[] oldChecked) {
        setCheckedItems(oldChecked);
        if (fShowAllItemsButton.getSelection()) {
            fTableViewer.setLabelProvider(getLabelProvider(false));
            Set<String> currentItems = new HashSet<String>();
            for (String item : getAllItems()) {
                currentItems.add(item);
            }
            fTableViewer.setInput(currentItems);
        } else {
            fTableViewer.setLabelProvider(getLabelProvider(true));
            fTableViewer.setInput(getSubSetItems());
        }
        if (fSingleSelect) {
            fTableViewer.setSelection(new StructuredSelection(fCheckedItems.toArray()));
            fCheckedItems.removeAll(((IStructuredSelection) fTableViewer.getSelection()).toList());
        } else {
            Iterator<String> itr = fCheckedItems.iterator();
            while (itr.hasNext()) {
                Object checked = itr.next();
                if (((CheckboxTableViewer) fTableViewer).setChecked(checked, true)) {
                    // stores items that cannot be checked because they are not
                    // displayed in the table, in the fCheckedItems set
                    itr.remove();
                }
            }
        }
    }

    
    private void createRadioButtonBar(Composite parent) {
        Composite composite = new Composite(parent, SWT.FILL);
        int defaultMargin = IDialogConstants.HORIZONTAL_SPACING;
        GridLayoutFactory.fillDefaults().numColumns(2).margins(defaultMargin, defaultMargin).equalWidth(true).applyTo(
                composite);
        GridDataFactory.fillDefaults().applyTo(composite);

        fShowSubSetItemsButton = createRadioButton(composite, getShowSubSetLabel(), getShowListener(), isStartInSubSet());
        fShowAllItemsButton = createRadioButton(composite, getShowAllLabel(), getShowListener(), !isStartInSubSet());
    }

    private Button createRadioButton(Composite parent, String text, SelectionAdapter adapter, boolean isSelected) {
        Button newButton = new Button(parent, SWT.RADIO);
        newButton.setText(text);
        newButton.addSelectionListener(adapter);
        newButton.setSelection(isSelected);
        GridDataFactory.fillDefaults().grab(true, false).align(SWT.BEGINNING, SWT.FILL).applyTo(newButton);
        return newButton;
    }

    /**
     * Not intended to be overridden by subclasses, set as protected for testing
     * purposes.
     */
    protected SelectionAdapter getShowListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (((Button) e.widget).getSelection()) {
                    updateTableItems(getSelectedElements());
                }
            }
        };
    }

    /**
     * Returns the items selected/checked in the table. 
     */
    private Object[] getSelectedElements() {
        if (fSingleSelect) {
            return ((IStructuredSelection) fTableViewer.getSelection()).toArray();
        } else {
            return ((CheckboxTableViewer) fTableViewer).getCheckedElements();
        }
    }

    /**
     * Not intended to be overridden by subclasses, set as protected for testing
     * purposes.
     */
    protected final void setCheckedItems(Object[] newChecked) {
        for (Object o :  newChecked) {
            fCheckedItems.add((String) o);
        }
    }

    private final void createTableRow(Composite parent) {
        Composite composite = new Composite(parent, SWT.FILL);
        GridDataFactory.createFrom(GRAB_TRUE_GRID_DATA).hint(parent.getSize().x, SWT.DEFAULT).applyTo(composite);
        int numColumns = fSingleSelect ? 1 : 2;
        GridLayoutFactory.swtDefaults().numColumns(numColumns).spacing(IDialogConstants.HORIZONTAL_MARGIN, 0).margins(
                IDialogConstants.HORIZONTAL_SPACING, 0).applyTo(composite);

        if (fSingleSelect) {
            fTableViewer = new TableViewer(composite, SWT.BORDER);
        } else {
            fTableViewer = CheckboxTableViewer.newCheckList(composite, SWT.BORDER);
        }
        if (!fSingleSelect) {
            createTableButtonBar(composite);
        }

        fTableViewer.setComparator(getTableViewerComparator());
        fTableViewer.setContentProvider(getContentProvider());
        fTableViewer.addSelectionChangedListener(getTableSelectionListener());
        fTableViewer.addOpenListener(getTableOpenListener());

        int hint = IDialogConstants.ENTRY_FIELD_WIDTH;
        GridDataFactory.createFrom(GRAB_TRUE_GRID_DATA).hint(hint, hint).applyTo(fTableViewer.getTable());
    }

    /**
     * Returns the selection changed listener for the table.
     */
    private ISelectionChangedListener getTableSelectionListener() {
        return new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                updateDescription((IStructuredSelection) event.getSelection());
            }
        };
    }

    /**
     * Returns the open changed listener for the table.
     */
    private IOpenListener getTableOpenListener() {
        return new IOpenListener() {
            public void open(OpenEvent event) {
                if (fSingleSelect) {
                    okPressed();
                }
            }
        };
    }

    /**
     * Updates the description text for the given selection.
     */
    protected void updateDescription(IStructuredSelection selection) {
        String description = getDescription(selection);
        if (description == null || description.length() == 0) {
            description = BuildUIDialogMessages.AbstractEditItemsDialog_NO_DESCRIPTION;
        }
        fDescriptionText.setText(description);
    }

    /**
     * Returns the description to show for the given selection.
     * 
     * @return the description or <code>null</code> if none
     */
    protected abstract String getDescription(IStructuredSelection selection);

    private final void createTableButtonBar(Composite parent) {
        Composite composite = new Composite(parent, SWT.FILL);
        GridDataFactory.fillDefaults().grab(false, true).applyTo(composite);
        GridLayoutFactory.fillDefaults().applyTo(composite);

        fSelectAllItemsButton = createPushButton(composite,
                BuildUIDialogMessages.AbstractEditItemsDialog_SELECT_ALL_LABEL, true, getSelectAllListener());
        fDeselectAllItemsButton = createPushButton(composite,
                BuildUIDialogMessages.AbstractEditItemsDialog_DESELECT_ALL_LABEL, true, getDeselectAllListener());

        GridDataFactory.fillDefaults().hint(BuildUIHelper.getPreferredButtonWidth(fSelectAllItemsButton),
                SWT.DEFAULT).grab(true, false).align(SWT.END, SWT.CENTER).applyTo(fSelectAllItemsButton);
        GridDataFactory.fillDefaults().hint(BuildUIHelper.getPreferredButtonWidth(fDeselectAllItemsButton),
                SWT.DEFAULT).grab(true, false).align(SWT.END, SWT.CENTER).applyTo(fDeselectAllItemsButton);
    }

    private Button createPushButton(Composite composite, String text, boolean isEnabled, SelectionListener listener) {
        Button newButton = new Button(composite, SWT.PUSH);
        newButton.setText(text);
        newButton.setEnabled(isEnabled);
        newButton.addSelectionListener(listener);
        return newButton;
    }

    /**
     * Not intended to be overridden by subclasses, set as protected for testing
     * purposes.
     */
    protected SelectionAdapter getSelectAllListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (!fSingleSelect) {
                    ((CheckboxTableViewer) fTableViewer).setAllChecked(true);
                }
            }
        };
    }

    /**
     * Not intended to be overridden by subclasses, set as protected for testing
     * purposes.
     */
    protected SelectionAdapter getDeselectAllListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (!fSingleSelect) {
                    ((CheckboxTableViewer) fTableViewer).setAllChecked(false);
                }
            }
        };
    }

    private final Label createLabel(Composite composite, String text, Color color) {
        Label label = new Label(composite, SWT.WRAP);
        int indent = IDialogConstants.HORIZONTAL_SPACING;
        label.setText(text);
        label.setForeground(color);
        GridDataFactory.fillDefaults().indent(indent, indent).applyTo(label);
        return label;
    }

    private final void createDescriptionRow(Composite parent) {
        Composite composite = new Composite(parent, SWT.FILL);
        GridDataFactory.fillDefaults().grab(true, false).applyTo(composite);
        GridLayoutFactory.swtDefaults().margins(
                IDialogConstants.HORIZONTAL_SPACING, 5).applyTo(composite);

        Label label = new Label(composite, SWT.NONE);
        label.setText(BuildUIDialogMessages.AbstractEditItemsDialog_DESCRIPTION);
        
        fDescriptionText = new Text(composite, SWT.READ_ONLY | SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
        fDescriptionText.setBackground(composite.getBackground());
        GridDataFactory.fillDefaults().grab(true, false).hint(100,
                fDescriptionText.computeTrim(0, 0, 100, fDescriptionText.getLineHeight() * 3).height).applyTo(
                fDescriptionText);
        
        updateDescription((IStructuredSelection) fTableViewer.getSelection());
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(getDialogTitle());
    }

    @Override
    protected boolean isResizable() {
        return true;
    }

    @Override
    protected void okPressed() {
        setCheckedItems(getSelectedElements());
        super.okPressed();
    }

    protected abstract ViewerComparator getTableViewerComparator();

    protected IContentProvider getContentProvider() {
        return new ArrayContentProvider();
    }

    /**
     * @return the title of the dialog. May not be <code>null</code>, but may be
     *         empty(not recommended).
     */
    protected abstract String getDialogTitle();

    /**
     * @return the description of what the dialog will do and what information
     *         is being displayed. May not be <code>null</code>, but may be
     *         empty(not recommended).
     */
    protected abstract String getDialogDescription();

    /**
     * @return the label to be attached to the show all items button, which
     *         causes the table to display the items returned in
     *         <code>getAllItems()</code> when selected. Should describe what
     *         happens when the radio button is selected (ie. "Show all items").
     *         May not be <code>null</code>, but may be empty(not recommended).
     */
    protected abstract String getShowAllLabel();

    /**
     * @return the label to be attached to the show sub set items button, which
     *         causes the table to display the items returned in
     *         <code>getSubSetItems()</code> when selected. Should describe what
     *         happens when the radio button is selected (ie.
     *         "Show sub set items"). May not be <code>null</code>, but may be
     *         empty(not recommended).
     */
    protected abstract String getShowSubSetLabel();

    /**
     * @return the array of items to be displayed when the showAllItemsButton is
     *         selected, generally is the array of all useful items in the
     *         repository. May not be <code>null</code>, but may be empty.
     */
    protected abstract String[] getAllItems();

    /**
     * @return the collection of items to be displayed when the
     *         showSubSetItemsButton is selected, generally is a subset of the
     *         array returned in <code>getAllItems()</code> that provides easier
     *         to use access for users. May not be <code>null</code>, but may be
     *         empty.
     */
    protected abstract Collection<String> getSubSetItems();

    /**
     * @returns the array of items that are initially to be checked. May not be
     *          null, but may be empty.
     */
    protected abstract String[] getInitialItems();

    /**
     * Can be implemented to have different label providers depending on what
     * content is displayed in the table at any moment (ie. if sub set items or
     * all items are displayed). If the same labels are expected, simply ignore
     * the parameter.
     * 
     * @param isSubSetShown
     *            boolean value representing whether or not the sub set is shown
     *            in the table or not.
     * @return the label provider for the table.
     */
    protected abstract LabelProvider getLabelProvider(boolean isSubSetShown);
    
    /**
     * @returns <code>true</code> if the items returned in the
     *          <code>getSubSetItems</code> method should be displayed in the
     *          table when the dialog is opened. <code>false</code> when the
     *          items returned by the <code>getAllItems</code> method should be
     *          displayed.
     */
    protected abstract boolean isStartInSubSet();
    
}