/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.internal.ui.AbstractBuildItemViewerComparator;
import com.ibm.team.build.internal.ui.BuildUIMessages;
import com.ibm.team.build.internal.ui.BuildUIPlugin;

/**
 * Dialog to add which build definitions are supported by a build engine.
 */
public class AddBuildDefinitionsDialog extends AbstractEditItemsDialog {

    private Map<String, IBuildDefinition> fAllBuildDefinitionsMap = new HashMap<String, IBuildDefinition>();
    private Map<String, String> fIdNameMap = new HashMap<String, String>();
    private Collection<String> fProjectAreaBuildDefinitionIds = new LinkedList<String>();
    private String[] fSupportedBuildDefinitionIds;
    private String fBuildProjectAreaName;

    /**
     * Create a dialog to add which build definitions are supported by a build
     * engine.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>.
     * @param supportedBuildDefinitions
     *            the array of supported build definition objects. May be
     *            <code>null</code>, or empty.
     * @param buildDefinitionToProjectAreaName
     *            the map of all build build definition objects in the
     *            repository to the name of the project area they are associated
     *            with. May be <code>null</code>, or empty. In addition, project
     *            area names (map values) can be <code>null</code>.
     * @param buildProjectAreaName
     *            the name of the project area which the build engine is
     *            associated with. May be <code>null</code>, however definitions
     *            will then only be displayed in the "Show All Definitions"
     *            table view.
     */
    public AddBuildDefinitionsDialog(Shell parentShell,
            Map<IBuildDefinition, String> buildDefinitionToProjectAreaName,
            IBuildDefinition[] supportedBuildDefinitions, String buildProjectAreaName) {
        super(parentShell);
        fBuildProjectAreaName = buildProjectAreaName;
        if (buildDefinitionToProjectAreaName != null) {
            if (fBuildProjectAreaName == null || fBuildProjectAreaName.equals(BuildUIMessages.UNKNOWN_PROJECT_AREA)) {
                for (IBuildDefinition definition : buildDefinitionToProjectAreaName.keySet()) {
                    addDefinition(definition, buildDefinitionToProjectAreaName.get(definition), supportedBuildDefinitions);
                }
            } else {
                for (IBuildDefinition definition : buildDefinitionToProjectAreaName.keySet()) {
                    String projectAreaName = buildDefinitionToProjectAreaName.get(definition);
                    boolean definitionAdded = addDefinition(definition, projectAreaName, supportedBuildDefinitions);
                    if (definitionAdded && fBuildProjectAreaName.equals(projectAreaName)) {
                        fProjectAreaBuildDefinitionIds.add(definition.getId());
                    }
                }
            }
        }

        if (supportedBuildDefinitions != null) {
            fSupportedBuildDefinitionIds = new String[supportedBuildDefinitions.length];
            for (int i = 0; i < supportedBuildDefinitions.length; i++) {
                String supportedId = supportedBuildDefinitions[i].getId();
                fSupportedBuildDefinitionIds[i] = supportedId;
            }
        } else {
            fSupportedBuildDefinitionIds = new String[0];
        }

    }

    private boolean addDefinition(IBuildDefinition definition, String projectAreaName, IBuildDefinition[] supportedBuildDefinitions) {
        String definitionId = definition.getId();
        for (IBuildDefinition supportedBuildDefinition : supportedBuildDefinitions) {
        	if (supportedBuildDefinition.getId().equals(definitionId)) {
        		return false;
        	}
        }
        fIdNameMap.put(definitionId, projectAreaName);
        fAllBuildDefinitionsMap.put(definitionId, definition);
        return true;
    }

    private String formatWithProjectArea(Object element) {
        String definitionId = element.toString();
        String areaName = fIdNameMap.get(definitionId);
        if (areaName == null) {
            areaName = BuildUIMessages.UNKNOWN_PROJECT_AREA;
        }
        return NLS.bind(BuildUIMessages.EXTENDED_LABEL, definitionId, areaName);
    }

    /**
     * @return the set of build definitions that are newly supported or no
     *         longer supported by the build engine after the dialog is
     *         finished.
     */
    public Set<IBuildDefinition> getChangedBuildDefinitions() {
        Set<IBuildDefinition> changedDefinitions = new HashSet<IBuildDefinition>();
        Iterator<String> changesAsItems = getChangedItems().iterator();
        while (changesAsItems.hasNext()) {
            changedDefinitions.add(fAllBuildDefinitionsMap.get(changesAsItems.next()));
        }
        return changedDefinitions;
    }

    @Override
    protected String getDialogDescription() {
        return BuildUIDialogMessages.AddBuildDefinitionsDialog_DESCRIPTION;
    }

    @Override
    protected String getDialogTitle() {
        return BuildUIDialogMessages.AddBuildDefinitionsDialog_TITLE;
    }

    @Override
    protected LabelProvider getLabelProvider(final boolean isSubSetDisplayed) {
        return new LabelProvider() {
            @Override
            public final String getText(Object element) {
                if (isSubSetDisplayed) {
                    return super.getText(element);
                } else {
                    return formatWithProjectArea(element);
                }
            }

            @Override
            public Image getImage(Object element) {
                return BuildUIPlugin.getImage("icons/obj16/build_type_obj.gif"); //$NON-NLS-1$
            }
        };
    }

    @Override
    protected String getShowAllLabel() {
        return BuildUIDialogMessages.AddBuildDefinitionsDialog_SHOW_ALL_LABEL;
    }

    @Override
    protected String getShowSubSetLabel() {
        return BuildUIDialogMessages.AddBuildDefinitionsDialog_SHOW_PROJECT_AREA_LABEL;
    }

    @Override
    protected String[] getAllItems() {
        return fAllBuildDefinitionsMap.keySet().toArray(new String[fAllBuildDefinitionsMap.size()]);
    }

    @Override
    protected String[] getInitialItems() {
        return new String[] {};
    }

    @Override
    protected Collection<String> getSubSetItems() {
        return fProjectAreaBuildDefinitionIds;
    }

    @Override
    protected boolean isStartInSubSet() {
        return fProjectAreaBuildDefinitionIds.size() > 0;
    }
    
    @Override
    protected ViewerComparator getTableViewerComparator() {
        return new AbstractBuildItemViewerComparator() {
            @Override
            protected String getTopProject() {
                return fBuildProjectAreaName;
            }

            @Override
            protected String getProject(Object element) {
                return fIdNameMap.get(element);
            }
        };
    }

    @Override
    protected String getDescription(IStructuredSelection selection) {
        if (selection.size() == 1) {
            String definitionId = (String) selection.getFirstElement();
            IBuildDefinition definition = fAllBuildDefinitionsMap.get(definitionId);
            if (definition != null) {
                return definition.getDescription();
            }
        }
        return null;
    }
}
