/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.model.IBuildFolder;
import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.internal.common.helper.ItemHelper;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.actions.BuildUIActionMessages;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.rcp.ui.ProcessAreaSelectionDialog;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.internal.ItemManager;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * A dialog for creating or editing a build folder.
 */
public class BuildFolderPropertiesDialog extends Dialog {

    private Text fTextName;
    private ControlDecoration fTextNameDecoration;
    private Text fTextProcessArea;

    private IProcessArea fProcessArea;

    private ITeamRepository fTeamRepository;

    private String fName;
    
    private IBuildFolder fBuildFolder;

    private Button fBrowseButton;
    
    private boolean fIsNewItem;
    
    private TeamBuildJob fTeamBuildJob;
    
    private IBuildFolderHandle fFolderHandle;
    
    private IProcessAreaHandle fInitialProcessAreaHandle;

    private BuildFolderPropertiesDialog(Shell parentShell, IBuildFolderHandle folderHandle, IProcessAreaHandle initialProcessArea, boolean isNewItem) {
        super(parentShell);
        setShellStyle(SWT.CLOSE | SWT.BORDER);
        setBlockOnOpen(true);
        fName = BuildUIDialogMessages.PENDING_LABEL;
        fIsNewItem = isNewItem;
        fFolderHandle = folderHandle;
        fInitialProcessAreaHandle = initialProcessArea;
    }

    /**
     * Create and open the build folder properties dialog with the specified
     * arguments to edit a build folder. Must be called from a UI thread.
     * 
     * @param parentShell
     *            The parent shell.
     * @param buildFolderHandle
     *            The handle to the build folder being edited.
     */
    public static void openEditFolderDialog(Shell parentShell, IBuildFolderHandle buildFolderHandle) {
        BuildFolderPropertiesDialog dialog = new BuildFolderPropertiesDialog(parentShell, buildFolderHandle, null, false);
        dialog.open();
    }

    /**
     * Create and open the build folder properties dialog with the specified
     * arguments to create a categorized build folder. Must be called from a UI
     * thread.
     * 
     * @param parentShell
     *            The parent shell.
     * @param parentBuildFolderHandle
     *            The handle to this new build folders parent folder.
     */
    public static void openNewFolderDialog(Shell parentShell, IBuildFolderHandle parentBuildFolderHandle) {
        BuildFolderPropertiesDialog dialog = new BuildFolderPropertiesDialog(parentShell, parentBuildFolderHandle, null, true);
        dialog.open();
    }
    
    /**
     * Create and open the build folder properties dialog with the specified
     * arguments to create an uncategorized build folder.
     * 
     * @param parentShell
     *            The parent shell.
     * @param processAreaHandle
     *            The handle to the initial process area of this new folder.
     */
    public static void openNewFolderDialog(Shell parentShell, IProcessAreaHandle processAreaHandle) {
        BuildFolderPropertiesDialog dialog = new BuildFolderPropertiesDialog(parentShell, null, processAreaHandle, true);
        dialog.open();
    }
    
    @Override
    public void create() {
        super.create();
        fetchAndUpdate(fFolderHandle);
    }
    
    private void fetchAndUpdate(final IBuildFolderHandle buildFolderHandle) {
        TeamBuildJob fetchFolderJob = new TeamBuildJob(
                BuildUIDialogMessages.BuildFolderPropertiesDialog_FETCHING_FOLDER_JOB_NAME, true,
                buildFolderHandle != null ? (ITeamRepository) buildFolderHandle.getOrigin() : (ITeamRepository) fInitialProcessAreaHandle.getOrigin()) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                monitor.beginTask(getName(), 2);
                try {
                    final IBuildFolder folderToModify;
                    final IProcessArea processArea;
                    
                    if (!fIsNewItem) {
                            folderToModify = (IBuildFolder) getJobTeamRepository().itemManager().fetchCompleteItem(
                                    buildFolderHandle, ItemManager.REFRESH, new SubProgressMonitor(monitor, 1));
        
                            processArea = (IProcessArea) getJobTeamRepository().itemManager().fetchCompleteItem(
                                    folderToModify.getProcessArea(), ItemManager.DEFAULT, new SubProgressMonitor(monitor, 1));

                    } else {
                        IBuildFolder parentFolder = null;
                        if (buildFolderHandle != null) {
                            parentFolder = (IBuildFolder) getJobTeamRepository().itemManager().fetchCompleteItem(
                                    buildFolderHandle, ItemManager.REFRESH, new SubProgressMonitor(monitor, 1));
                            
                            processArea = (IProcessArea) getJobTeamRepository().itemManager().fetchCompleteItem(
                                    parentFolder.getProcessArea(), ItemManager.DEFAULT, new SubProgressMonitor(monitor, 1));
                        } else {
                            processArea = (IProcessArea) getJobTeamRepository().itemManager().fetchCompleteItem(
                                    fInitialProcessAreaHandle, ItemManager.DEFAULT, new SubProgressMonitor(monitor, 2));
                        }

                        folderToModify = (IBuildFolder) BuildItemFactory.createBuildFolder(
                                parentFolder, BuildUIActionMessages.NewBuildFolderActionDelegate_DEFAULT_FOLDER_NAME,
                                processArea).getWorkingCopy();
                    }
    
                    
                    PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                        @Override
                        public void run() {
                            handleAsyncUpdateComplete(folderToModify, processArea);
                        }
                    });
                } catch (TeamRepositoryException e) {
                    Display.getDefault().asyncExec(new Runnable() {                        
                        @Override
                        public void run() {
                            if (getShell() != null && !getShell().isDisposed()) {
                                close();
                            }
                        }
                    });
                    return new Status(IStatus.ERROR, BuildUIPlugin.PLUGIN_ID,
                            BuildUIDialogMessages.BuildFolderPropertiesDialog_FETCHING_FOLDER_ERROR_MESSAGE, e);
                } finally {
                    monitor.done();
                }
                return Status.OK_STATUS;
            }
        };
        fTeamBuildJob = fetchFolderJob;
        fetchFolderJob.schedule();
    }
    
    private void handleAsyncUpdateComplete(IBuildFolder buildFolder, IProcessArea processArea) {
        if (getShell() != null && !getShell().isDisposed()) {
            fBuildFolder = buildFolder;
            fName = buildFolder.getName();
            fProcessArea = processArea;
            fTeamRepository = (ITeamRepository) fProcessArea.getOrigin();
            update();
            validate();
            fTextName.setEnabled(true);
            fBrowseButton.setEnabled(true);
            fTextProcessArea.setEnabled(true);
            fTextName.setSelection(0, fTextName.getText().length());
            fTextName.setFocus();
        }
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);
        GridLayout layout = new GridLayout(3, false);
        layout.marginLeft = IDialogConstants.HORIZONTAL_SPACING;
        layout.marginRight = IDialogConstants.HORIZONTAL_SPACING;
        layout.marginTop = IDialogConstants.VERTICAL_SPACING;
        composite.setLayout(layout);
        composite.setLayoutData(GridDataFactory.fillDefaults().align(SWT.FILL, SWT.FILL).hint(500, SWT.DEFAULT).create());

        Label labelName = new Label(composite, SWT.NONE);
        labelName.setText(BuildUIDialogMessages.BuildFolderPropertiesDialog_FOLDER_NAME_LABEL);
        labelName.setLayoutData(GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER).create());

        fTextName = new Text(composite, SWT.BORDER);
        fTextName.setEnabled(false);
        fTextName.setLayoutData(GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).span(2, 1).create());
        fTextName.setText(fName);
        fTextNameDecoration = new ControlDecoration(fTextName, SWT.CENTER);
        fTextName.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                fName = fTextName.getText();
                validate();
            }
        });

        Label labelprocessArea = new Label(composite, SWT.NONE);
        labelprocessArea.setText(BuildUIDialogMessages.BuildFolderPropertiesDialog_PROCESS_AREA_LABEL);
        labelprocessArea.setToolTipText(BuildUIDialogMessages.BuildFolderPropertiesDialog_PROCESS_AREA_TOOLTIP);
        labelprocessArea.setLayoutData(GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER).create());

        fTextProcessArea = new Text(composite, SWT.BORDER);
        fTextProcessArea.setText(BuildUIDialogMessages.PENDING_LABEL);
        fTextProcessArea.setEnabled(false);
        fTextProcessArea.setEditable(false);
        fTextProcessArea.setLayoutData(GridDataFactory.swtDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).create());

        fBrowseButton = new Button(composite, SWT.NONE);
        fBrowseButton.setEnabled(false);
        fBrowseButton.setText(BuildUIDialogMessages.BuildFolderPropertiesDialog_BROWSE_BUTTON_LABEL);
        fBrowseButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ProcessAreaSelectionDialog processAreaSelectionDialog = new ProcessAreaSelectionDialog(
                        getParentShell(), false);
                processAreaSelectionDialog.setInput((ITeamRepository) fProcessArea.getOrigin());
                processAreaSelectionDialog.addFilter(new ViewerFilter() {
                    @Override
                    public boolean select(Viewer viewer, Object parentElement, Object element) {
                        if (element instanceof IProcessArea) {
                            IProcessArea processArea = (IProcessArea) element;
                            return processArea.getProjectArea().sameItemId(fProcessArea.getProjectArea());
                        }
                        return true;
                    }
                });

                processAreaSelectionDialog.open();
                if (processAreaSelectionDialog.getSelectedProcessArea() != null) {
                    fProcessArea = processAreaSelectionDialog.getSelectedProcessArea();
                }

                update();
                validate();
            }
        });
        composite.pack();
        return composite;
    }
    
    @Override
    public Control createButtonBar(Composite parent) {
        Control control = super.createButtonBar(parent);
        getButton(IDialogConstants.OK_ID).setEnabled(false);
        return control;
        
    }

    private void saveFolder() {
        String jobName;
        if (fIsNewItem) {
            jobName = BuildUIDialogMessages.BuildFolderPropertiesDialog_SAVE_NEW_BUILD_FOLDER_JOB_NAME;
        } else {
            jobName = BuildUIDialogMessages.BuildFolderPropertiesDialog_SAVE_EXISTING_BUILD_FOLDER_JOB_NAME;
        }
        TeamBuildJob newFolderJob = new TeamBuildJob(jobName, true, fTeamRepository) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws TeamRepositoryException {
                monitor.beginTask(getName(), 1);
                try {
                    IBuildFolder workingFolder = (IBuildFolder) (fBuildFolder).getWorkingCopy();
                    workingFolder.setName(fName);
                    workingFolder.setProcessArea(fProcessArea);
                    ITeamBuildClient buildClient = ClientFactory.getTeamBuildClient(getJobTeamRepository());
                    buildClient.save(workingFolder, new SubProgressMonitor(monitor, 1));
                    Display.getDefault().asyncExec(new Runnable() {
                        @Override
                        public void run() {
                            BuildFolderPropertiesDialog.this.close();
                        }
                    });
                    return Status.OK_STATUS;
                } catch (final TeamRepositoryException e) {
                    Display.getDefault().asyncExec(new Runnable() {
                        @Override
                        public void run() {
                            validate();
                        }
                    });
                    throw e;
                } finally {
                    monitor.done();
                }
            }
        };
        newFolderJob.schedule();
    }
    
    @Override
    protected void cancelPressed() {
        fTeamBuildJob.cancel();
        super.cancelPressed();
    }
    
    @Override
    protected void okPressed() {
        getButton(IDialogConstants.OK_ID).setEnabled(false);
        saveFolder();
    }

    private void update() {
        if (fProcessArea != null && !fTextProcessArea.isDisposed()) {
            fTextProcessArea.setText(fProcessArea.getName());
        }
        if (fName != null && !fTextName.isDisposed()) {
            fTextName.setText(fName);
        }
    }

    private void validate() {
        if (!getShell().isDisposed()) {
            Button okButton = getButton(IDialogConstants.OK_ID);
            if (okButton != null) {
                boolean isValid = true;
                if (fTextName.getText().length() == 0) {
                    fTextNameDecoration.setDescriptionText(BuildUIDialogMessages.BuildFolderPropertiesDialog_NAME_DECORATOR_EMPTY_NAME);
                    isValid = false;
                } else if (!ItemHelper.validateStringAttributeLength(fTextName.getText(), IBuildFolder.ITEM_TYPE,
                        IBuildFolder.PROPERTY_NAME).equals(fTextName.getText())) {
                    fTextNameDecoration.setDescriptionText(BuildUIDialogMessages.BuildFolderPropertiesDialog_NAME_DECORATOR_TOO_LONG_NAME);
                    fTextNameDecoration.setImage(FieldDecorationRegistry.getDefault().getFieldDecoration(
                            FieldDecorationRegistry.DEC_ERROR).getImage());
                    isValid = false;
                }
                
                if (!isValid) {
                    fTextNameDecoration.setImage(FieldDecorationRegistry.getDefault().getFieldDecoration(
                            FieldDecorationRegistry.DEC_ERROR).getImage());
                } else {
                    fTextNameDecoration.setImage(null);
                }
                
                if (fProcessArea == null) {
                    isValid = false;
                }
                if (okButton.getEnabled() ^ isValid) {
                    okButton.setEnabled(isValid);
                }
            }
        }
    }
    
    @Override
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        if (fIsNewItem) {
            shell.setText(BuildUIDialogMessages.BuildFolderPropertiesDialog_TITLE_NEW_FOLDER);
        } else {
            shell.setText(BuildUIDialogMessages.BuildFolderPropertiesDialog_TITLE_EXISTING_FOLDER);
        }
        
    }
}
