/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.progress.DeferredTreeContentManager;
import org.eclipse.ui.progress.IDeferredWorkbenchAdapter;
import org.eclipse.ui.progress.IElementCollector;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.internal.common.model.dto.IBuildItemNamePair;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.domain.AbstractBuildDomainNode;
import com.ibm.team.build.internal.ui.domain.BuildDomain;
import com.ibm.team.build.internal.ui.domain.BuildDomainLabelProvider;
import com.ibm.team.build.internal.ui.domain.BuildFolderNode;
import com.ibm.team.build.internal.ui.domain.Messages;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;

/**
 * A dialog for moving build items to build folders.
 */
public class BuildFolderSelectionDialog extends Dialog implements IDeferredWorkbenchAdapter, ITreeContentProvider {

    private IItemHandle[] fItemsToMove;
    private IBuildFolderHandle fSelectedFolder;

    private final IProcessAreaHandle[] fProcessAreaHandles;
    
    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected TreeViewer fTreeViewer;

    private DeferredTreeContentManager fDeferredTreeManager;

    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected boolean fFetchInProgress;
    
    private static final Object[] EMPTY_ARRAY = new Object[0];
    
    private final Set<UUID> fFoldersToIgnore;
    
    private final AbstractBuildDomainNode rootNode = new AbstractBuildDomainNode() {
        @Override
        public String getLabel() {
            return Messages.BuildDomainContentProvider_BUILD_DOMAIN_NODE;
        }
    };

    /**
     * Create and open the build item move dialog with the specified arguments.
     * 
     * @param parentShell
     *            The parent shell.
     * @param itemsToMove
     *            The item to move.
     * @param processAreaHandles
     *            The process areas to limit folder selection to.
     */
    protected BuildFolderSelectionDialog(Shell parentShell, IItemHandle[] itemsToMove,
            IProcessAreaHandle[] processAreaHandles) {
        super(parentShell);
        setShellStyle(getShellStyle() | SWT.RESIZE | SWT.BORDER);
        setBlockOnOpen(true);
        fItemsToMove = itemsToMove;
        fProcessAreaHandles = processAreaHandles;
        
        fFoldersToIgnore = new HashSet<UUID>();
        for (IItemHandle itemHandle: itemsToMove) {
            if (itemHandle instanceof IBuildFolderHandle) {
                fFoldersToIgnore.add(itemHandle.getItemId());
            }
        }
    }

    protected TreeViewer getTreeViewer() {
        return fTreeViewer;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite control = (Composite) super.createDialogArea(parent);
        GridLayout gridLayout = new GridLayout(1, false);
        GridData controlGridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        control.setLayout(gridLayout);
        control.setLayoutData(controlGridData);

        Label instructionLabel = new Label(control, SWT.NONE);
        instructionLabel.setText(BuildUIDialogMessages.BuildFolderSelectionDialog_DESTINATION_FOLDER_LABEL);
        GridData labelData = new GridData();
        instructionLabel.setLayoutData(labelData);

        fTreeViewer = new TreeViewer(control, SWT.VIRTUAL | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
        fTreeViewer.setContentProvider(this);
        fTreeViewer.setSorter(new BuildDomain.BuildDomainNodeViewerSorter());

        // Use the standard build domain label provider, but add an image entry
        // for the 'Builds' node.
        fTreeViewer.setLabelProvider(new BuildDomainLabelProvider() {
            @Override
            public Image getImage(Object element) {
                if (element.equals(rootNode)) {
                    return BuildUIPlugin.getImage("icons/obj16/builds_obj.gif"); //$NON-NLS-1$
                }
                return super.getImage(element);
            }
        });

        GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);

        gridData.widthHint = 400;
        gridData.grabExcessHorizontalSpace = true;
        gridData.heightHint = 300;
        fTreeViewer.getControl().setLayoutData(gridData);
        fTreeViewer.getControl().setEnabled(true);
        fTreeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                if (event.getSelection() instanceof StructuredSelection) {
                    StructuredSelection structuredSelection = (StructuredSelection) event.getSelection();
                    if (structuredSelection.size() == 1) {
                        Object firstElement = structuredSelection.getFirstElement();
                        if (firstElement instanceof BuildFolderNode) {
                            BuildFolderNode folderNode = ((BuildFolderNode) structuredSelection.getFirstElement());
                            fSelectedFolder = folderNode.getBuildFolder();
                        } else if (firstElement.equals(rootNode)) {
                            fSelectedFolder = null;
                        }

                    }
                }
            }
        });
        fTreeViewer.addDoubleClickListener(new IDoubleClickListener() {
            @Override
            public void doubleClick(DoubleClickEvent event) {
                if (getButton(IDialogConstants.OK_ID).isEnabled()) {
                    getMoveBuildFolderJob().schedule();
                    BuildFolderSelectionDialog.this.close();
                }

            }
        });
        fTreeViewer.setInput(this);
        fTreeViewer.setExpandedElements(new Object[] { rootNode });
        return control;
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
        if (fDeferredTreeManager == null && viewer instanceof AbstractTreeViewer) {
            fDeferredTreeManager = new DeferredTreeContentManager((AbstractTreeViewer) viewer) {
                protected IDeferredWorkbenchAdapter getAdapter(Object element) {
                    if (hasChildren(element)) {
                        return BuildFolderSelectionDialog.this;
                    }
                    return super.getAdapter(element);
                }
            };
        }
    }

    /**
     * Create and open the build folder selection dialog with the specified
     * arguments.
     * 
     * @param parentShell
     *            The parent shell.
     * @param itemsToMove
     *            The build items that will be moved.
     * @param processAreaHandles
     *            An array of {@link IProcessAreaHandle}s to restrict folder
     *            visibility to.
     */
    public static void open(Shell parentShell, IItemHandle[] itemsToMove, IProcessAreaHandle[] processAreaHandles) {
        BuildFolderSelectionDialog dialog = new BuildFolderSelectionDialog(parentShell, itemsToMove, processAreaHandles);
        dialog.open();
    }

    @Override
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        if (fItemsToMove.length == 1) {
            if (fItemsToMove[0] instanceof IBuildDefinitionHandle) {
                shell.setText(BuildUIDialogMessages.BuildFolderSelectionDialog_MOVE_BUILD_DEFINITION_TITLE);
            } else if (fItemsToMove[0] instanceof IBuildFolderHandle) {
                shell.setText(BuildUIDialogMessages.BuildFolderSelectionDialog_MOVE_BUILD_FOLDER_TITLE);
            }
        } else {
            shell.setText(BuildUIDialogMessages.BuildFolderSelectionDialog_MOVE_BUILD_ITEMS_TITLE);
        }
    }

    @Override
    protected void okPressed() {
        getMoveBuildFolderJob().schedule();
        super.okPressed();
    }

    /**
     * Get the job for moving the items specified within the dialog to the
     * selected directory.
     * 
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected TeamBuildJob getMoveBuildFolderJob() {
        TeamBuildJob moveFolderJob = new TeamBuildJob(
                BuildUIDialogMessages.BuildFolderSelectionDialog_MOVE_BUILD_ITEM_JOB_NAME, true,
                (ITeamRepository) fItemsToMove[0].getOrigin()) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws TeamRepositoryException {
                monitor.beginTask(getName(), 1);
                try {
                    ITeamBuildClient recordClient = (ITeamBuildClient) getJobTeamRepository().getClientLibrary(
                            ITeamBuildClient.class);
                    recordClient.moveItemsToFolder(fItemsToMove, fSelectedFolder,
                            new SubProgressMonitor(monitor, 1));

                    return Status.OK_STATUS;
                } finally {
                    monitor.done();
                }
            }
        };
        return moveFolderJob;
    }

    @Override
    public Object[] getChildren(Object o) {
        if (o instanceof AbstractBuildDomainNode) {
            return fDeferredTreeManager.getChildren(o);
        } else {
            return EMPTY_ARRAY; 
        }
    }

    @Override
    public ImageDescriptor getImageDescriptor(Object object) {
        return null;
    }

    @Override
    public String getLabel(Object o) {
        // This label is used for the fetch job name.
        if (o instanceof AbstractBuildDomainNode) {
            return ((AbstractBuildDomainNode) o).getLabel();
        }
        return null;
    }

    @Override
    public Object getParent(Object o) {
        return null;
    }

    @Override
    public void fetchDeferredChildren(Object object, IElementCollector collector, IProgressMonitor monitor) {
        synchronized (this) {
            if (fFetchInProgress) {
                // A fetch is already in progress. Ignore this fetch request.
                return;
            }
            fFetchInProgress = true;
        }
        try {
            monitor.beginTask("", IProgressMonitor.UNKNOWN); //$NON-NLS-1$

            if (object == rootNode) {
                fetchAndCollectChildrenOfBuildFolder(null, collector, monitor);
            } else if (object instanceof BuildFolderNode) {
                fetchAndCollectChildrenOfBuildFolder((BuildFolderNode) object, collector, monitor);
            }
        } finally {
            synchronized (this) {
                fFetchInProgress = false;
            }
            collector.done();
            monitor.done();
        }
    }

    /**
     * Fetch and add the generated nodes to the collector for the specified
     * build folder node.
     */
    private void fetchAndCollectChildrenOfBuildFolder(BuildFolderNode node, IElementCollector collector, IProgressMonitor monitor) {
        IBuildFolderHandle parentFolderHandle;
        if (node == null) {
            parentFolderHandle = null;
        } else {
            parentFolderHandle = node.getBuildFolder();
        }

        try {
            ITeamBuildClient teamBuildClient = ClientFactory.getTeamBuildClient((ITeamRepository) fItemsToMove[0].getOrigin());

            IBuildItemNamePair[] childPairs = teamBuildClient.getChildrenOfFolder(parentFolderHandle,
                    fProcessAreaHandles, monitor);

            for (IBuildItemNamePair obj : childPairs) {
                IItemHandle itemHandle = obj.getItem();
                if (itemHandle instanceof IBuildFolderHandle && !fFoldersToIgnore.contains(itemHandle.getItemId())) {
                    BuildFolderNode folderNode = new BuildFolderNode((IBuildFolderHandle) itemHandle, obj.getName(),
                            node, null);
                    collector.add(folderNode, monitor);
                }
            }
        } catch (TeamRepositoryException exception) {
            BuildUIPlugin.log(exception);
        }
    }

    @Override
    public boolean isContainer() {
        return true;
    }

    @Override
    public ISchedulingRule getRule(Object object) {
        return null;
    }

    @Override
    public void dispose() {
        // Do nothing
    }

    public boolean hasChildren(Object o) {
        return (o instanceof BuildFolderNode || o == rootNode);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public Object[] getElements(Object inputElement) {
        if (inputElement == this) {
            return new Object[] { rootNode };
        } else {
            return getChildren(inputElement);
        }

    }

}
