/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.dialogs;

import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.resource.JFaceColors;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.internal.common.helper.BuildEngineHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.IHelpContextIds;
import com.ibm.team.build.internal.ui.properties.BuildPropertyEditorExtension;
import com.ibm.team.build.internal.ui.properties.BuildPropertyEditorExtensionManager;
import com.ibm.team.build.internal.ui.properties.BuildPropertyLabelHelper;
import com.ibm.team.build.internal.ui.properties.PulldownPropertyEditor;
import com.ibm.team.build.ui.properties.AbstractBuildPropertyEditor;
import com.ibm.team.build.ui.properties.IBuildPropertyEditorContext;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.util.ObfuscationHelper;

/**
 * A dialog for creating or editing a build property.
 */
public class BuildPropertyDialog extends Dialog {
    protected final String fTitle;
    protected Label fErrorMessageLabel;
    protected Text fPropertyNameText;
    protected Text fPropertyValueText;
    protected Text fPropertyDescriptionText;
    protected final List fExistingProperties;
    protected final IBuildProperty fBuildProperty;
    protected final boolean fIsNewProperty;
    protected final Shell fParentShell;
    protected final ITeamRepository fTeamRepository;
    protected AbstractBuildPropertyEditor fPropertyEditor;

    /**
     * Create a dialog to add or edit a build property.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param title
     *            the dialog title; never <code>null</code>
     * @param buildProperty
     *            the incoming property being added or edited
     * @param existingProperties
     *            A list of existing <code>IBuildProperty</code> objects. The
     *            list is used only for determining if the property being added
     *            already exists.
     * @param isNewProperty
     *            <code>true</code>, if the property is a new property being
     *            added; <code>false</code>, if the property is an existing
     *            property
     * @param teamRepository
     *            The team repository context.
     */
    public BuildPropertyDialog(Shell parentShell, String title, IBuildProperty buildProperty,
            List<IBuildProperty> existingProperties, boolean isNewProperty, ITeamRepository teamRepository) {
        super(parentShell);

        ValidationHelper.validateNotNull("title", title); //$NON-NLS-1$

        setShellStyle(getShellStyle() | SWT.RESIZE);
        fTitle = title;
        fParentShell = parentShell;
        fExistingProperties = existingProperties;
        fBuildProperty = buildProperty;
        fIsNewProperty = isNewProperty;
        fTeamRepository = teamRepository;
        fPropertyEditor = getPropertyEditor(buildProperty);
    }

    /**
     * Validates the state of the dialog input; disables the <code>OK</code>
     * button and optionally shows an error message if the dialog input state is
     * invalid.
     * 
     * @param suppressErrorMessage
     *            <code>true</code> to not show an error message even if the
     *            dialog state is invalid (used for initial display of the
     *            dialog); <code>false</code> to show the error message if the
     *            dialog state is invalid.
     */
    protected void validate(boolean suppressErrorMessage) {
        String errorMessage = getErrorMessage();

        if ((errorMessage == null) || (suppressErrorMessage)) {
            fErrorMessageLabel.setText(""); //$NON-NLS-1$
        } else {
            fErrorMessageLabel.setText(errorMessage);
        }
        fErrorMessageLabel.redraw();

        Button okButton = getButton(IDialogConstants.OK_ID);
        okButton.setEnabled(errorMessage == null);
    }

    private Button fEditButton;

    protected String getPropertyNameTextValue() {
        return fPropertyNameText.getText();
    }

    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);
        composite.setLayout(new GridLayout(3, false));
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.widthHint = 400;
        composite.setLayoutData(gridData);

        String message = BuildUIDialogMessages.PROPERTY_HEADER;
        if (fBuildProperty.isRequired() || fBuildProperty.isWellKnown()) {
            message = BuildUIDialogMessages.PROPERTY_HEADER_REQUIRED;
        }

        createLabel(composite, message, (Color) null);
        fErrorMessageLabel = createLabel(composite, "", JFaceColors.getErrorText(composite.getDisplay())); //$NON-NLS-1$

        int horizontalSpan = 2;
        boolean addEditButton = false;
        if (fPropertyEditor != null) {
            // Add an edit button that will open the custom edit dialog.
            addEditButton = true;
            horizontalSpan = 1;
        }

        String valueText = ""; //$NON-NLS-1$
        if (!fIsNewProperty) {
            valueText = BuildUIDialogMessages.BuildPropertyDialog_FETCHING_VALUE;
        }

        fPropertyNameText = createText(composite, BuildUIDialogMessages.PROPERTY_NAME, fBuildProperty.getName(), 2);
        fPropertyValueText = createText(composite, BuildUIDialogMessages.PROPERTY_VALUE, valueText, horizontalSpan);

        if (addEditButton) {
            fEditButton = new Button(composite, SWT.NONE);
            fEditButton.setText(BuildUIDialogMessages.BuildPropertyDialog_EDIT_BUTTON_LABEL);
            fEditButton.addSelectionListener(getEditButtonSelectionListener());

            IBuildPropertyEditorContext context = getPropertyEditorContext(fBuildProperty, fParentShell,
                    fTeamRepository);
            fPropertyValueText.setEnabled(false);

            if (!fIsNewProperty) {
                BuildPropertyLabelHelper labelHelper = getLabelHelper();
                labelHelper.getPropertyValueLabelInBackground(fPropertyEditor, context);
            }
        } else {
            fPropertyValueText.setText(getPropertyValueToEdit(fBuildProperty, fBuildProperty.getValue()));
        }
        
        fPropertyDescriptionText = createText(composite, BuildUIDialogMessages.DESCRIPTION_LABEL,
                fBuildProperty.getDescription(), 2);

        if (fBuildProperty.isRequired() || fBuildProperty.isWellKnown()) {
            fPropertyNameText.setEditable(false);
            fPropertyDescriptionText.setEditable(false);
        }

        fPropertyNameText.addModifyListener(getNameModifiedListener());
        fPropertyDescriptionText.addModifyListener(getDescriptionModifiedListener());
        fPropertyValueText.addModifyListener(getValueModifiedListener());

        applyDialogFont(composite);

        /*
         * If an existing string property is being edited, select the value
         * field.
         */
        if (!fIsNewProperty && !addEditButton) {
            fPropertyValueText.setFocus();
            fPropertyValueText.selectAll();
        } else {
            fPropertyNameText.setFocus();
        }

        PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, IHelpContextIds.HELP_CONTEXT_BUILD_PROPERTY_DIALOG); 

        return composite;
    }

    /**
     * Returns the value of the property to edit. For hidden properties, the
     * property's value is stored elsewhere, so return the empty string.
     */
    private String getPropertyValueToEdit(IBuildProperty buildProperty, String value) {
        if (BuildEngineHelper.isBuildForgeHiddenProperty(buildProperty) && value != null) {
            if (BuildEngineHelper.BUILD_FORGE_HIDDEN_PROPERTY_DUMMY_VALUE.equals(value)) {
                return ""; //$NON-NLS-1$
            }
            String deobfuscated = value;
            try {
                deobfuscated = ObfuscationHelper.decryptString(value);
            } catch (Exception e) {
                // not obfuscated to begin with
            }
            if (BuildEngineHelper.isDummyPassword(deobfuscated)) {
                return ""; //$NON-NLS-1$
            }
        }
        return value;
    }

    private BuildPropertyLabelHelper getLabelHelper() {
        return new BuildPropertyLabelHelper() {
            @Override
            public void labelAvailable(String label, IBuildProperty buildProperty) {
                if (!fPropertyValueText.isDisposed()) {
                    fPropertyValueText.setText(getPropertyValueToEdit(buildProperty, label));
                }
            }
        };
    }

    private ModifyListener getNameModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validate(false);
            }
        };
    }

    private ModifyListener getDescriptionModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validate(false);
            }
        };
    }

    private ModifyListener getValueModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validate(false);               
            }
        };
    }

    private SelectionListener getEditButtonSelectionListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                editProperty(fBuildProperty);
            }
        };
    }

    protected Label createLabel(Composite composite, String text, Color color) {
        Label label = new Label(composite, SWT.WRAP);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 3;
        label.setLayoutData(gridData);
        label.setText(text);
        label.setForeground(color);
        return label;
    }

    protected Text createText(Composite parent, String labelText, String defaultText, int textHorizontalSpan) {
        Label label = new Label(parent, SWT.NONE);
        label.setText(labelText);
        label.setLayoutData(new GridData());

        int style = SWT.BORDER;
        boolean isHiddenValue = labelText.equals(BuildUIDialogMessages.PROPERTY_VALUE) && BuildEngineHelper.isBuildForgeHiddenProperty(fBuildProperty);
        if (isHiddenValue) {
            style |= SWT.PASSWORD;
        }
        boolean isDescription = labelText.equals(BuildUIDialogMessages.DESCRIPTION_LABEL);
        if (isDescription) {
            style |= SWT.MULTI | SWT.WRAP;
        } else {
            style |= SWT.SINGLE;
        }

        Text text = new Text(parent, style);
        text.setText(defaultText);
        int heightHint = isDescription ? text.computeSize(300, text.getLineHeight() * 3).y : -1;
        GridDataFactory.fillDefaults().span(textHorizontalSpan, 1).hint(100, heightHint).grab(true, isDescription).applyTo(text);

        return text;
    }

    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(fTitle);
    }

    public void create() {
        super.create();
        validate(true);
    }

    protected List getExistingProperties() {
        return fExistingProperties;
    }

    protected String getErrorMessage() {
        String proposedPropertyName = getPropertyNameTextValue();

        if (proposedPropertyName.equals("")) { //$NON-NLS-1$
            return BuildUIDialogMessages.PROPERTY_NAME_REQUIRED;
        } else if (proposedPropertyName.indexOf(' ') > -1) {
            return BuildUIDialogMessages.PROPERTY_NAME_NO_SPACES;
        } else if (propertyNameConflict(proposedPropertyName)) {
            return BuildUIDialogMessages.PROPERTY_EXISTS;
        } else {
            return null;
        }
    }

    protected boolean propertyNameConflict(String name) {
        
        if(fBuildProperty.getName().length() != 0){//editing a property
            if (fBuildProperty.getName().equals(name)) {
                // allow the property to keep its incoming name
                return false;
            }
        }

        List properties = getExistingProperties();
        for (Object object : properties) {
            IBuildProperty property = (IBuildProperty) object;
            if (property.getName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    private void editProperty(final IBuildProperty buildProperty) {
        IBuildPropertyEditorContext context = getPropertyEditorContext(buildProperty, getShell(), fTeamRepository);

        if (fPropertyEditor.editProperty(context)) {
            fPropertyValueText.setText(BuildUIDialogMessages.BuildPropertyDialog_FETCHING_VALUE);
            BuildPropertyLabelHelper labelHelper = getLabelHelper();
            labelHelper.getPropertyValueLabelInBackground(fPropertyEditor, context);
        }

    }

    /**
     * @param buildProperty - The property for for which the editor will be used and is registered
     * @return The AbstractBuildPropertyEditor defined for the given IBuildProperty
     */
    public AbstractBuildPropertyEditor getPropertyEditor(final IBuildProperty buildProperty) {
        // Default to string editing.
        AbstractBuildPropertyEditor editor = null;

        String editorId = buildProperty.getKind();
        if (BuildEngineHelper.isBuildForgePulldownProperty(buildProperty)) {
            editorId = PulldownPropertyEditor.ID;
        }
        try {
            BuildPropertyEditorExtension extension = BuildPropertyEditorExtensionManager.getInstance().getBuildPropertyEditor(
                    editorId);
            if (extension != null) {
                editor = extension.createPropertyEditor();
            }
        } catch (CoreException exception) {
            handlePropertyEditorException(exception);
        }

        return editor;
    }

    protected IBuildPropertyEditorContext getPropertyEditorContext(final IBuildProperty buildProperty,
            final Shell parent, ITeamRepository teamRepository) {

        return new IBuildPropertyEditorContext() {
            public IBuildProperty getProperty() {
                return buildProperty;
            }

            public Shell getShell() {
                return parent;
            }

            public ITeamRepository getTeamRepository() {
                return fTeamRepository;
            }
        };

    }

    private void handlePropertyEditorException(CoreException exception) {
        BuildUIPlugin.log(exception);
    }

    /* (non-Javadoc)
     * Intentionally not documented. See parent.
     */
    @Override
    protected void okPressed() {
        fBuildProperty.setName(fPropertyNameText.getText());
        fBuildProperty.setDescription(fPropertyDescriptionText.getText());
        if (fPropertyEditor == null) {
            /*
             * Default string editor. Go ahead and update the property
             * directly from the text.
             */
            fBuildProperty.setValue(fPropertyValueText.getText());
        }
        super.okPressed();
    }
    
}
