/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.dialogs;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.SortedMap;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.resource.JFaceColors;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.Section;

import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.repository.common.util.NLS;

/**
 * A dialog for specifying a URL or a local file to attach to a build result
 * along with an optional description, component name, content type and
 * character encoding.
 */
public class BuildResultAttachmentFileDialog extends Dialog {

    protected Section fAdvancedSection;

    protected Label fErrorMessageLabel;
    protected Text fComponentNameText;
    protected Text fContentTypeText;
    protected Combo fCharacterEncodingDropdown;
    protected Text fFileNameText;
    protected Text fFileDescriptionText;

    protected String fOriginalFilePath;
    protected String fOriginalFileDescription;
    protected String fOriginalComponentName;
    protected String fOriginalContentType;
    protected String fOriginalCharacterEncoding;

    protected String fFilePath;
    protected String fFileDescription;
    protected String fComponentName;
    protected String fContentType;
    protected String fCharacterEncoding;

    protected static String[] fAvailableCharacterSets;

    protected boolean fFilePathChanged = false;
    protected boolean fIsLink;
    /*
     * true if this dialog is adding a new file attachment as opposed to editing
     * an existing one.
     */
    protected boolean fIsAdd;
    protected boolean fIsFileSaved;

    protected Button fBrowseButton;

    /**
     * Create a dialog to add a new file attachment.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param componentName
     *            the suggested component for the new file attachment
     * @return The newly created dialog.
     */
    public static BuildResultAttachmentFileDialog createAddFileDialog(Shell parentShell, String componentName) {
        return new BuildResultAttachmentFileDialog(parentShell, componentName, false);
    }

    /**
     * Create a dialog to add a new link attachment.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param componentName
     *            the suggested component for the new file attachment
     * @return The newly created dialog.
     */
    public static BuildResultAttachmentFileDialog createAddLinkDialog(Shell parentShell, String componentName) {
        return new BuildResultAttachmentFileDialog(parentShell, componentName, true);
    }

    /**
     * Create a dialog to edit an existing file attachment.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param filePath
     *            the file path of the existing file
     * @param fileDescription
     *            the description of the existing file
     * @param componentName
     *            the component name of the existing file
     * @param contentType
     *            the content type of the of the existing file
     * @param characterEncoding
     *            the character encoding of the of the existing file
     * @param isSaved
     *            if isSaved is true, the file being edited is already saved in
     *            the repository, so don't allow changes to content type or
     *            character encoding unless the file itself is changed.
     * @return The newly created dialog.
     */
    public static BuildResultAttachmentFileDialog createEditFileDialog(Shell parentShell, String filePath,
            String fileDescription, String componentName, String contentType, String characterEncoding, boolean isSaved) {
        return new BuildResultAttachmentFileDialog(parentShell, filePath, fileDescription, componentName, contentType,
                characterEncoding, false, isSaved);
    }

    /**
     * Create a dialog to edit an existing link attachment.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param linkURL
     *            the URL of the existing link
     * @param linkDescription
     *            the description of the existing link
     * @param componentName
     *            the component name of the existing link
     * @return The newly created dialog.
     */
    public static BuildResultAttachmentFileDialog createEditLinkDialog(Shell parentShell, String linkURL,
            String linkDescription, String componentName) {
        return new BuildResultAttachmentFileDialog(parentShell, linkURL, linkDescription, componentName, null, null,
                true, true);
    }

    /**
     * Create a dialog to edit an existing file attachment of a build result.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param filePath
     *            the file path of the existing file if creating an edit dialog
     * @param fileDescription
     *            the file description of the existing file if creating an edit
     *            dialog
     * @param componentName
     *            the file component name of the existing file if creating an
     *            edit dialog
     * @param contentType
     *            the content type of the of the existing file if creating an
     *            edit dialog
     * @param characterEncoding
     *            encoding the character encoding of the of the existing file if
     *            creating an edit dialog
     * @param isLink
     *            if isLink is true, create a dialog for adding a URL link,
     *            otherwise create a dialog for adding a file
     * @param isSaved
     *            if isSaved is true, the file being edited is already saved in
     *            the repository, so don't allow changes to content type or
     *            character encoding unless the file itself is changed.
     */
    protected BuildResultAttachmentFileDialog(Shell parentShell, String filePath, String fileDescription,
            String componentName, String contentType, String characterEncoding, boolean isLink, boolean isSaved) {
        super(parentShell);
        ValidationHelper.validateNotNull("parentShell", parentShell); //$NON-NLS-1$
        setShellStyle(getShellStyle() | SWT.RESIZE);

        fOriginalFilePath = fFilePath = filePath == null ? "" : filePath; //$NON-NLS-1$
        fOriginalFileDescription = fFileDescription = fileDescription == null ? "" : fileDescription; //$NON-NLS-1$
        fOriginalComponentName = fComponentName = componentName == null ? "" : componentName; //$NON-NLS-1$
        fOriginalContentType = fContentType = contentType == null ? "" : contentType; //$NON-NLS-1$
        fOriginalCharacterEncoding = fCharacterEncoding = characterEncoding == null ? "" : characterEncoding; //$NON-NLS-1$
        fIsLink = isLink;
        fIsFileSaved = isSaved;
        fIsAdd = false;
    }

    /**
     * Create a dialog to add an attachment to a build result.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param componentName
     *            the suggested component for the new file attachment
     * @param isLink
     *            if true, create a dialog to add a link, else create a dialog
     *            to add a file
     */
    protected BuildResultAttachmentFileDialog(Shell parentShell, String componentName, boolean isLink) {
        super(parentShell);
        ValidationHelper.validateNotNull("parentShell", parentShell); //$NON-NLS-1$
        setShellStyle(getShellStyle() | SWT.RESIZE);

        fOriginalFilePath = fFilePath = ""; //$NON-NLS-1$
        fOriginalFileDescription = fFileDescription = ""; //$NON-NLS-1$
        fOriginalComponentName = fComponentName = componentName == null ? "" : componentName; //$NON-NLS-1$
        fOriginalContentType = fContentType = ""; //$NON-NLS-1$
        fOriginalCharacterEncoding = fCharacterEncoding = ""; //$NON-NLS-1$
        fIsLink = isLink;
        fIsFileSaved = false;
        fIsAdd = true;
    }

    /**
     * Return true if any values were changed by the user.
     * 
     * @return true if any values were changed by the user, false otherwise.
     */
    public boolean isDirty() {
        /*
         * First, detect if the file name or URL is different. If this is a
         * link, we can simply compare the current URL to the original. If this
         * is a file name however, the file name could be the same while the
         * underlying file has changed. We cover this case by always setting the
         * fFilePathChanged field to true whenever the user selects a new file,
         * regardless of the name.
         */
        boolean filePathChanged = fIsLink ? !fOriginalFilePath.equals(fFilePath) : fFilePathChanged;
        return filePathChanged || !fOriginalFileDescription.equals(fFileDescription)
                || !fOriginalComponentName.equals(fComponentName) || !fOriginalContentType.equals(fContentType)
                || !fOriginalCharacterEncoding.equals(fCharacterEncoding);
    }

    /**
     * Return whether or not the file path was changed by the user. This is only
     * set when the dialog is created for a file entry, not a link entry.
     * 
     * @return True if the file path was edited, false otherwise.
     */
    public boolean isFilePathChanged() {
        return fFilePathChanged;
    }

    /**
     * Get the file path or URL.
     * 
     * @return The file path or URL entered by the user.
     */
    public String getFilePath() {
        return fFilePath;
    }

    /**
     * Get the description of the file or URL.
     * 
     * @return The description of the file or URL entered by the user or the
     *         empty string if none was specified.
     */
    public String getFileDescription() {
        return fFileDescription;
    }

    /**
     * Get the component name.
     * 
     * @return The component name entered by the user or the empty string if
     *         none was specified.
     */
    public String getComponentName() {
        return fComponentName;
    }

    /**
     * Get the content type for the file attachment.
     * 
     * @return the content type determined for the file attachment.
     */
    public String getContentType() {
        return fContentType;
    }

    /**
     * Get the character encoding specified for the file attachment.
     * 
     * @return the character encoding specified for the file attachment.
     */
    public String getCharacterEncoding() {
        return fCharacterEncoding;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public int open() {
        if (!fIsLink && fOriginalFilePath.length() == 0) {
            String filePath = promptUserForFile();
            if (filePath == null) {
                return CANCEL;
            }
            fFilePath = filePath;
        }
        return super.open();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void createButtonsForButtonBar(Composite parent) {
        super.createButtonsForButtonBar(parent);
        if (fIsLink) {
            getButton(IDialogConstants.OK_ID).setEnabled(fOriginalFilePath.trim().length() != 0);
        } else if (fIsAdd) {
            validateFileName(fFilePath);
        } else {
            updateContentTypeAndCharacterEncoding(!fIsFileSaved, false);
        }
    }

    protected Control createDialogArea(Composite parent) {
        Composite dialogComposite = (Composite) super.createDialogArea(parent);

        String message = fIsLink ? BuildUIDialogMessages.BuildResultAttachmentFileDialog_ADD_LINK_MESSAGE
                : BuildUIDialogMessages.BuildResultAttachmentFileDialog_ADD_FILE_MESSAGE;

        new Label(dialogComposite, SWT.NONE).setText(message);
        fErrorMessageLabel = new Label(dialogComposite, SWT.NONE);
        fErrorMessageLabel.setForeground(JFaceColors.getErrorText(dialogComposite.getDisplay()));
        GridDataFactory.fillDefaults().grab(true, false).applyTo(fErrorMessageLabel);

        Composite optionsComposite = new Composite(dialogComposite, SWT.NONE);
        GridLayoutFactory.fillDefaults().numColumns(3).applyTo(optionsComposite);
        GridDataFactory.fillDefaults().applyTo(optionsComposite);

        String nameLabel = fIsLink ? BuildUIDialogMessages.BuildResultAttachmentFileDialog_LINK_URL_LABEL
                : BuildUIDialogMessages.BuildResultAttachmentFileDialog_FILE_NAME_LABEL;

        String filePath = fIsLink ? fOriginalFilePath : new File(fOriginalFilePath).getName();
        fFileNameText = createText(optionsComposite, nameLabel, filePath, fIsLink ? 2 : 1);
        fFileNameText.setEditable(fIsLink);

        if (!fIsLink) {
            fBrowseButton = new Button(optionsComposite, SWT.NONE);
            fBrowseButton.setText(BuildUIDialogMessages.BuildResultAttachmentFileDialog_BROWSE_BUTTON_LABEL);
            fBrowseButton.addSelectionListener(getBrowseButtonSelectionListener());
        }

        fFileDescriptionText = createText(optionsComposite, BuildUIDialogMessages.DESCRIPTION_LABEL,
                fOriginalFileDescription, 2);
        fComponentNameText = createText(optionsComposite,
                BuildUIDialogMessages.BuildResultAttachmentFileDialog_COMPONENT_NAME_LABEL, fOriginalComponentName, 2);

        if (!fIsLink) {
            fAdvancedSection = new Section(dialogComposite, ExpandableComposite.TWISTIE
                    | ExpandableComposite.CLIENT_INDENT | ExpandableComposite.COMPACT);
            GridLayoutFactory.fillDefaults().applyTo(fAdvancedSection);
            GridDataFactory.fillDefaults().indent(0, 10).grab(true, true).applyTo(fAdvancedSection);
            fAdvancedSection.setText(BuildUIDialogMessages.BuildResultAttachmentFileDialog_ADVANCED_OPTIONS);

            Composite advancedComposite = new Composite(fAdvancedSection, SWT.NONE);
            GridLayoutFactory.fillDefaults().extendedMargins(0, 0, 10, 0).numColumns(3).applyTo(advancedComposite);
            GridDataFactory.fillDefaults().grab(true, true).applyTo(advancedComposite);

            fAdvancedSection.setClient(advancedComposite);
            fAdvancedSection.addExpansionListener(new ExpansionAdapter() {
                public void expansionStateChanged(ExpansionEvent event) {
                    handleExpansionStateChanged();
                }
            });

            fContentTypeText = createText(advancedComposite,
                    BuildUIDialogMessages.BuildResultAttachmentFileDialog_CONTENT_TYPE_LABEL, fOriginalContentType, 2);
            fContentTypeText.setToolTipText(BuildUIDialogMessages.BuildResultAttachmentFileDialog_CONTENT_TYPE_TOOLTIP);

            if (fAvailableCharacterSets == null) {
                SortedMap<String, Charset> availableCharsets = Charset.availableCharsets();
                fAvailableCharacterSets = availableCharsets.keySet().toArray(new String[availableCharsets.size()]);
            }

            fCharacterEncodingDropdown = createDropdown(advancedComposite,
                    BuildUIDialogMessages.BuildResultAttachmentFileDialog_CHARACTER_ENCODING_LABEL,
                    fOriginalCharacterEncoding, fAvailableCharacterSets,
                    BuildUIDialogMessages.BuildResultAttachmentFileDialog_CHARACTER_ENCODING_TOOLTIP, 2);

            fContentTypeText.addModifyListener(getContentTypeModifiedListener());
            fCharacterEncodingDropdown.addModifyListener(getCharacterEncodingModifiedListener());
            fCharacterEncodingDropdown.addSelectionListener(getCharacterEncodingSelectionListener());
        } else {
            fFileNameText.addModifyListener(getFileNameModifiedListener());
        }

        fFileDescriptionText.addModifyListener(getDescriptionModifiedListener());
        fComponentNameText.addModifyListener(getComponentNameModifiedListener());

        applyDialogFont(dialogComposite);

        if (fIsLink) {
            fFileNameText.setFocus();
        } else {
            fFileDescriptionText.setFocus();
        }
        
        return dialogComposite;
    }

    private void handleExpansionStateChanged() {
        Display.getDefault().asyncExec(new Runnable() {
            public void run() {
                getContents().getParent().pack();
            }
        });
    }

    protected ModifyListener getFileNameModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validateUrl(fFileNameText.getText());
            }
        };
    }

    protected ModifyListener getDescriptionModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                fFileDescription = fFileDescriptionText.getText();
            }
        };
    }

    protected ModifyListener getComponentNameModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                fComponentName = fComponentNameText.getText();
            }
        };
    }

    protected ModifyListener getContentTypeModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                fContentType = fContentTypeText.getText();
                updateCharacterEncodingEnablement(true);
            }
        };
    }

    protected SelectionListener getCharacterEncodingSelectionListener() {
        return new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                validateCharacterEncoding(fCharacterEncodingDropdown.getText());
            }

            public void widgetDefaultSelected(SelectionEvent e) {
                validateCharacterEncoding(fCharacterEncodingDropdown.getText());
            }
        };
    }

    protected ModifyListener getCharacterEncodingModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validateCharacterEncoding(fCharacterEncodingDropdown.getText());
            }
        };
    }

    protected void validateCharacterEncoding(String characterEncoding) {
        boolean enabled = characterEncoding.length() == 0 || Charset.isSupported(characterEncoding);
        String errorMessage = ""; //$NON-NLS-1$

        if (enabled) {
            fCharacterEncoding = characterEncoding;
        } else {
            errorMessage = NLS.bind(
                    BuildUIDialogMessages.BuildResultAttachmentFileDialog_CHARACTER_SET_NOT_SUPPORTED_ERROR_MESSAGE,
                    characterEncoding);
        }

        if (getButton(IDialogConstants.OK_ID) != null) {
            getButton(IDialogConstants.OK_ID).setEnabled(enabled);
        }
        fErrorMessageLabel.setText(errorMessage);
    }

    protected void validateUrl(String url) {
        boolean enabled = true;
        String errorMessage = ""; //$NON-NLS-1$

        if (url.trim().length() == 0) {
            enabled = false;
        } else {
            try {
                new URL(url);
                fFilePath = url;
            } catch (MalformedURLException exception) {
                enabled = false;
                errorMessage = BuildUIDialogMessages.BuildResultAttachmentFileDialog_MALFORMED_URL;
            }
        }

        getButton(IDialogConstants.OK_ID).setEnabled(enabled);
        fErrorMessageLabel.setText(errorMessage);
    }

    protected boolean fileExists(String fileName) {
        return new File(fileName).exists();
    }

    protected void validateFileName(String fileName) {
        boolean enabled = true;
        String errorMessage = ""; //$NON-NLS-1$

        fFileNameText.setText(new File(fileName).getName());

        if (!fileExists(fileName)) {
            enabled = false;
            errorMessage = BuildUIDialogMessages.BuildResultAttachmentFileDialog_FILE_NOT_EXIST;
        } else {
            fFilePathChanged = true;
            fFilePath = fileName;
            fIsFileSaved = false;
        }

        updateContentTypeAndCharacterEncoding(enabled && !fIsFileSaved, true);
        fErrorMessageLabel.setText(errorMessage);
        getButton(IDialogConstants.OK_ID).setEnabled(enabled);
    }

    protected void updateContentTypeAndCharacterEncoding(boolean enabled, boolean updateContentType) {
        if (enabled) {
            fContentTypeText.setEnabled(true);

            if (updateContentType) {
                fContentType = ContentUtil.getContentTypeFromFileName(fFilePath);
                if (fContentType == null) {
                    fContentType = ""; //$NON-NLS-1$
                }
                fContentTypeText.setText(fContentType);
            } else {
                updateCharacterEncodingEnablement(false);
            }
        } else {
            fContentTypeText.setEnabled(false);
            fCharacterEncodingDropdown.setEnabled(false);
        }
    }

    protected void updateCharacterEncodingEnablement(boolean updateEncoding) {
        boolean isTextContent = ContentUtil.isTextContentType(fContentType);

        if (updateEncoding) {
            fCharacterEncoding = isTextContent ? Charset.defaultCharset().name() : ""; //$NON-NLS-1$
            fCharacterEncodingDropdown.setText(fCharacterEncoding);
        }

        fCharacterEncodingDropdown.setEnabled(isTextContent);
    }

    protected SelectionListener getBrowseButtonSelectionListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                String filePath = promptUserForFile();
                if (filePath != null) {
                    validateFileName(filePath);
                }
            }
        };
    }

    protected String promptUserForFile() {
        FileDialog fileDialog = new FileDialog(getParentShell(), SWT.OPEN);
        fileDialog.setText(BuildUIDialogMessages.BuildResultAttachmentFileDialog_ADD_FILE_SELECTION_DIALOG_TITLE);
        return fileDialog.open();
    }

    protected Label createLabel(Composite composite, String text, Color color) {
        Label label = new Label(composite, SWT.WRAP);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        label.setLayoutData(gridData);
        label.setText(text);
        label.setForeground(color);
        return label;
    }

    protected Combo createDropdown(Composite parent, String labelText, String defaultText, String[] items,
            String tooltip, int textHorizontalSpan) {
        Label label = new Label(parent, SWT.NONE);
        label.setText(labelText);
        label.setLayoutData(new GridData());

        Combo combo = new Combo(parent, SWT.BORDER);
        combo.setVisibleItemCount(Math.min(20, items.length));
        combo.setItems(items);
        combo.setText(defaultText);
        combo.setToolTipText(tooltip);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = textHorizontalSpan;
        combo.setLayoutData(gridData);

        return combo;
    }

    protected Text createText(Composite parent, String labelText, String defaultText, int textHorizontalSpan) {
        Label label = new Label(parent, SWT.NONE);
        label.setText(labelText);
        label.setLayoutData(new GridData());

        Text text = new Text(parent, SWT.BORDER | SWT.SINGLE);
        text.setText(defaultText);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = textHorizontalSpan;
        text.setLayoutData(gridData);

        return text;
    }

    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);

        if (fIsLink) {
            newShell.setText(BuildUIDialogMessages.BuildResultAttachmentFileDialog_LINK_TITLE);
        } else {
            newShell.setText(BuildUIDialogMessages.BuildResultAttachmentFileDialog_FILE_TITLE);
        }
    }
}
