/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * Dialog for requesting a build.
 */
public class CreateReleaseDialog extends Dialog {

    protected Text fReleaseNameText;
    protected Text fBuildTagText;
    protected Button fPrivateButton;

    private String fReleaseName;
    private String fBuildTag = ""; //$NON-NLS-1$
    private boolean fIsPrivate;

    /**
     * Create the request build dialog. The dialog requests a build of the given
     * definition.
     * 
     * @param parentShell
     *            The parent shell for the dialog.
     * @param defaultReleaseName
     *            The default name to fill in the name field.
     */
    public CreateReleaseDialog(Shell parentShell, String defaultReleaseName) {
        super(parentShell);
        fReleaseName = defaultReleaseName;
        fBuildTag = defaultReleaseName.replace(" ", "_"); //$NON-NLS-1$//$NON-NLS-2$
        setShellStyle(getShellStyle() | SWT.RESIZE | SWT.MAX);
    }

    /**
     * Get the name of the release.
     * 
     * @return The name of the release.
     */
    public String getReleaseName() {
        return fReleaseName;
    }

    /**
     * Get the build tag.
     * 
     * @return The build tag. May be empty, but never <code>null</code>.
     */
    public String getBuildTag() {
        return fBuildTag;
    }

    /**
     * Whether or not the release is private. Private releases are only shown to
     * members of project teams.
     * 
     * @return Whether or not the release is private.
     */
    public boolean isReleasePrivate() {
        return fIsPrivate;
    }

    protected Control createDialogArea(Composite parent) {
        Composite dialogAreaComposite = (Composite) super.createDialogArea(parent);
        dialogAreaComposite.setLayout(new GridLayout(2, false));
        GridData gridData = new GridData(GridData.FILL_BOTH);
        dialogAreaComposite.setLayoutData(gridData);

        createLabel(dialogAreaComposite, BuildUIDialogMessages.CreateReleaseDialog_DESCRIPTION, (Color) null);

        // Vertical space.
        Label label = new Label(dialogAreaComposite, SWT.NONE);
        gridData = new GridData();
        gridData.horizontalSpan = 2;
        gridData.heightHint = 10;
        label.setLayoutData(gridData);

        label = new Label(dialogAreaComposite, SWT.NONE);
        label.setText(BuildUIDialogMessages.CreateReleaseDialog_RELEASE_NAME);
        label.setToolTipText(BuildUIDialogMessages.CreateReleaseDialog_RELEASE_NAME_TOOLTIP);
        label.setLayoutData(new GridData());

        fReleaseNameText = new Text(dialogAreaComposite, SWT.BORDER);
        fReleaseNameText.setText(fReleaseName);
        fReleaseNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        fReleaseNameText.addModifyListener(getNameTextModifyListener());
        fReleaseNameText.selectAll();

        label = new Label(dialogAreaComposite, SWT.NONE);
        label.setText(BuildUIDialogMessages.CreateReleaseDialog_BUILD_TAG);
        label.setToolTipText(BuildUIDialogMessages.CreateReleaseDialog_BUILD_TAG_TOOLTIP);
        label.setLayoutData(new GridData());

        fBuildTagText = new Text(dialogAreaComposite, SWT.BORDER);
        fBuildTagText.setText(fBuildTag);
        fBuildTagText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        fBuildTagText.addModifyListener(getTagTextModifyListener());

        // Vertical space.
        label = new Label(dialogAreaComposite, SWT.NONE);
        gridData = new GridData();
        gridData.horizontalSpan = 2;
        gridData.heightHint = 10;
        label.setLayoutData(gridData);

        fPrivateButton = new Button(dialogAreaComposite, SWT.CHECK);
        fPrivateButton.setText(BuildUIDialogMessages.CreateReleaseDialog_PRIVATE);
        gridData = new GridData();
        gridData.horizontalSpan = 2;
        fPrivateButton.setLayoutData(gridData);
        fPrivateButton.addSelectionListener(getPrivateButtonListener());

        applyDialogFont(dialogAreaComposite);

        fReleaseNameText.setFocus();

        if (fReleaseNameText.getText().equals("")) { //$NON-NLS-1$
            Button okButton = getButton(IDialogConstants.OK_ID);
            okButton.setEnabled(false);
        }

        return dialogAreaComposite;
    }

    private ModifyListener getNameTextModifyListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                fReleaseName = fReleaseNameText.getText();
                Button okButton = getButton(IDialogConstants.OK_ID);
                okButton.setEnabled(fReleaseNameText.getText().length() > 0);
            }
        };
    }

    protected ModifyListener getTagTextModifyListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                // Spaces not allowed in tags.
                fBuildTag = fBuildTagText.getText().replace(" ", "_"); //$NON-NLS-1$//$NON-NLS-2$
            }
        };
    }

    protected SelectionListener getPrivateButtonListener() {
        return new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                fIsPrivate = fPrivateButton.getSelection();
            }
        };
    }

    protected Label createLabel(Composite composite, String text, Color color) {
        Label label = new Label(composite, SWT.WRAP);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 2;
        label.setLayoutData(gridData);
        label.setText(text);
        label.setForeground(color);
        return label;
    }

    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(BuildUIDialogMessages.CreateReleaseDialog_TITLE);
    }

}
