/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.internal.ui.AbstractBuildItemViewerComparator;
import com.ibm.team.build.internal.ui.BuildUIMessages;
import com.ibm.team.build.internal.ui.BuildUIPlugin;

/**
 * Dialog to edit which build engines support a build definition.
 */
public class EditBuildEnginesDialog extends AbstractEditItemsDialog {

    // internal use only fields
    private Map<String, String> fIdNameMap;

    // used to return information to the abstract portion of the dialog
    private String[] fSupportingBuildEngineIds;
    private String fCurrentProjectAreaName;
    private Collection<String> fProjectAreaEngineIds;

    private Map<String, IBuildEngine> fBuildEngines;

    /**
     * Create a dialog to edit which build engines support a build definition.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>.
     * @param buildEngineIdsToProjectAreaName
     *            the map of all build engine ids in the repository to the name
     *            of the project area that the build engine is associated with.
     *            May be <code>null</code>, or empty. In addition, project area
     *            names (map values) can be <code>null</code>.
     * @param supportingBuildEngineIds
     *            a collection of the id's for all of the build engines that
     *            support the build definition. May be <code>null</code>, or
     *            empty.
     * @param currentProjectAreaName
     *            the name of the project area which the build definition is
     *            associated with. May be <code>null</code>, however ids will
     *            then only be displayed in the "Show All Engines" table view.
     * @param buildEngines
     *            the map of all build engine ids in the repository to the
     *            engine item
     */
    public EditBuildEnginesDialog(Shell parentShell, Map<String, String> buildEngineIdsToProjectAreaName,
            Collection<String> supportingBuildEngineIds, String currentProjectAreaName, Map<String, IBuildEngine> buildEngines) {
        super(parentShell);
        fCurrentProjectAreaName = currentProjectAreaName;
        fBuildEngines = buildEngines;
        fProjectAreaEngineIds = new LinkedList<String>();
        if (buildEngineIdsToProjectAreaName != null) {
            fIdNameMap = buildEngineIdsToProjectAreaName;
            if (currentProjectAreaName != null && !fCurrentProjectAreaName.equals(BuildUIMessages.UNKNOWN_PROJECT_AREA)) {
                for (String engineId : buildEngineIdsToProjectAreaName.keySet()) {
                    String projectAreaName = buildEngineIdsToProjectAreaName.get(engineId);
                    if (fCurrentProjectAreaName.equals(projectAreaName)) {
                        fProjectAreaEngineIds.add(engineId);
                    }
                }
            }
        } else {
            fIdNameMap = new HashMap<String, String>();
        }

        if (supportingBuildEngineIds != null) {
            fSupportingBuildEngineIds = supportingBuildEngineIds.toArray(new String[supportingBuildEngineIds.size()]);
        } else {
            fSupportingBuildEngineIds = new String[0];
        }
    }

    private String formatWithProjectArea(Object element) {
        String engineId = element.toString();
        String areaName = fIdNameMap.get(engineId);
        if (areaName == null) {
            areaName = BuildUIMessages.UNKNOWN_PROJECT_AREA;
        }
        return NLS.bind(BuildUIMessages.EXTENDED_LABEL, engineId, areaName);
    }

    /**
     * @return the set of build engine ids whose support status with this build
     *         definition changed in the dialog.
     */
    public Set<String> getChangedBuildEngineIds() {
        return getChangedItems();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected String getDialogDescription() {
        return BuildUIDialogMessages.EditBuildEnginesDialog_DESCRIPTION;
    }

    @Override
    protected String getDialogTitle() {
        return BuildUIDialogMessages.EditBuildEnginesDialog_TITLE;
    }

    @Override
    protected String getShowAllLabel() {
        return BuildUIDialogMessages.EditBuildEnginesDialog_SHOW_ALL_LABEL;
    }

    @Override
    protected String getShowSubSetLabel() {
        return BuildUIDialogMessages.EditBuildEnginesDialog_SHOW_PROJECT_AREA_LABEL;
    }

    @Override
    protected LabelProvider getLabelProvider(final boolean isSubSetDisplayed) {
        return new LabelProvider() {
            @Override
            public final String getText(Object element) {
                if (isSubSetDisplayed) {
                    return super.getText(element);
                } else {
                    return formatWithProjectArea(element);
                }
            }
            
            @Override 
            public Image getImage(Object element) { 
                return BuildUIPlugin.getImage("icons/obj16/build_engine_obj.gif");  //$NON-NLS-1$
            }
        };
    }
    
    @Override
    protected ViewerComparator getTableViewerComparator() {
        return new AbstractBuildItemViewerComparator() {

            @Override
            protected String getTopProject() {
                return fCurrentProjectAreaName;
            }

            @Override
            protected String getProject(Object element) {
                return fIdNameMap.get(element);
            }
        };
    }

    @Override
    protected String[] getAllItems() {
        return fIdNameMap.keySet().toArray(new String[fIdNameMap.size()]);
    }

    @Override
    protected String[] getInitialItems() {
        return fSupportingBuildEngineIds;
    }

    @Override
    protected Collection<String> getSubSetItems() {
        return fProjectAreaEngineIds;
    }

    @Override
    protected boolean isStartInSubSet() {
        return fProjectAreaEngineIds.size() > 0
                && fProjectAreaEngineIds.containsAll(Arrays.asList(fSupportingBuildEngineIds));
    }
    
    @Override
    protected String getDescription(IStructuredSelection selection) {
        if (selection.size() == 1) {
            String engineId = (String) selection.getFirstElement();
            IBuildEngine engine = fBuildEngines.get(engineId);
            if (engine != null) {
                return engine.getDescription();
            }
        }
        return null;
    }
}
