/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.JFaceColors;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.build.internal.common.MicrosoftBuildItem;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.IHelpContextIds;

/**
 * A dialog for creating or editing MS Build item information
 */
public class MicrosoftBuildInputDataDialog extends Dialog {
    protected String fTitle;
    protected Shell fParentShell;

    protected Label fErrorMessageLabel;
    protected Text fBuildFileNameText;
    protected Text fBuildConfigurationText;
    protected Combo fBuildTypeCombo;
    protected Text fBuildLogFileText;
    protected Text fAdditionalArgumentsText;
    protected Text fWorkingDirectoryText;

    protected MicrosoftBuildItem fBuildItem;

    private String EMPTY = ""; //$NON-NLS-1$

    /**
     * Create a dialog to add or edit a build property.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>
     * @param title
     *            the dialog title; never <code>null</code>
     */
    public MicrosoftBuildInputDataDialog(Shell parentShell, String title) {
        this(parentShell, title, null);
    }

    /**
     * @param parentShell
     * @param title
     * @param buildItem
     */
    public MicrosoftBuildInputDataDialog(Shell parentShell, String title, MicrosoftBuildItem buildItem) {
        super(parentShell);
        ValidationHelper.validateNotNull("title", title); //$NON-NLS-1$

        setShellStyle(getShellStyle() | SWT.RESIZE);
        fTitle = title;
        fParentShell = parentShell;
        fBuildItem = buildItem;
    }

    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(fTitle);
    }

    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);
        composite.setLayout(new GridLayout(3, false));
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.widthHint = 500;
        composite.setLayoutData(gridData);

        String message = BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_ADD_NEW_SOLUTION;

        createLabel(composite, message, (Color) null);
        fErrorMessageLabel = createLabel(composite, "", JFaceColors.getErrorText(composite.getDisplay())); //$NON-NLS-1$

        fBuildFileNameText = createText(composite,
                BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_SOLUTION_LABEL, EMPTY, 2);
        fBuildFileNameText.addModifyListener(getBuildFileNameModifiedListener());
        fBuildFileNameText.setToolTipText(BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_SOLUTION_DESC);

        fBuildConfigurationText = createText(composite,
                BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_CONFIG_LABEL, EMPTY, 2);
        fBuildConfigurationText.addModifyListener(getBuildConfigModifiedListener());
        fBuildConfigurationText.setToolTipText(BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_CONFIG_DESC);

        fBuildTypeCombo = createBuildTypeCombo(composite,
                BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_BUILD_TYPE_LABEL, MicrosoftBuildItem.BUILD_STR,
                2);
        fBuildTypeCombo.setToolTipText(BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_BUILD_TYPE_DESC);

        fBuildLogFileText = createText(composite,
                BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_BUILD_OUTPUT_LABEL, EMPTY, 2);
        fBuildLogFileText.setToolTipText(BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_BUILD_OUTPUT_DESC);

        fAdditionalArgumentsText = createText(composite,
                BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_BUILD_ADDITIONAL_ARG_LABEL, EMPTY, 2);
        fAdditionalArgumentsText.setToolTipText(BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_BUILD_ADDITIONAL_ARG_DESC);

        fWorkingDirectoryText = createText(composite,
                BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_WORKING_DIR_LABEL, EMPTY, 2);
        fWorkingDirectoryText.setToolTipText(BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_WORKING_DIR_DESC);

        applyDialogFont(composite);

        PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, IHelpContextIds.HELP_CONTEXT_MS_BUILD_ITEM_DIALOG);

        return composite;
    }

    public void create() {
        super.create();
        updateValues();
        validate(true);
    }

    private void updateValues() {
        if (fBuildItem != null) {
            fBuildFileNameText.setText(fBuildItem.getBuildFileName());
            fBuildConfigurationText.setText(fBuildItem.getBuildConfig());
            if (fBuildItem.getBuildType() != null) {
                fBuildTypeCombo.setText(fBuildItem.getBuildType());
            }
            fAdditionalArgumentsText.setText(fBuildItem.getAdditionalArguments());
            fBuildLogFileText.setText(fBuildItem.getBuildLogFile());
            fWorkingDirectoryText.setText(fBuildItem.getWorkingDir());
        }
    }

    @Override
    protected void okPressed() {
        if (getErrorMessage() == null) {
            String buildType = fBuildTypeCombo.getText();
            if (fBuildItem == null) {
                fBuildItem = new MicrosoftBuildItem(fBuildFileNameText.getText().trim(),
                        fBuildConfigurationText.getText().trim(), buildType, fAdditionalArgumentsText.getText().trim(),
                        fBuildLogFileText.getText().trim(), fWorkingDirectoryText.getText().trim());
            } else {
                fBuildItem.setBuildFileName(fBuildFileNameText.getText().trim());
                fBuildItem.setBuildConfig(fBuildConfigurationText.getText().trim());
                fBuildItem.setBuildLogFile(fBuildLogFileText.getText().trim());
                fBuildItem.setBuildType(buildType);
                fBuildItem.setAdditionalArguments(fAdditionalArgumentsText.getText().trim());
                fBuildItem.setWorkingDir(fWorkingDirectoryText.getText().trim());
            }
        }
        super.okPressed();
    }

    protected Label createLabel(Composite composite, String text, Color color) {
        Label label = new Label(composite, SWT.WRAP);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 3;
        label.setLayoutData(gridData);
        label.setText(text);
        label.setForeground(color);
        return label;
    }

    protected Text createText(Composite parent, String labelText, String defaultText, int textHorizontalSpan) {
        Label label = new Label(parent, SWT.NONE);
        label.setText(labelText);
        label.setLayoutData(new GridData());

        Text text = new Text(parent, SWT.BORDER | SWT.SINGLE);
        text.setText(defaultText);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = textHorizontalSpan;
        text.setLayoutData(gridData);

        return text;
    }

    protected Combo createBuildTypeCombo(Composite parent, String labelText, String defaultText, int textHorizontalSpan) {
        Label label = new Label(parent, SWT.NONE);
        label.setText(labelText);
        label.setLayoutData(new GridData());

        Combo combo = new Combo(parent, SWT.NONE);
        String items[] = { MicrosoftBuildItem.BUILD_STR, MicrosoftBuildItem.REBUILD_STR };
        combo.setItems(items);
        combo.setText(defaultText);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = textHorizontalSpan;
        combo.setLayoutData(gridData);

        return combo;
    }

    private ModifyListener getBuildFileNameModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validate(false);
            }
        };
    }

    private ModifyListener getBuildConfigModifiedListener() {
        return new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                validate(false);
            }
        };
    }

    /**
     * Validates the state of the dialog input; disables the <code>OK</code>
     * button and optionally shows an error message if the dialog input state is
     * invalid.
     * 
     * @param suppressErrorMessage
     *            <code>true</code> to not show an error message even if the
     *            dialog state is invalid (used for initial display of the
     *            dialog); <code>false</code> to show the error message if the
     *            dialog state is invalid.
     */
    protected void validate(boolean suppressErrorMessage) {
        String errorMessage = getErrorMessage();

        if ((errorMessage == null) || (suppressErrorMessage)) {
            fErrorMessageLabel.setText(""); //$NON-NLS-1$
        } else {
            fErrorMessageLabel.setText(errorMessage);
        }
        fErrorMessageLabel.redraw();

        Button okButton = getButton(IDialogConstants.OK_ID);
        okButton.setEnabled(errorMessage == null);
    }

    protected String getErrorMessage() {

        if (fBuildFileNameText.getText().trim().equals(EMPTY)) {
            return BuildUIDialogMessages.MicrosoftSolutionInputDataDialog_SOLUTION_FILE_REQUIRED;
        } else {
            return null;
        }
    }

    /**
     * 
     * @return the <code>MicrosoftBuildItem</code> object
     */
    public MicrosoftBuildItem getBuildItem() {
        return fBuildItem;
    }

}
