/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2005, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.dialogs;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.IBuildAction;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildRequestHandle;
import com.ibm.team.build.common.model.IBuildRequestParams;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.common.model.query.IBaseBuildEngineQueryModel.IBuildEngineQueryModel;
import com.ibm.team.build.common.model.query.IBaseBuildResultQueryModel.IBuildResultQueryModel;
import com.ibm.team.build.internal.common.schedule.IBuildScheduleTaskProperties;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.IHelpContextIds;
import com.ibm.team.build.internal.ui.dialogs.definitions.BuildDefinitionSelectionDialog;
import com.ibm.team.build.internal.ui.helper.BuildDefinitionHandleHelper;
import com.ibm.team.build.internal.ui.helper.BuildUIPreferencesHelper;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.build.internal.ui.views.query.BuildQueryView;
import com.ibm.team.build.internal.ui.views.query.ShowBuildQueueAction;
import com.ibm.team.build.internal.ui.views.query.ShowHistoryAction;
import com.ibm.team.build.ui.dialogs.requestbuild.RequestBuildSection;
import com.ibm.team.build.ui.dialogs.requestbuild.RequestBuildSectionSite;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.query.IDataQuery;
import com.ibm.team.repository.common.query.IDataQueryPage;
import com.ibm.team.repository.common.query.IDataRow;

/**
 * Dialog for requesting a build.
 */
public class RequestBuildDialog extends TitleAreaDialog {
    
    /**
     * The key for persisting the most recent definition chosen.
     */
    public static final String PREFERENCE_DEFINITION = "mostRecentDefinition"; //$NON-NLS-1$

    /**
     * The key for the "showing the builds view after requesting a build"
     * preference.
     */
    public static final String PREFERENCE_SHOW_BUILDS_VIEW = "showBuildsViewAfterRequest"; //$NON-NLS-1$

    /**
     * The index position of the show the builds of the requested type in the
     * builds view in the combo box
     */
    public static final int SHOW_BUILDS_OF_REQUEST_TYPE = 0;

    /**
     * The index position of the show the build queue in the builds view in the
     * combo box
     */
    public static final int SHOW_BUILD_QUEUE = 1;

    /**
     * The index position of the option that submits the request and does
     * nothing in the combo box
     */
    public static final int DO_NOTHING = 2;

    /**
     * Track the original list of build properties
     * <p>
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected List<IBuildProperty> fOriginalBuildProperties;

    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected IBuildDefinitionHandle fBuildDefinitionHandle;

    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected IBuildDefinition fBuildDefinition;

    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected IBuildResult fBuildResult;

    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected IBuildResultHandle fBuildResultHandle;

    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected boolean fIsRebuild;

    /**
     * This field is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used by subclasses.
     */
    protected IBuildRequest fBuildRequest;

    private Job fRequestBuildJob;

    private Job fInitializationJob;

    protected Text fBuildDefinitionText;

    protected Combo fShowBuildsViewCombo;

    protected Button fBrowseButton;

    protected boolean fIsPersonalBuild;
    
    protected IBuildEngine fHandlingEngine;
    
    private IWorkbenchPartSite fPartSite;

    protected Map<Object, String> fWarningMessagesMap = new LinkedHashMap<Object, String>();
    protected Map<Object, String> fErrorMessagesMap = new LinkedHashMap<Object, String>();

    protected List<RequestBuildSection> fSections = new ArrayList<RequestBuildSection>();
    
    /**
     * Create the request build dialog. The dialog requests a build of the given
     * definition.
     * 
     * @param parentShell
     *            The parent shell for the dialog.
     * @param buildDefinitionHandle
     *            The type of build to request. May be <code>null</code>. If
     *            <code>null</code>, an attempt is made to retreive a definition
     *            from persisted storage. If that fails, the user is prompted to
     *            choose a definition.
     */
    public RequestBuildDialog(Shell parentShell, IBuildDefinitionHandle buildDefinitionHandle) {
        super(parentShell);
        fBuildDefinitionHandle = buildDefinitionHandle;

        setShellStyle(getShellStyle() | SWT.RESIZE | SWT.MAX);
        BuildUIPlugin.getDefault().getPreferenceStore().setDefault(PREFERENCE_SHOW_BUILDS_VIEW, true);
    }

    /**
     * Create the request build dialog using an existing build result. Creating
     * the dialog in this way requests a rebuild using the same generic
     * properties in the given build result.
     * 
     * @param parentShell
     *            The parent shell for the dialog.
     * @param buildResultHandle
     *            The build result. Request a new build of the same type.
     * @param isRebuild
     *            Whether or not to rebuild using the same properties as the
     *            given build result.
     */
    public RequestBuildDialog(Shell parentShell, IBuildResultHandle buildResultHandle, boolean isRebuild) {
        super(parentShell);
        fBuildResultHandle = buildResultHandle;
        fIsRebuild = isRebuild;

        setShellStyle(getShellStyle() | SWT.RESIZE | SWT.MAX);
        BuildUIPlugin.getDefault().getPreferenceStore().setDefault(PREFERENCE_SHOW_BUILDS_VIEW, true);
    }
    
    
    /**
     * Get the text to be displayed at the top of the request dialog. Subclasses extending this dialog can provide
     * their own header text.
     * @return The header text to be displayed at the top of the request dialog
     */
    protected String getHeaderText() {
        return BuildUIDialogMessages.REQUEST_BUILD_HEADER;
    }
    
    /**
     * Get the ID for the dialog help context.  Subclasses extending this dialog can provide their own F1 help. 
     * @return The dialog help context ID
     */
    protected String getHelpId() {
        return IHelpContextIds.HELP_CONTEXT_REQUEST_BUILD_DIALOG;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        setTitle(BuildUIDialogMessages.RequestBuildDialog_REQUEST_BUILD_TITLE); 
        setMessage(getHeaderText(), IMessageProvider.NONE);
        
        Composite dialogAreaComposite = (Composite) super.createDialogArea(parent);
        Composite mainComposite = new Composite(dialogAreaComposite, SWT.NONE);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(mainComposite);
        GridLayoutFactory.swtDefaults().numColumns(3).equalWidth(false).applyTo(mainComposite);
        
        // Vertical space.
        Label label = new Label(mainComposite, SWT.NONE);
        GridData gridData = new GridData();
        gridData.horizontalSpan = 3;
        gridData.heightHint = 10;
        label.setLayoutData(gridData);

        label = new Label(mainComposite, SWT.NONE);
        label.setText(BuildUIDialogMessages.BUILD_DEFINITION_LABEL);
        label.setLayoutData(new GridData());

        fBuildDefinitionText = new Text(mainComposite, SWT.BORDER);
        fBuildDefinitionText.setText(BuildUIDialogMessages.PENDING_LABEL);
        fBuildDefinitionText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        fBuildDefinitionText.setEditable(false);

        fBrowseButton = new Button(mainComposite, SWT.NONE);
        fBrowseButton.setText(BuildUIDialogMessages.RequestBuildDialog_BROWSE);
        fBrowseButton.addSelectionListener(getBrowseButtonListener());
        fBrowseButton.setEnabled(!fIsRebuild);

        // Vertical space.
        label = new Label(mainComposite, SWT.NONE);
        gridData = new GridData();
        gridData.horizontalSpan = 3;
        gridData.heightHint = 10;
        label.setLayoutData(gridData);

        fSections = createSections();
        
        Composite sectionsComposite = new Composite(mainComposite, SWT.NONE);
        GridDataFactory.fillDefaults().grab(true, true).span(3, 1).applyTo(sectionsComposite);
        GridLayoutFactory.fillDefaults().applyTo(sectionsComposite);
        
        for (RequestBuildSection section : fSections) {
            section.createContent(sectionsComposite);
        }

        // Vertical space.
        label = new Label(mainComposite, SWT.NONE);
        gridData = new GridData();
        gridData.horizontalSpan = 3;
        gridData.heightHint = 10;
        label.setLayoutData(gridData);

        Composite composite = new Composite(mainComposite, SWT.NONE);
        GridLayoutFactory.fillDefaults().extendedMargins(0, 0, 0, 5).numColumns(2).applyTo(composite);
        GridDataFactory.fillDefaults().grab(true, false).span(3, SWT.DEFAULT).applyTo(composite);

        label = new Label(composite, SWT.NONE);
        label.setText(BuildUIDialogMessages.RequestBuildDialog_AFTER_REQUEST_LABEL);
        GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(label);

        fShowBuildsViewCombo = new Combo(composite, SWT.READ_ONLY);
        fShowBuildsViewCombo.setItems(new String[] {
                BuildUIDialogMessages.RequestBuildDialog_SHOW_BUILDS_OF_REQUESTED_TYPE,
                BuildUIDialogMessages.RequestBuildDialog_SHOW_BUILD_QUEUE,
                BuildUIDialogMessages.RequestBuildDialog_DO_NOTHING, });
        fShowBuildsViewCombo.setEnabled(false);
        fShowBuildsViewCombo.select(BuildUIPlugin.getDefault().getPreferenceStore().getInt(PREFERENCE_SHOW_BUILDS_VIEW));
        fShowBuildsViewCombo.addSelectionListener(getShowBuildsViewButtonListener());
        GridDataFactory.fillDefaults().grab(true, false).applyTo(fShowBuildsViewCombo);

        initializeInBackground();

        applyDialogFont(dialogAreaComposite);

        PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, getHelpId());

        return dialogAreaComposite;
    }

    // Methods that display/remove warning/error messages
    private void addWarningMessage(Object id, String warningMessage) {
        fWarningMessagesMap.put(id, warningMessage);
        updateMessage();
    }

    private void removeWarningMessage(Object id) {
        fWarningMessagesMap.remove(id);
        updateMessage();
    }

    private void addErrorMessage(Object id, String errorMessage) {
        fErrorMessagesMap.put(id, errorMessage);
        updateMessage();
    }

    private void removeErrorMessage(Object id) {
        fErrorMessagesMap.remove(id);
        updateMessage();
    }

    protected String getWarningMessageString() {
        return fWarningMessagesMap.values().iterator().next();
    }

    protected String getErrorMessageString() {
        return fErrorMessagesMap.values().iterator().next();
    }

    protected void updateMessage() {
        if (!fErrorMessagesMap.isEmpty()) {
            setMessage(getErrorMessageString(), IMessageProvider.ERROR);
        } else if (!fWarningMessagesMap.isEmpty()) {
            setMessage(getWarningMessageString(), IMessageProvider.WARNING);
        } else {
            setMessage(getHeaderText(), IMessageProvider.NONE);
        }
    }
    
    /**
     * Adds the registered sections to the list of sections.
     */
    protected List<RequestBuildSection> createSections() {
        RequestBuildSectionSite site = createSectionSite();

        List<RequestBuildSection> sections = new ArrayList<RequestBuildSection>();
        RequestBuildDialogSectionExtension[] extensions = RequestBuildDialogSectionExtensionManager.getInstance().getRequestBuildDialogSectionExtensions();
        for (RequestBuildDialogSectionExtension extension : extensions) {
            try {
                RequestBuildSection buildSection = extension.createSectionFactory().createRequestBuildSection(site);
                // Handle special case where the properties section needs to
                // register a context menu for extension, not the best solution.
                if (buildSection instanceof PropertiesSection) {
                    ((PropertiesSection) buildSection).setWorkbenchPartSite(fPartSite);
                }
                sections.add(buildSection);
            } catch (Exception e) {
                BuildUIPlugin.log(e);
            }
        }
        
        return sections;
    }

    /**
     * Creates the site for sections.
     */
    protected RequestBuildSectionSite createSectionSite() {
        return new RequestBuildSectionSite() {

            @Override
            public void setProperty(String propertyName, Object value) {
                if (RequestBuildSectionSite.PROP_PERSONAL_BUILD.equals(propertyName)) {
                    fIsPersonalBuild = (Boolean) value;
                } else if (RequestBuildSectionSite.PROP_HANDLING_ENGINE.equals(propertyName)) {
                    fHandlingEngine = (IBuildEngine) value;
                }
                notifyPropertyChanged(propertyName);
                RequestBuildDialog.this.revalidate();
            }
            
            @Override
            public void revalidate() {
                RequestBuildDialog.this.revalidate();
            }
            
            @Override
            public void layoutChanged() {
                handleLayoutChanged();
            }
            
            @Override
            public boolean isRebuild() {
                return fIsRebuild;
            }
            
            @Override
            public boolean isPersonalBuild() {
                return fIsPersonalBuild;
            }
            
            @Override
            public IBuildResult getBuildResult() {
                return fBuildResult;
            }
            
            @Override
            public IBuildRequest getBuildRequest() {
                return fBuildRequest;
            }
            
            @Override
            public IBuildDefinition getBuildDefinition() {
                return fBuildDefinition;
            }

            @Override
            public ITeamRepository getTeamRepository() {
                return fBuildDefinition == null ? null : (ITeamRepository) fBuildDefinition.getOrigin();
            }

            @Override
            public List<IBuildProperty> getOriginalBuildProperties() {
                return fOriginalBuildProperties;
            }

            @Override
            public IBuildEngine getHandlingEngine() {
                return fHandlingEngine;
            }

            @Override
            public void addErrorMessage(Object id, String errorMessage) {
                RequestBuildDialog.this.addErrorMessage(id, errorMessage);                
            }
            
            @Override
            public void removeErrorMessage(Object id) {
                RequestBuildDialog.this.removeErrorMessage(id);                
            } 
            
            @Override
            public void addWarningMessage(Object id, String warningMessage) {
                RequestBuildDialog.this.addWarningMessage(id, warningMessage);                
            } 
            
            @Override
            public void removeWarningMessage(Object id) {
                RequestBuildDialog.this.removeWarningMessage(id);                
            } 
        };
    }

    protected SelectionListener getBrowseButtonListener() {
        return new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                BuildDefinitionSelectionDialog selectionDialog = getSelectionDialog();

                if (selectionDialog.open() == Window.OK) {
                    IBuildDefinition definition = selectionDialog.getFirstSelectedBuildDefinition();
                    fBuildDefinitionHandle = definition;
                    fBuildDefinition = definition;

                    notifyPropertyChanged(RequestBuildSectionSite.PROP_BUILD_DEFINITION);
                    
                    handleInitializationComplete();
                }
            }
        };
    }

    /**
     * Notifies sections that the specified dialog property has changed.
     * 
     * @param propertyName the name of the dialog property
     */
    protected void notifyPropertyChanged(String propertyName) {
        for (RequestBuildSection section : fSections) {
            section.handlePropertyChanged(propertyName);
        }
    }

    protected SelectionListener getShowBuildsViewButtonListener() {
        return new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                BuildUIPlugin.getDefault().getPreferenceStore().setValue(PREFERENCE_SHOW_BUILDS_VIEW,
                        fShowBuildsViewCombo.getSelectionIndex());
                BuildUIPlugin.getDefault().savePluginPreferences();
            }
        };
    }

    protected BuildDefinitionSelectionDialog getSelectionDialog() {
        return new BuildDefinitionSelectionDialog(getShell(),
                BuildUIDialogMessages.RequestBuildDialog_SELECT_DEFINITION_TITLE,
                BuildUIDialogMessages.RequestBuildDialog_SELECT_DEFINITION_MESSAGE, false, false);
    }

    
    private void handleLayoutChanged() {
        getShell().pack(true);
    }

    protected void handleInitializationComplete() {
        if (!fBuildDefinitionText.isDisposed()) {
            fShowBuildsViewCombo.setEnabled(true);

            fBuildDefinitionText.setText(fBuildDefinition.getId());

            // Save the definition properties so we can later determine if any
            // properties have been added or removed.
            fOriginalBuildProperties = fBuildDefinition.getProperties();

            // Get a working copy, so any property changes made by the user are
            // performed on the working copy.
            fBuildDefinition = (IBuildDefinition) fBuildDefinition.getWorkingCopy();

            if (fIsRebuild) {
                // Since it's a rebuild, discard the properties from the
                // definition and use the properties from the request's
                // definition instance.

                List<IBuildProperty> requestProperties = fBuildRequest.getBuildDefinitionInstance().getProperties();

                // clear the properties in the new working copy
                fBuildDefinition.getProperties().clear();
                for (IBuildProperty requestProperty : requestProperties) {
                    // Copy each property from the original request to the
                    // new working copy. Omit the scheduled build property.

                    if (!requestProperty.getName().equals(IBuildScheduleTaskProperties.PROPERTY_SCHEDULED_BUILD)) {
                        IBuildProperty newProperty = requestProperty.copyProperty();
                        fBuildDefinition.getProperties().add(newProperty);
                    }
                }
            }

            for (RequestBuildSection section : fSections) {
                section.initializationComplete();
            }

            revalidate();

            getContents().getParent().pack();
        }
    }

    protected IBuildDefinition getBuildDefinition(IBuildDefinitionHandle buildDefinitionHandle, IProgressMonitor monitor)
            throws IllegalArgumentException, TeamRepositoryException {
        ITeamRepository teamRepository = (ITeamRepository) buildDefinitionHandle.getOrigin();
        return (IBuildDefinition) teamRepository.itemManager().fetchCompleteItem(buildDefinitionHandle,
                IItemManager.REFRESH, monitor);
    }

    private void initializeInBackground() {
        ITeamRepository jobTeamRepository = null;

        if (fBuildResultHandle != null) {
            jobTeamRepository = (ITeamRepository) fBuildResultHandle.getOrigin();
        } else if (fBuildDefinitionHandle != null) {
            jobTeamRepository = (ITeamRepository) fBuildDefinitionHandle.getOrigin();
        } else if (getPersistedDefinition() != null) {
            jobTeamRepository = (ITeamRepository) getPersistedDefinition().getOrigin();
        }

        String jobName = BuildUIDialogMessages.RequestBuildDialog_JOB_LABEL_INITIALIZING;
        fInitializationJob = new TeamBuildJob(jobName, true, jobTeamRepository) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                if (fBuildResultHandle != null) {
                    ITeamRepository teamRepository = (ITeamRepository) fBuildResultHandle.getOrigin();
                    fBuildResult = (IBuildResult) teamRepository.itemManager().fetchCompleteItem(fBuildResultHandle,
                            IItemManager.REFRESH, monitor);
                    fBuildDefinitionHandle = fBuildResult.getBuildDefinition();
                }

                boolean isPersistedDefinition = false;

                if (fBuildDefinitionHandle == null) {
                    fBuildDefinitionHandle = getPersistedDefinition();
                    isPersistedDefinition = true;
                }

                /*
                 * Try to fetch the definition.
                 */
                if (fBuildDefinitionHandle != null) {
                    try {
                        fBuildDefinition = getBuildDefinition(fBuildDefinitionHandle, monitor);
                    } catch (ItemNotFoundException exception) {
                        if (isPersistedDefinition) {
                            /*
                             * Persisted definition no longer exists, so clear
                             * it.
                             */
                            fBuildDefinitionHandle = null;
                            BuildUIPreferencesHelper.storeBuildDefinitionPreferences(new IBuildDefinition[0],
                                    PREFERENCE_DEFINITION);
                        } else {
                            /*
                             * The user-specified definition was deleted, throw
                             * exception.
                             */
                            throw exception;
                        }
                    }
                }

                if (fBuildDefinitionHandle == null) {
                    /*
                     * No build definition was initially provided to the dialog,
                     * and no definition could be retrieved from the persisted
                     * store. The user will have to browse to select a
                     * definition.
                     */
                    Display.getDefault().asyncExec(new Runnable() {
                        public void run() {
                            if (!fBuildDefinitionText.isDisposed()) {
                                fBuildDefinitionText.setText(BuildUIDialogMessages.RequestBuildDialog_NO_DEFINITION_SELECTED);
                            }
                        }
                    });

                    return Status.OK_STATUS;
                }

                /*
                 * If this is a rebuild, we need to fetch the build request object
                 * containing the build definition instance so we can access the
                 * build properties.
                 */
                if (fIsRebuild) {
                    fBuildRequest = getBuildRequest(fBuildResult, monitor);
                    fIsPersonalBuild = fBuildResult.isPersonalBuild();
                }

                for (RequestBuildSection section : fSections) {
                    section.initialize(monitor);
                }
                
                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        handleInitializationComplete();
                    }
                });
                return Status.OK_STATUS;
            }

            @Override
            protected void jobFinished(final IStatus status) {
                if (status.getSeverity() != IStatus.OK) {
                    Display.getDefault().asyncExec(new Runnable() {
                        public void run() {
                            close();
                        }
                    });
                }
                handleInitializationJobDone(this, status);
            }
        };

        fInitializationJob.schedule();
    }


    /**
     * Get a persisted definition.
     * 
     * @return The definition. If a persisted definition is not found, return
     *         <code>null</code>.
     */
    private IBuildDefinitionHandle getPersistedDefinition() {
        IBuildDefinitionHandle handle = null;

        List<BuildDefinitionHandleHelper> handleHelpers = BuildUIPreferencesHelper.getBuildDefinitionPreferences(PREFERENCE_DEFINITION);
        if (!handleHelpers.isEmpty()) {
            BuildDefinitionHandleHelper handleHelper = handleHelpers.get(0);
            IBuildDefinitionHandle definitionHandle = handleHelper.getBuildDefinitionHandle();

            ITeamRepository teamRepository = (ITeamRepository) definitionHandle.getOrigin();
            if (teamRepository != null) {
                if (teamRepository.loggedIn()) {
                    handle = definitionHandle;
                }
            }
        }

        return handle;
    }

    protected boolean isDefinitionSupported(IBuildDefinition buildDefinition, IProgressMonitor monitor)
            throws IllegalArgumentException, TeamRepositoryException {

        ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();

        ITeamBuildClient client = (ITeamBuildClient) teamRepository.getClientLibrary(ITeamBuildClient.class);

        IBuildEngineQueryModel buildEngineQueryModel = IBuildEngineQueryModel.ROOT;
        IDataQuery query = IDataQuery.FACTORY.newInstance(buildEngineQueryModel);
        query.filter(buildEngineQueryModel.active()._isTrue()._and(
                buildEngineQueryModel.supportedBuildDefinitions().itemId()._eq(query.newUUIDArg())));
        query.select(query.count());

        IDataQueryPage page = client.queryData(query, new Object[] { buildDefinition.getItemId() }, 1, monitor);
        IDataRow row = page.getRow(0);
        return row.getInt(0) > 0;

    }

    /**
     * This method is not meant to be overridden by clients. It is protected to
     * enable testing.
     */
    protected void handleInitializationJobDone(TeamBuildJob job, IStatus status) {
        // Intentionally empty
    }

    protected void handleUnsupportedDefinition(IBuildDefinition buildDefinition) {
        MessageDialog dialog = getErrorDialog(BuildUIDialogMessages.RequestBuildDialog_ERROR_DIALOG_TITLE, NLS.bind(
                BuildUIDialogMessages.RequestBuildDialog_NO_SUPPORTING_ENGINES, buildDefinition.getId()));
        dialog.open();
    }

    protected void handleNoEnginesSupportingRequests(IBuildDefinition buildDefinition) {
        MessageDialog dialog = getErrorDialog(BuildUIDialogMessages.RequestBuildDialog_ERROR_DIALOG_TITLE, NLS.bind(
                BuildUIDialogMessages.RequestBuildDialog_NO_ENGINES_SUPPORT_REQUESTS, buildDefinition.getId()));
        dialog.open();
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected MessageDialog getErrorDialog(String title, String message) {
        return new MessageDialog(getParentShell(), title, null, message, MessageDialog.ERROR,
                new String[] { IDialogConstants.OK_LABEL }, 0);
    }

    /**
     * Get the build request associated with the result.
     * 
     * @param buildResult
     *            The result to look in.
     * @param monitor
     *            Progress monitor.
     * @return The build request.
     * @throws TeamRepositoryException
     */
    protected IBuildRequest getBuildRequest(IBuildResult buildResult, IProgressMonitor monitor)
            throws TeamRepositoryException {

        List<IBuildRequestHandle> buildRequestHandles = buildResult.getBuildRequests();
        ITeamRepository teamRepository = (ITeamRepository) buildResult.getOrigin();

        List<IBuildRequest> buildRequests = teamRepository.itemManager().fetchCompleteItems(buildRequestHandles,
                IItemManager.DEFAULT, monitor);

        IBuildRequest buildRequest = null;

        for (IBuildRequest request : buildRequests) {
            if (request != null && request.getBuildAction().getAction().equals(IBuildAction.REQUEST_BUILD)) {
                buildRequest = request;
            }
        }

        return buildRequest;
    }

    protected Label createLabel(Composite composite, String text, Color color) {
        Label label = new Label(composite, SWT.WRAP);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 3;
        label.setLayoutData(gridData);
        label.setText(text);
        label.setForeground(color);
        return label;
    }

    @Override
    protected void okPressed() {
        requestBuild();
        super.okPressed();
    }

    /**
     * Issue the build request.  Called when the OK button is pressed.
     */
    protected void requestBuild() {
        requestBuild(fBuildDefinition);
    }

    @Override
    protected void cancelPressed() {
        if (fInitializationJob != null) {
            fInitializationJob.cancel();
        }

        if (fRequestBuildJob != null) {
            fRequestBuildJob.cancel();
        }

        super.cancelPressed();
    }

    /**
     * Get a message for the confirmation dialog.
     * 
     * @param buildDefinition
     *            The build definition.
     * @param inProgress
     *            Whether or not a build is currently in progress.
     * @param pendingRequest
     *            Whether or not a request is already in the queue.
     * @return The confirmation message.
     */
    private String getConfirmationMessage(IBuildDefinition buildDefinition, boolean inProgress, boolean pendingRequest) {
        String message = null;
        if (inProgress) {
            message = NLS.bind(BuildUIDialogMessages.RequestBuildDialog_BUILD_ALREADY_IN_PROGRESS,
                    buildDefinition.getId());
        } else if (pendingRequest) {
            message = NLS.bind(BuildUIDialogMessages.RequestBuildDialog_REQUEST_ALREADY_IN_QUEUE,
                    buildDefinition.getId());
        }
        return message;
    }

    protected boolean isBuildInState(BuildState buildState, IBuildDefinition buildDefinition,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {
        ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();
        ITeamBuildClient buildClient = (ITeamBuildClient) teamRepository.getClientLibrary(ITeamBuildClient.class);
        IBuildResultQueryModel buildResultQueryModel = IBuildResultQueryModel.ROOT;
        IDataQuery query = IDataQuery.FACTORY.newInstance(buildResultQueryModel);
        query.filter(buildResultQueryModel.buildState()._eq(buildState.name())._and( // hardcode state for perf (only 2 cases)
                buildResultQueryModel.buildDefinition()._eq(query.newItemHandleArg())));

        query.select(query.count());
        IDataQueryPage page = buildClient.queryData(query, new Object[] { buildDefinition }, 1,
                progressMonitor);
        IDataRow row = page.getRow(0);
        return row.getInt(0) > 0;
    }

    /**
     * Display a confirmation dialog asking the user whether they really want to
     * request a build, because there are already existing builds running or in
     * the queue.
     * <p>
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     * 
     * @param buildDefinition
     *            The build definition.
     * @param inProgress
     *            Whether or not a build is currently in progress.
     * @param pendingRequest
     *            Whether or not a request is already in the queue.
     * @return True if the user still wants to request a build; false otherwise.
     */
    protected boolean displayConfirmationPreExistingBuilds(final IBuildDefinition buildDefinition,
            final boolean inProgress, final boolean pendingRequest) {

        final boolean[] shouldMakeNewRequest = new boolean[1];
        shouldMakeNewRequest[0] = true;

        if (inProgress || pendingRequest) {
            Display.getDefault().syncExec(new Runnable() {
                public void run() {
                    MessageDialog dialog = getMessageDialog(getParentShell(),
                            BuildUIDialogMessages.RequestBuildDialog_REQUEST_BUILD_TITLE, null, getConfirmationMessage(
                                    buildDefinition, inProgress, pendingRequest), MessageDialog.QUESTION, new String[] {
                                    IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL }, 1);
                    if (dialog.open() != Window.OK) {
                        shouldMakeNewRequest[0] = false;
                    }
                }
            });
        }
        return shouldMakeNewRequest[0];
    }

    /**
     * Display a confirmation dialog asking the user whether they really want to
     * request a build, because there is no build engine that is responsive.
     * <p>
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     * 
     * @param buildDefinition
     *            The build definition.
     * @return True if the user still wants to request a build; false otherwise.
     */
    protected boolean displayConfirmationUnresponsiveEngine(final IBuildDefinition buildDefinition) {

        final boolean[] shouldMakeNewRequest = new boolean[1];
        shouldMakeNewRequest[0] = true;

        Display.getDefault().syncExec(new Runnable() {
            public void run() {
                String message = NLS.bind(BuildUIDialogMessages.RequestBuildDialog_ENGINES_NOT_PROCESSING,
                        buildDefinition.getId());

                MessageDialog dialog = getMessageDialog(getParentShell(),
                        BuildUIDialogMessages.RequestBuildDialog_REQUEST_BUILD_TITLE, null, message,
                        MessageDialog.WARNING, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL },
                        1);

                if (dialog.open() != Window.OK) {
                    shouldMakeNewRequest[0] = false;
                }
            }
        });
        return shouldMakeNewRequest[0];
    }

    /**
     * Display a confirmation dialog asking the user whether they really want to
     * request a build, because there are already existing builds running or in
     * the queue.
     * <p>
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     * 
     * @param buildDefinition
     *            The build definition.
     * @param inProgress
     *            Whether or not a build is currently in progress.
     * @param pendingRequest
     *            Whether or not a request is already in the queue.
     * @return True if the user still wants to request a build; false otherwise.
     */
    protected boolean displayChangeShowPersonalBuildsInViewConfirmation() {
        final boolean[] shouldShowPersonalBuilds = { false };
        Display.getDefault().syncExec(new Runnable() {
            public void run() {
                MessageDialog dialog = getMessageDialog(getParentShell(),
                        BuildUIDialogMessages.RequestBuildDialog_OPENING_BUILDS_VIEW_DIALOG_TITLE, null,
                        BuildUIDialogMessages.RequestBuildDialog_SHOULD_SHOW_PERSONAL_BUILDS_QUESTION_MESSAGE,
                        MessageDialog.QUESTION, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL },
                        1);
                if (dialog.open() == Window.OK) {
                    shouldShowPersonalBuilds[0] = true;
                }
            }
        });
        
        return shouldShowPersonalBuilds[0];
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected MessageDialog getMessageDialog(Shell parentShell, String dialogTitle, Image dialogTitleImage,
            String dialogMessage, int dialogImageType, String[] dialogButtonLabels, int defaultIndex) {

        return new MessageDialog(parentShell, dialogTitle, dialogTitleImage, dialogMessage, dialogImageType,
                dialogButtonLabels, defaultIndex);
    }

    /**
     * Request the build with the specified build definition.
     * 
     * @param buildDefinition 
     *      the build definition to be requested
     */
    protected void requestBuild(final IBuildDefinition buildDefinition) {
        final ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();

        // Persist the definition that will be requested.
        BuildUIPreferencesHelper.storeBuildDefinitionPreferences(new IBuildDefinition[] { buildDefinition },
                PREFERENCE_DEFINITION);

        fRequestBuildJob = new TeamBuildJob(BuildUIDialogMessages.REQUEST_BUILD_JOB_TITLE, true) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                return requestBuildJobImpl(buildDefinition, teamRepository, monitor);
            }
        };

        fRequestBuildJob.addJobChangeListener(new JobChangeAdapter() {
            @Override
            public void done(IJobChangeEvent event) {
                event.getJob().removeJobChangeListener(this);
                handleRequestJobCompleted(event);
            }
        });

        fRequestBuildJob.schedule();
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected IStatus requestBuildJobImpl(final IBuildDefinition buildDefinition, ITeamRepository teamRepository,
            IProgressMonitor monitor) throws Exception {

        // TODO: if handling engine given, check whether it supports the
        // definition, is active, has request processing enabled, and is
        // responsive, warning the user if not, but
        // allowing the request to go ahead if they say so.
        
        if (fHandlingEngine == null) {
            
            /*
             * Check to make sure there is an engine supporting this definition.
             */
            if (!isDefinitionSupported(buildDefinition, monitor)) {
                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        handleUnsupportedDefinition(buildDefinition);
                    }
                });
                return Status.OK_STATUS;
            }
    
            /*
             * Check to make sure there is an engine that supports requests.
             */
            if (!anyEnginesSupportingRequests(buildDefinition, monitor)) {
                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        handleNoEnginesSupportingRequests(buildDefinition);
                    }
                });
                return Status.OK_STATUS;
            }
    
            /*
             * Check to see if there are any responsive engines.
             */
            if (!anyResponsiveEngines(buildDefinition, monitor)) {
                if (!displayConfirmationUnresponsiveEngine(buildDefinition)) {
                    // Don't request a build.
                    return Status.OK_STATUS;
                }
            }
        }

        if (!fIsPersonalBuild) {
            /*
             * For non-personal builds, check for in progress or pending builds.
             */

            boolean inProgress = isBuildInState(BuildState.IN_PROGRESS, buildDefinition, monitor);
            boolean pendingRequest = false;

            if (!inProgress) {
                // Nothing in progress. Check for a pending request.
                pendingRequest = isBuildInState(BuildState.NOT_STARTED, buildDefinition, monitor);
            }

            if (inProgress || pendingRequest) {
                if (!displayConfirmationPreExistingBuilds(buildDefinition, inProgress, pendingRequest)) {
                    // Don't request a build.
                    return Status.OK_STATUS;
                }
            }
        }
        
        for (RequestBuildSection section : fSections) {
            section.applyProperties(buildDefinition);
        }

        // compute the build properties change sets

        // get the new build properties from the working copy we've been editing
        List<IBuildProperty> newBuildProperties = buildDefinition.getProperties();

        Vector<IBuildProperty> deletedBuildProperties = new Vector<IBuildProperty>();

        if (fOriginalBuildProperties != null) {
            for (IBuildProperty originalProperty : fOriginalBuildProperties) {
                boolean deleted = true;
                for (IBuildProperty newProperty : newBuildProperties) {
                    if (newProperty.getName().equals(originalProperty.getName())) {
                        deleted = false;
                        break;
                    }
                }

                if (deleted) {
                    deletedBuildProperties.add(originalProperty);
                }
            }
        }

        IBuildProperty[] newPropertiesArray = newBuildProperties.toArray(new IBuildProperty[newBuildProperties.size()]);
        IBuildProperty[] deletedPropertiesArray = deletedBuildProperties.toArray(new IBuildProperty[deletedBuildProperties.size()]);

        requestBuild(teamRepository, buildDefinition, newPropertiesArray, deletedPropertiesArray, true, monitor);

        final IProcessArea processArea = (IProcessArea) teamRepository.itemManager().fetchCompleteItem(
                buildDefinition.getProcessArea(), IItemManager.DEFAULT, monitor);
        Display.getDefault().asyncExec(new Runnable() {
            public void run() {
                handleAsyncShowBuildsView(processArea.getProjectArea(), buildDefinition);
            }
        });
        return Status.OK_STATUS;
    }

    private boolean anyEnginesSupportingRequests(IBuildDefinition buildDefinition, IProgressMonitor monitor)
            throws IllegalArgumentException, TeamRepositoryException {

        ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();
        ITeamBuildClient client = (ITeamBuildClient) teamRepository.getClientLibrary(ITeamBuildClient.class);
        return client.anyBuildEnginesWithRequestProcessingEnabled(buildDefinition, monitor);
    }

    protected boolean anyResponsiveEngines(IBuildDefinitionHandle buildDefinition, IProgressMonitor monitor)
            throws IllegalArgumentException, TeamRepositoryException {

        ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();
        ITeamBuildClient client = (ITeamBuildClient) teamRepository.getClientLibrary(ITeamBuildClient.class);
        return client.anyResponsiveBuildEngines(buildDefinition, monitor);
    }

    protected void handleAsyncShowBuildsView(IProjectAreaHandle projectAreaHandle, IBuildDefinition buildDefinition) {
        int buildsViewPreference = BuildUIPlugin.getDefault().getPreferenceStore().getInt(PREFERENCE_SHOW_BUILDS_VIEW);
        if (buildsViewPreference != DO_NOTHING) {
            showBuildsView(projectAreaHandle, buildDefinition, buildsViewPreference);
        }
    }

    protected void showBuildsView(IProjectAreaHandle projectAreaHandle, IBuildDefinition buildDefinition,
            int buildsViewOption) {
        Boolean showPersonalBuilds = null;
        if (fIsPersonalBuild && !BuildQueryView.globalShowPersonalBuilds() && !BuildQueryView.globalShowMyPersonalBuilds()) {
            showPersonalBuilds = displayChangeShowPersonalBuildsInViewConfirmation();
        }
        IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        if (window != null) {
            IWorkbenchPage page = window.getActivePage();
            if (page != null) {
                Action action;
                if (buildsViewOption == SHOW_BUILD_QUEUE) {
                    action = new ShowBuildQueueAction(projectAreaHandle, showPersonalBuilds);
                } else {
                    action = new ShowHistoryAction(buildDefinition, showPersonalBuilds);
                }
                action.run();
            }
        }
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected void handleRequestJobCompleted(IJobChangeEvent event) {
        /*
         * Default implementation does nothing. Test classes should override to
         * be notified when the job is completed.
         */
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code> to
     * enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected IBuildRequest requestBuild(ITeamRepository teamRepository, IBuildDefinitionHandle buildDefinitionHandle,
            IBuildProperty[] newAndModifiedBuildProperties, IBuildProperty[] deletedBuildProperties,
            boolean allowDuplicateRequests, IProgressMonitor monitor) throws IllegalArgumentException,
            TeamRepositoryException {

        IBuildRequestParams params = BuildItemFactory.createBuildRequestParams();
        params.setBuildDefinition(buildDefinitionHandle);
        params.getNewOrModifiedBuildProperties().addAll(Arrays.asList(newAndModifiedBuildProperties));
        params.getDeletedBuildProperties().addAll(Arrays.asList(deletedBuildProperties));
        params.setAllowDuplicateRequests(allowDuplicateRequests);
        params.setPersonalBuild(fIsPersonalBuild);
        if (fHandlingEngine != null) {
            params.getPotentialHandlers().add(fHandlingEngine);
        }
        return ClientFactory.getTeamBuildRequestClient(teamRepository).requestBuild(params, monitor);
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        if (fIsRebuild) {
            newShell.setText(BuildUIDialogMessages.RequestBuildDialog_DIALOG_TITLE_REQUEST_REBUILD);
        } else {
            newShell.setText(BuildUIDialogMessages.RequestBuildDialog_DIALOG_TITLE_REQUEST_BUILD);
        }
    }

    @Override
    public void create() {
        super.create();
        Button okButton = getButton(IDialogConstants.OK_ID);
        okButton.setText(BuildUIDialogMessages.REQUEST_BUILD_SUBMIT);
        okButton.setEnabled(false);

        Button cancelButton = getButton(IDialogConstants.CANCEL_ID);
        getShell().setDefaultButton(cancelButton);
        // Set the focus on the cancel button, so it's not in the build
        // definition text field.
        cancelButton.setFocus();
    }

    private void revalidate() {
        boolean valid = true;
        for (RequestBuildSection section : fSections) {
            if (!section.validate2()) {
                valid = false;
            }
        }
        Button okButton = getButton(IDialogConstants.OK_ID);
        okButton.setEnabled(valid);
    }
    
    /**
     * Sets the workbench part site. This is required if the context menus
     * within the dialog must be registered for extension.
     * 
     * @param partSite
     *            The workbench part site. May be <code>null</code>. If
     *            <code>null</code>, context menus within the section are not
     *            registered for extension.
     */
    public void setWorkbenchPartSite(IWorkbenchPartSite partSite) {
        fPartSite = partSite;
    }
}
