/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;

import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.ui.dialogs.requestbuild.IRequestBuildSectionFactory;

/**
 * Wrapper class representing a request build dialog section extension.
 */
public class RequestBuildDialogSectionExtension {

    /**
     * The class attribute.
     */
    public static final String CLASS_ATTRIBUTE = "class"; //$NON-NLS-1$

    /**
     * The order attribute.
     */
    public static final String ORDER_ATTRIBUTE = "order"; //$NON-NLS-1$

    private String fClassAttribute;
    private String fOrderAttribute;
    private int fOrderNumber = -1;

    private IConfigurationElement fPluginConfigurationElement;

    /**
     * Constructs the <code>RequestBuildDialogSectionExtension</code> with a
     * plugin configuration element that represents a request build dialog
     * section. It automatically validates the attributes.
     * 
     * @param pluginConfigurationElement
     *            The plugin configuration element.
     */
    public RequestBuildDialogSectionExtension(IConfigurationElement pluginConfigurationElement) {

        ValidationHelper.validateNotNull("configElement", pluginConfigurationElement); //$NON-NLS-1$

        fPluginConfigurationElement = pluginConfigurationElement;
        fClassAttribute = pluginConfigurationElement.getAttribute(CLASS_ATTRIBUTE);
        fOrderAttribute = pluginConfigurationElement.getAttribute(ORDER_ATTRIBUTE);

        validateAttributes();
    }

    /**
     * Retrieves the class attribute for the plugin extension.
     * 
     * @return The class attribute for the plugin extension.
     */
    public String getClassAttribute() {
        return fClassAttribute;
    }

    /**
     * Retrieves the order attribute for the plugin extension.
     * 
     * @return The order attribute for the plugin extension.
     */
    public String getOrderAttribute() {
        return fOrderAttribute;
    }

    /**
     * Retrieves the order number for the plugin extension.
     * 
     * @return The order number for the plugin extension.
     */
    public int getOrderNumber() {
        if (fOrderNumber == -1) {
            fOrderNumber = Integer.parseInt(getOrderAttribute().trim());
        }
        return fOrderNumber;
    }

    /**
     * Creates a factory for the plugin extension. A new object is returned
     * each time this method is called.
     * 
     * @return A provider for the plugin extension.
     * @throws CoreException
     *             If an executable extension for the extension point could not
     *             be created.
     * @throws IllegalArgumentException
     *             <ul>
     *             <li>If any of the attributes of the extension point are
     *             <code>null</code> or empty.</li>
     *             <li>If the class attribute of the extension point is not of
     *             type <code>IConfigurationElementEditorFactory</code>.</li>
     *             </ul>
     */
    public IRequestBuildSectionFactory createSectionFactory() throws IllegalArgumentException, CoreException {
        return createExecutableExtension();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public int hashCode() {
        return fClassAttribute.hashCode();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (getClass() != obj.getClass()) {
            return false;
        }

        RequestBuildDialogSectionExtension other = (RequestBuildDialogSectionExtension) obj;

        if (!fClassAttribute.equals(other.fClassAttribute)) {
            return false;
        }

        return true;
    }

    public String toString() {
        return fClassAttribute;
    }

    /**
     * Validates the attributes of the extension point and throws an
     * <code>IllegalArgumentException</code> if any of them are
     * <code>null</code> or empty.
     */
    private void validateAttributes() {
        ValidationHelper.validateNotNull(CLASS_ATTRIBUTE, fClassAttribute);
        ValidationHelper.validateNotEmpty(CLASS_ATTRIBUTE, fClassAttribute.trim());
        ValidationHelper.validateNotNull(ORDER_ATTRIBUTE, fOrderAttribute);
        ValidationHelper.validateNotEmpty(ORDER_ATTRIBUTE, fOrderAttribute.trim());
        ValidationHelper.validateIsInteger(ORDER_ATTRIBUTE, fOrderAttribute.trim());
    }

    /**
     * Creates an <code>IRequestBuildSectionFactory</code>
     * instance.
     * 
     * @throws CoreException
     *             If an executable extension for the extension point could not
     *             be created.
     * @throws IllegalArgumentException
     *             If the created executable extension is not of type
     *             <code>IRequestBuildSectionFactory</code>.
     */
    private IRequestBuildSectionFactory createExecutableExtension() throws CoreException,
            IllegalArgumentException {
        Object executableExtension = fPluginConfigurationElement.createExecutableExtension(CLASS_ATTRIBUTE);
        ValidationHelper.validateIsType(CLASS_ATTRIBUTE, IRequestBuildSectionFactory.class, executableExtension);
        return (IRequestBuildSectionFactory) executableExtension;
    }

}
