/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedList;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.RegistryFactory;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorMessages;

/**
 * Manages the loading of the request build dialog section extensions.
 */
public class RequestBuildDialogSectionExtensionManager {

    /**
     * ID for the request build dialog section extension point.
     */
    public static final String EXTENSION_POINT_ID = "com.ibm.team.build.ui.requestBuildDialogSections"; //$NON-NLS-1$

    /**
     * The singleton instance of the manager.
     */
    protected static RequestBuildDialogSectionExtensionManager fManager = null;

    private RequestBuildDialogSectionExtension[] fExtensions;

    /**
     * Singleton constructor.
     */
    protected RequestBuildDialogSectionExtensionManager() {
        fExtensions = loadExtensions();
        Arrays.sort(fExtensions, new Comparator<RequestBuildDialogSectionExtension>() {
            public int compare(RequestBuildDialogSectionExtension e1, RequestBuildDialogSectionExtension e2) {
                return e1.getOrderNumber() - e2.getOrderNumber();
            }
        });
    }

    /**
     * Return the singleton instance of the manager.
     * 
     * @return The singleton instance of the manager.
     */
    public static RequestBuildDialogSectionExtensionManager getInstance() {
        synchronized (RequestBuildDialogSectionExtensionManager.class) {
            if (fManager == null) {
                fManager = new RequestBuildDialogSectionExtensionManager();
            }
        }
        return fManager;
    }

    /**
     * Get the extensions. Extensions are only loaded once, the first time this
     * class is instantiated.
     * 
     * @return The extensions.
     */
    public RequestBuildDialogSectionExtension[] getRequestBuildDialogSectionExtensions() {
        return fExtensions;
    }

    /**
     * Retrieves the configuration elements for the extension point.
     * 
     * @param extensionPoint
     *            The extension point to retrieve the configuration elements
     *            from.
     * @return The configuration elements for the extension point.
     */
    protected IConfigurationElement[] getConfigurationElements(IExtensionPoint extensionPoint) {
        return extensionPoint.getConfigurationElements();
    }

    /**
     * Load all the build configuration element editor extensions, ignoring any
     * that fail to load properly or are duplicates of others.
     */
    private RequestBuildDialogSectionExtension[] loadExtensions() {
        LinkedList<RequestBuildDialogSectionExtension> extensions = new LinkedList<RequestBuildDialogSectionExtension>();

        IExtensionPoint extensionPoint = RegistryFactory.getRegistry().getExtensionPoint(
                EXTENSION_POINT_ID);
        if (extensionPoint != null) {
            IConfigurationElement[] configurationElements = getConfigurationElements(extensionPoint);
            for (int i = 0; i < configurationElements.length; i++) {
                IConfigurationElement configElement = configurationElements[i];
                try {
                    RequestBuildDialogSectionExtension extension = new RequestBuildDialogSectionExtension(configElement);
                    if (!extensions.contains(extension)) {
                        extensions.add(extension);
                    } else {
                        handleDuplicateExtensions(configElement);
                    }
                } catch (IllegalArgumentException exception) {
                    handleIllegalArgumentException(exception, configElement);
                }
            }
        }

        return (RequestBuildDialogSectionExtension[]) extensions.toArray(new RequestBuildDialogSectionExtension[extensions.size()]);
    }

    /**
     * Helper to handle duplicate extensions.
     * 
     * @param configElement
     *            The configuration element that is .
     */
    private void handleDuplicateExtensions(IConfigurationElement configElement) {
        String[] params = { getExtensionFQN(configElement),
                configElement.getAttribute(RequestBuildDialogSectionExtension.CLASS_ATTRIBUTE),
                configElement.getContributor().getName() };
        String message = NLS.bind(BuildResultEditorMessages.BuildContributionManager_DUPLICATE_EXTENSION_POINT, params);
        BuildUIPlugin.log(message);
    }

    /**
     * Helper to handle consistent processing of illegal argument exceptions.
     * 
     * @param exception
     *            The illegal argument exception that occurred.
     * @param configElement
     *            The configuration element that generated the exception.
     */
    private void handleIllegalArgumentException(IllegalArgumentException exception, IConfigurationElement configElement) {
        String[] params = { getExtensionFQN(configElement), configElement.getContributor().getName(), };
        String message = NLS.bind(BuildResultEditorMessages.BuildContributionManager_MISSING_OR_INVALID, params);
        BuildUIPlugin.log(message);
    }

    /**
     * Get the fully qualified name of the extension for this configuration
     * element.
     * 
     * @param configElement
     *            The configuration element to get the fully qualified name of
     *            the extension from.
     * @return The fully qualified name of the extension for this configuration
     *         element.
     */
    private String getExtensionFQN(IConfigurationElement configElement) {
        return configElement.getNamespaceIdentifier() + "." + configElement.getName(); //$NON-NLS-1$
    }
}
