/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.util.Util;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.Section;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.model.IBuildConfigurationElement;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionInstance;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.query.IBaseBuildEngineQueryModel.IBuildEngineQueryModel;
import com.ibm.team.build.internal.client.iterator.ItemQueryIterator;
import com.ibm.team.build.internal.common.builddefinition.IJazzScmConfigurationElement;
import com.ibm.team.build.internal.common.helper.ItemHandleAwareHashMap;
import com.ibm.team.build.internal.scm.ComponentLoadRules;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.editors.builddefinition.BuildDefinitionEditorMessages;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.build.ui.dialogs.requestbuild.IRequestBuildSectionFactory;
import com.ibm.team.build.ui.dialogs.requestbuild.RequestBuildSection;
import com.ibm.team.build.ui.dialogs.requestbuild.RequestBuildSectionSite;
import com.ibm.team.filesystem.client.FileSystemCore;
import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.filesystem.client.ILocalChange;
import com.ibm.team.filesystem.client.ILocalChangeManager;
import com.ibm.team.filesystem.client.ISandbox;
import com.ibm.team.filesystem.client.ISharingManager;
import com.ibm.team.filesystem.common.IFileItemHandle;
import com.ibm.team.filesystem.ui.FileSelectionDialog;
import com.ibm.team.filesystem.ui.WORKSPACES_OR_STREAMS;
import com.ibm.team.filesystem.ui.WorkspaceAndStreamSelectionDialog;
import com.ibm.team.filesystem.ui.WorkspaceAndStreamSelectionDialogOptions;
import com.ibm.team.filesystem.ui.wrapper.AbstractPlaceWrapper;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IProjectArea;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.ITeamArea;
import com.ibm.team.process.common.ITeamAreaHandle;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.repository.common.query.IItemQuery;
import com.ibm.team.repository.common.service.IQueryService;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.client.IWorkspaceManager;
import com.ibm.team.scm.client.SCMPlatform;
import com.ibm.team.scm.client.internal.IConfigurationDescriptor;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IWorkspace;
import com.ibm.team.scm.common.IWorkspaceHandle;
import com.ibm.team.scm.common.dto.IPrivateScope;
import com.ibm.team.scm.common.dto.IReadScope;

/**
 * Section for SCM options (personal build, etc) in the build request dialog.
 */
@SuppressWarnings("restriction")
public class ScmOptionsSection extends RequestBuildSection {

    /**
     * Factory interface for creating the section.
     */
    public static class Factory implements IRequestBuildSectionFactory {
        public RequestBuildSection createRequestBuildSection(RequestBuildSectionSite site) {
            return new ScmOptionsSection(site);
        }
    }

    /**
     * The key prefix for the persisted personal build workspace. Combine this
     * key with the build definition id to form a key which can be used to look
     * up a persisted personal build workspace.
     */
    public static final String PREFERENCE_WORKSPACE_PREFIX = "personalBuildWorkspace"; //$NON-NLS-1$

    private Composite fBuildOptionsComposite;
    protected Button fPersonalBuildButton;
    protected Text fPersonalBuildWorkspaceText;
    protected Button fBrowseWorkspaceButton;
    protected Text fComponentLoadRulesText;
    protected Button fBrowseComponentLoadRulesButton;
    private IWorkspace fPersonalBuildWorkspace;
    private ComponentLoadRules fComponentLoadRules;

    private Button fBrowseEngineButton;

    private Text fEngineText;
    
    private boolean fUncheckedInChangesInWorkspace;

    /**
     * Wrapper of personal build check box and information label. 
     */
    private Composite fPersonalBuildCheckBoxComposite;

    private boolean fPersonalBuildWorkspaceHasPrivateComponents;

    /**
     * Create the section.
     * 
     * @param site
     *            the section site
     */
    public ScmOptionsSection(RequestBuildSectionSite site) {
        super(site);
    }

    @Override
    protected String getSectionName() {
        if (getSite().isRebuild() && getSite().getBuildResult() != null) {
            return NLS.bind(BuildUIDialogMessages.RequestBuildDialog_REBUILD_OPTIONS_LABEL,
                    getSite().getBuildResult().getLabel());
        }
        return BuildUIDialogMessages.RequestBuildDialog_BUILD_OPTIONS_LABEL;
    }

    @Override
    protected void createSectionContent(Section section) {

        // Dispose any pre existing options composite. There may be one if
        // the user is clicking Browse to select a different build.
        if (fBuildOptionsComposite != null && !fBuildOptionsComposite.isDisposed()) {
            fBuildOptionsComposite.dispose();
        }

        fBuildOptionsComposite = new Composite(section, SWT.NONE);
        fBuildOptionsComposite.setLayoutData(new GridData());
        GridLayout layout = new GridLayout(3, false);
        layout.marginLeft = 10;
        fBuildOptionsComposite.setLayout(layout);
        section.setClient(fBuildOptionsComposite);

        Label engineLabel = new Label(fBuildOptionsComposite, SWT.NONE);
        engineLabel.setText(BuildUIDialogMessages.RequestBuildDialog_BUILD_ENGINE_LABEL);

        fEngineText = new Text(fBuildOptionsComposite, SWT.READ_ONLY | SWT.SINGLE | SWT.BORDER);
        GridDataFactory.fillDefaults().grab(true, false).applyTo(fEngineText);
        setPotentialHandler(null);

        fBrowseEngineButton = new Button(fBuildOptionsComposite, SWT.PUSH);
        fBrowseEngineButton.setText(BuildUIDialogMessages.RequestBuildDialog_BUILD_ENGINE_BROWSE);
        fBrowseEngineButton.addSelectionListener(getBrowseEngineButtonListener());

        if (hasScmConfiguration()) {
            fPersonalBuildCheckBoxComposite = new Composite(fBuildOptionsComposite, SWT.NONE);
            layout = new GridLayout(2, false);
            layout.horizontalSpacing = 0;
            layout.marginWidth = 0;
            fPersonalBuildCheckBoxComposite.setLayout(layout);
            
            GridData gridData = new GridData();
            gridData.horizontalSpan = 3;
            fPersonalBuildCheckBoxComposite.setLayoutData(gridData);
            
            fPersonalBuildButton = new Button(fPersonalBuildCheckBoxComposite, SWT.CHECK);
            fPersonalBuildButton.setText(BuildUIDialogMessages.RequestBuildDialog_PERSONAL_BUILD_BUTTON);
            GridDataFactory.swtDefaults().applyTo(fPersonalBuildButton);
            
            fPersonalBuildButton.addSelectionListener(getPersonalBuildButtonListener());
            boolean isPersonalBuild = getSite().isPersonalBuild();
            fPersonalBuildButton.setSelection(isPersonalBuild);
            
            /**
             * Information label for personal build check box.
            */
            Label label = new Label(fPersonalBuildCheckBoxComposite, SWT.NONE);
            label.setImage(BuildUIPlugin.getImage("icons/obj16/info.gif")); //$NON-NLS-1$
            label.setToolTipText(BuildUIDialogMessages.RequestBuildDialog_PERSONAL_BUILD_DESCRIPTION);
            GridDataFactory.swtDefaults().applyTo(label);
            
            label = new Label(fBuildOptionsComposite, SWT.NONE);
            label.setText(BuildUIDialogMessages.RequestBuildDialog_REPOSITORY_WORKSPACE);
            GridDataFactory.swtDefaults().applyTo(label);

            fPersonalBuildWorkspaceText = new Text(fBuildOptionsComposite, SWT.READ_ONLY | SWT.BORDER);
            GridDataFactory.fillDefaults().grab(true, false).indent(10, 0).applyTo(fPersonalBuildWorkspaceText);
            fPersonalBuildWorkspaceText.setEnabled(isPersonalBuild);
            IWorkspace personalBuildWorkspace = getPersonalBuildWorkspace();
            if (personalBuildWorkspace != null) {
                fPersonalBuildWorkspaceText.setText(personalBuildWorkspace.getName());
            }

            fBrowseWorkspaceButton = new Button(fBuildOptionsComposite, SWT.PUSH);
            fBrowseWorkspaceButton.setText(BuildUIDialogMessages.RequestBuildDialog_BROWSE_WORKSPACE);
            fBrowseWorkspaceButton.addSelectionListener(getBrowseWorkspaceButtonListener());
            fBrowseWorkspaceButton.setEnabled(isPersonalBuild);
            
            // We might be dealing with a rebuild case in which the selection for a personal
            // build has already been made. We would still like to tell them if they have 
            // pending local changes.
            if (isPersonalBuild) {
                createJobToCheckWorkspace(fPersonalBuildButton.getDisplay());
            } else {
                setUncheckedInChangesExist(false);
                setPersonalBuildWorkspaceHasPrivateComponents(false);
            }

            label = new Label(fBuildOptionsComposite, SWT.NONE);
            label.setText(BuildUIDialogMessages.RequestBuildDialog_COMPONENT_LOAD_RULES);
            GridDataFactory.swtDefaults().applyTo(label);

            fComponentLoadRulesText = new Text(fBuildOptionsComposite, SWT.READ_ONLY | SWT.BORDER);
            GridDataFactory.fillDefaults().grab(true, false).indent(10, 0).applyTo(fComponentLoadRulesText);
            fComponentLoadRulesText.setEnabled(isPersonalBuild);
            if (getComponentLoadRules() != null) {
                updateComponentLoadRules(getComponentLoadRules());
            }

            fBrowseComponentLoadRulesButton = new Button(fBuildOptionsComposite, SWT.PUSH);
            fBrowseComponentLoadRulesButton.setText(BuildUIDialogMessages.RequestBuildDialog_BROWSE_COMPONENT_LOAD_RULES);
            fBrowseComponentLoadRulesButton.addSelectionListener(getBrowseComponentLoadRulesButtonListener());
            fBrowseComponentLoadRulesButton.setEnabled(isPersonalBuild && personalBuildWorkspace != null);
            
            if (isPersonalBuild) {
                getSection().setExpanded(true);
            }
        } 
    }

    protected boolean hasScmConfiguration() {
        return getSite().getBuildDefinition().getConfigurationElement(IJazzScmConfigurationElement.ELEMENT_ID) != null;
    }

    protected SelectionListener getPersonalBuildButtonListener() {
        return new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                boolean isPersonalBuild = fPersonalBuildButton.getSelection();
                IWorkspace personalBuildWorkspace = getPersonalBuildWorkspace();
                if (isPersonalBuild) {
                    createJobToCheckWorkspace(fPersonalBuildButton.getDisplay());
                } else {
                    setUncheckedInChangesExist(false);
                    setPersonalBuildWorkspaceHasPrivateComponents(false);
                }
                getSite().setProperty(RequestBuildSectionSite.PROP_PERSONAL_BUILD, isPersonalBuild);
                fPersonalBuildWorkspaceText.setEnabled(isPersonalBuild);
                fBrowseWorkspaceButton.setEnabled(isPersonalBuild);
                fComponentLoadRulesText.setEnabled(isPersonalBuild && personalBuildWorkspace != null);
                fBrowseComponentLoadRulesButton.setEnabled(isPersonalBuild && personalBuildWorkspace != null);
            }
        };
    }

    protected SelectionListener getBrowseEngineButtonListener() {
        return new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                Button button = (Button) e.getSource();
                button.setEnabled(false);
                ITeamRepository teamRepository = getSite().getTeamRepository();
                IProcessAreaHandle processAreaHandle = getSite().getBuildDefinition().getProcessArea();
                openBuildEngineChooserDialog(fBrowseEngineButton.getShell(), teamRepository, processAreaHandle, button);
            }
        };
    }

    protected SelectionListener getBrowseWorkspaceButtonListener() {
        return new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ITeamRepository teamRepository = getSite().getTeamRepository();
                IWorkspace workspace = openWorkspaceChooserDialog(fBrowseWorkspaceButton.getShell(), teamRepository);
                if (workspace != null) {
                    setPersonalBuildWorkspace(workspace);
                    fPersonalBuildWorkspaceText.setText(workspace.getName());
                    createJobToCheckWorkspace(fPersonalBuildButton.getDisplay());
                    fBrowseComponentLoadRulesButton.setEnabled(true);
                    getSite().revalidate();

                    BuildUIPlugin.getDefault().getPreferenceStore().setValue(
                            PREFERENCE_WORKSPACE_PREFIX + getSite().getBuildDefinition().getId(),
                            workspace.getItemId().getUuidValue());
                    BuildUIPlugin.getDefault().savePluginPreferences();
                } else {
                    setUncheckedInChangesExist(false);
                    setPersonalBuildWorkspaceHasPrivateComponents(false);
                }
            }
        };
    }
    
    protected void createJobToCheckWorkspace(final Display display) {
        String jobNameCheckWorkspace = BuildUIDialogMessages.ScmOptionsSection_CHECK_PERSONAL_BUILD_WORKSPACE_JOB_NAME;
        TeamBuildJob jobCheckWorkspace = new TeamBuildJob(jobNameCheckWorkspace, false, getSite().getTeamRepository()) {
            @Override
            protected IStatus runProtected(final IProgressMonitor monitor) throws Exception {
                checkPersonalBuildWorkspace(display, monitor);
                return Status.OK_STATUS;
            }
        };
        jobCheckWorkspace.schedule();
    }

    /**
     * NON-UI THREAD
     */
    protected void checkPersonalBuildWorkspace(Display display, IProgressMonitor monitor) {
        SubMonitor subMonitor = SubMonitor.convert(monitor, 100);
        setUncheckedInChangesExist(hasUncheckedInChangesInWorkspace(subMonitor.newChild(50)));
        setPersonalBuildWorkspaceHasPrivateComponents(hasPrivateComponents(subMonitor.newChild(50)));
        handleWorkspaceCheckDone(display);
    }

    /**
     * NON-UI THREAD
     */
    protected boolean hasPrivateComponents(IProgressMonitor monitor) {
        SubMonitor subMonitor = SubMonitor.convert(monitor, 100);
        
        boolean privateComponents = false;
        IWorkspace workspace = getPersonalBuildWorkspace();
        if (workspace != null && workspace.getOrigin() instanceof ITeamRepository) {
            ITeamRepository repo = (ITeamRepository) workspace.getOrigin();
            IWorkspaceManager workspaceManager = SCMPlatform.getWorkspaceManager(repo);
            try {
                IWorkspaceConnection workspaceConnection = workspaceManager.getWorkspaceConnection(workspace, subMonitor.newChild(10));
                List<IReadScope> readScopes = workspaceManager.findReadScopeForComponents(workspaceConnection.getComponents(), subMonitor.newChild(90));
                for (IReadScope readScope : readScopes) {
                    // Note: As of RTC 6.0.1, in addition to "Public" or "Private" visibility, a component
                    // can use an Access Group for visibility. Without knowing 'who' the build user is
                    // it would be difficult to tell if 'they' are in the Access Group, and so there is
                    // always the case that builds may fail due to this...
                    // We probably do not need to warn for this case though; we can assume users have thought
                    // of this when creating an Access Group, whereas it probably is the case that if a component
                    // has a Private visibility, then the build user would not be able to see it (thus we warn).
                    if (readScope instanceof IPrivateScope) {
                        privateComponents = true;
                        break;
                    }
                }
            } catch (TeamRepositoryException exception) {
                BuildUIPlugin.log(exception);
            }
        }
        return privateComponents;
    }

    /**
     * NON-UI THREAD
     */
    protected boolean hasUncheckedInChangesInWorkspace(IProgressMonitor monitor) {
        boolean uncheckedInChangesExist = false;
        IWorkspace personalBuildWorkspace = getPersonalBuildWorkspace();
        if (personalBuildWorkspace != null) {
            ILocalChangeManager lcm = FileSystemCore.getSharingManager().getLocalChangeManager();
            ISharingManager sm = FileSystemCore.getSharingManager();
            Collection<ISandbox> sandboxes = sm.getRegisteredSandboxes();
            SubMonitor subMonitor = SubMonitor.convert(monitor, sandboxes.size());
            outter: for (ISandbox sandbox : sandboxes) {
                Collection<? extends IConfigurationDescriptor> loadedComponents = null;
                try {
                    loadedComponents = sandbox.allLoadedConfigurations(subMonitor.newChild(1));
                } catch (FileSystemException exception) {
                    BuildUIPlugin.log(exception);
                }
                for (IConfigurationDescriptor loaded : loadedComponents) {
                    if (loaded.getConnectionHandle() instanceof IWorkspaceHandle) {
                        IComponentHandle component = loaded.getComponentHandle();
                        ILocalChange[] localChanges = lcm.getPendingChanges(personalBuildWorkspace, component, sandbox);
                        if (localChanges.length > 0) {
                            uncheckedInChangesExist = true;
                            break outter;
                        }
                    }
                }
            }
        }
        return uncheckedInChangesExist;
    }

    /**
     * NON-UI THREAD
     */
    protected void handleWorkspaceCheckDone(Display display) {        
        display.asyncExec(new Runnable() {
            @Override
            public void run() {
                if (!fPersonalBuildButton.isDisposed()) {
                    getSite().revalidate();
                }
            }
        });
    }
    
    protected SelectionListener getBrowseComponentLoadRulesButtonListener() {
        return new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ITeamRepository teamRepository = getSite().getTeamRepository();
                Map<IComponentHandle, IFileItemHandle> componentLoadRuleFiles = openComponentLoadRulesChooserDialog(
                        fBrowseComponentLoadRulesButton.getShell(), teamRepository, getPersonalBuildWorkspace());
                if (componentLoadRuleFiles != null) {
                    setComponentLoadRules(new ComponentLoadRules(componentLoadRuleFiles));
                    updateComponentLoadRules(getComponentLoadRules());
                }
            }
        };
    }

    protected void updateComponentLoadRules(ComponentLoadRules rules) {
        int numLoadRules = getComponentLoadRules().getLoadRuleFiles().values().size();
        if (numLoadRules == 0) {
            fComponentLoadRulesText.setText(Util.ZERO_LENGTH_STRING);
        } else {
            fComponentLoadRulesText.setEnabled(true);
            String messageId = numLoadRules > 1 ? BuildDefinitionEditorMessages.JazzScmConfigurationEditor_NUMBER_OF_LOADRULE_FILES_TEXT_MULTI
                    : BuildDefinitionEditorMessages.JazzScmConfigurationEditor_NUMBER_OF_LOADRULE_FILES_TEXT;
            fComponentLoadRulesText.setText(NLS.bind(messageId, numLoadRules));
        }
    }

    @Override
    public boolean validate2() {
        boolean isValid = true;
        boolean isEmptyBuildWorkspace = false;
        boolean isBuildWorkspaceSteam = false;
        boolean isBuildWorkspacePrivate = false;

        if (hasScmConfiguration() && getSite().isPersonalBuild()) {
            IWorkspace buildWorkspace = getPersonalBuildWorkspace();
            if (buildWorkspace == null) {
                isValid = false;
                isEmptyBuildWorkspace = true;
            } else if (buildWorkspace.isStream()) {
                isBuildWorkspaceSteam = true;
            } else if (buildWorkspace.getReadScope() instanceof IPrivateScope) {
                isBuildWorkspacePrivate = true;
            }
        }

        // Empty build workspaces are not allowed.
        if (isEmptyBuildWorkspace) {
            getSite().addErrorMessage(BuildUIDialogMessages.ScmOptionsSection_ERROR_MESSAGE_NO_BUILD_WORKSPACE,
                    BuildUIDialogMessages.ScmOptionsSection_ERROR_MESSAGE_NO_BUILD_WORKSPACE);
        } else {
            getSite().removeErrorMessage(BuildUIDialogMessages.ScmOptionsSection_ERROR_MESSAGE_NO_BUILD_WORKSPACE);
        }

        // It's not recommended to use a stream as the build workspace.
        if (isBuildWorkspaceSteam) {
            getSite().addWarningMessage(
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_BUILD_WORKSPACE_IS_STREAM,
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_BUILD_WORKSPACE_IS_STREAM);
        } else {
            getSite().removeWarningMessage(
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_BUILD_WORKSPACE_IS_STREAM);
        }

        // It's not recommended to build against a private build workspace.
        if (isBuildWorkspacePrivate) {
            getSite().addWarningMessage(
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_PRIVATE_BUILD_WORKSPACE,
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_PRIVATE_BUILD_WORKSPACE);
        } else {
            getSite().removeWarningMessage(
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_PRIVATE_BUILD_WORKSPACE);
        }

        // Warning message will be generated if there are unchecked in changes
        // in the build workspace.
        if (getUncheckedInChangesExist()) {
            getSite().addWarningMessage(
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_UNCHECKED_IN_CHANGES_EXIST_IN_BUILD_WORKSPACE,
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_UNCHECKED_IN_CHANGES_EXIST_IN_BUILD_WORKSPACE);
        } else {
            getSite().removeWarningMessage(
                    BuildUIDialogMessages.ScmOptionsSection_WARNING_MESSAGE_UNCHECKED_IN_CHANGES_EXIST_IN_BUILD_WORKSPACE);

        }

        if (getPersonalBuildWorkspaceHasPrivateComponents()) {
            getSite().addWarningMessage(BuildUIDialogMessages.ScmOptionsSection_PRIVATE_COMPONENTS_IN_PERSONAL_WORKSPACE,
                    BuildUIDialogMessages.ScmOptionsSection_PRIVATE_COMPONENTS_IN_PERSONAL_WORKSPACE);
        } else {
            getSite().removeWarningMessage(BuildUIDialogMessages.ScmOptionsSection_PRIVATE_COMPONENTS_IN_PERSONAL_WORKSPACE);
        }
        return isValid;
    }

    @Override
    public void initialize(IProgressMonitor monitor) {
        if (getSite().isRebuild()) {
            setPersonalBuildWorkspace(getWorkspace(getSite().getBuildRequest(), monitor));
            setComponentLoadRules(getComponentLoadRules(getSite().getBuildRequest()));
        } else {
            setPersonalBuildWorkspace(getPersistedWorkspace(getSite().getBuildDefinition(), monitor));
            setComponentLoadRules(getComponentLoadRules(getSite().getBuildDefinition()));
        }
    }

    private ComponentLoadRules getComponentLoadRules(IBuildRequest buildRequest) {
        ComponentLoadRules componentLoadRules = null;

        IBuildDefinitionInstance instance = buildRequest.getBuildDefinitionInstance();
        IBuildProperty property = instance.getProperty(IJazzScmConfigurationElement.PROPERTY_COMPONENT_LOAD_RULES);
        if (property != null) {
            componentLoadRules = new ComponentLoadRules(property.getValue());
        }
        return componentLoadRules;
    }

    private ComponentLoadRules getComponentLoadRules(IBuildDefinition buildDefinition) {
        ComponentLoadRules componentLoadRules = null;

        IBuildConfigurationElement element = buildDefinition.getConfigurationElement(IJazzScmConfigurationElement.ELEMENT_ID);
        if (element != null) {
            IBuildProperty property = buildDefinition.getProperty(IJazzScmConfigurationElement.PROPERTY_COMPONENT_LOAD_RULES);
            // older build definitions may not have the property
            if (property != null) {
                componentLoadRules = new ComponentLoadRules(property.getValue());
            }
        }
        return componentLoadRules;
    }

    protected Map<IComponentHandle, IFileItemHandle> openComponentLoadRulesChooserDialog(Shell shell,
            ITeamRepository teamRepository, IWorkspace workspace) {
        Map<IComponentHandle, IFileItemHandle> initialSelection;
        if (getComponentLoadRules() != null) {
            initialSelection = getComponentLoadRules().getLoadRuleFiles();
        } else {
            initialSelection = Collections.EMPTY_MAP;
        }
        Map<IComponentHandle, IFileItemHandle> pickedFiles = FileSelectionDialog.pickFilePerComponent(shell,
                teamRepository, workspace, BuildUIDialogMessages.RequestBuildDialog_LOAD_RULES_DIALOG_MESSAGE,
                initialSelection);
        return pickedFiles;
    }

    protected IWorkspace openWorkspaceChooserDialog(Shell shell, ITeamRepository teamRepository) {
        WorkspaceAndStreamSelectionDialogOptions options = new WorkspaceAndStreamSelectionDialogOptions()
        .setTarget(null)
        .setAllowMultiSelect(false)
        .setComponents(null)
        .setShowWorkspaces(WORKSPACES_OR_STREAMS.WORKSPACES)
        .setShowWorkspacesRadio(true)
        .setExactOwnerName(null)
        .setToIgnore(null)
        .setForceRepo(true);

        AbstractPlaceWrapper wrapper = WorkspaceAndStreamSelectionDialog.getWorkspaceOrStream(shell, teamRepository, options);

        if (wrapper != null) {
            return wrapper.getWorkspace();
        }

        return null;
    }

    protected void openBuildEngineChooserDialog(final Shell shell, final ITeamRepository teamRepository,
            final IProcessAreaHandle processAreaHandle, final Button button) {
        final Display display = shell.getDisplay();
        TeamBuildJob job = new TeamBuildJob(BuildUIDialogMessages.RequestBuildDialog_FETCHING_ENGINES_JOB_NAME, true,
                teamRepository) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                SubMonitor progress = SubMonitor.convert(monitor, 100);
                IProcessArea processArea = (IProcessArea) teamRepository.itemManager().fetchCompleteItem(
                        processAreaHandle, IItemManager.DEFAULT, progress.newChild(10));
                final IProjectArea projectArea = (IProjectArea) teamRepository.itemManager().fetchCompleteItem(
                        processArea.getProjectArea(), IItemManager.DEFAULT, progress.newChild(10));
                final List<IBuildEngine> buildEngines = getBuildEngines(teamRepository, progress.newChild(40));
                final Map<String, String> buildEngineIdsToProjectAreaName = fetchEngineIdToProjectName(teamRepository,
                        buildEngines, progress.newChild(40));
                display.asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        if (!shell.isDisposed()) {
                            button.setEnabled(true);
                            Map<String, IBuildEngine> buildEnginesById = getBuildEnginesById(buildEngines);
                            EditBuildEnginesDialog dialog = new EditBuildEnginesDialog(shell,
                                    buildEngineIdsToProjectAreaName, null, projectArea.getName(), buildEnginesById) {
                                @Override
                                protected String getDialogTitle() {
                                    return BuildUIDialogMessages.RequestBuildDialog_SELECT_ENGINE_DIALOG_TITLE;
                                }

                                @Override
                                protected String getDialogDescription() {
                                    return BuildUIDialogMessages.RequestBuildDialog_SELECT_ENGINE_DIALOG_DESCRIPTION;
                                };
                            };
                            dialog.setSingleSelect(true);
                            if (dialog.open() == Window.OK) {
                                Set<String> changedBuildEngineIds = dialog.getChangedBuildEngineIds();
                                IBuildEngine buildEngine = changedBuildEngineIds.isEmpty() ? null
                                        : buildEnginesById.get(changedBuildEngineIds.iterator().next());
                                setPotentialHandler(buildEngine);
                            }
                        }
                    }
                });
                return Status.OK_STATUS;
            }
        };
        job.schedule();
    }

    private void setPotentialHandler(IBuildEngine buildEngine) {
        fEngineText.setText(buildEngine == null ? BuildUIDialogMessages.RequestBuildDialog_ANY_SUPPORTING_ENGINE_TEXT
                : buildEngine.getId());
        getSite().setProperty(RequestBuildSectionSite.PROP_HANDLING_ENGINE, buildEngine);
    }

    protected Map<String, IBuildEngine> getBuildEnginesById(List<IBuildEngine> buildEngines) {
        Map<String, IBuildEngine> buildEnginesById = new HashMap<String, IBuildEngine>();
        for (IBuildEngine engine : buildEngines) {
            buildEnginesById.put(engine.getId(), engine);
        }
        return buildEnginesById;
    }

    protected List<IBuildEngine> getBuildEngines(ITeamRepository teamRepository, IProgressMonitor monitor)
            throws TeamRepositoryException {
        IItemQuery itemQuery = IItemQuery.FACTORY.newInstance(IBuildEngineQueryModel.ROOT);
        ItemQueryIterator<IBuildEngineHandle> iter = new ItemQueryIterator<IBuildEngineHandle>(
                ClientFactory.getTeamBuildClient(teamRepository), itemQuery, IQueryService.EMPTY_PARAMETERS);
        LinkedList<IBuildEngine> buildEngines = new LinkedList<IBuildEngine>();

        SubMonitor loopProgress = SubMonitor.convert(monitor, 100);
        while (iter.hasNext(loopProgress.newChild(20))) {
            loopProgress.setWorkRemaining(100);
            List<IBuildEngineHandle> engineHandles = iter.next(IQueryService.ITEM_QUERY_MAX_PAGE_SIZE, loopProgress.newChild(40));
            List<IBuildEngine> items = teamRepository.itemManager().fetchCompleteItems(engineHandles, IItemManager.DEFAULT, loopProgress.newChild(40));
            for (IBuildEngine item : items) {
                if (item != null) {
                    buildEngines.add(item);
                }
            }
        }
        loopProgress.done();
        
        return buildEngines;
    }

    protected Map<String, String> fetchEngineIdToProjectName(ITeamRepository teamRepository,
            List<IBuildEngine> buildEngines, IProgressMonitor monitor) throws TeamRepositoryException {
        Map<String, String> idToProjectName = new HashMap<String, String>();

        if (buildEngines.size() > 0) {

            SubMonitor progress = SubMonitor.convert(monitor, 100);
            
            // partition into team area handles and project area handles
            List<ITeamAreaHandle> teamAreaHandles = new ArrayList<ITeamAreaHandle>();
            List<IProjectAreaHandle> projectAreaHandles = new ArrayList<IProjectAreaHandle>();
            for (IBuildEngine engine : buildEngines) {
                IProcessAreaHandle handle = engine.getProcessArea();
                if (handle instanceof ITeamAreaHandle) {
                    teamAreaHandles.add((ITeamAreaHandle) handle);
                } else if (handle instanceof IProjectAreaHandle) {
                    projectAreaHandles.add((IProjectAreaHandle) handle);
                }
            }

            ItemHandleAwareHashMap<IProcessAreaHandle, IProcessArea> processAreaMap = new ItemHandleAwareHashMap<IProcessAreaHandle, IProcessArea>();

            // fetch the team areas and add to map
            List<ITeamArea> teamAreas = teamRepository.itemManager().fetchPartialItems(teamAreaHandles,
                    IItemManager.DEFAULT, Collections.singletonList(IProcessArea.PROJECT_AREA_PROPERTY_ID), progress.newChild(50));
            for (ITeamArea teamArea : teamAreas) {
                if (teamArea != null) {
                    processAreaMap.put(teamArea, teamArea);
                    // need to fetch the project area below
                    if (teamArea.getProjectArea() != null) {
                        projectAreaHandles.add(teamArea.getProjectArea());
                    }
                }
            }

            // fetch the project areas and add to map
            List<IProjectArea> projectAreas = teamRepository.itemManager().fetchPartialItems(projectAreaHandles,
                    IItemManager.DEFAULT, Collections.singletonList(IProcessArea.NAME_PROPERTY_ID), progress.newChild(50));
            for (IProjectArea projectArea : projectAreas) {
                if (projectArea != null) {
                    processAreaMap.put(projectArea, projectArea);
                }
            }

            // populate the id to project name map by navigating through the
            // process area map
            for (IBuildEngine engine : buildEngines) {
                IProcessArea processArea = processAreaMap.get(engine.getProcessArea());
                IProjectArea projectArea = (processArea instanceof ITeamArea) ? (IProjectArea) processAreaMap.get(processArea.getProjectArea())
                        : (IProjectArea) processArea;
                String projectName = projectArea == null ? null : projectArea.getName();
                idToProjectName.put(engine.getId(), projectName);
            }
        }
        return idToProjectName;
    }

    @Override
    public void applyProperties(IBuildDefinition buildDefinition) {
        if (hasScmConfiguration()) {
            if (getSite().isPersonalBuild()) {
                setPersonalBuildProperties(buildDefinition);
            } else {
                // If the original build was a personal build, but the rebuild
                // is not, restore the workspace and load rules properties from
                // the build definition
                if (getSite().isRebuild()) {
                    IBuildResult buildResult = getSite().getBuildResult();
                    if (buildResult != null && buildResult.isPersonalBuild()) {
                        for (IBuildProperty origProperty : getSite().getOriginalBuildProperties()) {
                            if (origProperty.getName().equals(IJazzScmConfigurationElement.PROPERTY_WORKSPACE_UUID)
                                    || origProperty.getName().equals(
                                            IJazzScmConfigurationElement.PROPERTY_COMPONENT_LOAD_RULES)) {
                                IBuildProperty newProperty = buildDefinition.getProperty(origProperty.getName());
                                if (newProperty != null) {
                                    newProperty.setValue(origProperty.getValue());
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private void setPersonalBuildProperties(IBuildDefinition buildDefinition) {
        IWorkspace personalBuildWorkspace = getPersonalBuildWorkspace();
        if (personalBuildWorkspace != null) {
            buildDefinition.setProperty(IJazzScmConfigurationElement.PROPERTY_WORKSPACE_UUID,
                    personalBuildWorkspace.getItemId().getUuidValue());
        }

        ComponentLoadRules loadRules = getComponentLoadRules();
        if (loadRules != null) {
            IBuildProperty property = buildDefinition.getProperty(IJazzScmConfigurationElement.PROPERTY_COMPONENT_LOAD_RULES);
            // Load rules property was not created in migration so need to check
            // for null.
            if (property == null) {
                property = BuildItemFactory.createBuildProperty();
                property.setName(IJazzScmConfigurationElement.PROPERTY_COMPONENT_LOAD_RULES);
                property.setGenericEditAllowed(false);
                buildDefinition.getProperties().add(property);
            }
            property.setValue(loadRules.getBuildPropertySetting());
        }
    }

    /**
     * Get the persisted workspace.
     * 
     * @return The workspace, or <code>null</code> if there was no persisted
     *         workspace.
     */
    protected IWorkspace getPersistedWorkspace(IBuildDefinition buildDefinition, IProgressMonitor monitor) {

        String workspaceUuid = BuildUIPlugin.getDefault().getPreferenceStore().getString(
                PREFERENCE_WORKSPACE_PREFIX + buildDefinition.getId());

        ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();

        return getWorkspace(workspaceUuid, teamRepository, monitor);
    }

    protected IWorkspace getWorkspace(IBuildRequest buildRequest, IProgressMonitor monitor) {
        IWorkspace workspace = null;

        IBuildDefinitionInstance instance = buildRequest.getBuildDefinitionInstance();
        IBuildProperty property = instance.getProperty(IJazzScmConfigurationElement.PROPERTY_WORKSPACE_UUID);
        if (property != null) {
            ITeamRepository teamRepository = (ITeamRepository) buildRequest.getOrigin();
            String workspaceUuid = property.getValue();
            workspace = getWorkspace(workspaceUuid, teamRepository, monitor);
        }
        return workspace;
    }

    protected IWorkspace getWorkspace(String workspaceUuid, ITeamRepository teamRepository, IProgressMonitor monitor) {
        IWorkspace workspace = null;

        try {
            if (workspaceUuid != null && workspaceUuid.length() > 0) {
                IWorkspaceHandle workspaceHandle = (IWorkspaceHandle) IWorkspace.ITEM_TYPE.createItemHandle(
                        UUID.valueOf(workspaceUuid), null);
                workspace = fetchWorkspace(workspaceHandle, teamRepository, monitor);
            }
        } catch (TeamRepositoryException exception) {
            // Workspace was probably deleted. We just won't show one initially.
        }

        return workspace;
    }

    protected IWorkspace fetchWorkspace(IWorkspaceHandle workspaceHandle, ITeamRepository teamRepository,
            IProgressMonitor monitor) throws TeamRepositoryException {
        return (IWorkspace) teamRepository.itemManager().fetchCompleteItem(workspaceHandle, IItemManager.DEFAULT,
                monitor);
    }

    @Override
    public void handlePropertyChanged(String propertyName) {
        if (RequestBuildSectionSite.PROP_BUILD_DEFINITION.equals(propertyName)) {
            getSite().setProperty(RequestBuildSectionSite.PROP_PERSONAL_BUILD, false);

            // Reset the workspace, so the wrong persisted workspace
            // isn't used by default.
            setPersonalBuildWorkspace(null);
            setPersonalBuildWorkspaceHasPrivateComponents(false);
        }
    }

    /**
     * Sets the component load rules.
     * 
     * @param componentLoadRules
     *            the component load rules to set
     */
    protected synchronized void setComponentLoadRules(ComponentLoadRules componentLoadRules) {
        fComponentLoadRules = componentLoadRules;
    }

    /**
     * Returns the component load rules.
     * 
     * @return the component load rules
     */
    protected synchronized ComponentLoadRules getComponentLoadRules() {
        return fComponentLoadRules;
    }

    /**
     * Sets the personal build workspace.
     * 
     * @param personalBuildWorkspace
     *            the personal build workspace to set
     */
    protected synchronized void setPersonalBuildWorkspace(IWorkspace personalBuildWorkspace) {
        fPersonalBuildWorkspace = personalBuildWorkspace;
    }

    /**
     * Returns the personal build workspace.
     * 
     * @return The personal build workspace. Can be {@code null}.
     */
    protected synchronized IWorkspace getPersonalBuildWorkspace() {
        return fPersonalBuildWorkspace;
    }

    /**
     * Sets the flag indicating if any components in the personal build workspace have private read scope
     * 
     * @param hasPrivateComponents
     *            <code>true</code> if a component in the build workspace has a private read scope
     */
    protected synchronized void setPersonalBuildWorkspaceHasPrivateComponents(boolean hasPrivateComponents) {
        fPersonalBuildWorkspaceHasPrivateComponents = hasPrivateComponents;
    }

    /**
     * Returns the component scopes of the components in the personal build workspace.
     * 
     * @return the personal build workspace
     */
    protected synchronized boolean getPersonalBuildWorkspaceHasPrivateComponents() {
        return fPersonalBuildWorkspaceHasPrivateComponents;
    }

    /**
     * Sets the unchecked-in changes flag.
     * 
     * @param uncheckedInChangesExist
     *            the unchecked-in change flag to set, <code>true</code> if
     *            there are unchecked in changes in the build workspace.
     */
    protected synchronized void setUncheckedInChangesExist(boolean uncheckedInChangesExist) {
        fUncheckedInChangesInWorkspace = uncheckedInChangesExist;
    }

    /**
     * Gets the value of the unchecked-in changes flag.
     * 
     * @return <code>true</code> if there are unchecked in changes in the build
     *         workspace.
     */
    protected synchronized boolean getUncheckedInChangesExist() {
        return fUncheckedInChangesInWorkspace;
    }
}
