/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.build.internal.ui.BuildUIPlugin;

/**
 * Modal dialog that contains an expandable details section and a toggle control.
 */
public class ToggleStatusNotificationDialog extends ErrorDialog {
    
    /**
     * This field is <code>protected</code> instead of <code>private</code>
     * to enable testing.
     */
    protected boolean fToggleState;
    
    /**
     * This field is <code>protected</code> instead of <code>private</code>
     * to enable testing.
     */
    protected String fToggleMessage;
    
    /**
     * This field is <code>protected</code> instead of <code>private</code>
     * to enable testing.
     */
    protected Button fToggleButton;
    
    /**
     * Create a modal dialog with a toggle control and a details section.
     * 
     * @param parentShell
     *            The parent shell for the dialog.
     * @param displayMask
     *            The mask to use to filter the displaying of child items, as
     *            per <code>IStatus.matches</code>.
     * @param dialogTitle
     *            The title to use for the dialog.
     * @param message
     *            The message to use for the dialog.
     * @param status
     *            The {@link IStatus} reason for the dialog.
     * @param toggleMessage
     *            The message for the toggle control.
     * @param toggleState
     *            The initial state for the toggle.
     */
    public ToggleStatusNotificationDialog(Shell parentShell, int displayMask, String dialogTitle, String message, IStatus status,
            String toggleMessage, boolean toggleState) {
        super(parentShell, dialogTitle, message, status, displayMask);
        this.fToggleMessage = toggleMessage;
        this.fToggleState = toggleState;
        super.setBlockOnOpen(true);
    }
    
    /**
     * Convenience method that opens a modal dialog with status details and a
     * toggle option.
     * <p>
     * Must be called from the UI thread.
     * 
     * @param parentShell
     *            The parent shell for the dialog.
     * @param displayMask
     *            The {@link IStatus} type of the dialog.
     * @param dialogTitle
     *            The title to use for the dialog.
     * @param message
     *            The message describing the purpose of the dialog.
     * @param details
     *            The contents of the details section of the page.
     * @param toggleMessage
     *            The message for the toggle control.
     * @param toggleState
     *            The initial state for the toggle.
     * @return the state of the toggle component after the dialog closes
     */
    public static boolean openDialog(Shell parentShell, int displayMask, String dialogTitle, String message, String[] details, String toggleMessage, boolean toggleState) {
        MultiStatus info = new MultiStatus(BuildUIPlugin.PLUGIN_ID, 1, message, null);
        for (String detail: details) {
            info.add(new Status(displayMask, BuildUIPlugin.PLUGIN_ID, detail));
        }
        
        ToggleStatusNotificationDialog dialog = new ToggleStatusNotificationDialog(parentShell, displayMask, dialogTitle, null, info,  toggleMessage, toggleState);
        dialog.open();
        return dialog.getToggleState();
    }
    
    /**
     * @see Dialog#createDialogArea(Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite dialogAreaComposite = (Composite) super.createDialogArea(parent);
        createToggleButton(dialogAreaComposite);
        dialogAreaComposite.pack();
        return dialogAreaComposite;
    }

    /**
     * Creates a toggle button without any text or state.  The text and state
     * will be created by <code>createDialogArea</code>. 
     * 
     * @param parent
     *            The composite in which the toggle button should be placed;
     *            must not be <code>null</code>.
     * @return The added toggle button; never <code>null</code>.
     */
    protected Button createToggleButton(Composite parent) {
        final Button button = new Button(parent, SWT.CHECK | SWT.LEFT);
        fToggleButton = button;
        button.setText(fToggleMessage);
        GridData data = new GridData(SWT.NONE);
        data.verticalIndent = 10;
        data.horizontalSpan = 2;
        button.setLayoutData(data);
        button.setFont(parent.getFont());
        button.setSelection(fToggleState);
        button.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                fToggleState = button.getSelection();
            }

        });

        return button;
    }
    
    /**
     * Returns the toggle state. This can be called even after the dialog is
     * closed.
     * 
     * @return <code>true</code> if the toggle button is checked,
     *         <code>false</code> if not
     */
    public boolean getToggleState() {
        return fToggleState;
    }
    
}
