/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.build.internal.ui.notification;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.ibm.team.build.common.builddefinition.IEmailNotificationConfigurationElement;
import com.ibm.team.build.common.notification.NotificationCriteria;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.rcp.ui.TeamContributorSelectionDialog;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * This dialog allows the editing and saving of NotificationCriteria objects
 */
public class EditEmailNotificationCriteriaDialog extends Dialog {

    private NotificationCriteria fCriteria;

    private Button fPersonalBuildButton;

    private Text fCriteriaText;

    private Button fCompletedBuildButton;
    private Button fCompletedBuildReportAlwaysButton;
    private Button fCompletedBuildReportErrorsButton;

    private Button fAbandonedBuildButton;
    private Button fAbandonedBuildReportAlwaysButton;
    private Button fAbandonedBuildReportErrorsButton;

    private TableViewer fEmailsViewer;
    private Button fAddEmailButton;
    private Button fRemoveEmailButton;

    private TableViewer fUsersViewer;
    private Button fAddUsersButton;
    private Button fRemoveUserButton;

    private TableViewer fRolesViewer;
    private Button fAddRolesButton;
    private Button fRemoveRolesButton;

    private Button fChangesDelivererButton;
    private Button fBuildRequestorButton;

    private Button okButton;

    private ContributorsQueryJob fContributorsQueryJob = new ContributorsQueryJob();

    private ITeamRepository fTeamRepository;
    private IProcessAreaHandle fProcessAreaHandle;

    private String[] fRoleDefinitions;
    private Set<String> fSelectedRoles;
    private Map<String, String> fUserIds;
    private Set<String> fEmails;

    public EditEmailNotificationCriteriaDialog(Control control, ITeamRepository teamRepository, IProcessAreaHandle processAreaHandle, NotificationCriteria criteria,
            String[] roleDefinitions) {
        super(control.getShell());
        fTeamRepository = teamRepository;
        fProcessAreaHandle = processAreaHandle;
        fCriteria = criteria;
        fRoleDefinitions = roleDefinitions;

        fSelectedRoles = new HashSet<String>();
        fUserIds = new HashMap<String, String>();
        fEmails = new HashSet<String>();

        fSelectedRoles.addAll(Arrays.asList(fCriteria.getRoles()));
        for (String user : fCriteria.getUsers()) {
            fUserIds.put(user, ""); //$NON-NLS-1$
        }
        fEmails.addAll(Arrays.asList(fCriteria.getEmails()));
    }

    @Override
    public void create() {
        super.create();
        okButton.setEnabled(!fCriteria.getCriteriaName().trim().equals("")); //$NON-NLS-1$

        final String[] userIds = fUserIds.keySet().toArray(new String[fUserIds.size()]);
        ContributorFetchJob job = new ContributorFetchJob(fTeamRepository, fUsersViewer.getTable(),
                new AbstractUserNameRetrievalRunnable() {
                    public void run() {
                        Map<String, String> userNameMap = getUserNameMap();
                        for (int i = 0; i < userIds.length; i++) {
                            if (fUserIds.containsKey(userIds[i])) {
                                fUserIds.put(userIds[i], userNameMap.get(userIds[i]));
                            }
                        }
                        fUsersViewer.refresh();
                    }
                });
        job.setUserIds(userIds);
        job.schedule();
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(Messages.EditEmailNotificationCriteriaDialog_DialogTitle);
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);
        ((GridLayout) composite.getLayout()).numColumns = 2;
        ((GridLayout) composite.getLayout()).makeColumnsEqualWidth = true;

        GC gc = new GC(composite);
        gc.setFont(JFaceResources.getDialogFont());
        FontMetrics metrics = gc.getFontMetrics();
        int widthHint = metrics.getAverageCharWidth() * 8;
        gc.dispose();

        fPersonalBuildButton = new Button(composite, SWT.CHECK);
        fPersonalBuildButton.setText(Messages.EditEmailNotificationCriteriaDialog_PersonalBuildButtonText);
        fPersonalBuildButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1));
        fPersonalBuildButton.setSelection(fCriteria.isPersonalCriteria());

        createNotificationCriteriaGroup(composite, widthHint);
        createEmailAddressesGroup(composite);
        createUsersGroup(composite);
        createRolesGroup(composite);
        fCriteriaText.setFocus();

        addListeners();

        Dialog.applyDialogFont(composite);

        return composite;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        super.createButtonsForButtonBar(parent);
        okButton = getButton(IDialogConstants.OK_ID);
    }

    private void createNotificationCriteriaGroup(Composite composite, int widthHint) {
        Group group = new Group(composite, SWT.NONE);
        group.setText(Messages.EditEmailNotificationCriteriaDialog_EmailNotificationCriteriaHeader);
        group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        group.setLayout(new GridLayout(2, false));

        Label label = new Label(group, SWT.LEAD);
        label.setText(Messages.EditEmailNotificationCriteriaDialog_CriteriaNameLabel);
        label.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));

        fCriteriaText = new Text(group, SWT.BORDER);
        fCriteriaText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        fCriteriaText.setText(fCriteria.getCriteriaName());

        Composite nested = new Composite(group, SWT.NONE);
        nested.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
        GridLayout layout = new GridLayout(3, false);
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        nested.setLayout(layout);

        new Label(nested, SWT.SEPARATOR | SWT.HORIZONTAL).setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true,
                3, 1));

        fCompletedBuildButton = new Button(nested, SWT.CHECK);
        fCompletedBuildButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3, 1));
        fCompletedBuildButton.setText(Messages.EditEmailNotificationCriteriaDialog_NotificationCompletedCriteria);
        fCompletedBuildButton.setSelection(fCriteria.getNotifyCompletedCondition() != NotificationCriteria.NotifyCondition.NEVER);

        GridData data = new GridData(SWT.BEGINNING, SWT.BEGINNING, false, false);
        data.widthHint = widthHint;

        Composite radioComposite = new Composite(nested, SWT.NONE);
        layout = new GridLayout(3, false);
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        radioComposite.setLayout(layout);

        new Label(radioComposite, SWT.NONE).setLayoutData(data);

        fCompletedBuildReportAlwaysButton = new Button(radioComposite, SWT.RADIO);
        fCompletedBuildReportAlwaysButton.setText(Messages.EditEmailNotificationCriteriaDialog_NotificationAlways);
        fCompletedBuildReportAlwaysButton.setEnabled(fCriteria.getNotifyCompletedCondition() != NotificationCriteria.NotifyCondition.NEVER);

        fCompletedBuildReportErrorsButton = new Button(radioComposite, SWT.RADIO);
        fCompletedBuildReportErrorsButton.setText(Messages.EditEmailNotificationCriteriaDialog_NotificationErrors);
        fCompletedBuildReportErrorsButton.setEnabled(fCriteria.getNotifyCompletedCondition() != NotificationCriteria.NotifyCondition.NEVER);

        if (fCriteria.getNotifyCompletedCondition() != NotificationCriteria.NotifyCondition.NEVER) {
            fCompletedBuildReportAlwaysButton.setEnabled(true);
            fCompletedBuildReportErrorsButton.setEnabled(true);

            fCompletedBuildReportAlwaysButton.setSelection(fCriteria.getNotifyCompletedCondition() == NotificationCriteria.NotifyCondition.ALWAYS);
            fCompletedBuildReportErrorsButton.setSelection(fCriteria.getNotifyCompletedCondition() == NotificationCriteria.NotifyCondition.ON_ERROR);
        } else {
            fCompletedBuildReportAlwaysButton.setEnabled(false);
            fCompletedBuildReportErrorsButton.setEnabled(false);

            fCompletedBuildReportAlwaysButton.setSelection(false);
            fCompletedBuildReportErrorsButton.setSelection(true);
        }

        fAbandonedBuildButton = new Button(nested, SWT.CHECK);
        fAbandonedBuildButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3, 1));
        fAbandonedBuildButton.setText(Messages.EditEmailNotificationCriteriaDialog_NotificationAbandonedCriteria);
        fAbandonedBuildButton.setSelection(fCriteria.getNotifyAbandonedCondition() != NotificationCriteria.NotifyCondition.NEVER);

        radioComposite = new Composite(nested, SWT.NONE);
        layout = new GridLayout(3, false);
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        radioComposite.setLayout(layout);

        data = new GridData(SWT.BEGINNING, SWT.BEGINNING, false, false);
        data.widthHint = widthHint;

        new Label(radioComposite, SWT.NONE).setLayoutData(data);

        fAbandonedBuildReportAlwaysButton = new Button(radioComposite, SWT.RADIO);
        fAbandonedBuildReportAlwaysButton.setText(Messages.EditEmailNotificationCriteriaDialog_NotificationAlways);

        fAbandonedBuildReportErrorsButton = new Button(radioComposite, SWT.RADIO);
        fAbandonedBuildReportErrorsButton.setText(Messages.EditEmailNotificationCriteriaDialog_NotificationErrors);

        if (fCriteria.getNotifyAbandonedCondition() != NotificationCriteria.NotifyCondition.NEVER) {
            fAbandonedBuildReportAlwaysButton.setEnabled(true);
            fAbandonedBuildReportErrorsButton.setEnabled(true);

            fAbandonedBuildReportAlwaysButton.setSelection(fCriteria.getNotifyAbandonedCondition() == NotificationCriteria.NotifyCondition.ALWAYS);
            fAbandonedBuildReportErrorsButton.setSelection(fCriteria.getNotifyAbandonedCondition() == NotificationCriteria.NotifyCondition.ON_ERROR);
        } else {
            fAbandonedBuildReportAlwaysButton.setEnabled(false);
            fAbandonedBuildReportErrorsButton.setEnabled(false);

            fAbandonedBuildReportAlwaysButton.setSelection(true);
            fAbandonedBuildReportErrorsButton.setSelection(false);
        }
    }

    private void createEmailAddressesGroup(Composite composite) {
        Group group = new Group(composite, SWT.NONE);
        group.setText(Messages.EditEmailNotificationCriteriaDialog_EmailAddressesHeader);
        group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        group.setLayout(new GridLayout(2, false));

        fEmailsViewer = new TableViewer(group, SWT.BORDER | SWT.MULTI);
        fEmailsViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2));
        fEmailsViewer.setLabelProvider(new LabelProvider());
        fEmailsViewer.setContentProvider(new ArrayContentProvider());
        fEmailsViewer.setComparator(new ViewerComparator());
        fEmailsViewer.setInput(fEmails);

        fAddEmailButton = new Button(group, SWT.PUSH);
        fAddEmailButton.setText(Messages.EditEmailNotificationCriteriaDialog_EmailAddressesAddButton);
        fAddEmailButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));

        fRemoveEmailButton = new Button(group, SWT.PUSH);
        fRemoveEmailButton.setText(Messages.EditEmailNotificationCriteriaDialog_EmailAddressesRemoveButton);
        fRemoveEmailButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
        fRemoveEmailButton.setEnabled(false);
    }

    private void createUsersGroup(Composite composite) {
        Group group = new Group(composite, SWT.NONE);
        group.setText(Messages.EditEmailNotificationCriteriaDialog_UsersHeader);
        group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        group.setLayout(new GridLayout(2, false));

        fUsersViewer = new TableViewer(group, SWT.BORDER | SWT.MULTI);
        fUsersViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2));
        fUsersViewer.setLabelProvider(new LabelProvider() {
            @Override
            public String getText(Object element) {
                Entry<String, String> entry = (Entry<String, String>) element;
                String key = entry.getKey();
                String value = entry.getValue();
                if (value.equals("")) { //$NON-NLS-1$
                    return key;
                } else {
                    return key + " (" + value + ")"; //$NON-NLS-1$ //$NON-NLS-2$
                }
            }
        });
        fUsersViewer.setContentProvider(new IStructuredContentProvider() {
            public Object[] getElements(Object inputElement) {
                return ((Map<?, ?>) inputElement).entrySet().toArray();
            }

            public void dispose() {
                // nothing to do
            }

            public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
                // nothing to do
            }

        });
        fUsersViewer.setComparator(new ViewerComparator());
        fUsersViewer.setInput(fUserIds);

        fAddUsersButton = new Button(group, SWT.PUSH);
        fAddUsersButton.setText(Messages.EditEmailNotificationCriteriaDialog_UsersAddButton);
        fAddUsersButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));

        fRemoveUserButton = new Button(group, SWT.PUSH);
        fRemoveUserButton.setText(Messages.EditEmailNotificationCriteriaDialog_UsersRemoveButton);
        fRemoveUserButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
        fRemoveUserButton.setEnabled(false);

        fChangesDelivererButton = new Button(group, SWT.CHECK);
        fChangesDelivererButton.setText(Messages.EditEmailNotificationCriteriaDialog_UsersChangesCriteria);
        fChangesDelivererButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1));
        fChangesDelivererButton.setSelection(fCriteria.isNotifySCM());

        fBuildRequestorButton = new Button(group, SWT.CHECK);
        fBuildRequestorButton.setText(Messages.EditEmailNotificationCriteriaDialog_UsersRequestCriteria);
        fBuildRequestorButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1));
        fBuildRequestorButton.setSelection(fCriteria.isNotifyRequestor());
    }

    private void createRolesGroup(Composite composite) {
        Group group = new Group(composite, SWT.NONE);
        group.setText(Messages.EditEmailNotificationCriteriaDialog_RolesHeader);
        group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        group.setLayout(new GridLayout(2, false));

        fRolesViewer = new TableViewer(group, SWT.BORDER | SWT.MULTI);
        fRolesViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2));
        fRolesViewer.setLabelProvider(new LabelProvider());
        fRolesViewer.setContentProvider(new ArrayContentProvider());
        fRolesViewer.setComparator(new ViewerComparator());
        fRolesViewer.setInput(fSelectedRoles);

        fAddRolesButton = new Button(group, SWT.PUSH);
        fAddRolesButton.setText(Messages.EditEmailNotificationCriteriaDialog_RolesAddButton);
        fAddRolesButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));

        fRemoveRolesButton = new Button(group, SWT.PUSH);
        fRemoveRolesButton.setText(Messages.EditEmailNotificationCriteriaDialog_RolesRemoveButton);
        fRemoveRolesButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
        fRemoveRolesButton.setEnabled(false);
    }

    protected void handleCompletedBuildButtonSelected() {
        boolean enabled = fCompletedBuildButton.getSelection();
        fCompletedBuildReportAlwaysButton.setEnabled(enabled);
        fCompletedBuildReportErrorsButton.setEnabled(enabled);
    }

    protected void handleAbandonedBuildButtonSelected() {
        boolean enabled = fAbandonedBuildButton.getSelection();
        fAbandonedBuildReportAlwaysButton.setEnabled(enabled);
        fAbandonedBuildReportErrorsButton.setEnabled(enabled);
    }

    protected void handleEmailSelectionChanged() {
        IStructuredSelection selection = (IStructuredSelection) fEmailsViewer.getSelection();
        fRemoveEmailButton.setEnabled(!selection.isEmpty());
    }

    protected IInputValidator getEmailValidator() {
        return new IInputValidator() {
            public String isValid(String email) {

                if (email.trim().length() == 0) {
                    // Do not display an error initially
                    return ""; //$NON-NLS-1$
                } else {
                    if (email.contains(IEmailNotificationConfigurationElement.EMAILS_TO_NOTIFY_SEPARATOR)) {
                        // Ensure the email separator characters are not used (normally a comma).
                        return NLS.bind(Messages.EditEmailNotificationCriteriaDialog_EmailAddressesSeparatorCharactersNotAllowedMessage,
                                IEmailNotificationConfigurationElement.EMAILS_TO_NOTIFY_SEPARATOR);
                    }
                    // Otherwise attempt to match a valid email sequence
                    Pattern p = Pattern.compile(".+@.+\\.[a-z]+"); //$NON-NLS-1$
                    Matcher m = p.matcher(email);

                    boolean matchFound = m.matches();

                    if (matchFound) {
                        return null;
                    } else {
                        return Messages.EditEmailNotificationCriteriaDialog_EmailAddressesInvalidEmailMessage;
                    }
                }
            }
        };
    }

    protected InputDialog getEmailInputDialog() {
        return new InputDialog(fAddEmailButton.getShell(),
                Messages.EditEmailNotificationCriteriaDialog_EmailAddressesPromptTitle,
                Messages.EditEmailNotificationCriteriaDialog_EmailAddressesPromptDescription, null, getEmailValidator());
    }

    protected void handleAddEmailButtonSelected() {
        InputDialog inputDialog = getEmailInputDialog();
        if (inputDialog.open() == Window.OK) {
            if (fEmails.add(inputDialog.getValue().trim())) {
                fEmailsViewer.refresh();
            }
        }
    }

    protected void handleRemoveEmailButtonSelected() {
        IStructuredSelection selection = (IStructuredSelection) fEmailsViewer.getSelection();
        fEmails.removeAll(selection.toList());
        fEmailsViewer.refresh();
    }

    protected void handleUserSelectionChanged() {
        IStructuredSelection selection = (IStructuredSelection) fUsersViewer.getSelection();
        fRemoveUserButton.setEnabled(!selection.isEmpty());
    }

    protected void handleAddUserButtonSelected() {
        TeamContributorSelectionDialog dialog = new TeamContributorSelectionDialog(fAddUsersButton.getShell(),
                fTeamRepository);
        dialog.setShellTitle(Messages.EditEmailNotificationCriteriaDialog_TeamContributorSelectionDialogTitle);
        dialog.setTitle(Messages.EditEmailNotificationCriteriaDialog_TeamContributorSelectionDialogTitle);
        dialog.setMessage(Messages.EditEmailNotificationCriteriaDialog_TeamContributorSelectionDialogMessage);
        if (dialog.open() == Window.OK) {
            IContributorHandle[] result = dialog.getContributorResult();
            addSelectedContributors(result);
        }
    }

    protected void handleRemoveUserButtonSelected() {
        IStructuredSelection selection = (IStructuredSelection) fUsersViewer.getSelection();
        for (Iterator<?> iterator = selection.iterator(); iterator.hasNext();) {
            Entry<String, String> entry = (Entry<String, String>) iterator.next();
            fUserIds.remove(entry.getKey());
        }
        fUsersViewer.refresh();
    }

    protected void handleRoleSelectionChanged() {
        IStructuredSelection selection = (IStructuredSelection) fRolesViewer.getSelection();
        fRemoveRolesButton.setEnabled(!selection.isEmpty());
    }

    protected void handleRemoveRoleButtonSelected() {
        IStructuredSelection selection = (IStructuredSelection) fRolesViewer.getSelection();
        fSelectedRoles.removeAll(selection.toList());
        fRolesViewer.refresh();
    }

    private void addListeners() {
        fCriteriaText.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                okButton.setEnabled(!fCriteriaText.getText().trim().equals("")); //$NON-NLS-1$
            }
        });

        fCompletedBuildButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleCompletedBuildButtonSelected();
            }
        });

        fAbandonedBuildButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleAbandonedBuildButtonSelected();
            }
        });

        fEmailsViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                handleEmailSelectionChanged();
            }
        });

        fAddEmailButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleAddEmailButtonSelected();
            }
        });

        fRemoveEmailButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleRemoveEmailButtonSelected();
            }
        });

        fUsersViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                handleUserSelectionChanged();
            }
        });

        fAddUsersButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleAddUserButtonSelected();
            }
        });

        fRemoveUserButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleRemoveUserButtonSelected();
            }
        });

        fRolesViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                handleRoleSelectionChanged();
            }
        });

        fAddRolesButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleAddRolesButtonSelected();
            }
        });

        fRemoveRolesButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleRemoveRoleButtonSelected();
            }
        });
    }

    protected NotificationCriteria getCriteria() {
        return fCriteria;
    }

    protected Button getPersonalBuildButton() {
        return fPersonalBuildButton;
    }

    protected Text getCriteriaText() {
        return fCriteriaText;
    }

    protected Button getCompletedBuildButton() {
        return fCompletedBuildButton;
    }

    protected Button getCompletedBuildReportAlwaysButton() {
        return fCompletedBuildReportAlwaysButton;
    }

    protected Button getCompletedBuildReportErrorsButton() {
        return fCompletedBuildReportErrorsButton;
    }

    protected Button getAbandonedBuildButton() {
        return fAbandonedBuildButton;
    }

    protected Button getAbandonedBuildReportAlwaysButton() {
        return fAbandonedBuildReportAlwaysButton;
    }

    protected Button getAbandonedBuildReportErrorsButton() {
        return fAbandonedBuildReportErrorsButton;
    }

    protected TableViewer getEmailsViewer() {
        return fEmailsViewer;
    }

    protected Button getAddEmailButton() {
        return fAddEmailButton;
    }

    protected Button getRemoveEmailButton() {
        return fRemoveEmailButton;
    }

    protected TableViewer getUsersViewer() {
        return fUsersViewer;
    }

    protected Button getAddUsersButton() {
        return fAddUsersButton;
    }

    protected Button getRemoveUserButton() {
        return fRemoveUserButton;
    }

    protected TableViewer getRolesViewer() {
        return fRolesViewer;
    }

    protected Button getAddRolesButton() {
        return fAddRolesButton;
    }

    protected Button getRemoveRolesButton() {
        return fRemoveRolesButton;
    }

    protected Button getChangesDelivererButton() {
        return fChangesDelivererButton;
    }

    protected Button getBuildRequestorButton() {
        return fBuildRequestorButton;
    }

    @Override
    protected Button getOKButton() {
        return okButton;
    }

    @Override
    protected void okPressed() {
        fCriteria.setPersonalCriteria(fPersonalBuildButton.getSelection());

        fCriteria.setCriteriaName(fCriteriaText.getText());

        NotificationCriteria.NotifyCondition completedCondition;
        if (!fCompletedBuildButton.getSelection()) {
            completedCondition = NotificationCriteria.NotifyCondition.NEVER;
        } else {
            completedCondition = fCompletedBuildReportAlwaysButton.getSelection() ? NotificationCriteria.NotifyCondition.ALWAYS : NotificationCriteria.NotifyCondition.ON_ERROR;
        }
        fCriteria.setNotifyCompletedCondition(completedCondition);
        
        NotificationCriteria.NotifyCondition abandonedCondition;
        if (!fAbandonedBuildButton.getSelection()) {
            abandonedCondition = NotificationCriteria.NotifyCondition.NEVER;
        } else {
            abandonedCondition = fAbandonedBuildReportAlwaysButton.getSelection() ? NotificationCriteria.NotifyCondition.ALWAYS : NotificationCriteria.NotifyCondition.ON_ERROR;
        }
        fCriteria.setNotifyAbandonedCondition(abandonedCondition);

        fCriteria.setEmails(fEmails.toArray(new String[fEmails.size()]));

        fCriteria.setNotifySCM(fChangesDelivererButton.getSelection());
        fCriteria.setNotifyRequestor(fBuildRequestorButton.getSelection());
        fCriteria.setUsers(fUserIds.keySet().toArray(new String[fUserIds.size()]));

        fCriteria.setRoles(fSelectedRoles.toArray(new String[fSelectedRoles.size()]));

        super.okPressed();
    }

    private void addSelectedContributors(final IContributorHandle[] result) {
        fContributorsQueryJob.addHandles(result);
        fContributorsQueryJob.setSystem(true);
        fContributorsQueryJob.schedule();
    }

    private void refreshContributors() {
        Control control = fUsersViewer.getControl();
        if (!control.isDisposed()) {
            Display display = control.getDisplay();
            display.asyncExec(new Runnable() {
                public void run() {
                    if (!fUsersViewer.getControl().isDisposed()) {
                        fUsersViewer.refresh();
                    }
                }
            });
        }
    }

    protected RoleSelectionDialog getRoleSelectionDialog(Shell shell) {
        return new RoleSelectionDialog(shell, fTeamRepository, fProcessAreaHandle, fRoleDefinitions);
    }

    protected void handleAddRolesButtonSelected() {
        final RoleSelectionDialog dialog = getRoleSelectionDialog(fAddRolesButton.getShell());
        if (dialog.open() == Window.OK) {
            boolean modified = false;
            for (Object o : dialog.getResult()) {
                modified |= fSelectedRoles.add((String) o);
            }

            if (modified) {
                fRolesViewer.refresh();
            }
        }
    }

    class ContributorsQueryJob extends Job {

        private List<IContributorHandle> fHandles = Collections.synchronizedList(new ArrayList<IContributorHandle>());

        public ContributorsQueryJob() {
            super(Messages.EditEmailNotificationCriteriaDialog_ContributorsQueryJobName);
        }

        public void addHandles(IContributorHandle[] handles) {
            synchronized (fHandles) {
                fHandles.addAll(Arrays.asList(handles));
            }
        }

        public List<IContributorHandle> getHandles() {
            synchronized (fHandles) {
                List<IContributorHandle> handles = new ArrayList<IContributorHandle>(fHandles);
                fHandles.clear();
                return handles;
            }
        }

        @Override
        protected IStatus run(IProgressMonitor monitor) {
            try {
                List<IContributorHandle> handles = getHandles();
                List<?> items = fTeamRepository.itemManager().fetchCompleteItems(handles, IItemManager.DEFAULT, monitor);

                for (Object object : items) {
                    IContributor contributor = (IContributor) object;
                    if (!fUserIds.containsKey(contributor.getUserId())) {
                        fUserIds.put(contributor.getUserId(), contributor.getName());
                    }
                }

                refreshContributors();

                if (!fHandles.isEmpty()) {
                    schedule();
                }
            } catch (TeamRepositoryException e) {
                return new Status(IStatus.ERROR, "com.ibm.team.build.notification.ui", //$NON-NLS-1$
                        Messages.EditEmailNotificationCriteriaDialog_ContributorsQueryJobError, e);
            }

            return Status.OK_STATUS;
        }

    }
}
