/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.notification;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.accessibility.ACC;
import org.eclipse.swt.accessibility.AccessibleAdapter;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import com.ibm.team.build.common.builddefinition.IEmailNotificationConfigurationElement;
import com.ibm.team.build.common.notification.NotificationCriteriaFromBuildDefinition;
import com.ibm.team.build.common.notification.NotificationCriteriaFromBuildDefinitionHelper;
import com.ibm.team.build.internal.ui.IHelpContextIds;
import com.ibm.team.build.internal.ui.editors.builddefinition.BasicConfigurationElementEditor;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * An editor for the email notification configuration element.
 * 
 * @since 0.8
 */
public class NotificationConfigurationEditor extends BasicConfigurationElementEditor {

    private FormToolkit fToolkit;
    
    private Section fNotificationCriteriaSection;

    private CheckboxTableViewer fCriteriasViewer;

    private Button fAddCriteriaButton;

    private Button fEditCriteriaButton;
    
    private Button fRemoveCriteriaButton;

    private Text fCriteriaSummaryText;

    private ContributorFetchJob fSummaryJob;

    private List<String> fRoleDefinitions;

    private List<NotificationCriteriaFromBuildDefinition> fCriterias;
    
    /**
     * Create the editor.
     * 
     * @param elementId
     *            The id of the build configuration element being edited.
     * @param title
     *            The editor title.
     */
    public NotificationConfigurationEditor(String elementId, String title) {
        super(elementId, title);

        fCriterias = new ArrayList<NotificationCriteriaFromBuildDefinition>();
        fRoleDefinitions = new ArrayList<String>();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void createContent(Composite parent, FormToolkit toolkit) {
        reset();
        
        fToolkit = toolkit;

        parent.setLayout(new TableWrapLayout());
        
        fNotificationCriteriaSection = createSection(parent,
                Messages.NotificationConfigurationEditor_NotificationSectionHeader,
                Messages.NotificationConfigurationEditor_NotificationSectionDescription, true, 1);

        fNotificationCriteriaSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB, 1, 1));
        
        Composite sectionContents = fToolkit.createComposite(fNotificationCriteriaSection);
        GridLayout sectionContentsLayout = new GridLayout(3, false);
        sectionContents.setLayout(sectionContentsLayout);

        fCriteriasViewer = new CheckboxTableViewer(fToolkit.createTable(sectionContents, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL
                | SWT.H_SCROLL | SWT.CHECK));
        GridDataFactory.fillDefaults().grab(true, true).span(1, 3).hint(100, fCriteriasViewer.getTable().getItemHeight() * 25).applyTo(fCriteriasViewer.getTable());
        fCriteriasViewer.setContentProvider(new ArrayContentProvider());
        fCriteriasViewer.setLabelProvider(new LabelProvider() {
            @Override
            public String getText(Object element) {
                NotificationCriteriaFromBuildDefinition criteria = (NotificationCriteriaFromBuildDefinition) element;
                return criteria.getCriteriaName();
            }
        });
        fCriteriasViewer.setInput(fCriterias);

        fAddCriteriaButton = fToolkit.createButton(sectionContents,
                Messages.NotificationConfigurationEditor_CriteriaAddButton, SWT.PUSH);
        fAddCriteriaButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));

        fCriteriaSummaryText = fToolkit.createText(sectionContents, null, SWT.LEAD | SWT.WRAP | SWT.V_SCROLL | SWT.READ_ONLY);
        GridDataFactory.fillDefaults().grab(true, true).span(1, 3).hint(100, fCriteriaSummaryText.getLineHeight() * 25).applyTo(fCriteriaSummaryText);
        fCriteriaSummaryText.getAccessible().addAccessibleListener(new AccessibleAdapter() {
            @Override
            public void getName(AccessibleEvent e) {
                if (e.childID == ACC.CHILDID_SELF) {
                    e.result = Messages.NotificationConfigurationEditor_CriteriaSummaryAccessibilityLabel;
                }
            }
        });
        
        fEditCriteriaButton = fToolkit.createButton(sectionContents,
                Messages.NotificationConfigurationEditor_CriteriaEditButton, SWT.PUSH);
        fEditCriteriaButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
        fEditCriteriaButton.setEnabled(false);

        fRemoveCriteriaButton = fToolkit.createButton(sectionContents,
                Messages.NotificationConfigurationEditor_CriteriaRemoveButton, SWT.PUSH);
        fRemoveCriteriaButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
        fRemoveCriteriaButton.setEnabled(false);

        fNotificationCriteriaSection.setClient(sectionContents);

        createNewCriteriasFromProperties();
        
        attachListeners();

        fSummaryJob = new ContributorFetchJob(getTeamRepository(), fCriteriaSummaryText,
                new AbstractUserNameRetrievalRunnable() {
                    public void run() {
                        updateCriteriaSummary(getUserNameMap());
                    }
                });
        
        handleCriteriaViewersSelectionChanged();
    }
    
    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected String getContextHelpId() {
        return IHelpContextIds.HELP_CONTEXT_BUILD_DEFINITION_EDITOR_EMAIL_NOTIFICATION;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public boolean validate() {
        return true;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public Control getFocusControl() {
        return fCriteriasViewer.getTable();
    }
    
    private void reset() {
        fCriterias.clear();
        fRoleDefinitions.clear();
    }

    protected void updateCriteriaSummary(Map<String, String> userNameMap) {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        NotificationCriteriaFromBuildDefinition criteria = (NotificationCriteriaFromBuildDefinition) selection.getFirstElement();
        if (criteria != null) {
            String summary = criteria.getCriteriaSummary(userNameMap, Text.DELIMITER);
            fCriteriaSummaryText.setText(summary);
        }
    }

    protected void handleCriteriaViewersSelectionChanged() {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        int size = selection.size();
        if (size == 1) {
            NotificationCriteriaFromBuildDefinition criteria = (NotificationCriteriaFromBuildDefinition) selection.getFirstElement();
            fSummaryJob.cancel();
            fSummaryJob.setUserIds(criteria.getUsers());
            fSummaryJob.schedule();

            String summary = criteria.getCriteriaSummary(null, Text.DELIMITER);
            fCriteriaSummaryText.setText(summary);

            // can only edit one criteria at a time
            fEditCriteriaButton.setEnabled(true);
        } else {
            fCriteriaSummaryText.setText(""); //$NON-NLS-1$

            // can only edit one criteria at a time
            fEditCriteriaButton.setEnabled(false);
        }

        // can't remove if nothing is selected
        fRemoveCriteriaButton.setEnabled(size != 0);
    }

    protected void handleCriteriaViewerCheckStateChanged(NotificationCriteriaFromBuildDefinition criteria, boolean checked) {
        criteria.setEnabled(checked);
        NotificationCriteriaFromBuildDefinitionHelper.createPropertiesFromCriteria(fWorkingCopy, criteria);
        setDirty(true);
    }

    protected EditEmailNotificationCriteriaDialog createDialog(Control control, ITeamRepository teamRepository,
            IProcessAreaHandle processArea, NotificationCriteriaFromBuildDefinition criteria, String[] roleDefinitions) {
        return new EditEmailNotificationCriteriaDialog(control.getShell(), teamRepository, processArea, criteria,
                roleDefinitions);
    }

    private EditEmailNotificationCriteriaDialog createDialog(Control control, NotificationCriteriaFromBuildDefinition criteria) {
        return createDialog(control.getShell(), getTeamRepository(), fWorkingCopy.getProcessArea(), criteria,
                fRoleDefinitions.toArray(new String[fRoleDefinitions.size()]));
    }

    protected void handleAddCriteriaButtonSelected() {
        NotificationCriteriaFromBuildDefinition newCriteria = new NotificationCriteriaFromBuildDefinition();
        EditEmailNotificationCriteriaDialog dialog = createDialog(fAddCriteriaButton, newCriteria);
        if (dialog.open() == Window.OK) {
            fCriterias.add(newCriteria);
            fCriteriasViewer.add(newCriteria);
            if (newCriteria.isEnabled()) {
                fCriteriasViewer.setChecked(newCriteria, true);
            }
            fCriteriasViewer.setSelection(new StructuredSelection(newCriteria), true);

            fCriteriasViewer.update(newCriteria, null);
            // set the selection again so that the summary can be updated
            fCriteriasViewer.setSelection(new StructuredSelection(newCriteria));

            int numCriterias = NotificationCriteriaFromBuildDefinitionHelper.getNumCriteriasProperty(fWorkingCopy);

            newCriteria.setPropertyNamePrefixNumber(numCriterias);
            NotificationCriteriaFromBuildDefinitionHelper.createPropertiesFromCriteria(fWorkingCopy, newCriteria);

            fWorkingCopy.setConfigurationProperty(IEmailNotificationConfigurationElement.ELEMENT_ID,
                    IEmailNotificationConfigurationElement.PROPERTY_NUMBER_OF_CRITERIA, Integer.toString(numCriterias + 1));

            setDirty(true);
        }
    }

    protected void handleEditCriteriaButtonSelected() {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        editCriteria(selection);
    }

    protected void handleOpenCriteria(IStructuredSelection selection) {
        editCriteria(selection);
    }

    protected void editCriteria(IStructuredSelection selection) {
        if (selection.size() != 1) {
            return;
        }
        NotificationCriteriaFromBuildDefinition modifiedCriteria = (NotificationCriteriaFromBuildDefinition) selection.getFirstElement();
        EditEmailNotificationCriteriaDialog dialog = createDialog(fEditCriteriaButton, modifiedCriteria);
        if (dialog.open() == Window.OK) {
            fCriteriasViewer.update(modifiedCriteria, null);
            // set the selection again so that the summary can be updated
            fCriteriasViewer.refresh();
            fCriteriasViewer.setSelection(new StructuredSelection(modifiedCriteria));
            NotificationCriteriaFromBuildDefinitionHelper.createPropertiesFromCriteria(fWorkingCopy, modifiedCriteria);
            setDirty(true);
        }   
    }

    protected void handleRemoveCriteriaButtonSelected() {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        fCriterias.removeAll(selection.toList());
        fCriteriasViewer.refresh();
        createNewPropertiesFromExistingCriteria();
        setDirty(true);
    }

    private void attachListeners() {
        fCriteriasViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                handleCriteriaViewersSelectionChanged();
            }
        });

        fCriteriasViewer.addCheckStateListener(new ICheckStateListener() {
            public void checkStateChanged(CheckStateChangedEvent event) {
                handleCriteriaViewerCheckStateChanged((NotificationCriteriaFromBuildDefinition) event.getElement(),
                        event.getChecked());
            }
        });

        fCriteriasViewer.addOpenListener(new IOpenListener() {
            public void open(OpenEvent event) {
                handleOpenCriteria((IStructuredSelection) event.getSelection());
            }
        });

        fAddCriteriaButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleAddCriteriaButtonSelected();
            }
        });

        fEditCriteriaButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleEditCriteriaButtonSelected();
            }
        });

        fRemoveCriteriaButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleRemoveCriteriaButtonSelected();
            }
        });
    }

    protected List<NotificationCriteriaFromBuildDefinition> getCriterias() {
        return fCriterias;
    }

    protected Collection<String> getRoleDefinitions() {
        return fRoleDefinitions;
    }

    protected CheckboxTableViewer getCriteriasViewer() {
        return fCriteriasViewer;
    }

    protected Button getAddCriteriaButton() {
        return fAddCriteriaButton;
    }

    protected Button getEditCriteriaButton() {
        return fEditCriteriaButton;
    }

    protected Button getRemoveCriteriaButton() {
        return fRemoveCriteriaButton;
    }
    
    protected Text getSummaryLabel() {
        return fCriteriaSummaryText;
    }
    
    /**
     * Creates criteria objects from the build definition configuration element
     * properties, adds them to the CheckboxTableViewer and sets their "checked"
     * (enabled) state appropriatly.
     */
    protected void createNewCriteriasFromProperties() {

        int numCriterias = NotificationCriteriaFromBuildDefinitionHelper.getNumCriteriasProperty(fWorkingCopy);

        for (int i = 0; i < numCriterias; i++) {
            NotificationCriteriaFromBuildDefinition newCriteria = NotificationCriteriaFromBuildDefinitionHelper.getCriteriaFromProperties(fWorkingCopy, i);

            if (newCriteria != null) {
                fCriterias.add(newCriteria);
                fCriteriasViewer.add(newCriteria);
                if (newCriteria.isEnabled()) {
                    fCriteriasViewer.setChecked(newCriteria, true);
                }
                fCriteriasViewer.update(newCriteria, null);
                // set the selection again so that the summary can be updated
                fCriteriasViewer.setSelection(new StructuredSelection(newCriteria));
            }
        }
    }
    
    protected void createNewPropertiesFromExistingCriteria() {

        int numCriterias = NotificationCriteriaFromBuildDefinitionHelper.getNumCriteriasProperty(fWorkingCopy);

        // Delete all existing criteria properties
        for (int i = 0; i < numCriterias; i++) {
            NotificationCriteriaFromBuildDefinitionHelper.deleteCriteriaProperties(fWorkingCopy, i);
        }

        fWorkingCopy.setConfigurationProperty(IEmailNotificationConfigurationElement.ELEMENT_ID,
                IEmailNotificationConfigurationElement.PROPERTY_NUMBER_OF_CRITERIA, Integer.toString(0));

        // Find all existing criteria objects
        TableItem[] children = fCriteriasViewer.getTable().getItems();
        ArrayList<NotificationCriteriaFromBuildDefinition> criterias = new ArrayList<NotificationCriteriaFromBuildDefinition>();
        for (int i = 0; i < children.length; i++) {
            TableItem item = children[i];
            if (item.getData() instanceof NotificationCriteriaFromBuildDefinition) {
                criterias.add((NotificationCriteriaFromBuildDefinition) item.getData());
            }
        }

        for (int i = 0; i < criterias.size(); i++) {
            // Since the order is re-aranged, update the property name prefix
            // number in the criteria object first
            criterias.get(i).setPropertyNamePrefixNumber(i);
            NotificationCriteriaFromBuildDefinitionHelper.createPropertiesFromCriteria(fWorkingCopy, criterias.get(i));

            // Also update the numCriteria property with each entry
            fWorkingCopy.setConfigurationProperty(IEmailNotificationConfigurationElement.ELEMENT_ID,
                    IEmailNotificationConfigurationElement.PROPERTY_NUMBER_OF_CRITERIA, Integer.toString(i + 1));
        }
    }
}