/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.build.internal.ui.notification;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.accessibility.ACC;
import org.eclipse.swt.accessibility.AccessibleAdapter;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.ibm.team.build.common.notification.NotificationCriteria;
import com.ibm.team.build.common.notification.NotificationCriteriaFromProjectConfigHelper;
import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.process.ide.ui.AbstractConfigurationDataAspectEditor;
import com.ibm.team.process.ide.ui.IProcessAspectEditorSite;
import com.ibm.team.process.ide.ui.ProcessAspect;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Aspect editor for the configuration data for Build email notifications.
 */
public class NotificationProcessAspectEditor extends AbstractConfigurationDataAspectEditor {

    public final static String STATIC_CONFIGURATION_DATA_ID = "com.ibm.team.build.configuration.notification"; //$NON-NLS-1$

    public final static String PROCESS_ROLE_DEFINITIONS_NODE_NAME = "role-definitions"; //$NON-NLS-1$

    public final static String PROCESS_ROLE_ID_ATTRIBUTE_NAME = "role-id"; //$NON-NLS-1$

    private FormToolkit fToolkit;

    private CheckboxTableViewer fCriteriasViewer;

    private Button fAddCriteriaButton;
    private Button fEditCriteriaButton;
    private Button fRemoveCriteriaButton;

    private Text fCriteriaSummaryText;

    private ContributorFetchJob fSummaryJob;

    private List<String> fRoleDefinitions;

    private List<NotificationCriteria> fCriterias;

    private String fConfigurationDataId;

    public NotificationProcessAspectEditor(String configurationDataId) {
        if (!configurationDataId.equals(STATIC_CONFIGURATION_DATA_ID)) {
            throw new IllegalArgumentException("The configuration data id should be " //$NON-NLS-1$
                    + STATIC_CONFIGURATION_DATA_ID + ", not " //$NON-NLS-1$
                    + configurationDataId);
        }

        fConfigurationDataId = configurationDataId;

        fCriterias = new ArrayList<NotificationCriteria>();
        fRoleDefinitions = new ArrayList<String>();
    }

    private ITeamRepository getTeamRepository() {
        return (ITeamRepository) getAspect().getProcessContainerWorkingCopy().getUnderlyingProcessItem().getOrigin();
    }

    @Override
    public void init(IProcessAspectEditorSite site, ProcessAspect inputAspect) {
        super.init(site, inputAspect);
        readXML();
    }

    private void reset() {
        fCriterias.clear();
        fRoleDefinitions.clear();
    }

    private void readXML() {
        reset();
        readRoleDefinitionsFromProcess();

        ModelElement configurationData = getSite().getConfigurationData(fConfigurationDataId);
        // our configuration data may be undefined in the process XML
        if (configurationData != null) {
            fCriterias.addAll(NotificationCriteriaFromProjectConfigHelper.getAllCriterias(configurationData));
        }
    }

    /**
     * Processes the role definitions from the process's XML source.
     */
    private void readRoleDefinitionsFromProcess() {
        ModelElement configurationElement = getAspect().getConfigurationRoot();
        List<?> elements = configurationElement.getChildElements();
        for (Object element : elements) {
            ModelElement me = (ModelElement) element;
            if (me.getName().equals(PROCESS_ROLE_DEFINITIONS_NODE_NAME)) {
                elements = me.getChildElements();
                for (Object e : elements) {
                    String role = ((ModelElement) e).getAttribute(PROCESS_ROLE_ID_ATTRIBUTE_NAME);
                    if (role != null) {
                        fRoleDefinitions.add(role);
                    }
                }
                break;
            }
        }
    }

    @Override
    public void createControl(Composite parent, FormToolkit toolkit) {
        fToolkit = toolkit;

        GridLayout layout = new GridLayout();
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        parent.setLayout(layout);

        Composite composite = fToolkit.createComposite(parent);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        layout = new GridLayout(1, true);
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        composite.setLayout(layout);

        Section notificationCriteriaSection = fToolkit.createSection(composite, ExpandableComposite.TITLE_BAR | Section.DESCRIPTION);
        notificationCriteriaSection.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        notificationCriteriaSection.setText(Messages.NotificationProcessAspectEditor_NotificationSectionHeader);

        Composite outer = fToolkit.createComposite(notificationCriteriaSection);
        layout = new GridLayout(3, false);
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        outer.setLayout(layout);
        outer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        fCriteriasViewer = new CheckboxTableViewer(fToolkit.createTable(outer, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL
                | SWT.H_SCROLL | SWT.CHECK));
        fCriteriasViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 3));
        fCriteriasViewer.setContentProvider(new ArrayContentProvider());
        fCriteriasViewer.setLabelProvider(new LabelProvider() {
            @Override
            public String getText(Object element) {
                NotificationCriteria criteria = (NotificationCriteria) element;
                return criteria.getCriteriaName();
            }
        });
        fCriteriasViewer.setInput(fCriterias);

        for (NotificationCriteria criteria : fCriterias) {
            if (criteria.isEnabled()) {
                fCriteriasViewer.setChecked(criteria, true);
            }
        }

        fAddCriteriaButton = fToolkit.createButton(outer, Messages.NotificationProcessAspectEditor_CriteriaAddButton,
                SWT.PUSH);
        fAddCriteriaButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));

        fCriteriaSummaryText = fToolkit.createText(outer, null, SWT.LEAD | SWT.WRAP | SWT.V_SCROLL | SWT.READ_ONLY);
        fCriteriaSummaryText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 3));
        fCriteriaSummaryText.getAccessible().addAccessibleListener(new AccessibleAdapter() {
            @Override
            public void getName(AccessibleEvent e) {
                if (e.childID == ACC.CHILDID_SELF) {
                    e.result = Messages.NotificationProcessAspectEditor_CriteriaSummaryAccessibilityLabel;
                }
            }
        });

        fEditCriteriaButton = fToolkit.createButton(outer, Messages.NotificationProcessAspectEditor_CriteriaEditButton,
                SWT.PUSH);
        fEditCriteriaButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
        fEditCriteriaButton.setEnabled(false);

        fRemoveCriteriaButton = fToolkit.createButton(outer,
                Messages.NotificationProcessAspectEditor_CriteriaRemoveButton, SWT.PUSH);
        fRemoveCriteriaButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false, false));
        fRemoveCriteriaButton.setEnabled(false);

        notificationCriteriaSection.setClient(outer);

        attachListeners();

        fSummaryJob = new ContributorFetchJob(getTeamRepository(), fCriteriaSummaryText,
                new AbstractUserNameRetrievalRunnable() {
                    public void run() {
                        updateCriteriaSummary(getUserNameMap());
                    }
                });

        getSite().reflow();
    }

    protected void updateCriteriaSummary(Map<String, String> userNameMap) {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        NotificationCriteria criteria = (NotificationCriteria) selection.getFirstElement();
        if (criteria != null) {
            String summary = criteria.getCriteriaSummary(userNameMap, Text.DELIMITER);
            fCriteriaSummaryText.setText(summary);
        }
    }

    protected void handleCriteriaViewersSelectionChanged() {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        int size = selection.size();
        if (size == 1) {
            NotificationCriteria criteria = (NotificationCriteria) selection.getFirstElement();
            fSummaryJob.cancel();
            fSummaryJob.setUserIds(criteria.getUsers());
            fSummaryJob.schedule();

            String summary = criteria.getCriteriaSummary(null, Text.DELIMITER);
            fCriteriaSummaryText.setText(summary);

            // can only edit one criteria at a time
            fEditCriteriaButton.setEnabled(true);
        } else {
            fCriteriaSummaryText.setText(""); //$NON-NLS-1$

            // can only edit one criteria at a time
            fEditCriteriaButton.setEnabled(false);
        }

        // can't remove if nothing is selected
        fRemoveCriteriaButton.setEnabled(size != 0);
    }

    protected void handleCriteriaViewerCheckStateChanged(NotificationCriteria criteria, boolean checked) {
        criteria.setEnabled(checked);
        setDirty();
    }

    protected EditEmailNotificationCriteriaDialog createDialog(Control control, ITeamRepository teamRepository,
            IProcessAreaHandle processAreaHandle, NotificationCriteria criteria, String[] roleDefinitions) {
        return new EditEmailNotificationCriteriaDialog(control.getShell(), teamRepository, null, criteria,
                roleDefinitions);
    }

    private EditEmailNotificationCriteriaDialog createDialog(Control control, NotificationCriteria criteria) {
        return createDialog(control.getShell(), getTeamRepository(), null, criteria,
                fRoleDefinitions.toArray(new String[fRoleDefinitions.size()]));
    }

    protected void handleAddCriteriaButtonSelected() {
        NotificationCriteria criteria = new NotificationCriteria();
        EditEmailNotificationCriteriaDialog dialog = createDialog(fAddCriteriaButton, criteria);
        if (dialog.open() == Window.OK) {
            fCriterias.add(criteria);
            fCriteriasViewer.add(criteria);
            if (criteria.isEnabled()) {
                fCriteriasViewer.setChecked(criteria, true);
            }
            fCriteriasViewer.setSelection(new StructuredSelection(criteria), true);
            setDirty();
        }
    }

    protected void handleEditCriteriaButtonSelected() {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        editCriteria(selection);
    }

    protected void handleOpenCriteria(IStructuredSelection selection) {
        editCriteria(selection);
    }

    protected void editCriteria(IStructuredSelection selection) {
        if (selection.size() != 1) {
            return;
        }
        NotificationCriteria criteria = (NotificationCriteria) selection.getFirstElement();
        EditEmailNotificationCriteriaDialog dialog = createDialog(fEditCriteriaButton, criteria);
        if (dialog.open() == Window.OK) {
            fCriteriasViewer.update(criteria, null);
            // set the selection again so that the summary can be updated
            fCriteriasViewer.setSelection(new StructuredSelection(criteria));
            setDirty();
        }
    }

    protected void handleRemoveCriteriaButtonSelected() {
        IStructuredSelection selection = (IStructuredSelection) fCriteriasViewer.getSelection();
        fCriterias.removeAll(selection.toList());
        fCriteriasViewer.refresh();
        setDirty();
    }

    private void attachListeners() {
        fCriteriasViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                handleCriteriaViewersSelectionChanged();
            }
        });

        fCriteriasViewer.addCheckStateListener(new ICheckStateListener() {
            public void checkStateChanged(CheckStateChangedEvent event) {
                handleCriteriaViewerCheckStateChanged((NotificationCriteria) event.getElement(), event.getChecked());
            }
        });

        fCriteriasViewer.addOpenListener(new IOpenListener() {
            public void open(OpenEvent event) {
                handleOpenCriteria((IStructuredSelection) event.getSelection());
            }
        });

        fAddCriteriaButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleAddCriteriaButtonSelected();
            }
        });

        fEditCriteriaButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleEditCriteriaButtonSelected();
            }
        });

        fRemoveCriteriaButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                handleRemoveCriteriaButtonSelected();
            }
        });
    }

    @Override
    public void dispose() {
        // nothing to dispose
    }

    @Override
    public boolean needsApplyAndRevertButtons() {
        return false;
    }

    @Override
    protected boolean saveState(IMemento memento) {
        if (fCriterias.isEmpty()) {
            // nothing to save
            return false;
        }
        NotificationCriteriaFromProjectConfigHelper.saveCriteriasIntoMemento(fConfigurationDataId, memento, fCriterias);
        return true;
    }


    @Override
    public void revert() {
        super.revert();
        readXML();

        fCriteriasViewer.refresh();

        // check state is cleared when refreshed, need to set it back
        for (NotificationCriteria criteria : fCriterias) {
            if (criteria.isEnabled()) {
                fCriteriasViewer.setChecked(criteria, true);
            }
        }

        fEditCriteriaButton.setEnabled(false);
        fRemoveCriteriaButton.setEnabled(false);
    }

    protected List<NotificationCriteria> getCriterias() {
        return fCriterias;
    }

    protected Collection<String> getRoleDefinitions() {
        return fRoleDefinitions;
    }

    protected CheckboxTableViewer getCriteriasViewer() {
        return fCriteriasViewer;
    }

    protected Button getAddCriteriaButton() {
        return fAddCriteriaButton;
    }

    protected Button getEditCriteriaButton() {
        return fEditCriteriaButton;
    }

    protected Button getRemoveCriteriaButton() {
        return fRemoveCriteriaButton;
    }

}
