/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.build.internal.ui.notification;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;

import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Dialog that allows the user to choose role types from a given process area.
 * 
 * @since 0.8
 */
public class RoleSelectionDialog extends ElementListSelectionDialog {

    private Job fRoleFetchingJob;
    private boolean fJobRequired = false;;
    private boolean fFetchingDone;
    private final ITeamRepository fTeamRepository;
    private final IProcessAreaHandle fProcessAreaHandle;
    private final String[] fRoleDefinitions;

    /**
     * Creates the dialog that allows the user to choose role types from a given
     * process area.
     * 
     * @param parent
     *            The parent shell for this dialog.
     * 
     * @param teamRepository
     *            The repository of the process area.
     * @param processAreaHandle
     *            The handle to the current process area.
     * @param roleDefinitions
     *            The role types to show. If this is <code>null</code> or empty,
     *            a job will be scheduled to fetch role types from the given
     *            process area.
     */
    public RoleSelectionDialog(Shell parent, ITeamRepository teamRepository, IProcessAreaHandle processAreaHandle,
            String[] roleDefinitions) {
        super(parent, new LabelProvider());
        fTeamRepository = teamRepository;
        fProcessAreaHandle = processAreaHandle;
        fRoleDefinitions = roleDefinitions;
        String title = Messages.EditEmailNotificationCriteriaDialog_RolesSelectionDialogTitle;
        String message = Messages.EditEmailNotificationCriteriaDialog_RolesSelectionDialogMessage;
        init(title, message);
    }

    protected void init(String title, String message) {
        setTitle(title);
        setMessage(message);
        setStatusLineAboveButtons(true);
        setMultipleSelection(true);
        setEmptyListMessage(Messages.EditEmailNotificationCriteriaDialog_RolesSelectionDialogNoRolesFoundMessage);

        if (fRoleDefinitions == null || fRoleDefinitions.length == 0) {
            fJobRequired = true;
            fRoleFetchingJob = createFetchRoleJob();
        }
    }

    /**
     * Create a new job that fetches the roles.
     * 
     * @return the job, never <code>null</code>.
     */
    protected Job createFetchRoleJob() {
        return new RoleTypesFetchJob(fTeamRepository, fProcessAreaHandle, this);
    }

    @Override
    public void setListElements(Object[] elements) {
        super.setListElements(elements);
    }

    /*
     * Hack to keep our status showing in the dialog.
     */
    IStatus fStatus = null;
    IStatus fRealStatus = null;

    @Override
    protected void updateStatus(IStatus status) {
        fRealStatus = status;
        super.updateStatus(status);
    }

    /*
     * Can be used to set an error or warning status. We have to keep track of
     * this status and keep reapplying it whenever the real status is updated
     * because the selection changed.
     */
    protected void setStatus(IStatus status) {
        super.updateStatus(status);
        fStatus = status;
    }

    @Override
    protected void handleSelectionChanged() {
        if (fJobRequired) {
            if (fFetchingDone) {
                super.handleSelectionChanged();
                /*
                 * The selection change may have overwritten our status. If we
                 * set one, reset it by calling updateStatus, but keep the
                 * correct button enablement by calling
                 * updateButtonsEnablementState with the "real" status.
                 */
                if (fStatus != null) {
                    super.updateStatus(fStatus);
                    updateButtonsEnableState(fRealStatus);
                }
            } else {
                getOkButton().setEnabled(false);
            }
        } else {
            // A job will not be required if we were already given a list of
            // roles
            super.handleSelectionChanged();
        }
    }

    @Override
    public int open() {
        if (fJobRequired) {
            fRoleFetchingJob.schedule();
            fJobRequired = true;
            setElements(new Object[] { Messages.EditEmailNotificationCriteriaDialog_RolesSelectionDialogFetchingRolesMessage });
        } else {
            setElements(fRoleDefinitions);
        }
        return super.open();
    }

    @Override
    public boolean close() {
        if (fRoleFetchingJob != null) {
            fRoleFetchingJob.cancel();
        }
        return super.close();
    }

    /**
     * Set the flag that indicates that the build definitions have been fetched.
     */
    public void setFetchingDone() {
        fFetchingDone = true;
    }

    /**
     * @return whether fetching is done
     */
    public boolean isFetchingDone() {
        return fFetchingDone;
    }
}
